@@ -70,10 +70,6 @@
 /* Maximum poll wait time for child processes, in milliseconds */
 #define MAX_POLL_WAIT 500
 
-#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
-#define POLLRDHUP POLLHUP
-#endif
-
 /* Default and minimum timeout parameters for the capmc command */
 #define DEFAULT_CAPMC_RETRIES 4
 #define DEFAULT_CAPMC_TIMEOUT 60000	/* 60 seconds */
@@ -136,8 +132,7 @@ static s_p_hashtbl_t *_config_make_tbl(char *filename)
 		return tbl;
 	}
 
-	if (s_p_parse_file(tbl, NULL, filename, false, NULL, false) ==
-			   SLURM_ERROR) {
+	if (s_p_parse_file(tbl, NULL, filename, 0, NULL) == SLURM_ERROR) {
 		error("%s: s_p_parse_file error: %s", prog_name,
 		      slurm_strerror(slurm_get_errno()));
 		s_p_hashtbl_destroy(tbl);
