@@ -151,11 +151,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 	if (conf->job_container_plugin)
 		STORE_FIELD(hv, conf, job_container_plugin, charp);
 
-	if (conf->job_credential_private_key)
-		STORE_FIELD(hv, conf, job_credential_private_key, charp);
-	if (conf->job_credential_public_certificate)
-		STORE_FIELD(hv, conf, job_credential_public_certificate, charp);
-
 	STORE_FIELD(hv, conf, job_file_append, uint16_t);
 
 	STORE_FIELD(hv, conf, job_requeue, uint16_t);
@@ -284,9 +279,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 
 	STORE_FIELD(hv, conf, ret2service, uint16_t);
 
-	if (conf->route_plugin)
-		STORE_FIELD(hv, conf, route_plugin, charp);
-
 	if (conf->sched_logfile)
 		STORE_FIELD(hv, conf, sched_logfile, charp);
 	STORE_FIELD(hv, conf, sched_log_level, uint16_t);
@@ -486,8 +478,6 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, job_comp_type, charp, FALSE);
 	FETCH_FIELD(hv, conf, job_comp_user, charp, FALSE);
 	FETCH_FIELD(hv, conf, job_container_plugin, charp, FALSE);
-	FETCH_FIELD(hv, conf, job_credential_private_key, charp, FALSE);
-	FETCH_FIELD(hv, conf, job_credential_public_certificate, charp, FALSE);
 	FETCH_FIELD(hv, conf, job_file_append, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, job_requeue, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, job_submit_plugins, charp, FALSE);
@@ -555,7 +545,6 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, resv_over_run, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, resv_prolog, charp, FALSE);
 	FETCH_FIELD(hv, conf, ret2service, uint16_t, TRUE);
-	FETCH_FIELD(hv, conf, route_plugin, charp, FALSE);
 
 	FETCH_FIELD(hv, conf, sched_logfile, charp, FALSE);
 	FETCH_FIELD(hv, conf, sched_log_level, uint16_t, TRUE);
