@@ -66,7 +66,7 @@ static int _job_resrcs_to_hv(job_info_t *job_info, HV *hv)
 	uint64_t *last_mem_alloc_ptr = NULL;
 	uint64_t last_mem_alloc = NO_VAL64;
 	char *last_hosts;
-	hostlist_t hl, hl_last;
+	hostlist_t *hl, *hl_last;
 	uint32_t threads;
 
 	if (!job_resrcs || !job_resrcs->core_bitmap
@@ -218,6 +218,8 @@ job_info_to_hv(job_info_t *job_info, HV *hv)
 	hv_store_sv(hv, "exc_node_inx", newRV_noinc((SV*)av));
 
 	STORE_FIELD(hv, job_info, exit_code, uint32_t);
+	if (job_info->extra)
+		STORE_FIELD(hv, job_info, extra, charp);
 	if (job_info->features)
 		STORE_FIELD(hv, job_info, features, charp);
 	if (job_info->tres_per_node)
@@ -282,14 +284,13 @@ job_info_to_hv(job_info_t *job_info, HV *hv)
 	STORE_FIELD(hv, job_info, restart_cnt, uint16_t);
 	if(job_info->resv_name)
 		STORE_FIELD(hv, job_info, resv_name, charp);
-	STORE_PTR_FIELD(hv, job_info, select_jobinfo, "Slurm::dynamic_plugin_data_t");
 	STORE_PTR_FIELD(hv, job_info, job_resrcs, "Slurm::job_resources_t");
 	STORE_FIELD(hv, job_info, shared, uint16_t);
 	STORE_FIELD(hv, job_info, show_flags, uint16_t);
 	STORE_FIELD(hv, job_info, start_time, time_t);
 	if(job_info->state_desc)
 		STORE_FIELD(hv, job_info, state_desc, charp);
-	STORE_FIELD(hv, job_info, state_reason, uint16_t);
+	STORE_FIELD(hv, job_info, state_reason, uint32_t);
 	if(job_info->std_in)
 		STORE_FIELD(hv, job_info, std_in, charp);
 	if(job_info->std_out)
@@ -354,6 +355,7 @@ hv_to_job_info(HV *hv, job_info_t *job_info)
 		/* nothing to do */
 	}
 	FETCH_FIELD(hv, job_info, exit_code, uint32_t, TRUE);
+	FETCH_FIELD(hv, job_info, extra, charp, FALSE);
 	FETCH_FIELD(hv, job_info, features, charp, FALSE);
 	FETCH_FIELD(hv, job_info, tres_per_node, charp, FALSE);
 	FETCH_FIELD(hv, job_info, group_id, uint32_t, TRUE);
@@ -416,13 +418,12 @@ hv_to_job_info(HV *hv, job_info_t *job_info)
 	FETCH_FIELD(hv, job_info, resize_time, time_t, TRUE);
 	FETCH_FIELD(hv, job_info, restart_cnt, uint16_t, TRUE);
 	FETCH_FIELD(hv, job_info, resv_name, charp, FALSE);
-	FETCH_PTR_FIELD(hv, job_info, select_jobinfo, "Slurm::dynamic_plugin_data_t", FALSE);
 	FETCH_PTR_FIELD(hv, job_info, job_resrcs, "Slurm::job_resources_t", FALSE);
 	FETCH_FIELD(hv, job_info, shared, uint16_t, TRUE);
 	FETCH_FIELD(hv, job_info, show_flags, uint16_t, TRUE);
 	FETCH_FIELD(hv, job_info, start_time, time_t, TRUE);
 	FETCH_FIELD(hv, job_info, state_desc, charp, FALSE);
-	FETCH_FIELD(hv, job_info, state_reason, uint16_t, TRUE);
+	FETCH_FIELD(hv, job_info, state_reason, uint32_t, TRUE);
 	FETCH_FIELD(hv, job_info, std_in, charp, FALSE);
 	FETCH_FIELD(hv, job_info, std_out, charp, FALSE);
 	FETCH_FIELD(hv, job_info, std_err, charp, FALSE);
