@@ -106,7 +106,6 @@ hv_to_cluster_cond(HV* hv, slurmdb_cluster_cond_t* cluster_cond)
     FETCH_FIELD(hv, cluster_cond, with_usage,     uint16_t, FALSE);
 
     FETCH_LIST_FIELD(hv, cluster_cond, cluster_list);
-    FETCH_LIST_FIELD(hv, cluster_cond, plugin_id_select_list);
     FETCH_LIST_FIELD(hv, cluster_cond, rpc_version_list);
 
     return 0;
@@ -270,8 +269,7 @@ report_acct_grouping_to_hv(slurmdb_report_acct_grouping_t* rec, HV* hv)
 
     STORE_FIELD(hv, rec, acct,     charp);
     STORE_FIELD(hv, rec, count,    uint32_t);
-    STORE_FIELD(hv, rec, lft,      uint32_t);
-    STORE_FIELD(hv, rec, rgt,      uint32_t);
+    STORE_FIELD(hv, rec, lineage,  charp);
 
     my_av = (AV*)sv_2mortal((SV*)newAV());
     if (rec->groups) {
@@ -707,12 +705,8 @@ job_rec_to_hv(slurmdb_job_rec_t* rec, HV* hv)
     slurmdb_step_rec_t *step;
     ListIterator itr = NULL;
     AV* steps_av = (AV*)sv_2mortal((SV*)newAV());
-    HV* stats_hv = (HV*)sv_2mortal((SV*)newHV());
     HV* step_hv;
 
-    stats_to_hv(&rec->stats, stats_hv);
-    hv_store_sv(hv, "stats", newRV((SV*)stats_hv));
-
     if (rec->steps) {
 	itr = slurm_list_iterator_create(rec->steps);
 	while ((step = slurm_list_next(itr))) {
@@ -740,11 +734,12 @@ job_rec_to_hv(slurmdb_job_rec_t* rec, HV* hv)
     STORE_FIELD(hv, rec, end,             time_t);
     STORE_FIELD(hv, rec, env,             charp);
     STORE_FIELD(hv, rec, exitcode,        uint32_t);
+    STORE_FIELD(hv, rec, failed_node,     charp);
     /*STORE_FIELD(hv, rec, first_step_ptr,  void*);*/
     STORE_FIELD(hv, rec, gid,             uint32_t);
     STORE_FIELD(hv, rec, jobid,           uint32_t);
     STORE_FIELD(hv, rec, jobname,         charp);
-    STORE_FIELD(hv, rec, lft,             uint32_t);
+    STORE_FIELD(hv, rec, lineage,         charp);
     STORE_FIELD(hv, rec, partition,       charp);
     STORE_FIELD(hv, rec, nodes,           charp);
     STORE_FIELD(hv, rec, priority,        uint32_t);
@@ -766,7 +761,6 @@ job_rec_to_hv(slurmdb_job_rec_t* rec, HV* hv)
     STORE_FIELD(hv, rec, timelimit,       uint32_t);
     STORE_FIELD(hv, rec, tot_cpu_sec,     uint32_t);
     STORE_FIELD(hv, rec, tot_cpu_usec,    uint32_t);
-    STORE_FIELD(hv, rec, track_steps,     uint16_t);
     STORE_FIELD(hv, rec, tres_alloc_str,  charp);
     STORE_FIELD(hv, rec, tres_req_str,    charp);
     STORE_FIELD(hv, rec, uid,             uint32_t);
