@@ -166,7 +166,7 @@ extern void slurm_allocation_msg_thr_destroy(
 static void _handle_node_fail(struct allocation_msg_thread *msg_thr,
 			      slurm_msg_t *msg)
 {
-	srun_node_fail_msg_t *nf = (srun_node_fail_msg_t *)msg->data;
+	srun_node_fail_msg_t *nf = msg->data;
 
 	if (msg_thr->callback.node_fail != NULL)
 		(msg_thr->callback.node_fail)(nf);
@@ -180,7 +180,7 @@ static void _handle_node_fail(struct allocation_msg_thread *msg_thr,
 static void _handle_timeout(struct allocation_msg_thread *msg_thr,
 			    slurm_msg_t *msg)
 {
-	srun_timeout_msg_t *to = (srun_timeout_msg_t *)msg->data;
+	srun_timeout_msg_t *to = msg->data;
 
 	debug3("received timeout message");
 
@@ -191,7 +191,7 @@ static void _handle_timeout(struct allocation_msg_thread *msg_thr,
 static void _handle_user_msg(struct allocation_msg_thread *msg_thr,
 			     slurm_msg_t *msg)
 {
-	srun_user_msg_t *um = (srun_user_msg_t *)msg->data;
+	srun_user_msg_t *um = msg->data;
 	debug3("received user message");
 
 	if (msg_thr->callback.user_msg != NULL)
@@ -208,7 +208,7 @@ static void _handle_ping(struct allocation_msg_thread *msg_thr,
 static void _handle_job_complete(struct allocation_msg_thread *msg_thr,
 				 slurm_msg_t *msg)
 {
-	srun_job_complete_msg_t *comp = (srun_job_complete_msg_t *)msg->data;
+	srun_job_complete_msg_t *comp = msg->data;
 	debug3("job complete message received");
 
 	if (msg_thr->callback.job_complete != NULL)
@@ -218,7 +218,7 @@ static void _handle_job_complete(struct allocation_msg_thread *msg_thr,
 static void _handle_suspend(struct allocation_msg_thread *msg_thr,
 			    slurm_msg_t *msg)
 {
-	suspend_msg_t *sus_msg = (suspend_msg_t *)msg->data;
+	suspend_msg_t *sus_msg = msg->data;
 	debug3("received suspend message");
 
 	if (msg_thr->callback.job_suspend != NULL)
@@ -228,7 +228,7 @@ static void _handle_suspend(struct allocation_msg_thread *msg_thr,
 static void _net_forward(struct allocation_msg_thread *msg_thr,
 			 slurm_msg_t *forward_msg)
 {
-	net_forward_msg_t *msg = (net_forward_msg_t *) forward_msg->data;
+	net_forward_msg_t *msg = forward_msg->data;
 	int *local, *remote;
 	eio_obj_t *e1, *e2;
 
@@ -329,8 +329,8 @@ _handle_msg(void *arg, slurm_msg_t *msg)
 		_net_forward(msg_thr, msg);
 		break;
 	default:
-		error("%s: received spurious message type: %u",
-		      __func__, msg->msg_type);
+		error("%s: received spurious message type: %s",
+		      __func__, rpc_num2string(msg->msg_type));
 		break;
 	}
 	return;
