@@ -50,13 +50,14 @@
 #include "src/common/list.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_auth.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_resource_info.h"
-#include "src/common/slurm_selecttype_info.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/auth.h"
+#include "src/interfaces/select.h"
+
 /* Local functions */
 static void _write_group_header(FILE* out, char * header);
 static void _write_key_pairs(FILE* out, void *key_pairs);
@@ -107,7 +108,7 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 			    partition_info_msg_t * part_info_ptr)
 {
 	int i = 0;
-	char time_str[32];
+	char time_str[256];
 	char *tmp_str = NULL;
 	char *base_path = NULL;
 	char *path = NULL;
@@ -117,7 +118,7 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 	partition_info_t *p = NULL;
 	struct records {
 	  char *rec;
-	  hostlist_t hostlist;
+	  hostlist_t *hostlist;
 	  struct records *next;
 	} *rp = NULL;
 	struct records *crp;
@@ -334,6 +335,10 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 			fprintf(fp, " MaxCPUsPerNode=%u",
 				p[i].max_cpus_per_node);
 
+		if (p[i].max_cpus_per_socket != INFINITE)
+			fprintf(fp, " MaxCPUsPerSocket=%u",
+				p[i].max_cpus_per_socket);
+
 		if (p[i].max_mem_per_cpu & MEM_PER_CPU) {
 		        if (p[i].max_mem_per_cpu != MEM_PER_CPU)
 		                fprintf(fp, " MaxMemPerCPU=%"PRIu64"",
@@ -384,6 +389,9 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 		else if (p[i].cr_type & CR_SOCKET)
 			fprintf(fp, " SelectTypeParameters=CR_SOCKET");
 
+		if (p[i].flags & PART_FLAG_PDOI)
+			fprintf(fp, " PowerDownOnIdle=YES");
+
 		force = p[i].max_share & SHARED_FORCE;
 		val = p[i].max_share & (~SHARED_FORCE);
 		if (val == 0)
@@ -461,7 +469,7 @@ static void _print_config_plugin_params_list(FILE* out, List l, char *title)
 void slurm_print_ctl_conf ( FILE* out,
 			    slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr )
 {
-	char time_str[32], tmp_str[128];
+	char time_str[32], tmp_str[256];
 	void *ret_list = NULL;
 	char *select_title = "Select Plugin Configuration";
 	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
@@ -500,11 +508,6 @@ void slurm_print_ctl_conf ( FILE* out,
 		 (List) slurm_ctl_conf_ptr->node_features_conf, tmp2_str);
 	xfree(tmp2_str);
 
-	xstrcat(tmp2_str, "\nSlurmctld Plugstack Plugins Configuration:");
-	_print_config_plugin_params_list(out,
-		 (List) slurm_ctl_conf_ptr->slurmctld_plugstack_conf, tmp2_str);
-	xfree(tmp2_str);
-
 	slurm_print_key_pairs(out, slurm_ctl_conf_ptr->select_conf_key_pairs,
 			      select_title);
 
@@ -513,9 +516,10 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 {
 	List ret_list = NULL;
 	config_key_pair_t *key_pair;
-	char tmp_str[128];
+	char tmp_str[256];
 	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
 	int i;
+	bool format_stderr = false;
 
 	if ( slurm_ctl_conf_ptr == NULL )
 		return NULL;
@@ -582,6 +586,9 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJE)
 		xstrfmtcat(key_pair->value, "%sjob_env",
 			   key_pair->value ? "," : "");
+	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJX)
+		xstrfmtcat(key_pair->value, "%sjob_extra",
+			   key_pair->value ? "," : "");
 	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJS)
 		xstrfmtcat(key_pair->value, "%sjob_script",
 			   key_pair->value ? "," : "");
@@ -920,6 +927,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->job_comp_loc);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(*key_pair));
+	key_pair->name = xstrdup("JobCompParams");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->job_comp_params);
+	list_append(ret_list, key_pair);
+
 	snprintf(tmp_str, sizeof(tmp_str), "%u",
 		 slurm_ctl_conf_ptr->job_comp_port);
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -942,18 +954,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->job_container_plugin);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("JobCredentialPrivateKey");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->
-				  job_credential_private_key);
-	list_append(ret_list, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("JobCredentialPublicCertificate");
-	key_pair->value = xstrdup(
-		slurm_ctl_conf_ptr->job_credential_public_certificate);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("JobDefaults");
 	key_pair->value =
@@ -998,11 +998,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->launch_params);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("LaunchType");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->launch_type);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("Licenses");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->licenses);
@@ -1010,6 +1005,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("LogTimeFormat");
+	if (slurm_ctl_conf_ptr->log_fmt & LOG_FMT_FORMAT_STDERR) {
+		format_stderr = true;
+		slurm_ctl_conf_ptr->log_fmt &= (~LOG_FMT_FORMAT_STDERR);
+	}
+
 	if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_ISO8601_MS)
 		key_pair->value = xstrdup("iso8601_ms");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_ISO8601)
@@ -1018,12 +1018,19 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 		key_pair->value = xstrdup("rfc5424_ms");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_RFC5424)
 		key_pair->value = xstrdup("rfc5424");
+	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_RFC3339)
+		key_pair->value = xstrdup("rfc3339");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_CLOCK)
 		key_pair->value = xstrdup("clock");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_SHORT)
 		key_pair->value = xstrdup("short");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_THREAD_ID)
 		key_pair->value = xstrdup("thread_id");
+
+	if (format_stderr) {
+		xstrcat(key_pair->value, ",format_stderr");
+		slurm_ctl_conf_ptr->log_fmt |= LOG_FMT_FORMAT_STDERR;
+	}
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -1043,6 +1050,13 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
+	snprintf(tmp_str, sizeof(tmp_str), "%u",
+		 slurm_ctl_conf_ptr->max_batch_requeue);
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("MaxBatchRequeue");
+	key_pair->value = xstrdup(tmp_str);
+	list_append(ret_list, key_pair);
+
 	snprintf(tmp_str, sizeof(tmp_str), "%u",
 		 slurm_ctl_conf_ptr->max_dbd_msgs);
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -1195,6 +1209,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 						      preempt_mode));
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("PreemptParameters");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->preempt_params);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("PreemptType");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->preempt_type);
@@ -1462,11 +1481,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("RoutePlugin");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->route_plugin);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SchedulerParameters");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->sched_params);
@@ -1666,11 +1680,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->slurmctld_pidfile);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("SlurmctldPlugstack");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->slurmctld_plugstack);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SLURM_CONF");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->slurm_conf);
@@ -1717,6 +1726,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->suspend_exc_parts);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("SuspendExcStates");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->suspend_exc_states);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SuspendProgram");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->suspend_program);
@@ -1924,8 +1938,23 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 
 	if (cluster_flags & CLUSTER_FLAG_MULTSD) {
 		if ((this_addr = getenv("SLURMD_NODENAME"))) {
-			slurm_conf_get_addr(this_addr, &req_msg.address,
-					    req_msg.flags);
+			if (slurm_conf_get_addr(this_addr, &req_msg.address,
+						req_msg.flags)) {
+				/*
+				 * The node isn't in the conf, see if the
+				 * controller has an address for it.
+				 */
+				slurm_node_alias_addrs_t *alias_addrs;
+				if (!slurm_get_node_alias_addrs(this_addr,
+								&alias_addrs)) {
+					add_remote_nodes_to_conf_tbls(
+						alias_addrs->node_list,
+						alias_addrs->node_addrs);
+				}
+				slurm_free_node_alias_addrs(alias_addrs);
+				slurm_conf_get_addr(this_addr, &req_msg.address,
+						    req_msg.flags);
+			}
 		} else {
 			this_addr = "localhost";
 			slurm_set_addr(&req_msg.address, slurm_conf.slurmd_port,
@@ -1951,7 +1980,7 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 
 	rc = slurm_send_recv_node_msg(&req_msg, &resp_msg, 0);
 
-	if ((rc != 0) || !resp_msg.auth_cred) {
+	if (rc != SLURM_SUCCESS) {
 		error("slurm_slurmd_info: %m");
 		if (resp_msg.auth_cred)
 			auth_g_destroy(resp_msg.auth_cred);
@@ -1987,7 +2016,7 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 void slurm_print_slurmd_status (FILE* out,
 				slurmd_status_t * slurmd_status_ptr)
 {
-	char time_str[32];
+	char time_str[256];
 
 	if (slurmd_status_ptr == NULL )
 		return ;
@@ -2196,6 +2225,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 
 		if (!xstrcasecmp(key_pair->name, "SuspendExcNodes") ||
 		    !xstrcasecmp(key_pair->name, "SuspendExcParts") ||
+		    !xstrcasecmp(key_pair->name, "SuspendExcStates") ||
 		    !xstrcasecmp(key_pair->name, "SuspendProgram") ||
 		    !xstrcasecmp(key_pair->name, "SuspendRate") ||
 		    !xstrcasecmp(key_pair->name, "SuspendTime") ||
@@ -2214,6 +2244,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 		    !xstrcasecmp(key_pair->name, "SchedulerType") ||
 		    !xstrcasecmp(key_pair->name, "SlurmSchedLogLevel") ||
 		    !xstrcasecmp(key_pair->name, "PreemptMode") ||
+		    !xstrcasecmp(key_pair->name, "PreemptParameters") ||
 		    !xstrcasecmp(key_pair->name, "PreemptType") ||
 		    !xstrcasecmp(key_pair->name, "PreemptExemptTime") ||
 		    !xstrcasecmp(key_pair->name, "PriorityType") ||
