@@ -41,11 +41,11 @@
 #include <ctype.h>
 #include <errno.h>
 #include <grp.h>
+#include <limits.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <time.h>
@@ -59,8 +59,8 @@
 #include "src/common/macros.h"
 #include "src/common/parse_time.h"
 #include "src/common/proc_args.h"
-#include "src/common/select.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/strlcpy.h"
 #include "src/common/uid.h"
@@ -139,52 +139,87 @@ static void _free_node_info(void)
 static void _fname_format(char *buf, int buf_size, job_info_t * job_ptr,
 			  char *fname)
 {
-	char *ptr, *tmp, *tmp2 = NULL, *user;
+	char *q, *p, *tmp, *tmp2 = NULL, *user;
+	unsigned int wid, offset;
 
 	tmp = xstrdup(fname);
-	while ((ptr = strstr(tmp, "%A"))) {	/* Array job ID */
-		ptr[0] = '\0';
-		if (job_ptr->array_task_id == NO_VAL) {
-			/* Not a job array */
-			xstrfmtcat(tmp2, "%s%u%s", tmp, job_ptr->job_id, ptr+2);
-		} else {
-			xstrfmtcat(tmp2, "%s%u%s", tmp, job_ptr->array_job_id,
-				   ptr+2);
-		}
-		xfree(tmp);	/* transfer the results */
-		tmp = tmp2;
-		tmp2 = NULL;
-	}
-	while ((ptr = strstr(tmp, "%a"))) {	/* Array task ID */
-		ptr[0] = '\0';
-		xstrfmtcat(tmp2, "%s%u%s", tmp, job_ptr->array_task_id, ptr+2);
-		xfree(tmp);	/* transfer the results */
-		tmp = tmp2;
-		tmp2 = NULL;
-	}
-	while ((ptr = strstr(tmp, "%j"))) {	/* Job ID */
-		ptr[0] = '\0';
-		xstrfmtcat(tmp2, "%s%u%s", tmp, job_ptr->job_id, ptr+2);
-		xfree(tmp);	/* transfer the results */
-		tmp = tmp2;
-		tmp2 = NULL;
-	}
-	while ((ptr = strstr(tmp, "%u"))) {	/* User name */
-		ptr[0] = '\0';
-		user = uid_to_string((uid_t) job_ptr->user_id);
-		xstrfmtcat(tmp2, "%s%s%s", tmp, user, ptr+2);
-		xfree(user);
-		xfree(tmp);	/* transfer the results */
-		tmp = tmp2;
-		tmp2 = NULL;
-	}
-	xstrsubstituteall(tmp, "%x", job_ptr->name);	/* Job name */
+	q = p  = tmp;
+	while (*p != '\0') {
+		if (*p == '%') {
+			offset = 1;
+			wid = 0;
+			if (*(p + 1) == '%') {
+				p++;
+				xmemcat(tmp2, q, p);
+				q = ++p;
+				continue;
+			}
+			if (isdigit(*(p + 1))) {
+				unsigned long in_width = 0;
+				if ((in_width = strtoul(p + 1, &p, 10)) > 9) {
+					/* Remove % and double digit 10 */
+					wid = 10;
+					offset = 3;
+				} else {
+					wid = (unsigned int)in_width;
+					offset = 2;
+				}
+				if (*p == '\0')
+					break;
+			} else
+				p++;
 
-	if (tmp[0] == '/')
-		snprintf(buf, buf_size, "%s", tmp);
-	else
-		snprintf(buf, buf_size, "%s/%s", job_ptr->work_dir, tmp);
+			switch (*p) {
+				case 'A': /* Array job ID */
+					xmemcat(tmp2, q, p - offset);
+					q = p + 1;
+					if (job_ptr->array_task_id == NO_VAL) {
+						/* Not a job array */
+						xstrfmtcat(tmp2, "%0*u", wid,
+							   job_ptr->job_id);
+					} else {
+						xstrfmtcat(tmp2, "%0*u", wid,
+							job_ptr->array_job_id);
+					}
+					break;
+				case 'a': /* Array task ID */
+					xmemcat(tmp2, q, p - offset);
+					xstrfmtcat(tmp2, "%0*u", wid,
+						   job_ptr->array_task_id);
+					q = p + 1;
+					break;
+				case 'j': /* Job ID */
+					xmemcat(tmp2, q, p - offset);
+					xstrfmtcat(tmp2, "%0*u", wid,
+						   job_ptr->job_id);
+					q = p + 1;
+					break;
+				case 'u': /* User name */
+					xmemcat(tmp2, q, p - offset);
+					user = uid_to_string(
+						(uid_t) job_ptr->user_id);
+					xstrfmtcat(tmp2, "%s", user);
+					xfree(user);
+					q = p + 1;
+					break;
+				case 'x':
+					xmemcat(tmp2, q, p - offset);
+					xstrfmtcat(tmp2, "%s", job_ptr->name);
+					q = p + 1;
+					break;
+			}
+		} else
+			p++;
+	}
+	if (p != q)
+		xmemcat(tmp2, q, p);
 	xfree(tmp);
+
+	if (tmp2[0] == '/')
+		snprintf(buf, buf_size, "%s", tmp2);
+	else
+		snprintf(buf, buf_size, "%s/%s", job_ptr->work_dir, tmp2);
+	xfree(tmp2);
 }
 
 /* Given a job record pointer, return its stderr path in buf */
@@ -278,6 +313,14 @@ extern uint32_t slurm_xlate_job_id(char *job_id_str)
 				job_id = job_ptr->job_id;
 				break;
 			}
+			if (job_ptr->array_bitmap) {
+				int array_len = bit_size(job_ptr->array_bitmap);
+				if (array_id < array_len &&
+				    bit_test(job_ptr->array_bitmap, array_id)) {
+					job_id = job_ptr->job_id;
+					break;
+				}
+			}
 		}
 
 		slurm_free_job_info_msg(resp);
@@ -310,7 +353,7 @@ slurm_print_job_info_msg ( FILE* out, job_info_msg_t *jinfo, int one_liner )
 {
 	int i;
 	job_info_t *job_ptr = jinfo->job_array;
-	char time_str[32];
+	char time_str[256];
 
 	slurm_make_time_str ((time_t *)&jinfo->last_update, time_str,
 		sizeof(time_str));
@@ -361,13 +404,14 @@ extern char *
 slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 {
 	int i, j, k;
-	char time_str[32], *group_name, *user_name;
+	char time_str[256], *group_name, *user_name;
 	char *gres_last = "", tmp1[128], tmp2[128];
 	char *tmp6_ptr;
 	char tmp_line[1024 * 128];
-	char tmp_path[MAXPATHLEN];
+	char tmp_path[PATH_MAX];
 	uint16_t exit_status = 0, term_sig = 0;
 	job_resources_t *job_resrcs = job_ptr->job_resrcs;
+	char *job_size_str = NULL;
 	char *out = NULL;
 	time_t run_time;
 	uint32_t min_nodes, max_nodes = 0;
@@ -382,7 +426,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	uint64_t *last_mem_alloc_ptr = NULL;
 	uint64_t last_mem_alloc = NO_VAL64;
 	char *last_hosts;
-	hostlist_t hl, hl_last;
+	hostlist_t *hl, *hl_last;
 	uint32_t threads;
 	char *line_end = (one_liner) ? " " : "\n   ";
 
@@ -450,6 +494,9 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	} else
 		xstrfmtcat(out, "Reason=%s ", job_reason_string(job_ptr->state_reason));
 
+	if (job_ptr->failed_node)
+		xstrfmtcat(out, "FailedNode=%s ", job_ptr->failed_node);
+
 	xstrfmtcat(out, "Dependency=%s", job_ptr->dependency);
 	xstrcat(out, line_end);
 
@@ -477,7 +524,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	}
 
 	/****** Line 6 ******/
-	if (IS_JOB_PENDING(job_ptr))
+	if (IS_JOB_PENDING(job_ptr) || !job_ptr->start_time)
 		run_time = 0;
 	else if (IS_JOB_SUSPENDED(job_ptr))
 		run_time = job_ptr->pre_sus_time;
@@ -647,6 +694,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	if (IS_JOB_PENDING(job_ptr)) {
 		min_nodes = job_ptr->num_nodes;
 		max_nodes = job_ptr->max_nodes;
+		job_size_str = job_ptr->job_size_str;
 		if (max_nodes && (max_nodes < min_nodes))
 			min_nodes = max_nodes;
 	} else {
@@ -654,7 +702,10 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		max_nodes = 0;
 	}
 
-	_sprint_range(tmp_line, sizeof(tmp_line), min_nodes, max_nodes);
+	if (job_size_str)
+		snprintf(tmp_line, sizeof(tmp_line), "%s", job_size_str);
+	else
+		_sprint_range(tmp_line, sizeof(tmp_line), min_nodes, max_nodes);
 	xstrfmtcat(out, "NumNodes=%s ", tmp_line);
 	_sprint_range(tmp_line, sizeof(tmp_line), job_ptr->num_cpus, job_ptr->max_cpus);
 	xstrfmtcat(out, "NumCPUs=%s ", tmp_line);
@@ -693,9 +744,11 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 
 	/****** Line 16 ******/
 	/* Tres should already of been converted at this point from simple */
-	xstrfmtcat(out, "TRES=%s",
-		   job_ptr->tres_alloc_str ? job_ptr->tres_alloc_str
-					   : job_ptr->tres_req_str);
+	xstrfmtcat(out, "ReqTRES=%s", job_ptr->tres_req_str);
+	xstrcat(out, line_end);
+
+	/****** Line ******/
+	xstrfmtcat(out, "AllocTRES=%s", job_ptr->tres_alloc_str);
 	xstrcat(out, line_end);
 
 	/****** Line 17 ******/
@@ -929,6 +982,12 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		xstrfmtcat(out, "Comment=%s ", job_ptr->comment);
 	}
 
+	/****** Line (optional) ******/
+	if (job_ptr->extra) {
+		xstrcat(out, line_end);
+		xstrfmtcat(out, "Extra=%s ", job_ptr->extra);
+	}
+
 	/****** Line 30 (optional) ******/
 	if (job_ptr->batch_flag) {
 		xstrcat(out, line_end);
@@ -986,19 +1045,29 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 
 	/****** Line 38 (optional) ******/
 	if (job_ptr->bitflags &
-	    (GRES_DISABLE_BIND | GRES_ENFORCE_BIND | KILL_INV_DEP |
-	     NO_KILL_INV_DEP | SPREAD_JOB)) {
+	    (GRES_DISABLE_BIND | GRES_ENFORCE_BIND |
+	     GRES_MULT_TASKS_PER_SHARING |
+	     GRES_ONE_TASK_PER_SHARING | KILL_INV_DEP | NO_KILL_INV_DEP |
+	     SPREAD_JOB)) {
 		xstrcat(out, line_end);
+		if (job_ptr->bitflags & GRES_ALLOW_TASK_SHARING)
+			xstrcat(out, "GresAllowTaskSharing=Yes,");
 		if (job_ptr->bitflags & GRES_DISABLE_BIND)
-			xstrcat(out, "GresEnforceBind=No");
+			xstrcat(out, "GresEnforceBind=No,");
 		if (job_ptr->bitflags & GRES_ENFORCE_BIND)
-			xstrcat(out, "GresEnforceBind=Yes");
+			xstrcat(out, "GresEnforceBind=Yes,");
+		if (job_ptr->bitflags & GRES_MULT_TASKS_PER_SHARING)
+			xstrcat(out, "GresOneTaskPerSharing=No,");
+		if (job_ptr->bitflags & GRES_ONE_TASK_PER_SHARING)
+			xstrcat(out, "GresOneTaskPerSharing=Yes,");
 		if (job_ptr->bitflags & KILL_INV_DEP)
-			xstrcat(out, "KillOInInvalidDependent=Yes");
+			xstrcat(out, "KillOInInvalidDependent=Yes,");
 		if (job_ptr->bitflags & NO_KILL_INV_DEP)
-			xstrcat(out, "KillOInInvalidDependent=No");
+			xstrcat(out, "KillOInInvalidDependent=No,");
 		if (job_ptr->bitflags & SPREAD_JOB)
-			xstrcat(out, "SpreadJob=Yes");
+			xstrcat(out, "SpreadJob=Yes,");
+
+		out[strlen(out)-1] = '\0'; /* remove trailing ',' */
 	}
 
 	/****** Line (optional) ******/
@@ -1066,9 +1135,10 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	}
 
 	/****** Line (optional) ******/
-	if (job_ptr->container) {
+	if (job_ptr->container || job_ptr->container_id) {
 		xstrcat(out, line_end);
-		xstrfmtcat(out, "Container=%s", job_ptr->container);
+		xstrfmtcat(out, "Container=%s ContainerID=%s",
+			   job_ptr->container, job_ptr->container_id);
 	}
 
 	/****** Line (optional) ******/
@@ -1150,7 +1220,7 @@ static void *_load_job_thread(void *args)
 	}
 	xfree(args);
 
-	return (void *) NULL;
+	return NULL;
 }
 
 
@@ -1530,8 +1600,23 @@ slurm_pid2jobid (pid_t job_pid, uint32_t *jobid)
 
 	if (cluster_flags & CLUSTER_FLAG_MULTSD) {
 		if ((this_addr = getenv("SLURMD_NODENAME"))) {
-			slurm_conf_get_addr(this_addr, &req_msg.address,
-					    req_msg.flags);
+			if (slurm_conf_get_addr(this_addr, &req_msg.address,
+						req_msg.flags)) {
+				/*
+				 * The node isn't in the conf, see if the
+				 * controller has an address for it.
+				 */
+				slurm_node_alias_addrs_t *alias_addrs;
+				if (!slurm_get_node_alias_addrs(this_addr,
+								&alias_addrs)) {
+					add_remote_nodes_to_conf_tbls(
+						alias_addrs->node_list,
+						alias_addrs->node_addrs);
+				}
+				slurm_free_node_alias_addrs(alias_addrs);
+				slurm_conf_get_addr(this_addr, &req_msg.address,
+						    req_msg.flags);
+			}
 		} else {
 			this_addr = "localhost";
 			slurm_set_addr(&req_msg.address, slurm_conf.slurmd_port,
@@ -1559,7 +1644,7 @@ slurm_pid2jobid (pid_t job_pid, uint32_t *jobid)
 
 	rc = slurm_send_recv_node_msg(&req_msg, &resp_msg, 0);
 
-	if ((rc != 0) || !resp_msg.auth_cred) {
+	if (rc != SLURM_SUCCESS) {
 		if (resp_msg.auth_cred)
 			auth_g_destroy(resp_msg.auth_cred);
 		return SLURM_ERROR;
@@ -1788,7 +1873,7 @@ extern int slurm_job_cpus_allocated_on_node_id(
 extern int slurm_job_cpus_allocated_on_node(job_resources_t *job_resrcs_ptr,
 					    const char *node)
 {
-	hostlist_t node_hl;
+	hostlist_t *node_hl;
 	int node_id;
 
 	if (!job_resrcs_ptr || !node || !job_resrcs_ptr->nodes)
@@ -1861,7 +1946,7 @@ int slurm_job_cpus_allocated_str_on_node(char *cpus,
 					 job_resources_t *job_resrcs_ptr,
 					 const char *node)
 {
-	hostlist_t node_hl;
+	hostlist_t *node_hl;
 	int node_id;
 
 	if (!job_resrcs_ptr || !node || !job_resrcs_ptr->nodes)
@@ -2034,7 +2119,7 @@ static void *_load_job_prio_thread(void *args)
 	}
 	xfree(args);
 
-	return (void *) NULL;
+	return NULL;
 }
 
 static int _load_fed_job_prio(slurm_msg_t *req_msg,
@@ -2204,23 +2289,17 @@ static int _load_fed_job_prio(slurm_msg_t *req_msg,
 }
 
 /*
- * slurm_load_job_prio - issue RPC to get job priority information for
- *	jobs which pass filter test
+ * slurm_load_job_prio - issue RPC to get job priority information for jobs
  * OUT factors_resp - job priority factors
- * IN job_id_list - list of job IDs to be reported
- * IN partitions - comma delimited list of partition names to be reported
- * IN uid_list - list of user IDs to be reported
- * IN show_flags -  job filtering option: 0, SHOW_LOCAL and/or SHOW_SIBLING
+ * IN show_flags -  job filtering option: 0 or SHOW_LOCAL
  * RET 0 or -1 on error
  * NOTE: free the response using slurm_free_priority_factors_response_msg()
  */
 extern int
 slurm_load_job_prio(priority_factors_response_msg_t **factors_resp,
-		    List job_id_list, char *partitions, List uid_list,
 		    uint16_t show_flags)
 {
 	slurm_msg_t req_msg;
-	priority_factors_request_msg_t factors_req;
 	void *ptr = NULL;
 	slurmdb_federation_rec_t *fed;
 	int rc;
@@ -2236,14 +2315,8 @@ slurm_load_job_prio(priority_factors_response_msg_t **factors_resp,
 		show_flags &= (~SHOW_FEDERATION);
 	}
 
-	memset(&factors_req, 0, sizeof(factors_req));
-	factors_req.job_id_list = job_id_list;
-	factors_req.partitions  = partitions;
-	factors_req.uid_list    = uid_list;
-
 	slurm_msg_t_init(&req_msg);
 	req_msg.msg_type = REQUEST_PRIORITY_FACTORS;
-	req_msg.data     = &factors_req;
 
 	/* With -M option, working_cluster_rec is set and  we only get
 	 * information for that cluster */
