@@ -78,7 +78,8 @@
  * seen any common systems where this is not the correct path.
  */
 #define LDD_PATH "/usr/bin/ldd"
-#define MAX_THREADS      8	/* These can be huge messages, so
+#define DEFAULT_THREADS 8
+#define MAX_THREADS     64	/* These can be huge messages, so
 				 * only run MAX_THREADS at one time */
 
 int block_len;				/* block size */
@@ -96,7 +97,7 @@ static List _fill_in_excluded_paths(struct bcast_parameters *params);
 static int _find_subpath(void *x, void *key);
 static int _foreach_shared_object(void *x, void *y);
 static int   _get_job_info(struct bcast_parameters *params);
-static int _get_lib_paths(char *filename, List lib_paths);
+static List _get_lib_paths(char *filename);
 
 static int _file_state(struct bcast_parameters *params)
 {
@@ -113,15 +114,15 @@ static int _file_state(struct bcast_parameters *params)
 	}
 
 	verbose("modes    = %o", (unsigned int) f_stat.st_mode);
-	verbose("uid      = %d", (int) f_stat.st_uid);
-	verbose("gid      = %d", (int) f_stat.st_gid);
+	verbose("uid      = %u", f_stat.st_uid);
+	verbose("gid      = %u", f_stat.st_gid);
 	verbose("atime    = %s", slurm_ctime2(&f_stat.st_atime));
 	verbose("mtime    = %s", slurm_ctime2(&f_stat.st_mtime));
 	verbose("ctime    = %s", slurm_ctime2(&f_stat.st_ctime));
 	verbose("size     = %ld", (long) f_stat.st_size);
 
 	if (!f_stat.st_size) {
-		error("Warning: file `%s` is empty.", params->src_fname);
+		warning("file `%s` is empty.", params->src_fname);
 		return SLURM_SUCCESS;
 	}
 	src = mmap(NULL, f_stat.st_size, PROT_READ, MAP_SHARED, fd, 0);
@@ -174,9 +175,10 @@ static int _file_bcast(struct bcast_parameters *params,
 	slurm_msg_t msg;
 
 	slurm_msg_t_init(&msg);
+	slurm_msg_set_r_uid(&msg, SLURM_AUTH_UID_ANY);
 	msg.data = bcast_msg;
 	msg.flags = USE_BCAST_NETWORK;
-	msg.forward.tree_width = params->fanout;
+	msg.forward.tree_width = params->tree_width;
 	msg.msg_type = REQUEST_FILE_BCAST;
 
 	ret_list = slurm_send_recv_msgs(sbcast_cred->node_list, &msg,
@@ -341,10 +343,10 @@ static int _bcast_file(struct bcast_parameters *params)
 		bcast_msg.mtime     = f_stat.st_mtime;
 	}
 
-	if (!params->fanout)
-		params->fanout = MAX_THREADS;
-	else
-		params->fanout = MIN(MAX_THREADS, params->fanout);
+	if (!params->tree_width)
+		params->tree_width = DEFAULT_THREADS;
+	else if (params->tree_width != 0xfffd)
+		params->tree_width = MIN(MAX_THREADS, params->tree_width);
 
 	while (more) {
 		START_TIMER;
@@ -418,22 +420,26 @@ static int _decompress_data_lz4(file_bcast_msg_t *req)
 
 /*
  * IN: char pointer with the filename.
- * IN/OUT: List of shared object direct and indirect dependencies.
- *
- * RET:	SLURM_[SUCCESS|ERROR]
+ * OUT: List of shared object direct and indirect dependencies.
+ *      or NULL on error.
  */
-static int _get_lib_paths(char *filename, List lib_paths)
+static List _get_lib_paths(char *filename)
 {
+	List lib_paths = NULL;
 	char **ldd_argv;
 	char *result = NULL;
 	char *lpath = NULL, *lpath_end = NULL;
 	char *tok = NULL, *save_ptr = NULL;
-	int status = SLURM_ERROR, rc = SLURM_SUCCESS;
+	int status = SLURM_ERROR;
+	run_command_args_t run_command_args = {
+		.max_wait = 5000,
+		.script_path = LDD_PATH,
+		.script_type = "ldd",
+		.status = &status,
+	};
 
-	if (!filename || !lib_paths) {
-		rc = SLURM_ERROR;
+	if (!filename)
 		goto fini;
-	}
 
 	ldd_argv = xcalloc(3, sizeof(char *));
 	ldd_argv[0] = xstrdup("ldd");
@@ -448,20 +454,21 @@ static int _get_lib_paths(char *filename, List lib_paths)
 	 * search for non-direct dependencies and knowing where to find them by
 	 * doing something similar to the search order of the dynamic linker.
 	 */
-	result = run_command("ldd", LDD_PATH, ldd_argv, NULL, 5000, 0, &status);
-	free_command_argv(ldd_argv);
+	run_command_args.script_argv = ldd_argv;
+	result = run_command(&run_command_args);
+	xfree_array(ldd_argv);
 
 	if (status) {
-		error("Cannot autodetect libraries for '%s' with ldd command",
+		error("Cannot autodetect libraries for '%s' with ldd command; still sending the file but ignoring send_libs",
 		      filename);
-		rc = SLURM_ERROR;
 		goto fini;
 	} else if (!result) {
 		verbose("ldd exited normally but returned no libraries");
-		rc = SLURM_SUCCESS;
 		goto fini;
 	}
 
+	lib_paths = list_create(xfree_ptr);
+
 	/*
 	 * FIXME: does not handle spaces in library paths correctly.
 	 * (Although libtool itself doesn't love that either.)
@@ -479,7 +486,7 @@ static int _get_lib_paths(char *filename, List lib_paths)
 
 fini:
 	xfree(result);
-	return rc;
+	return lib_paths;
 }
 
 /*
@@ -565,7 +572,7 @@ static List _fill_in_excluded_paths(struct bcast_parameters *params)
 	tok = strtok_r(tmp_str, ",", &saveptr);
 	while (tok) {
 		if (tok[0] != '/')
-			error("Ignorning non-absolute excluded path: '%s'",
+			error("Ignoring non-absolute excluded path: '%s'",
 			      tok);
 		else
 			list_append(excl_paths, xstrdup(tok));
@@ -581,20 +588,16 @@ static List _fill_in_excluded_paths(struct bcast_parameters *params)
  *
  * RET: SLURM_[ERROR|SUCCESS]
  */
-static int _bcast_shared_objects(struct bcast_parameters *params)
+static int _bcast_shared_objects(struct bcast_parameters *params,
+				 List lib_paths)
 {
-	foreach_shared_object_t args;
-	int rc;
-	List lib_paths = NULL, excl_paths = NULL;
+	foreach_shared_object_t args =
+		{ .return_code = SLURM_SUCCESS };
+	List excl_paths = NULL;
 	char *save_dst = params->dst_fname;
 	char *save_src = params->src_fname;
 
 	memset(&args, 0, sizeof(args));
-	lib_paths = list_create(xfree_ptr);
-	if ((rc = _get_lib_paths(params->src_fname, lib_paths)) !=
-	    SLURM_SUCCESS)
-		goto fini;
-
 	if (!(args.bcast_total_cnt = list_count(lib_paths))) {
 		verbose("No shared objects detected for '%s'",
 			params->src_fname);
@@ -607,30 +610,47 @@ static int _bcast_shared_objects(struct bcast_parameters *params)
 	args.excluded_paths = excl_paths;
 
 	list_for_each(lib_paths, _foreach_shared_object, &args);
-	rc = args.return_code;
 	params->flags &= ~BCAST_FLAG_SHARED_OBJECT;
 	params->dst_fname = save_dst;
 	params->src_fname = save_src;
 
 fini:
-	FREE_NULL_LIST(excl_paths);
-	FREE_NULL_LIST(lib_paths);
-	return rc;
+	return args.return_code;
 }
 
 extern int bcast_file(struct bcast_parameters *params)
 {
+	List lib_paths = NULL;
 	int rc;
 
 	if ((rc = _file_state(params)) != SLURM_SUCCESS)
 		return rc;
 	if ((rc = _get_job_info(params)) != SLURM_SUCCESS)
 		return rc;
+
+	/*
+	 * If getting the shared libraries fail, still send the file but
+	 * ignore send_libs.
+	 */
+	if (params->flags & BCAST_FLAG_SEND_LIBS) {
+		if (!(lib_paths = _get_lib_paths(params->src_fname))) {
+			/*
+			 * Clear BCAST_FLAG_SEND_LIBS to avoid creating the
+			 * empty "<filename>_libs" directory
+			 */
+			params->flags &= ~BCAST_FLAG_SEND_LIBS;
+		}
+	}
+
+	/* Now send files */
 	if ((rc = _bcast_file(params)) != SLURM_SUCCESS)
 		return rc;
-	if ((params->flags & BCAST_FLAG_SEND_LIBS) &&
-	    ((rc = _bcast_shared_objects(params)) != SLURM_SUCCESS))
-		return rc;
+
+	/* Send libraries, if required */
+	if (lib_paths) {
+		rc = _bcast_shared_objects(params, lib_paths);
+		FREE_NULL_LIST(lib_paths);
+	}
 
 /*	slurm_free_sbcast_cred_msg(sbcast_cred); */
 	return rc;
