@@ -40,7 +40,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <limits.h>     /* for PATH_MAX */
+#include <limits.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -97,7 +97,7 @@ static uint16_t	_cpu_freq_next_cpu(char **core_range, uint16_t *cpx,
 				   uint16_t *start, uint16_t *end);
 static uint32_t	_cpu_freq_get_scaling_freq(int cpuidx, char* option);
 static void     _cpu_freq_init_data(int cpx);
-static void     _cpu_freq_setup_data(stepd_step_rec_t *job, int cpx);
+static void     _cpu_freq_setup_data(stepd_step_rec_t *step, int cpx);
 static bool	_cpu_freq_test_scaling_freq(int cpuidx, char *option);
 static int	_derive_avail_freq(int cpuidx);
 static int	_fd_lock_retry(int fd);
@@ -432,7 +432,7 @@ rwfail:
  *  a pointer to a hex map string of the cpus to be used by this step
  */
 extern void
-cpu_freq_cpuset_validate(stepd_step_rec_t *job)
+cpu_freq_cpuset_validate(stepd_step_rec_t *step)
 {
 	int cpuidx, cpu_num;
 	bitstr_t *cpus_to_set;
@@ -450,38 +450,39 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 			set_batch_freq = 0;
 	}
 
-	if (((job->step_id.step_id == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
-	    (job->step_id.step_id == SLURM_EXTERN_CONT))
+	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
+	     !set_batch_freq) ||
+	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
-	slurm_sprint_cpu_bind_type(cpu_bind_type_string, job->cpu_bind_type);
+	slurm_sprint_cpu_bind_type(cpu_bind_type_string, step->cpu_bind_type);
 
 	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
-		 __func__, job->cpu_freq_min, job->cpu_freq_min,
-		 job->cpu_freq_max, job->cpu_freq_max, job->cpu_freq_gov);
+		 __func__, step->cpu_freq_min, step->cpu_freq_min,
+		 step->cpu_freq_max, step->cpu_freq_max, step->cpu_freq_gov);
 	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		 job->step_id.job_id, job->step_id.step_id,
-		 job->node_tasks, job->cpus_per_task,
-		 job->cpus);
+		 step->step_id.job_id, step->step_id.step_id,
+		 step->node_tasks, step->cpus_per_task,
+		 step->cpus);
 	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
-		 job->cpu_bind_type, job->cpu_bind);
+		 step->cpu_bind_type, step->cpu_bind);
 
 	if (!cpu_freq_count)
 		return;
 
-	if (job->cpu_bind == NULL) {
+	if (step->cpu_bind == NULL) {
 		/*
 		 * slurm_verify_cpu_bind will set cpu_bind to NULL for manual
 		 * binding that doesn't require an argument
 		 */
-		if ((job->cpu_bind_type != CPU_BIND_NONE) &&
-		    (job->cpu_bind_type != CPU_BIND_RANK) &&
-		    (job->cpu_bind_type != CPU_BIND_LDRANK))
+		if ((step->cpu_bind_type != CPU_BIND_NONE) &&
+		    (step->cpu_bind_type != CPU_BIND_RANK) &&
+		    (step->cpu_bind_type != CPU_BIND_LDRANK))
 			error("cpu_freq_cpuset_validate: cpu_bind string is null");
 		return;
 	}
-	cpu_bind = xstrdup(job->cpu_bind);
+	cpu_bind = xstrdup(step->cpu_bind);
 
 	if ( (cpu_str = strtok_r(cpu_bind, ",", &savestr) ) == NULL) {
 		error("cpu_freq_cpuset_validate: cpu_bind string invalid");
@@ -495,13 +496,13 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 	do {
 		debug3("  cpu_str = %s", cpu_str);
 
-		if ((job->cpu_bind_type & CPU_BIND_MAP) == CPU_BIND_MAP) {
+		if ((step->cpu_bind_type & CPU_BIND_MAP) == CPU_BIND_MAP) {
 			cpu_num = atoi(cpu_str);
 			if (cpu_num >= cpu_freq_count) {
 				error("cpu_freq_cpuset_validate: invalid cpu "
 				      "number %d", cpu_num);
-				bit_free(cpu_map);
-				bit_free(cpus_to_set);
+				FREE_NULL_BITMAP(cpu_map);
+				FREE_NULL_BITMAP(cpus_to_set);
 				xfree(cpu_bind);
 				return;
 			}
@@ -510,8 +511,8 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 			if (bit_unfmt_hexmask(cpu_map, cpu_str) == -1) {
 				error("cpu_freq_cpuset_validate: invalid cpu "
 				      "mask %s", cpu_bind);
-				bit_free(cpu_map);
-				bit_free(cpus_to_set);
+				FREE_NULL_BITMAP(cpu_map);
+				FREE_NULL_BITMAP(cpus_to_set);
 				xfree(cpu_bind);
 				return;
 			}
@@ -521,13 +522,13 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 
 	for (cpuidx = 0; cpuidx < cpu_freq_count; cpuidx++) {
 		if (bit_test(cpus_to_set, cpuidx)) {
-			_cpu_freq_setup_data(job, cpuidx);
+			_cpu_freq_setup_data(step, cpuidx);
 		}
 	}
-	cpu_freq_set(job);
+	cpu_freq_set(step);
 
-	bit_free(cpu_map);
-	bit_free(cpus_to_set);
+	FREE_NULL_BITMAP(cpu_map);
+	FREE_NULL_BITMAP(cpus_to_set);
 	xfree(cpu_bind);
 	return;
 }
@@ -538,7 +539,7 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
  *  the list of cpus to be used by this step
  */
 extern void
-cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
+cpu_freq_cgroup_validate(stepd_step_rec_t *step, char *step_alloc_cores)
 {
 	uint16_t start  = USHRT_MAX;
 	uint16_t end    = USHRT_MAX;
@@ -553,22 +554,23 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
 			set_batch_freq = 0;
 	}
 
-	if (((job->step_id.step_id == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
-	    (job->step_id.step_id == SLURM_EXTERN_CONT))
+	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
+	     !set_batch_freq) ||
+	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
 	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
-		 __func__, job->cpu_freq_min, job->cpu_freq_min,
-		 job->cpu_freq_max, job->cpu_freq_max, job->cpu_freq_gov);
+		 __func__, step->cpu_freq_min, step->cpu_freq_min,
+		 step->cpu_freq_max, step->cpu_freq_max, step->cpu_freq_gov);
 	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		 job->step_id.job_id, job->step_id.step_id,
-		 job->node_tasks, job->cpus_per_task,
-		 job->cpus);
+		 step->step_id.job_id, step->step_id.step_id,
+		 step->node_tasks, step->cpus_per_task,
+		 step->cpus);
 	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
-		 job->cpu_bind_type, job->cpu_bind);
+		 step->cpu_bind_type, step->cpu_bind);
 	log_flag(CPU_FREQ, "  step logical cores = %s, step physical cores = %s",
-		 job->step_alloc_cores, step_alloc_cores);
+		 step->step_alloc_cores, step_alloc_cores);
 
 	if (!cpu_freq_count)
 		return;
@@ -582,9 +584,9 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
 			  cpuidx, cpu_freq_count);
 		    return;
 		}
-		_cpu_freq_setup_data(job, cpuidx);
+		_cpu_freq_setup_data(step, cpuidx);
 	}
-	cpu_freq_set(job);
+	cpu_freq_set(step);
 	return;
 }
 
@@ -701,7 +703,7 @@ _cpu_freq_get_cur_gov(int cpuidx)
  * set cpu governor
  */
 static int
-_cpu_freq_set_gov(stepd_step_rec_t *job, int cpuidx, char* gov )
+_cpu_freq_set_gov(stepd_step_rec_t *step, int cpuidx, char *gov)
 {
 	char path[PATH_MAX];
 	FILE *fp;
@@ -709,12 +711,12 @@ _cpu_freq_set_gov(stepd_step_rec_t *job, int cpuidx, char* gov )
 	uint32_t jobid;
 
 #ifdef HAVE_NATIVE_CRAY
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->step_id.job_id;
+		jobid = step->step_id.job_id;
 #else
-	jobid = job->step_id.job_id;
+	jobid = step->step_id.job_id;
 #endif
 
 	rc = SLURM_SUCCESS;
@@ -764,7 +766,7 @@ _cpu_freq_get_scaling_freq(int cpuidx, char* option)
 }
 
 /*
- * test for existance of cpufreq file
+ * test for existence of cpufreq file
  *
  * Return: true if file found
  */
@@ -788,7 +790,7 @@ _cpu_freq_test_scaling_freq(int cpuidx, char *option)
  *
  */
 static int
-_cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
+_cpu_freq_set_scaling_freq(stepd_step_rec_t *step, int cpx, uint32_t freq,
 		char* option)
 {
 	char path[PATH_MAX];
@@ -798,12 +800,12 @@ _cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
 	uint32_t jobid;
 
 #ifdef HAVE_NATIVE_CRAY
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->step_id.job_id;
+		jobid = step->step_id.job_id;
 #else
-	jobid = job->step_id.job_id;
+	jobid = step->step_id.job_id;
 #endif
 
 	rc = SLURM_SUCCESS;
@@ -1029,54 +1031,47 @@ _cpu_freq_init_data(int cpx)
  * Or min/max governor base on --cpu-freq parameter
  */
 static void
-_cpu_freq_setup_data(stepd_step_rec_t *job, int cpx)
+_cpu_freq_setup_data(stepd_step_rec_t *step, int cpx)
 {
 	uint32_t freq;
 
-	if (   (job->cpu_freq_min == NO_VAL || job->cpu_freq_min==0)
-	    && (job->cpu_freq_max == NO_VAL || job->cpu_freq_max==0)
-	    && (job->cpu_freq_gov == NO_VAL || job->cpu_freq_gov==0)) {
-		/* If no --cpu-freq, use default governor from conf file.  */
-		slurm_conf_t *conf = slurm_conf_lock();
-		job->cpu_freq_gov = conf->cpu_freq_def;
-		slurm_conf_unlock();
-		if (job->cpu_freq_gov == NO_VAL)
-			return;
-	}
+	/* If no --cpu-freq, use default governor from conf file. */
+	if (step->cpu_freq_gov == NO_VAL)
+		step->cpu_freq_gov = slurm_conf.cpu_freq_def;
 
 	/* Get current state */
 	if (_cpu_freq_current_state(cpx) == SLURM_ERROR)
 		return;
 
-	if (job->cpu_freq_min == NO_VAL &&
-	    job->cpu_freq_max != NO_VAL &&
-	    job->cpu_freq_gov == NO_VAL) {
+	if (step->cpu_freq_min == NO_VAL &&
+	    step->cpu_freq_max != NO_VAL &&
+	    step->cpu_freq_gov == NO_VAL) {
 		/* Pre version 15.08 behavior */
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_frequency = freq;
 		goto newfreq;
 	}
-	if (job->cpu_freq_gov == CPU_FREQ_USERSPACE) {
-		_cpu_freq_govspec_string(job->cpu_freq_gov, cpx);
-		if (job->cpu_freq_max == NO_VAL) {
+	if (step->cpu_freq_gov == CPU_FREQ_USERSPACE) {
+		_cpu_freq_govspec_string(step->cpu_freq_gov, cpx);
+		if (step->cpu_freq_max == NO_VAL) {
 			return; /* pre version 15.08 behavior. */
 		}
 		/* Power capping */
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_frequency = freq;
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_min, cpx);
 		cpufreq[cpx].new_min_freq = freq;
-		goto newfreq;
+		cpufreq[cpx].new_max_freq = freq;
+		return;
 	}
-	if (job->cpu_freq_min != NO_VAL && job->cpu_freq_max != NO_VAL) {
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_min, cpx);
+	if (step->cpu_freq_min != NO_VAL && step->cpu_freq_max != NO_VAL) {
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_min, cpx);
 		cpufreq[cpx].new_min_freq = freq;
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_max_freq = freq;
 	}
 
-	if (job->cpu_freq_gov != NO_VAL) {
-		_cpu_freq_govspec_string(job->cpu_freq_gov, cpx);
+	if (step->cpu_freq_gov != NO_VAL) {
+		_cpu_freq_govspec_string(step->cpu_freq_gov, cpx);
 	}
 newfreq:
 	/* Make sure a 'new' frequency is within scaling min/max */
@@ -1146,18 +1141,20 @@ _cpu_freq_check_freq(const char* arg)
 	} else if (xstrncasecmp(arg, "med", 3) == 0) {
 		return CPU_FREQ_MEDIUM;
 	}
-	if ( (frequency = strtoul(arg, &end, 10) )) {
-		return frequency;
+	frequency = strtoul(arg, &end, 10);
+	if ((*end != '\0') ||
+	    ((frequency == 0) && (errno == EINVAL))) {
+		error("unrecognized --cpu-freq argument \"%s\"", arg);
+		return 0;
 	}
-	error("unrecognized --cpu-freq argument \"%s\"", arg);
-	return 0;
+	return frequency;
 }
 
 /*
  * set cpu frequency if possible for each cpu of the job step
  */
 extern void
-cpu_freq_set(stepd_step_rec_t *job)
+cpu_freq_set(stepd_step_rec_t *step)
 {
 	char freq_detail[100];
 	uint32_t freq;
@@ -1186,22 +1183,22 @@ cpu_freq_set(stepd_step_rec_t *job)
 				/* The current frequency is > requested max,
 				 * Set it so it is in range
 				 * have to go to UserSpace to do it. */
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					return;
-				rc = _cpu_freq_set_scaling_freq(job, i, freq,
+				rc = _cpu_freq_set_scaling_freq(step, i, freq,
 						         "scaling_setspeed");
 				if (rc == SLURM_ERROR)
 					continue;
 				if (cpufreq[i].new_governor[0] == '\0') {
 					/* Not requesting new gov, so restore */
-					rc = _cpu_freq_set_gov(job, i,
+					rc = _cpu_freq_set_gov(step, i,
 						cpufreq[i].org_governor);
 					if (rc == SLURM_ERROR)
 						continue;
 				}
 			}
-			rc = _cpu_freq_set_scaling_freq(job, i, freq,
+			rc = _cpu_freq_set_scaling_freq(step, i, freq,
 							"scaling_max_freq");
 			if (rc == SLURM_ERROR)
 				continue;
@@ -1212,40 +1209,40 @@ cpu_freq_set(stepd_step_rec_t *job)
 				/* The current frequency is < requested min,
 				 * Set it so it is in range
 				 * have to go to UserSpace to do it. */
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					continue;
-				rc = _cpu_freq_set_scaling_freq(job, i, freq,
+				rc = _cpu_freq_set_scaling_freq(step, i, freq,
 						         "scaling_setspeed");
 				if (rc == SLURM_ERROR)
 					continue;
 				if (cpufreq[i].new_governor[0] == '\0') {
 					/* Not requesting new gov, so restore */
-					rc= _cpu_freq_set_gov(job, i,
+					rc= _cpu_freq_set_gov(step, i,
 						cpufreq[i].org_governor);
 					if (rc == SLURM_ERROR)
 						continue;
 				}
 			}
-			rc= _cpu_freq_set_scaling_freq(job, i, freq,
+			rc= _cpu_freq_set_scaling_freq(step, i, freq,
 						       "scaling_min_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_frequency != NO_VAL) {
 			if (xstrcmp(cpufreq[i].org_governor,"userspace")) {
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					continue;
 			}
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].new_frequency,
 					"scaling_setspeed");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_governor[0] != '\0') {
-			rc = _cpu_freq_set_gov(job, i, cpufreq[i].new_governor);
+			rc = _cpu_freq_set_gov(step, i, cpufreq[i].new_governor);
 			if (rc == SLURM_ERROR)
 				continue;
 		}
@@ -1271,7 +1268,7 @@ cpu_freq_set(stepd_step_rec_t *job)
  * default frequency and governor type
  */
 extern void
-cpu_freq_reset(stepd_step_rec_t *job)
+cpu_freq_reset(stepd_step_rec_t *step)
 {
 	int i, rc, fd;
 	char freq_detail[100];
@@ -1281,12 +1278,12 @@ cpu_freq_reset(stepd_step_rec_t *job)
 		return;
 
 #ifdef HAVE_NATIVE_CRAY
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->step_id.job_id;
+		jobid = step->step_id.job_id;
 #else
-	jobid = job->step_id.job_id;
+	jobid = step->step_id.job_id;
 #endif
 
 	for (i = 0; i < cpu_freq_count; i++) {
@@ -1301,10 +1298,10 @@ cpu_freq_reset(stepd_step_rec_t *job)
 			continue;
 
 		if (cpufreq[i].new_frequency != NO_VAL) {
-			rc = _cpu_freq_set_gov(job, i, "userspace");
+			rc = _cpu_freq_set_gov(step, i, "userspace");
 			if (rc == SLURM_ERROR)
 				continue;
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_frequency,
 					"scaling_setspeed");
 			if (rc == SLURM_ERROR)
@@ -1315,21 +1312,22 @@ cpu_freq_reset(stepd_step_rec_t *job)
 		 * www.kernel.org/doc/Documentation/cpu-freq/user-guide.txt
 		 */
 		if (cpufreq[i].new_max_freq != NO_VAL) {
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_max_freq,
 					"scaling_max_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_min_freq != NO_VAL) {
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_min_freq,
 					"scaling_min_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_governor[0] != '\0') {
-			rc = _cpu_freq_set_gov(job, i, cpufreq[i].org_governor);
+			rc = _cpu_freq_set_gov(
+				step, i, cpufreq[i].org_governor);
 			if (rc == SLURM_ERROR)
 				continue;
 		}
@@ -1502,10 +1500,10 @@ cpu_freq_govlist_to_string(char* buf, uint16_t bufsz, uint32_t govs)
 /*
  * Verify slurm.conf CpuFreqDef option
  *
- * Input:  - arg  - frequency value to check
+ * Input:  - arg  - governor/frequency value to check:
  * 		    valid governor, low, medium, highm1, high,
  * 		    or numeric frequency
- *	   - freq - pointer to corresponging enum or numberic value
+ *	   - freq - pointer to corresponding enum or numeric value
  * Returns - -1 on error, else 0
  */
 extern int
@@ -1566,7 +1564,7 @@ cpu_freq_verify_govlist(const char *arg, uint32_t *govs)
  * Verify cpu_freq command line option
  *
  * --cpu-freq=arg
- *   where arg is p1{-p2{:p3}}
+ *   where arg is p1[-p2][:p3]
  *
  * - p1 can be  [#### | low | medium | high | highm1]
  * 	which will set the current frequency, and set the governor to
@@ -1574,13 +1572,14 @@ cpu_freq_verify_govlist(const char *arg, uint32_t *govs)
  * - p1 can be [Conservative | OnDemand | Performance | PowerSave | UserSpace]
  *      which will set the governor to the corresponding value.
  * - When p2 is present, p1 will be the minimum frequency and p2 will be
- *   the maximum. The governor will not be changed.
+ *   the maximum. The governor cannot be UserSpace, so CpuFreqDef must be set in
+ *   slurm.conf if there's no p3.
  * - p2 can be  [#### | medium | high | highm1] p2 must be greater than p1.
  * - If the current frequency is < min, it will be set to min.
  *   Likewise, if the current frequency is > max, it will be set to max.
  * - p3 can be [Conservative | OnDemand | Performance | PowerSave | UserSpace]
  *   which will set the governor to the corresponding value.
- *   When p3 is UserSpace, the current frequency is set to p2.
+ *   When p3 is UserSpace, p2 must be empty.
  *   p2 will have been set by PowerCapping.
  *
  * returns -1 on error, 0 otherwise
@@ -1641,6 +1640,18 @@ cpu_freq_verify_cmdline(const char *arg,
 		*cpu_freq_max = frequency;
 	}
 	if (p2) {
+		if (!p3 && (slurm_conf.cpu_freq_def == NO_VAL)) {
+			/*
+			 * If the user specified a range without a governor,
+			 * (even if userspace is not set), we won't accept the
+			 * request. We don't know how the cpus are set and we
+			 * won't decide which one to set for the user. Note that
+			 * a range is valid for multiple governors.
+			 */
+			error("You must explicitly choose a governor when defining a range. Please specify only one value for the desired frequency (p1) or choose a specific governor (p3).");
+			rc = -1;
+			goto clean;
+		}
 		frequency = _cpu_freq_check_freq(p2);
 		if (frequency == 0) {
 			rc = -1;
@@ -1655,22 +1666,43 @@ cpu_freq_verify_cmdline(const char *arg,
 			goto clean;
 		}
 	}
-
 	if (p3) {
-		if (!p2) {
-			error("gov on cpu-frec (%s) illegal without max", p3);
-			rc = -1;
-			goto clean;
-		}
 		frequency = _cpu_freq_check_gov(p3, 0);
 		if (frequency == 0) {
 			error("illegal governor: %s on --cpu-freq", p3);
 			rc = -1;
 			goto clean;
 		}
+		if (!p2) {
+			if (frequency != CPU_FREQ_USERSPACE) {
+				error("gov on cpu-frec (%s) illegal without max",
+				      p3);
+				rc = -1;
+				goto clean;
+			}
+		} else {
+			if (frequency == CPU_FREQ_USERSPACE) {
+				error("%s governor does not support a range. Please specify only one value for the desired frequency (p1) or choose a different governor.",
+				      p3);
+				rc = -1;
+				goto clean;
+			}
+		}
 		*cpu_freq_gov = frequency;
+	} else if (p2 && (*cpu_freq_gov == NO_VAL) &&
+		   (slurm_conf.cpu_freq_def != NO_VAL)) {
+		/*
+		 * No governor specified and a range is specified.
+		 * Use slurm.conf CpuFreqDef if defined. Note that this cannot
+		 * be UserSpace.
+		 */
+		*cpu_freq_gov = slurm_conf.cpu_freq_def;
 	}
 
+	/* Also force this in case we specify just one frequency. */
+	if ((*cpu_freq_gov == NO_VAL) && !p2 && !p3)
+		*cpu_freq_gov = CPU_FREQ_USERSPACE;
+
 clean:
 	if (*cpu_freq_gov != NO_VAL) {
 		if (((*cpu_freq_gov & slurm_conf.cpu_freq_govs)
@@ -1703,7 +1735,7 @@ clean:
  * freq_str is a buffer to hold the composite string for all input values.
  * freq_len is length of freq_str
  * gov is a governor value
- * min is a minumum value
+ * min is a minimum value
  * max is a maximum value
  * freq is a (current) frequency value.
  *
