@@ -72,6 +72,7 @@ extern void free_cron_entry(void *in)
 	xfree(entry->day_of_month);
 	xfree(entry->month);
 	xfree(entry->day_of_week);
+	xfree(entry->cronspec);
 	xfree(entry->command);
 	xfree(entry);
 }
@@ -253,12 +254,20 @@ static int _next_day_of_month(cron_entry_t *entry, struct tm *tm)
 
 	/* february == 1 */
 	if (tm->tm_mon != 1) {
-		if (bit_test(entry->day_of_month, 29))
+		if (tm->tm_mday > 29)
+			/* no-op */;
+		else if (bit_test(entry->day_of_month, 29))
 			return days_to_advance;
-		days_to_advance++;
-		if (bit_test(entry->day_of_month, 30))
+		else
+			days_to_advance++;
+
+		if (tm->tm_mday > 30)
+			/* no-op */;
+		else if (bit_test(entry->day_of_month, 30))
 			return days_to_advance;
-		days_to_advance++;
+		else
+			days_to_advance++;
+
 		if ((tm->tm_mon == 0) || (tm->tm_mon == 2) ||
 		    (tm->tm_mon == 4) || (tm->tm_mon == 6) ||
 		    (tm->tm_mon == 7) || (tm->tm_mon == 9) ||
@@ -291,18 +300,24 @@ static int _next_day_of_month(cron_entry_t *entry, struct tm *tm)
 	return days_to_advance;
 }
 
-extern time_t calc_next_cron_start(cron_entry_t *entry)
+extern time_t calc_next_cron_start(cron_entry_t *entry, time_t next)
 {
 	struct tm tm;
 	time_t now = time(NULL);
-	localtime_r(&now, &tm);
 	int validated_month, days_to_add;
 
 	/*
-	 * Always move into the next minute to avoid running again this minute.
+	 * Avoid running twice in the same minute.
 	 */
-	tm.tm_sec = 0;
-	tm.tm_min++;
+	if (next && next > now + 60) {
+		now = next;
+		localtime_r(&now, &tm);
+		tm.tm_sec = 0;
+	} else {
+		localtime_r(&now, &tm);
+		tm.tm_sec = 0;
+		tm.tm_min++;
+	}
 
 month:
 	_next_month(entry, &tm);
