@@ -50,6 +50,7 @@
 #ifndef   __list_datatypes_defined
 #  define __list_datatypes_defined
 typedef struct xlist *List;
+typedef struct xlist list_t;
 
 /*
  *  List opaque data type.
@@ -59,6 +60,7 @@ typedef struct xlist *List;
  *  List Iterator opaque data type.
  */
 typedef struct listIterator *ListIterator;
+typedef struct listIterator list_itr_t;
 
 /*
  *  Function prototype to deallocate data stored in a list.
@@ -100,25 +102,25 @@ typedef int (*ListForF) (void *x, void *arg);
  *  Note: Abandoning a list without calling list_destroy() will result
  *    in a memory leak.
  */
-List list_create(ListDelF f);
+extern list_t *list_create(ListDelF f);
 
 /*
  *  Destroys list [l], freeing memory used for list iterators and the
  *    list itself; if a deletion function was specified when the list
  *    was created, it will be called for each item in the list.
  */
-void list_destroy(List l);
+extern void list_destroy(list_t *l);
 
 /*
  *  Returns non-zero if list [l] is empty; o/w returns zero.
  */
-int list_is_empty(List l);
+extern int list_is_empty(list_t *l);
 
 /*
  * Return the number of items in list [l].
  * If [l] is NULL, return 0.
  */
-int list_count(List l);
+extern int list_count(list_t *l);
 
 /*
  *  Create new shallow copy of list [l] pointers, without destructor.
@@ -129,7 +131,7 @@ int list_count(List l);
  *  Warning: destruction of this list will not free members of [l].
  *  Warning: This list is only valid while [l] is unchanged.
  */
-List list_shallow_copy(List l);
+extern list_t *list_shallow_copy(list_t *l);
 
 /***************************
  *  List Access Functions  *
@@ -137,16 +139,15 @@ List list_shallow_copy(List l);
 
 /*
  *  Inserts data [x] at the end of list [l].
- *  Returns the data's ptr.
  */
-void *list_append(List l, void *x);
+extern void list_append(list_t *l, void *x);
 
 /*
  *  Inserts list [sub] at the end of list [l].
  *  Note: list [l] must have a destroy function of NULL.
  *  Returns a count of the number of items added to list [l].
  */
-int list_append_list(List l, List sub);
+extern int list_append_list(list_t *l, List sub);
 
 /*
  *  Pops off list [sub] and appends data at the end of list [l].
@@ -154,7 +155,7 @@ int list_append_list(List l, List sub);
  *  Note: list [sub] will be returned empty, but not destroyed.
  *  Returns a count of the number of items added to list [l].
  */
-int list_transfer(List l, List sub);
+extern int list_transfer(list_t *l, List sub);
 
 /*
  *  Pop off elements in list [sub] to [l], unless already in [l].
@@ -164,7 +165,7 @@ int list_transfer(List l, List sub);
  *        but never destroyed.
  *  Returns a count of the number of items added to list [l].
  */
-int list_transfer_unique(List l, ListFindF f, List sub);
+extern int list_transfer_unique(list_t *l, ListFindF f, list_t *sub);
 
 /*
  *  Pops off list [sub] to [l] with maximum number of entries.
@@ -173,13 +174,20 @@ int list_transfer_unique(List l, ListFindF f, List sub);
  *  Note: list [sub] may be returned empty, but not destroyed.
  *  Returns a count of the number of items added to list [l].
  */
-int list_transfer_max(List l, List sub, int max);
+extern int list_transfer_max(list_t *l, list_t *sub, int max);
+
+/*
+ *  Traverses list [l] using [f] to match each item with [key].
+ *  Matching items are then transfered to [sub].
+ *  Note: list [l] must have the same destroy function as list [sub].
+ *  Returns a count of the number of items moved to list [sub] from list [l].
+ */
+extern int list_transfer_match(list_t *l, list_t *sub, ListFindF f, void *key);
 
 /*
  *  Inserts data [x] at the beginning of list [l].
- *  Returns the data's ptr.
  */
-void *list_prepend(List l, void *x);
+extern void list_prepend(list_t *l, void *x);
 
 /*
  *  Traverses list [l] using [f] to match each item with [key].
@@ -189,13 +197,12 @@ void *list_prepend(List l, void *x);
  *    a list iterator; it should only be used when all list items are known
  *    to be unique (according to the function [f]).
  */
-void *list_find_first(List l, ListFindF f, void *key);
+extern void *list_find_first(list_t *l, ListFindF f, void *key);
 
 /*
  * Same as list_find_first, but use rdlock instead of wrlock
  */
-void *list_find_first_ro(List l, ListFindF f, void *key);
-
+extern void *list_find_first_ro(list_t *l, ListFindF f, void *key);
 
 /*
  *  Traverses list [l] using [f] to match each item with [key].
@@ -206,7 +213,7 @@ void *list_find_first_ro(List l, ListFindF f, void *key);
  *    a list iterator; it should only be used when all list items are known
  *    to be unique (according to the function [f]).
  */
-void *list_remove_first(List l, ListFindF f, void *key);
+extern void *list_remove_first(list_t *l, ListFindF f, void *key);
 
 /*
  *  Traverses list [l] using [f] to match each item with [key].
@@ -215,7 +222,7 @@ void *list_remove_first(List l, ListFindF f, void *key);
  *    created, it will be called to deallocate each item being removed.
  *  Returns a count of the number of items removed from the list.
  */
-int list_delete_all(List l, ListFindF f, void *key);
+extern int list_delete_all(list_t *l, ListFindF f, void *key);
 
 /*
  *  Traverses list [l] using [f] to match each item with [key].
@@ -227,7 +234,7 @@ int list_delete_all(List l, ListFindF f, void *key);
  *  Returns 0 if no item was found, 1 if an item was removed, -1 if processing
  *    was stopped.
  */
-int list_delete_first(List l, ListFindF f, void *key);
+extern int list_delete_first(list_t *l, ListFindF f, void *key);
 
 /*
  *  Traverses list [l] and deletes 'key' from it.
@@ -236,7 +243,7 @@ int list_delete_first(List l, ListFindF f, void *key);
  *  removed.
  *  Returns 1 if found and 0 if not.
  */
-int list_delete_ptr(List l, void *key);
+extern int list_delete_ptr(list_t *l, void *key);
 
 /*
  *  For each item in list [l], invokes the function [f] with [arg].
@@ -244,8 +251,8 @@ int list_delete_ptr(List l, void *key);
  *  If [f] returns <0 for a given item, the iteration is aborted and the
  *    function returns the negative of that item's position in the list.
  */
-int list_for_each(List l, ListForF f, void *arg);
-int list_for_each_ro(List l, ListForF f, void *arg);
+extern int list_for_each(list_t *l, ListForF f, void *arg);
+extern int list_for_each_ro(list_t *l, ListForF f, void *arg);
 
 /*
  *  For each item in list [l], invokes the function [f] with [arg].
@@ -253,7 +260,7 @@ int list_for_each_ro(List l, ListForF f, void *arg);
  *  If [f] returns <0 for a given item, the iteration is NOT aborted but the
  *  return value (count of items processed) will be negated.
  */
-int list_for_each_nobreak(List l, ListForF f, void *arg);
+extern int list_for_each_nobreak(list_t *l, ListForF f, void *arg);
 
 /*
  *  For each item in list [l], invokes the function [f] with [arg].
@@ -265,8 +272,8 @@ int list_for_each_nobreak(List l, ListForF f, void *arg);
  *  If [f] returns <0 for a given item, the iteration is aborted and the
  *    function returns the negative of that item's position in the list.
  */
-int list_for_each_max(List l, int *max, ListForF f, void *arg,
-		      int break_on_fail, int write_lock);
+extern int list_for_each_max(list_t *l, int *max, ListForF f, void *arg,
+			     int break_on_fail, int write_lock);
 
 /*
  *  Traverses list [l] and removes all items in list
@@ -274,7 +281,7 @@ int list_for_each_max(List l, int *max, ListForF f, void *arg,
  *  created, it will be called to deallocate each item being removed.
  *  Returns a count of the number of items removed from the list.
  */
-int list_flush(List l);
+extern int list_flush(list_t *l);
 
 /*
  *  Traverses list [l] and removes items.
@@ -283,20 +290,20 @@ int list_flush(List l);
  *  created, it will be called to deallocate each item being removed.
  *  Returns a count of the number of items removed from the list.
  */
-int list_flush_max(List l, int max);
+extern int list_flush_max(list_t *l, int max);
 
 /*
  *  Sorts list [l] into ascending order according to the function [f].
  *  Note: Sorting a list resets all iterators associated with the list.
  *  This function uses the libC qsort() algorithm.
  */
-void list_sort(List l, ListCmpF f);
+extern void list_sort(list_t *l, ListCmpF f);
 
 /*
  * Reverses the order of the items in list [l].
  * Note: Reversing a list resets all iterators associated with the list.
  */
-void list_flip(List l);
+extern void list_flip(list_t *l);
 
 /****************************
  *  Stack Access Functions  *
@@ -304,22 +311,21 @@ void list_flip(List l);
 
 /*
  *  Pushes data [x] onto the top of stack [l].
- *  Returns the data's ptr.
  */
-void *list_push(List l, void *x);
+extern void list_push(list_t *l, void *x);
 
 /*
  *  Pops the data item at the top of the stack [l].
  *  Returns the data's ptr, or NULL if the stack is empty.
  */
-void *list_pop(List l);
+extern void *list_pop(list_t *l);
 
 /*
  *  Peeks at the data item at the top of the stack (or head of the queue) [l].
  *  Returns the data's ptr, or NULL if the stack (or queue) is empty.
  *  Note: The item is not removed from the list.
  */
-void *list_peek(List l);
+extern void *list_peek(list_t *l);
 
 /****************************
  *  Queue Access Functions  *
@@ -327,15 +333,14 @@ void *list_peek(List l);
 
 /*
  *  Enqueues data [x] at the tail of queue [l].
- *  Returns the data's ptr.
  */
-void *list_enqueue(List l, void *x);
+extern void list_enqueue(list_t *l, void *x);
 
 /*
  *  Dequeues the data item at the head of the queue [l].
  *  Returns the data's ptr, or NULL if the queue is empty.
  */
-void *list_dequeue(List l);
+extern void *list_dequeue(list_t *l);
 
 
 /*****************************
@@ -346,41 +351,40 @@ void *list_dequeue(List l);
  *  Creates and returns a list iterator for non-destructively traversing
  *    list [l].
  */
-ListIterator list_iterator_create(List l);
+extern list_itr_t *list_iterator_create(list_t *l);
 
 /*
  *  Resets the list iterator [i] to start traversal at the beginning
  *    of the list.
  */
-void list_iterator_reset(ListIterator i);
+extern void list_iterator_reset(list_itr_t *i);
 
 /*
  *  Destroys the list iterator [i]; list iterators not explicitly destroyed
  *    in this manner will be destroyed when the list is deallocated via
  *    list_destroy().
  */
-void list_iterator_destroy(ListIterator i);
+extern void list_iterator_destroy(list_itr_t *i);
 
 /*
  *  Returns a ptr to the next item's data,
  *    or NULL once the end of the list is reached.
  *  Example: i=list_iterator_create(i); while ((x=list_next(i))) {...}
  */
-void *list_next(ListIterator i);
+extern void *list_next(list_itr_t *i);
 
 /*
  *  Returns a ptr to the next item's data WITHOUT advancing the pointer,
  *    or NULL once the end of the list is reached.
  */
-void *list_peek_next(ListIterator i);
+extern void *list_peek_next(list_itr_t *i);
 
 /*
  *  Inserts data [x] immediately before the last item returned via list
  *    iterator [i]; once the list iterator reaches the end of the list,
  *    insertion is made at the list's end.
- *  Returns the data's ptr.
  */
-void *list_insert(ListIterator i, void *x);
+extern void list_insert(list_itr_t *i, void *x);
 
 /*
  *  Traverses the list from the point of the list iterator [i]
@@ -389,14 +393,14 @@ void *list_insert(ListIterator i, void *x);
  *    returns non-zero, or NULL once the end of the list is reached.
  *  Example: i=list_iterator_reset(i); while ((x=list_find(i,f,k))) {...}
  */
-void *list_find(ListIterator i, ListFindF f, void *key);
+extern void *list_find(list_itr_t *i, ListFindF f, void *key);
 
 /*
  *  Removes from the list the last item returned via list iterator [i]
  *    and returns the data's ptr.
  *  Note: The client is responsible for freeing the returned data.
  */
-void *list_remove(ListIterator i);
+extern void *list_remove(list_itr_t *i);
 
 /*
  *  Removes from the list the last item returned via list iterator [i];
@@ -405,6 +409,6 @@ void *list_remove(ListIterator i);
  *  Returns a count of the number of items removed from the list
  *    (ie, '1' if the item was removed, and '0' otherwise).
  */
-int list_delete_item(ListIterator i);
+extern int list_delete_item(list_itr_t *i);
 
 #endif /* !LSD_LIST_H */
