@@ -48,6 +48,7 @@
 #include <time.h>
 
 #include "src/common/bitstring.h"
+#include "src/common/extra_constraints.h"
 #include "src/common/hostlist.h"
 #include "src/common/list.h"
 #include "src/common/read_config.h"
@@ -107,6 +108,7 @@ struct node_record {
 	acct_gather_energy_t *energy;	/* power consumption data */
 	ext_sensors_data_t *ext_sensors; /* external sensor data */
 	char *extra;			/* arbitrary string */
+	data_t *extra_data;		/* Data serialized from extra */
 	char *features;			/* node's available features, used only
 					 * for state save/restore, DO NOT
 					 * use for scheduling purposes */
@@ -121,6 +123,8 @@ struct node_record {
 	List gres_list;			/* list of gres state info managed by
 					 * plugins */
 	uint32_t index;			/* Index into node_record_table_ptr */
+	char *instance_id;		/* cloud instance id */
+	char *instance_type;		/* cloud instance type */
 	time_t last_busy;		/* time node was last busy (no jobs) */
 	time_t last_response;		/* last response from the node */
 	uint32_t magic;			/* magic cookie for data integrity */
@@ -231,7 +235,7 @@ char * bitmap2node_name (bitstr_t *bitmap);
  * globals: node_record_table_ptr - pointer to node table
  * NOTE: the caller must xfree the memory at node_list when no longer required
  */
-hostlist_t bitmap2hostlist (bitstr_t *bitmap);
+hostlist_t *bitmap2hostlist(bitstr_t *bitmap);
 
 /*
  * build_all_nodeline_info - get a array of slurm_conf_node_t structures
@@ -257,14 +261,15 @@ extern int build_node_spec_bitmap(node_record_t *node_ptr);
 /*
  * Expand a nodeline's node names, host names, addrs, ports into separate nodes.
  */
-extern void expand_nodeline_info(slurm_conf_node_t *node_ptr,
-				 config_record_t *config_ptr,
-				 void (*_callback) (
-				       char *alias, char *hostname,
-				       char *address, char *bcast_addr,
-				       uint16_t port, int state_val,
-				       slurm_conf_node_t *node_ptr,
-				       config_record_t *config_ptr));
+extern int expand_nodeline_info(slurm_conf_node_t *node_ptr,
+				config_record_t *config_ptr,
+				char **err_msg,
+				int (*_callback) (
+					char *alias, char *hostname,
+					char *address, char *bcast_addr,
+					uint16_t port, int state_val,
+					slurm_conf_node_t *node_ptr,
+					config_record_t *config_ptr));
 
 /*
  * create_config_record - create a config_record entry and set is values to
@@ -290,18 +295,19 @@ extern config_record_t *config_record_from_conf_node(
 /*
  * Grow the node_record_table_ptr.
  */
-extern void grow_node_record_table_ptr();
+extern void grow_node_record_table_ptr(void);
 
 /*
  * create_node_record - create a node record and set its values to defaults
  * IN config_ptr - pointer to node's configuration information
  * IN node_name - name of the node
- * RET pointer to the record or NULL if error
+ * OUT node_ptr - node_record_t** with created node on SUCESS, NULL otherwise.
+ * RET SUCESS, or error code
  * NOTE: grows node_record_table_ptr if needed and appends a new node_record_t *
  *       to node_record_table_ptr and increases node_record_count.
  */
-extern node_record_t *create_node_record(config_record_t *config_ptr,
-					 char *node_name);
+extern int create_node_record(config_record_t *config_ptr, char *node_name,
+			      node_record_t **node_ptr);
 
 /*
  * Create a new node_record_t * at the specified index.
@@ -323,9 +329,11 @@ extern node_record_t *create_node_record_at(int index, char *node_name,
  *
  * IN alias - name of node.
  * IN config_ptr - config_record_t* to initialize node with.
- * RET node_record_t* on SUCESS, NULL otherwise.
+ * OUT node_ptr - node_record_t** with added node on SUCESS, NULL otherwise.
+ * RET SUCESS, or error code
  */
-extern node_record_t *add_node_record(char *alias, config_record_t *config_ptr);
+extern int add_node_record(char *alias, config_record_t *config_ptr,
+			   node_record_t **node_ptr);
 
 /*
  * Add existing record to node_record_table_ptr
@@ -375,7 +383,7 @@ extern node_record_t *find_node_record_no_alias(char *name);
  * OUT bitmap     - set to bitmap, may not have all bits set on error
  * RET 0 if no error, otherwise EINVAL
  */
-extern int hostlist2bitmap (hostlist_t hl, bool best_effort, bitstr_t **bitmap);
+extern int hostlist2bitmap(hostlist_t *hl, bool best_effort, bitstr_t **bitmap);
 
 /*
  * init_node_conf - initialize the node configuration tables and values.
@@ -424,10 +432,6 @@ extern void cr_fini_global_core_data(void);
 /*return the coremap index to the first core of the given node */
 extern uint32_t cr_get_coremap_offset(uint32_t node_index);
 
-/* Return a bitmap the size of the machine in cores. On a Bluegene
- * system it will return a bitmap in cnodes. */
-extern bitstr_t *cr_create_cluster_core_bitmap(int core_mult);
-
 /*
  * Determine maximum number of CPUs on this node usable by a job
  * ntasks_per_core IN - tasks-per-core to be launched by this job
@@ -504,4 +508,12 @@ extern void node_conf_set_all_active_bits(bitstr_t *b);
  * NOTE: Like strtok_r() characters in s may be modified.
  */
 extern char *node_conf_nodestr_tokenize(char *s, char **save_ptr);
+
+/*
+ * Make a bitmap the size of the full cluster.
+ * IN/OUT core_bitmap - If *core_bitmap noop, otherwise create a bitstr_t the
+ *                      size of the cluster.
+ */
+extern void node_conf_create_cluster_core_bitmap(bitstr_t **core_bitmap);
+
 #endif /* !_HAVE_NODE_CONF_H */
