@@ -63,7 +63,6 @@
 
 /* If we unpack a buffer that contains bad data, we want to avoid a memory
  * allocation error due to array or buffer sizes that are unreasonably large */
-#define MAX_PACK_ARRAY_LEN	(128 * 1024)
 #define MAX_PACK_MEM_LEN	(1024 * 1024 * 1024)
 
 typedef struct {
@@ -72,6 +71,7 @@ typedef struct {
 	uint32_t size;
 	uint32_t processed;
 	bool mmaped;
+	bool shadow;
 } buf_t;
 
 #define get_buf_data(__buf)		(__buf->head)
@@ -80,8 +80,15 @@ typedef struct {
 #define remaining_buf(__buf)		(__buf->size - __buf->processed)
 #define size_buf(__buf)			(__buf->size)
 
+typedef struct {
+	buf_t *header;
+	buf_t *auth;
+	buf_t *body;
+} msg_bufs_t;
+
 extern buf_t *create_buf(char *data, uint32_t size);
 extern buf_t *create_mmap_buf(const char *file);
+extern buf_t *create_shadow_buf(char *data, uint32_t size);
 extern void free_buf(buf_t *my_buf);
 extern buf_t *init_buf(uint32_t size);
 extern void grow_buf(buf_t *my_buf, uint32_t size);
@@ -131,11 +138,13 @@ extern void packlongdouble_array(long double *valp, uint32_t size_val,
 extern int unpacklongdouble_array(long double **valp, uint32_t *size_val,
 				  buf_t *buffer);
 
+extern void packbuf(buf_t *source, buf_t *buffer);
+
 extern void packmem(void *valp, uint32_t size_val, buf_t *buffer);
 extern int unpackmem_ptr(char **valp, uint32_t *size_valp, buf_t *buffer);
 extern int unpackmem_xmalloc(char **valp, uint32_t *size_valp, buf_t *buffer);
-extern int unpackmem_malloc(char **valp, uint32_t *size_valp, buf_t *buffer);
 
+extern int unpackstr_xmalloc(char **valp, uint32_t *size_valp, buf_t *buffer);
 extern int unpackstr_xmalloc_escaped(char **valp, uint32_t *size_valp,
 				     buf_t *buffer);
 extern int unpackstr_xmalloc_chooser(char **valp, uint32_t *size_valp,
@@ -259,13 +268,6 @@ extern int unpackmem_array(char *valp, uint32_t size_valp, buf_t *buffer);
 		goto unpack_error;			\
 } while (0)
 
-#define safe_unpackmem_malloc(valp,size_valp,buf) do {	\
-	xassert(sizeof(*size_valp) == sizeof(uint32_t));\
-	xassert(buf->magic == BUF_MAGIC);		\
-	if (unpackmem_malloc(valp,size_valp,buf))	\
-		goto unpack_error;			\
-} while (0)
-
 #define packstr(str,buf) do {				\
 	uint32_t _size = 0;				\
 	if((char *)str != NULL)				\
@@ -328,14 +330,13 @@ extern int unpackmem_array(char *valp, uint32_t size_valp, buf_t *buffer);
 	FREE_NULL_BITMAP(b);				\
 } while (0)
 
-#define unpackstr_malloc	                        \
-        unpackmem_malloc
-
-#define unpackstr_xmalloc	                        \
-        unpackmem_xmalloc
-
-#define safe_unpackstr_malloc	                        \
-        safe_unpackmem_malloc
+#define unpack_bit_str_hex_as_fmt_str(str, buf) do {	\
+	bitstr_t *b = NULL;				\
+	unpack_bit_str_hex(&b, buf);			\
+	if (b)						\
+		*str = bit_fmt_full(b);			\
+	FREE_NULL_BITMAP(b);				\
+} while (0)
 
 #define safe_unpackstr(valp, buf) do {				\
 	uint32_t size_valp;					\
