@@ -1202,8 +1202,20 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 			      temp.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO));
 		if (!last_ancestor)
 			last_ancestor = xbasename(slurm_conf_path);
-		rc = s_p_parse_file(hashtbl, hash_val, path_name, flags,
-				    last_ancestor);
+
+		if ((xstrstr(file_name, "*")) &&
+		    ((!xstrcasecmp(last_ancestor,"slurm.conf")) ||
+		     (!(slurm_conf.debug_flags & DEBUG_FLAG_GLOB_SILENCE)))) {
+			error("Slurm does not support glob parsing. %s from %s will be skipped over. If this expected, ignore this message and set DebugFlags=GLOB_SILENCE in your slurm.conf.",
+			      path_name, last_ancestor);
+			xfree(path_name);
+			xfree(file_name);
+			return -1;
+		} else {
+			rc = s_p_parse_file(hashtbl, hash_val, path_name, flags,
+					    last_ancestor);
+		}
+
 		xfree(path_name);
 		if (rc == SLURM_SUCCESS) {
 			if (!xstrstr(file_name, "/") && running_in_slurmctld())
