@@ -43,12 +43,11 @@
 #include <ctype.h>		/* isdigit    */
 #include <fcntl.h>
 #include <limits.h>
-#include <pwd.h>		/* getpwuid   */
+#include <signal.h>
 #include <stdarg.h>		/* va_start   */
 #include <stdio.h>
 #include <stdlib.h>		/* getenv, strtoll */
 #include <string.h>		/* strcpy */
-#include <sys/param.h>		/* MAXPATHLEN */
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
@@ -426,7 +425,7 @@ uint64_t str_to_mbytes(const char *arg)
 		;
 	else if (((endptr[0] == 'k') || (endptr[0] == 'K')) &&
 		 _end_on_byte(endptr))
-		result = (result + 1023) / 1024;	/* round up */
+		result = ROUNDUP(result, 1024);	/* round up */
 	else if (((endptr[0] == 'm') || (endptr[0] == 'M')) &&
 	         _end_on_byte(endptr))
 		;
@@ -439,9 +438,6 @@ uint64_t str_to_mbytes(const char *arg)
 	else
 		return NO_VAL64;
 
-	if (result < 0)
-		return NO_VAL64;
-
 	return (uint64_t) result;
 }
 
@@ -467,8 +463,7 @@ extern char *mbytes_to_str(uint64_t mbytes)
 }
 
 /* Convert a string into a node count */
-static int
-_str_to_nodes(const char *num_str, char **leftover)
+extern int str_to_nodes(const char *num_str, char **leftover)
 {
 	long int num;
 	char *endptr;
@@ -553,7 +548,7 @@ bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes,
 		xfree(tmp_str);
 	} else if ((ptr = xstrchr(arg, '-')) != NULL) {
 		min_str = xstrndup(arg, ptr-arg);
-		*min_nodes = _str_to_nodes(min_str, &leftover);
+		*min_nodes = str_to_nodes(min_str, &leftover);
 		if (!xstring_is_whitespace(leftover)) {
 			error("\"%s\" is not a valid node count", min_str);
 			xfree(min_str);
@@ -564,7 +559,7 @@ bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes,
 			*min_nodes = 1;
 
 		max_str = xstrndup(ptr+1, strlen(arg)-((ptr+1)-arg));
-		*max_nodes = _str_to_nodes(max_str, &leftover);
+		*max_nodes = str_to_nodes(max_str, &leftover);
 		if (!xstring_is_whitespace(leftover)) {
 			error("\"%s\" is not a valid node count", max_str);
 			xfree(max_str);
@@ -572,7 +567,7 @@ bool verify_node_count(const char *arg, int *min_nodes, int *max_nodes,
 		}
 		xfree(max_str);
 	} else {
-		*min_nodes = *max_nodes = _str_to_nodes(arg, &leftover);
+		*min_nodes = *max_nodes = str_to_nodes(arg, &leftover);
 		if (!xstring_is_whitespace(leftover)) {
 			error("\"%s\" is not a valid node count", arg);
 			return false;
@@ -666,7 +661,7 @@ bool get_resource_arg_range(const char *arg, const char *what, int* min,
 			exit(1);
 		return false;
 	} else if (result > INT_MAX) {
-		error ("Numeric argument (%ld) to big for %s.", result, what);
+		error("Numeric argument (%ld) too large for %s.", result, what);
 		if (isFatal)
 			exit(1);
 		return false;
@@ -694,7 +689,7 @@ bool get_resource_arg_range(const char *arg, const char *what, int* min,
 			exit(1);
 		return false;
 	} else if (result > INT_MAX) {
-		error ("Numeric argument (%ld) to big for %s.", result, what);
+		error("Numeric argument (%ld) too large for %s.", result, what);
 		if (isFatal)
 			exit(1);
 		return false;
@@ -1231,8 +1226,10 @@ static struct {
 	{ "KILL",	SIGKILL	},
 	{ "ALRM",	SIGALRM	},
 	{ "TERM",	SIGTERM	},
+	{ "CHLD",	SIGCHLD	},
 	{ "USR1",	SIGUSR1	},
 	{ "USR2",	SIGUSR2	},
+	{ "PIPE",	SIGPIPE	},
 	{ "URG",	SIGURG	},
 	{ "CONT",	SIGCONT	},
 	{ "STOP",	SIGSTOP	},
@@ -1404,7 +1401,7 @@ extern int parse_int(const char *name, const char *val, bool positive)
 		error ("Invalid numeric value \"%s\" for %s.", val, name);
 		exit(1);
 	} else if (result >= INT_MAX) {
-		error ("Numeric argument (%ld) to big for %s.", result, name);
+		error("Numeric argument (%ld) too large for %s.", result, name);
 		exit(1);
 	} else if (result <= INT_MIN) {
 		error ("Numeric argument (%ld) to small for %s.", result, name);
@@ -1578,10 +1575,6 @@ extern uint64_t parse_resv_flags(const char *flagstr, const char *msg,
 				outflags |= RESERVE_FLAG_NO_PURGE_COMP;
 			else
 				outflags |= RESERVE_FLAG_PURGE_COMP;
-		} else if (!xstrncasecmp(curr, "First_Cores", MAX(taglen,1)) &&
-			   op != RESV_REM) {
-			curr += taglen;
-			outflags |= RESERVE_FLAG_FIRST_CORES;
 		} else if (!xstrncasecmp(curr, "Time_Float", MAX(taglen,1)) &&
 			   op == RESV_NEW) {
 			curr += taglen;
@@ -1744,7 +1737,7 @@ extern int validate_acctg_freq(char *acctg_freq)
 /*
  * Format a tres_per_* argument
  * dest OUT - resulting string
- * prefix IN - TRES type (e.g. "gres:gpu")
+ * prefix IN - TRES type (e.g. "gres/gpu")
  * src IN - user input, can include multiple comma-separated specifications
  */
 extern void xfmt_tres(char **dest, char *prefix, char *src)
@@ -1771,7 +1764,7 @@ extern void xfmt_tres(char **dest, char *prefix, char *src)
 /*
  * Format a tres_freq argument
  * dest OUT - resulting string
- * prefix IN - TRES type (e.g. "gres:gpu")
+ * prefix IN - TRES type (e.g. "gres/gpu")
  * src IN - user input
  */
 extern void xfmt_tres_freq(char **dest, char *prefix, char *src)
