@@ -83,9 +83,11 @@ enum {
 	LONG_OPT_BURST_BUFFER_SPEC,
 	LONG_OPT_CLUSTER,
 	LONG_OPT_CLUSTER_CONSTRAINT,
+	LONG_OPT_COMPLETE_FLAG,
 	LONG_OPT_COMMENT,
 	LONG_OPT_COMPRESS,
 	LONG_OPT_CONTAINER,
+	LONG_OPT_CONTAINER_ID,
 	LONG_OPT_CONTEXT,
 	LONG_OPT_CONTIGUOUS,
 	LONG_OPT_CORE,
@@ -101,7 +103,8 @@ enum {
 	LONG_OPT_EXACT,
 	LONG_OPT_EXCLUSIVE,
 	LONG_OPT_EXPORT,
-	LONG_OPT_EXPORT_FILE,
+	LONG_OPT_EXTERNAL_LAUNCHER,
+	LONG_OPT_EXTRA,
 	LONG_OPT_GET_USER_ENV,
 	LONG_OPT_GID,
 	LONG_OPT_GPU_BIND,
@@ -177,7 +180,10 @@ enum {
 	LONG_OPT_THREADSPERCORE,
 	LONG_OPT_TIME_MIN,
 	LONG_OPT_TMP,
+	LONG_OPT_TREE_WIDTH,
+	LONG_OPT_TRES_BIND,
 	LONG_OPT_TRES_PER_JOB,
+	LONG_OPT_TRES_PER_TASK,
 	LONG_OPT_UID,
 	LONG_OPT_UMASK,
 	LONG_OPT_USAGE,
@@ -204,13 +210,8 @@ typedef struct {
  * options only processed by sbatch
  */
 typedef struct {
-	/* batch script argv and argc, if provided on the command line */
-	int script_argc;
-	char **script_argv;
-
 	char *array_inx;		/* --array			*/
 	char *batch_features;		/* --batch			*/
-	char *export_file;		/* --export-file=file		*/
 	bool ignore_pbs;		/* --ignore-pbs			*/
 	int minsockets;			/* --minsockets=n		*/
 	int mincores;			/* --mincores=n			*/
@@ -233,9 +234,6 @@ typedef struct {
  * options only processed by srun
  */
 typedef struct {
-	int argc;			/* length of argv array		*/
-	char **argv;			/* left over on command line	*/
-
 	uint16_t accel_bind_type;	/* --accel-bind			*/
 	char *alloc_nodelist;		/* grabbed from the environment	*/
 	char *bcast_exclude;		/* --bcast-exclude */
@@ -248,11 +246,13 @@ typedef struct {
 	cpu_bind_type_t cpu_bind_type;	/* --cpu-bind			*/
 	bool debugger_test;		/* --debugger-test		*/
 	bool disable_status;		/* --disable-status		*/
+	bool external_launcher;		/* --external-launcher		*/
 	char *epilog;			/* --epilog			*/
 	bool exact;			/* --exact			*/
 	bool exclusive;			/* --exclusive			*/
 	bool interactive;		/* --interactive		*/
 	uint32_t jobid;			/* --jobid			*/
+	uint32_t array_task_id;		/* --jobid			*/
 	int32_t kill_bad_exit;		/* --kill-on-bad-exit		*/
 	bool labelio;			/* --label-output		*/
 	int32_t max_threads;		/* --threads			*/
@@ -270,7 +270,7 @@ typedef struct {
 	bool preserve_env;		/* --preserve-env		*/
 	char *prolog;			/* --prolog			*/
 	char *propagate;		/* --propagate[=RLIMIT_CORE,...]*/
-	bool pty;			/* --pty			*/
+	char *pty;			/* --pty[=fd]			*/
 	bool quit_on_intr;		/* --quit-on-interrupt		*/
 	int relative;			/* --relative			*/
 	int resv_port_cnt;		/* --resv_ports			*/
@@ -280,6 +280,7 @@ typedef struct {
 	char *task_prolog;		/* --task-prolog		*/
 	bool test_exec;			/* test_exec set		*/
 	bool test_only;			/* --test-only			*/
+	uint16_t tree_width;		/* --treewidth			*/
 	bool unbuffered;		/* --unbuffered			*/
 	bool whole;			/* --whole			*/
 } srun_opt_t;
@@ -300,7 +301,10 @@ typedef struct {
 
 	void (*help_func)(void);	/* Print --help info		*/
 	void (*usage_func)(void);	/* Print --usage info		*/
+	void (*autocomplete_func)(const char *); /* Print --autocomplete= info*/
 
+	int argc;			/* command/script argc		*/
+	char **argv;			/* command/script argv		*/
 	char *burst_buffer;		/* --bb				*/
 	char *burst_buffer_file;	/* --bbf			*/
 	char *clusters;			/* cluster to run this on. */
@@ -313,10 +317,11 @@ typedef struct {
 	bool cpus_set;			/* cpus_per_task explicitly set	*/
 	int min_nodes;			/* --nodes=n			*/
 	int max_nodes;			/* --nodes=x-n			*/
+	char *job_size_str;		/* --nodes			*/
 	bool nodes_set;			/* nodes explicitly set		*/
 	int sockets_per_node;		/* --sockets-per-node=n		*/
 	int cores_per_socket;		/* --cores-per-socket=n		*/
-	uint32_t job_flags;		/* --kill_invalid_dep, --gres-flags */
+	uint64_t job_flags;		/* --kill_invalid_dep, --gres-flags */
 	int threads_per_core;		/* --threads-per-core=n		*/
 	int ntasks_per_node;		/* --ntasks-per-node=n		*/
 	int ntasks_per_gpu;		/* --ntasks-per-gpu=n		*/
@@ -377,6 +382,7 @@ typedef struct {
 	char *c_constraint;		/* --cluster-constraint		*/
 	char *gres;			/* --gres			*/
 	char *container;		/* --container			*/
+	char *container_id;		/* --container-id		*/
 	char *context;			/* --context			*/
 	bool contiguous;		/* --contiguous			*/
 	char *nodefile;			/* --nodefile			*/
@@ -387,7 +393,7 @@ typedef struct {
 	bool reboot;			/* --reboot			*/
 
 	time_t begin;			/* --begin			*/
-	char *extra;			/* unused			*/
+	char *extra;			/* --extra			*/
 	uint16_t mail_type;		/* --mail-type			*/
 	char *mail_user;		/* --mail-user			*/
 	int get_user_env_time;		/* --get-user-env[=timeout]	*/
@@ -413,6 +419,8 @@ typedef struct {
 	char *submit_line;		/* submit line of the caller	*/
 	char *tres_bind;		/* derived from gpu_bind	*/
 	char *tres_freq;		/* derived from gpu_freq	*/
+	char *tres_per_task;		/* --tres_per_task		*/
+
 	uint16_t x11;			/* --x11			*/
 	char *x11_magic_cookie;		/* cookie retrieved from xauth	*/
 	char *x11_target;		/* target host, or unix socket	*/
@@ -428,9 +436,28 @@ typedef struct {
 
 } slurm_opt_t;
 
+/*
+ * Parse a tres request and return the sum of the requested counts for a
+ * particular tres.
+ *
+ * IN in_val - The tres request of the format: "name[[:type]:count]". This can
+ *             be a comma-delimited list.
+ * IN tres_name - name of the tres to sum
+ * OUT cnt - total count
+ * IN/OUT save_ptr - pointer to make this function reentrant
+ * OUT rc - SLURM_SUCCESS if successful, an error code otherwise
+ *
+ * RET - true if there are additional values to parse, false if the whole string
+ *       has been parsed
+ */
+extern bool slurm_option_get_tres_per_tres(
+	char *in_val, char *tres_name, uint64_t *cnt, char **save_ptr, int *rc);
+
 extern struct option *slurm_option_table_create(slurm_opt_t *opt,
 						char **opt_string);
 extern void slurm_option_table_destroy(struct option *optz);
+extern void slurm_option_update_tres_per_task_cpu(int cpus_per_task,
+						  char **tres_per_task_p);
 
 /*
  * Process individual argument for the current job component
@@ -492,7 +519,7 @@ extern bool slurm_option_set_by_env(slurm_opt_t *opt, int optval);
 /*
  * Was the option set by an data_t value?
  */
-extern bool slurm_option_set_by_env(slurm_opt_t *opt, int optval);
+extern bool slurm_option_set_by_data(slurm_opt_t *opt, int optval);
 
 /*
  * Get option value by common option name.
@@ -568,4 +595,9 @@ extern char *slurm_option_get_argv_str(const int argc, char **argv);
 extern job_desc_msg_t *slurm_opt_create_job_desc(slurm_opt_t *opt_local,
 						 bool set_defaults);
 
+/*
+ * Compatible with shell/bash completions.
+ */
+extern void suggest_completion(struct option *opts, const char *query);
+
 #endif	/* _SLURM_OPT_H_ */
