@@ -81,8 +81,6 @@
 #define	bit_clear_all		slurm_bit_clear_all
 #define	bit_ffc			slurm_bit_ffc
 #define	bit_ffs			slurm_bit_ffs
-#define	bit_free		slurm_bit_free
-#define	bit_realloc		slurm_bit_realloc
 #define	bit_size		slurm_bit_size
 #define	bit_and			slurm_bit_and
 #define	bit_not			slurm_bit_not
@@ -104,6 +102,7 @@
 #define	bit_fmt_binmask		slurm_bit_fmt_binmask
 #define bit_unfmt_binmask	slurm_bit_unfmt_binmask
 #define	bit_fls			slurm_bit_fls
+#define	bit_fls_from_bit	slurm_bit_fls_from_bit
 #define	bit_fill_gaps		slurm_bit_fill_gaps
 #define	bit_super_set		slurm_bit_super_set
 #define	bit_overlap		slurm_bit_overlap
@@ -116,7 +115,6 @@
 #define bit_nffs		slurm_bit_nffs
 #define bit_copybits		slurm_bit_copybits
 #define	bit_get_bit_num		slurm_bit_get_bit_num
-#define	bit_get_pos_num		slurm_bit_get_pos_num
 
 /* fd.[ch] functions */
 #define closeall		slurm_closeall
@@ -125,6 +123,7 @@
 #define fd_get_socket_error	slurm_fd_get_socket_error
 #define send_fd_over_pipe	slurm_send_fd_over_pipe
 #define receive_fd_over_pipe	slurm_receive_fd_over_pipe
+#define rmdir_recursive		slurm_rmdir_recursive
 
 /* hostlist.[ch] functions */
 #define	hostlist_create_dims	slurm_hostlist_create_dims
@@ -137,8 +136,6 @@
 #define	hostlist_deranged_string_dims \
 				slurm_hostlist_deranged_string_dims
 #define	hostlist_deranged_string slurm_hostlist_deranged_string
-#define	hostlist_deranged_string_malloc \
-				slurm_hostlist_deranged_string_malloc
 #define	hostlist_deranged_string_xmalloc_dims \
 				slurm_hostlist_deranged_string_xmalloc_dims
 #define	hostlist_deranged_string_xmalloc \
@@ -149,10 +146,8 @@
 #define	hostlist_iterator_destroy slurm_hostlist_iterator_destroy
 #define	hostlist_iterator_reset	slurm_hostlist_iterator_reset
 #define	hostlist_next		slurm_hostlist_next
-#define	hostlist_next_range	slurm_hostlist_next_range
 #define	hostlist_nth		slurm_hostlist_nth
 #define	hostlist_pop            slurm_hostlist_pop
-#define	hostlist_pop_range      slurm_hostlist_pop_range
 #define	hostlist_push		slurm_hostlist_push
 #define	hostlist_push_host_dims	slurm_hostlist_push_host_dims
 #define	hostlist_push_host	slurm_hostlist_push_host
@@ -160,8 +155,6 @@
 #define hostlist_ranged_string_dims \
 				slurm_hostlist_ranged_string_dims
 #define	hostlist_ranged_string	slurm_hostlist_ranged_string
-#define	hostlist_ranged_string_malloc \
-				slurm_hostlist_ranged_string_malloc
 #define	hostlist_ranged_string_xmalloc_dims \
 				slurm_hostlist_ranged_string_xmalloc_dims
 #define	hostlist_ranged_string_xmalloc \
@@ -169,32 +162,34 @@
 #define	hostlist_remove		slurm_hostlist_remove
 #define	hostlist_shift		slurm_hostlist_shift
 #define	hostlist_shift_dims	slurm_hostlist_shift_dims
-#define	hostlist_shift_range	slurm_hostlist_shift_range
 #define	hostlist_sort		slurm_hostlist_sort
 #define	hostlist_cmp_first	slurm_hostlist_cmp_first
 #define	hostlist_uniq		slurm_hostlist_uniq
-#define	hostset_copy		slurm_hostset_copy
 #define	hostset_count		slurm_hostset_count
 #define	hostset_create		slurm_hostset_create
 #define	hostset_delete		slurm_hostset_delete
 #define	hostset_destroy		slurm_hostset_destroy
 #define	hostset_find		slurm_hostset_find
 #define	hostset_insert		slurm_hostset_insert
+#define hostset_deranged_string_xmalloc \
+				slurm_hostset_deranged_string_xmalloc
+#define hostset_ranged_string_xmalloc \
+				slurm_hostset_ranged_string_xmalloc
 #define	hostset_shift		slurm_hostset_shift
-#define	hostset_shift_range	slurm_hostset_shift_range
 #define	hostset_within		slurm_hostset_within
 #define	hostset_nth		slurm_hostset_nth
 
 /* gres.[ch] functions */
 #define gres_find_id		slurm_gres_find_id
+#define gres_find_job_by_key_exact_type slurm_gres_find_job_by_key_exact_type
 #define gres_find_sock_by_job_state slurm_gres_find_sock_by_job_state
 #define gres_get_node_used	slurm_gres_get_node_used
 #define gres_get_system_cnt	slurm_gres_get_system_cnt
 #define gres_get_value_by_type	slurm_gres_get_value_by_type
 #define gres_get_job_info	slurm_gres_get_job_info
 #define gres_get_step_info	slurm_gres_get_step_info
-#define gres_device_major	slurm_gres_device_major
 #define gres_sock_delete	slurm_gres_sock_delete
+#define gres_job_list_delete	slurm_gres_job_list_delete
 #define destroy_gres_device	slurm_destroy_gres_device
 #define destroy_gres_slurmd_conf slurm_destroy_gres_slurmd_conf
 
@@ -208,12 +203,15 @@
 #define	list_append_list	slurm_list_append_list
 #define	list_transfer		slurm_list_transfer
 #define	list_transfer_max	slurm_list_transfer_max
+#define	list_transfer_unique	slurm_list_transfer_unique
 #define	list_prepend		slurm_list_prepend
 #define	list_find_first		slurm_list_find_first
+#define	list_find_first_ro	slurm_list_find_first_ro
 #define	list_delete_all		slurm_list_delete_all
 #define	list_delete_first	slurm_list_delete_first
 #define	list_delete_ptr		slurm_list_delete_ptr
 #define	list_for_each		slurm_list_for_each
+#define	list_for_each_ro	slurm_list_for_each_ro
 #define	list_for_each_max	slurm_list_for_each_max
 #define	list_sort		slurm_list_sort
 #define	list_flip		slurm_list_flip
@@ -229,6 +227,7 @@
 #define	list_insert		slurm_list_insert
 #define	list_find		slurm_list_find
 #define	list_remove		slurm_list_remove
+#define	list_remove_first	slurm_list_remove_first
 #define	list_delete_item	slurm_list_delete_item
 #define list_flush		slurm_list_flush
 #define list_flush_max		slurm_list_flush_max
@@ -241,7 +240,7 @@
 #define	log_fini		slurm_log_fini
 #define	log_alter		slurm_log_alter
 #define	log_alter_with_fp	slurm_log_alter_with_fp
-#define	log_set_fpfx		slurm_log_set_fpfx
+#define	log_set_prefix		slurm_log_set_prefix
 #define	log_fp			slurm_log_fp
 #define	log_oom			slurm_log_oom
 #define	log_has_data		slurm_log_has_data
@@ -267,6 +266,8 @@
 #define build_all_nodeline_info slurm_build_all_nodeline_info
 #define rehash_node		slurm_rehash_node
 #define hostlist2bitmap		slurm_hostlist2bitmap
+#define bitmap2node_name	slurm_bitmap2node_name
+#define find_node_record	slurm_find_node_record
 
 /* pack.[ch] functions */
 #define	create_buf		slurm_create_buf
@@ -300,7 +301,7 @@
 #define	packmem			slurm_packmem
 #define	unpackmem_ptr		slurm_unpackmem_ptr
 #define	unpackmem_xmalloc	slurm_unpackmem_xmalloc
-#define	unpackmem_malloc	slurm_unpackmem_malloc
+#define	unpackstr_xmalloc	slurm_unpackstr_xmalloc
 #define	unpackstr_xmalloc_escaped slurm_unpackstr_xmalloc_escaped
 #define	unpackstr_xmalloc_chooser slurm_unpackstr_xmalloc_chooser
 #define	packstr_array		slurm_packstr_array
@@ -314,7 +315,6 @@
 #define time_str2secs           slurm_time_str2secs
 #define secs2time_str           slurm_secs2time_str
 #define mins2time_str           slurm_mins2time_str
-#define mon_abbr                slurm_mon_abbr
 
 /* env.[ch] functions */
 #define	setenvf 		slurm_setenvpf
@@ -337,10 +337,12 @@
 #define destroy_config_key_pair	slurm_destroy_config_key_pair
 #define get_extra_conf_path	slurm_get_extra_conf_path
 #define sort_key_pairs		slurm_sort_key_pairs
+#define conf_get_opt_str	slurm_conf_get_opt_str
 
 /* run_in_daemon.[ch] functions */
 #define run_in_daemon           slurm_run_in_daemon
 #define running_in_daemon	slurm_running_in_daemon
+#define running_in_sackd        slurm_running_in_sackd
 #define running_in_slurmctld    slurm_running_in_slurmctld
 #define running_in_slurmd       slurm_running_in_slurmd
 #define running_in_slurmdbd     slurm_running_in_slurmdbd
@@ -378,6 +380,7 @@
 
 /* xstring.[ch] functions */
 #define	_xstrcat		slurm_xstrcat
+#define	_xstrcatat		slurm_xstrcatat
 #define	_xstrncat		slurm_xstrncat
 #define	_xstrcatchar		slurm_xstrcatchar
 #define	_xstrftimecat		slurm_xstrftimecat
@@ -449,7 +452,6 @@
 /* plugin.[ch] functions */
 #define plugin_get_syms         slurm_plugin_get_syms
 #define plugin_load_and_link    slurm_plugin_load_and_link
-#define plugin_strerror         slurm_plugin_strerror
 #define plugin_unload           slurm_plugin_unload
 
 /* plugrack.[ch] functions */
@@ -520,39 +522,18 @@
 #define xfree_struct_passwd		slurm_xfree_struct_passwd
 #define stepd_getgr			slurm_stepd_getgr
 #define xfree_struct_group_array	slurm_xfree_struct_group_array
+#define stepd_gethostbyname		slurm_stepd_gethostbyname
+#define xfree_struct_hostent		slurm_xfree_struct_hostent
 #define stepd_get_namespace_fd		slurm_stepd_get_namespace_fd
 
 /* cgroup.[ch] functions */
 #define cgroup_conf_init		slurm_cgroup_conf_init
 #define cgroup_conf_destroy		slurm_cgroup_conf_destroy
+#define autodetect_cgroup_version       slurm_autodetect_cgroup_version
+
+/* topology.[ch] functions */
+#define topology_g_build_config slurm_topology_g_build_config
 
 #endif /* USE_ALIAS */
 
-/* Include the function definitions after redefining their names. */
-#include "src/common/bitstring.h"
-#include "src/common/callerid.h"
-#include "src/common/cgroup.h"
-#include "src/common/eio.h"
-#include "src/common/env.h"
-#include "src/common/hostlist.h"
-#include "src/common/list.h"
-#include "src/common/log.h"
-#include "src/common/macros.h"
-#include "src/common/node_select.h"
-#include "src/common/pack.h"
-#include "src/common/parse_config.h"
-#include "src/common/read_config.h"
-#include "src/common/slurm_auth.h"
-#include "src/common/slurm_jobacct_gather.h"
-#include "src/common/slurm_route.h"
-#include "src/common/slurm_step_layout.h"
-#include "src/common/strlcpy.h"
-#include "src/common/stepd_api.h"
-#include "src/common/switch.h"
-#include "src/common/working_cluster.h"
-#include "src/common/xassert.h"
-#include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
-#include "src/common/xstring.h"
-
 #endif /*__SLURM_XLATOR_H__*/
