@@ -57,6 +57,8 @@ extern slurmdbd_msg_type_t str_2_slurmdbd_msg_type(char *msg_type)
 		return DBD_FINI;
 	} else if (!xstrcasecmp(msg_type, "Add Accounts")) {
 		return DBD_ADD_ACCOUNTS;
+	} else if (!xstrcasecmp(msg_type, "Add Accounts Cond")) {
+		return DBD_ADD_ACCOUNTS_COND;
 	} else if (!xstrcasecmp(msg_type, "Add Account Coord")) {
 		return DBD_ADD_ACCOUNT_COORDS;
 	} else if (!xstrcasecmp(msg_type, "Add TRES")) {
@@ -71,6 +73,8 @@ extern slurmdbd_msg_type_t str_2_slurmdbd_msg_type(char *msg_type)
 		return DBD_ADD_RES;
 	} else if (!xstrcasecmp(msg_type, "Add Users")) {
 		return DBD_ADD_USERS;
+	} else if (!xstrcasecmp(msg_type, "Add Users Cond")) {
+		return DBD_ADD_USERS_COND;
 	} else if (!xstrcasecmp(msg_type, "Cluster TRES")) {
 		return DBD_CLUSTER_TRES;
 	} else if (!xstrcasecmp(msg_type, "Flush Jobs")) {
@@ -91,6 +95,8 @@ extern slurmdbd_msg_type_t str_2_slurmdbd_msg_type(char *msg_type)
 		return DBD_GET_EVENTS;
 	} else if (!xstrcasecmp(msg_type, "Get Federations")) {
 		return DBD_GET_FEDERATIONS;
+	} else if (!xstrcasecmp(msg_type, "Get Instances")) {
+		return DBD_GET_INSTANCES;
 	} else if (!xstrcasecmp(msg_type, "Reconfigure")) {
 		return DBD_RECONFIG;
 	} else if (!xstrcasecmp(msg_type, "Get Problems")) {
@@ -115,6 +121,8 @@ extern slurmdbd_msg_type_t str_2_slurmdbd_msg_type(char *msg_type)
 		return DBD_GOT_EVENTS;
 	} else if (!xstrcasecmp(msg_type, "Got Federations")) {
 		return DBD_GOT_FEDERATIONS;
+	} else if (!xstrcasecmp(msg_type, "Got Instances")) {
+		return DBD_GOT_INSTANCES;
 	} else if (!xstrcasecmp(msg_type, "Got Jobs")) {
 		return DBD_GOT_JOBS;
 	} else if (!xstrcasecmp(msg_type, "Got List")) {
@@ -254,6 +262,12 @@ extern char *slurmdbd_msg_type_2_str(slurmdbd_msg_type_t msg_type, int get_enum)
 		} else
 			return "Add Accounts";
 		break;
+	case DBD_ADD_ACCOUNTS_COND:
+		if (get_enum) {
+			return "DBD_ADD_ACCOUNTS_COND";
+		} else
+			return "Add Accounts Cond";
+		break;
 	case DBD_ADD_ACCOUNT_COORDS:
 		if (get_enum) {
 			return "DBD_ADD_ACCOUNT_COORDS";
@@ -296,6 +310,12 @@ extern char *slurmdbd_msg_type_2_str(slurmdbd_msg_type_t msg_type, int get_enum)
 		} else
 			return "Add Users";
 		break;
+	case DBD_ADD_USERS_COND:
+		if (get_enum) {
+			return "DBD_ADD_USERS_COND";
+		} else
+			return "Add Users Cond";
+		break;
 	case DBD_CLUSTER_TRES:
 		if (get_enum) {
 			return "DBD_CLUSTER_TRES";
@@ -356,6 +376,12 @@ extern char *slurmdbd_msg_type_2_str(slurmdbd_msg_type_t msg_type, int get_enum)
 		} else
 			return "Get Federations";
 		break;
+	case DBD_GET_INSTANCES:
+		if (get_enum) {
+			return "DBD_GET_INSTANCES";
+		} else
+			return "Get Instances";
+		break;
 	case DBD_RECONFIG:
 		if (get_enum) {
 			return "DBD_RECONFIG";
@@ -428,6 +454,12 @@ extern char *slurmdbd_msg_type_2_str(slurmdbd_msg_type_t msg_type, int get_enum)
 		} else
 			return "Got Federations";
 		break;
+	case DBD_GOT_INSTANCES:
+		if (get_enum) {
+			return "DBD_GOT_INSTANCES";
+		} else
+			return "Got Instances";
+		break;
 	case DBD_GOT_JOBS:
 		if (get_enum) {
 			return "DBD_GOT_JOBS";
@@ -845,6 +877,7 @@ extern void slurmdbd_free_msg(persist_msg_t *msg)
 	case DBD_GOT_CLUSTERS:
 	case DBD_GOT_EVENTS:
 	case DBD_GOT_FEDERATIONS:
+	case DBD_GOT_INSTANCES:
 	case DBD_GOT_JOBS:
 	case DBD_GOT_LIST:
 	case DBD_GOT_PROBS:
@@ -881,6 +914,7 @@ extern void slurmdbd_free_msg(persist_msg_t *msg)
 	case DBD_GET_CLUSTERS:
 	case DBD_GET_EVENTS:
 	case DBD_GET_FEDERATIONS:
+	case DBD_GET_INSTANCES:
 	case DBD_GET_JOBS_COND:
 	case DBD_GET_PROBS:
 	case DBD_GET_QOS:
@@ -923,6 +957,8 @@ extern void slurmdbd_free_msg(persist_msg_t *msg)
 	case DBD_JOB_SUSPEND:
 		slurmdbd_free_job_suspend_msg(msg->data);
 		break;
+	case DBD_ADD_ACCOUNTS_COND:
+	case DBD_ADD_USERS_COND:
 	case DBD_MODIFY_ACCOUNTS:
 	case DBD_MODIFY_ASSOCS:
 	case DBD_MODIFY_CLUSTERS:
@@ -1050,6 +1086,9 @@ extern void slurmdbd_free_cond_msg(dbd_cond_msg_t *msg,
 		case DBD_GET_EVENTS:
 			my_destroy = slurmdb_destroy_event_cond;
 			break;
+		case DBD_GET_INSTANCES:
+			my_destroy = slurmdb_destroy_instance_cond;
+			break;
 		default:
 			fatal("Unknown cond type");
 			return;
@@ -1147,6 +1186,14 @@ extern void slurmdbd_free_modify_msg(dbd_modify_msg_t *msg,
 
 	if (msg) {
 		switch (type) {
+		case DBD_ADD_ACCOUNTS_COND:
+			destroy_cond = slurmdb_destroy_add_assoc_cond;
+			destroy_rec = slurmdb_destroy_account_rec;
+			break;
+		case DBD_ADD_USERS_COND:
+			destroy_cond = slurmdb_destroy_add_assoc_cond;
+			destroy_rec = slurmdb_destroy_user_rec;
+			break;
 		case DBD_MODIFY_ACCOUNTS:
 			destroy_cond = slurmdb_destroy_account_cond;
 			destroy_rec = slurmdb_destroy_account_rec;
@@ -1196,6 +1243,9 @@ extern void slurmdbd_free_node_state_msg(dbd_node_state_msg_t *msg)
 {
 	if (msg) {
 		xfree(msg->hostlist);
+		xfree(msg->extra);
+		xfree(msg->instance_id);
+		xfree(msg->instance_type);
 		xfree(msg->reason);
 		xfree(msg->tres_str);
 		xfree(msg);
