@@ -44,12 +44,13 @@
 #include "slurm/slurm.h"
 
 #include "src/common/list.h"
-#include "src/common/slurm_accounting_storage.h"
+#include "src/interfaces/accounting_storage.h"
 
 /* Slurm DBD message types */
 /* ANY TIME YOU ADD TO THIS LIST UPDATE THE CONVERSION FUNCTIONS! */
 typedef enum {
-	DEFUNCT_DBD_INIT = 1400,/* Connection initialization		*/
+	SLURM_DBD_MESSAGES_START = 1400, /* So that we don't overlap with any
+					  * slurm_msg_type_t numbers. */
 	DBD_FINI,       	/* Connection finalization		*/
 	DBD_ADD_ACCOUNTS,       /* Add new account to the mix           */
 	DBD_ADD_ACCOUNT_COORDS, /* Add new coordinatior to an account   */
@@ -85,7 +86,7 @@ typedef enum {
 	DBD_MODIFY_CLUSTERS,    /* #1430, Modify existing cluster       */
 	DBD_MODIFY_USERS,       /* Modify existing user                 */
 	DBD_NODE_STATE,		/* Record node state transition		*/
-	DBD_DEFUNCT_RPC_1433,	/* Free for reuse			*/
+	SLURM_PERSIST_RC,	/* To mirror the DBD_RC this is replacing */
 	DBD_REGISTER_CTLD,	/* Register a slurmctld's comm port	*/
 	DBD_REMOVE_ACCOUNTS,    /* Remove existing account              */
 	DBD_REMOVE_ACCOUNT_COORDS,/* Remove existing coordinator from
@@ -152,7 +153,17 @@ typedef enum {
 	DBD_GOT_FEDERATIONS,	/* Response to DBD_GET_FEDERATIONS 	*/
 	DBD_MODIFY_FEDERATIONS, /* Modify existing federation 		*/
 	DBD_REMOVE_FEDERATIONS, /* Removing existing federation 	*/
-
+	DBD_JOB_HEAVY,         /* Send job script/env  		*/
+	DBD_GOT_JOB_ENV,	/* Loading env hash table*/
+	DBD_GOT_JOB_SCRIPT,	/* #1450, Loading bash script hash table */
+	DBD_ADD_ACCOUNTS_COND,  /* Add new account to the mix with acct_rec and
+				 * add_assoc_cond */
+	DBD_ADD_USERS_COND,     /* Add new user to the mix with user_rec and
+				 * add_assoc_cond */
+	DBD_GET_INSTANCES,	/* Get instance information */
+	DBD_GOT_INSTANCES,	/* Response to DBD_GET_INSTANCES */
+	SLURM_DBD_MESSAGES_END = 2000, /* So that we don't overlap with any
+					* slurm_msg_type_t numbers. */
 	SLURM_PERSIST_INIT = 6500, /* So we don't use the
 				    * REQUEST_PERSIST_INIT also used here.
 				    */
@@ -229,7 +240,11 @@ typedef struct dbd_job_comp_msg {
 	uint64_t db_index;	/* index into the db for this job */
 	uint32_t derived_ec;	/* derived job exit code or signal */
 	time_t   end_time;	/* job termintation time */
+	char *failed_node;	/* Name of node that failed which caused
+				 * this job to be killed.
+				 * NULL in all other situations */
 	uint32_t exit_code;	/* job exit code or signal */
+	char *extra;		/* job extra field */
 	uint32_t job_id;	/* job ID */
 	uint32_t job_state;	/* job state */
 	char *   nodes;		/* hosts allocated to the job */
@@ -257,12 +272,13 @@ typedef struct dbd_job_start_msg {
 	uint32_t db_flags;      /* flags about job */
 	uint64_t db_index;	/* index into the db for this job */
 	time_t   eligible_time;	/* time job becomes eligible to run */
-	char *env;              /* job environment in text form */
+	char *env_hash;         /* hash value of env */
 	uint32_t gid;	        /* group ID */
 	uint32_t het_job_id;	/* ID of hetjob leader or 0 */
 	uint32_t het_job_offset; /* Hetjob component ID, zero-origin */
 	uint32_t job_id;	/* job ID */
 	uint32_t job_state;	/* job state */
+	char *licenses; 	/* job licenses */
 	char *   mcs_label;	/* job mcs_label */
 	char *   name;		/* job name */
 	char *   nodes;		/* hosts allocated to the job */
@@ -274,8 +290,7 @@ typedef struct dbd_job_start_msg {
 	uint32_t req_cpus;	/* count of req processors */
 	uint64_t req_mem;       /* requested minimum memory */
 	uint32_t resv_id;	/* reservation id */
-	char *script;           /* job_script in text form */
-	buf_t *script_buf;	/* job script as mmap buf */
+	char *script_hash;      /* hash value of script */
 	time_t   start_time;	/* job start time */
 	uint32_t state_reason_prev; /* Last reason of blocking before job
 				     * started */
@@ -296,9 +311,18 @@ typedef struct dbd_job_start_msg {
 typedef struct dbd_id_rc_msg {
 	uint32_t job_id;
 	uint64_t db_index;
+	uint64_t flags;
 	uint32_t return_code;
 } dbd_id_rc_msg_t;
 
+typedef struct {
+	char *env;              /* job environment in text form */
+	char *env_hash;         /* hash value of env */
+	char *script;           /* job_script in text form */
+	char *script_hash;      /* hash value of script */
+	buf_t *script_buf;	/* job script as mmap buf */
+} dbd_job_heavy_msg_t;
+
 typedef struct dbd_job_suspend_msg {
 	uint32_t assoc_id;	/* accounting association id needed
 				 * to find job record in db */
@@ -326,9 +350,14 @@ typedef struct {
 
 #define DBD_NODE_STATE_DOWN  1
 #define DBD_NODE_STATE_UP    2
+#define DBD_NODE_STATE_UPDATE 3
+
 typedef struct dbd_node_state_msg {
 	time_t event_time;	/* time of transition */
+	char *extra;		/* arbitrary sting */
 	char *hostlist;		/* name of hosts */
+	char *instance_id;	/* cloud instance id */
+	char *instance_type;	/* cloud instance type */
 	uint16_t new_state;	/* new state of host, see DBD_NODE_STATE_* */
 	char *reason;		/* explanation for the node's state */
 	uint32_t reason_uid;   	/* User that set the reason, ignore if
@@ -410,6 +439,7 @@ extern void slurmdbd_free_cond_msg(dbd_cond_msg_t *msg,
 extern void slurmdbd_free_fini_msg(dbd_fini_msg_t *msg);
 extern void slurmdbd_free_job_complete_msg(dbd_job_comp_msg_t *msg);
 extern void slurmdbd_free_job_start_msg(void *in);
+extern void slurmdbd_free_job_heavy_msg(void *in);
 extern void slurmdbd_free_id_rc_msg(void *in);
 extern void slurmdbd_free_job_suspend_msg(dbd_job_suspend_msg_t *msg);
 extern void slurmdbd_free_list_msg(dbd_list_msg_t *msg);
