@@ -43,11 +43,11 @@
 #include <dirent.h>
 #include <grp.h>
 #include <inttypes.h>
+#include <netdb.h>
 #include <regex.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/param.h>	/* MAXPATHLEN */
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -60,13 +60,14 @@
 #include "src/common/macros.h"
 #include "src/common/pack.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_auth.h"
-#include "src/common/slurm_cred.h"
-#include "src/common/slurm_jobacct_gather.h"
+#include "src/interfaces/auth.h"
+#include "src/interfaces/cred.h"
+#include "src/interfaces/jobacct_gather.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/stepd_api.h"
 #include "src/common/strlcpy.h"
 #include "src/common/xmalloc.h"
+#include "src/common/xregex.h"
 #include "src/common/xstring.h"
 
 strong_alias(stepd_available, slurm_stepd_available);
@@ -78,6 +79,8 @@ strong_alias(stepd_getpw, slurm_stepd_getpw);
 strong_alias(xfree_struct_passwd, slurm_xfree_struct_passwd);
 strong_alias(stepd_getgr, slurm_stepd_getgr);
 strong_alias(xfree_struct_group_array, slurm_xfree_struct_group_array);
+strong_alias(stepd_gethostbyname, slurm_stepd_gethostbyname);
+strong_alias(xfree_struct_hostent, slurm_xfree_struct_hostent);
 strong_alias(stepd_get_namespace_fd, slurm_stepd_get_namespace_fd);
 
 /*
@@ -105,8 +108,8 @@ _handle_stray_socket(const char *socket_name)
 	}
 
 	if ((uid = getuid()) != buf.st_uid) {
-		debug3("_handle_stray_socket: socket %s is not owned by uid %d",
-		       socket_name, (int)uid);
+		debug3("_handle_stray_socket: socket %s is not owned by uid %u",
+		       socket_name, uid);
 		return;
 	}
 
@@ -147,9 +150,7 @@ _step_connect(const char *directory, const char *nodename,
 	struct sockaddr_un addr;
 	char *name = NULL, *pos = NULL;
 	uint32_t stepid = step_id->step_id;
-	bool old_id_tied = false;
 
-try_old_id:
 	xstrfmtcatat(name, &pos, "%s/%s_%u.%u",
 		     directory, nodename, step_id->job_id, stepid);
 	if (step_id->step_het_comp != NO_VAL)
@@ -184,30 +185,12 @@ try_old_id:
 		      __func__, name);
 		if (errno == ECONNREFUSED && running_in_slurmd()) {
 			_handle_stray_socket(name);
-			/*
-			 * NOTE: Checking against NO_VAL can be removed after 21.08
-			 */
-			if ((step_id->step_id == SLURM_BATCH_SCRIPT) ||
-			    (step_id->step_id == NO_VAL))
+
+			if (step_id->step_id == SLURM_BATCH_SCRIPT)
 				_handle_stray_script(directory,
 						     step_id->job_id);
 		}
 
-		/* NOTE: This code can be removed after 21.08 */
-		if (errno == ENOENT && !old_id_tied &&
-		    ((step_id->step_id == SLURM_BATCH_SCRIPT) ||
-		     (step_id->step_id == SLURM_EXTERN_CONT))) {
-			debug("%s: Try to use old step_id", __func__);
-			close(fd);
-			if (stepid == SLURM_BATCH_SCRIPT)
-				stepid = NO_VAL;
-			else
-				stepid = INFINITE;
-			pos = name;
-			old_id_tied = true;
-			goto try_old_id;
-		}
-
 		xfree(name);
 		close(fd);
 		return -1;
@@ -221,17 +204,26 @@ try_old_id:
 static char *
 _guess_nodename(void)
 {
-	char host[256];
+	char host[HOST_NAME_MAX];
 	char *nodename = NULL;
 
-	if (gethostname_short(host, 256) != 0)
-		return NULL;
+	/* If we are in a step just grab it from the ENV */
+	if ((nodename = getenv("SLURMD_NODENAME")))
+		return xstrdup(nodename);
 
+	if (gethostname_short(host, sizeof(host)) != 0)
+		return NULL;
 	nodename = slurm_conf_get_nodename(host);
 	if (nodename == NULL)
 		nodename = slurm_conf_get_aliased_nodename();
 	if (nodename == NULL) /* if no match, try localhost */
 		nodename = slurm_conf_get_nodename("localhost");
+	/*
+	 * If nothing above has given us a name, just return what
+	 * gethostname_short. This is helpful for dynamic nodes.
+	 */
+	if (!nodename)
+		nodename = xstrdup(host);
 
 	return nodename;
 }
@@ -266,7 +258,7 @@ extern int stepd_connect(const char *directory, const char *nodename,
 	if (directory == NULL) {
 		slurm_conf_t *cf = slurm_conf_lock();
 		directory = slurm_conf_expand_slurmd_path(cf->slurmd_spooldir,
-							  nodename);
+							  nodename, NULL);
 		slurm_conf_unlock();
 	}
 
@@ -337,11 +329,10 @@ stepd_notify_job(int fd, uint16_t protocol_version, char *message)
 /*
  * Send a signal to the proctrack container of a job step.
  */
-int
-stepd_signal_container(int fd, uint16_t protocol_version, int signal, int flags,
-		       uid_t req_uid)
+int stepd_signal_container(int fd, uint16_t protocol_version, int signal,
+			   int flags, char *details, uid_t req_uid)
 {
-	int req = REQUEST_SIGNAL_CONTAINER;
+	int req = REQUEST_SIGNAL_CONTAINER, details_len = 0;
 	int rc;
 	int errnum = 0;
 
@@ -349,6 +340,10 @@ stepd_signal_container(int fd, uint16_t protocol_version, int signal, int flags,
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_write(fd, &signal, sizeof(int));
 		safe_write(fd, &flags, sizeof(int));
+		if (details)
+			details_len = strlen(details);
+		safe_write(fd, &details_len, sizeof(int));
+		safe_write(fd, details, details_len);
 		safe_write(fd, &req_uid, sizeof(uid_t));
 	} else {
 		error("%s: invalid protocol_version %u",
@@ -402,10 +397,9 @@ rwfail:
  * On success returns SLURM_SUCCESS and fills in resp->local_pids,
  * resp->gtids, resp->ntasks, and resp->executable.
  */
-int
-stepd_attach(int fd, uint16_t protocol_version,
-	     slurm_addr_t *ioaddr, slurm_addr_t *respaddr,
-	     void *job_cred_sig, reattach_tasks_response_msg_t *resp)
+int stepd_attach(int fd, uint16_t protocol_version, slurm_addr_t *ioaddr,
+		 slurm_addr_t *respaddr, void *job_cred_sig, uint32_t sig_len,
+		 uid_t uid, reattach_tasks_response_msg_t *resp)
 {
 	int req = REQUEST_ATTACH;
 	int rc = SLURM_SUCCESS;
@@ -414,7 +408,9 @@ stepd_attach(int fd, uint16_t protocol_version,
 		safe_write(fd, &req, sizeof(int));
 		safe_write(fd, ioaddr, sizeof(slurm_addr_t));
 		safe_write(fd, respaddr, sizeof(slurm_addr_t));
-		safe_write(fd, job_cred_sig, SLURM_IO_KEY_SIZE);
+		safe_write(fd, &sig_len, sizeof(uint32_t));
+		safe_write(fd, job_cred_sig, sig_len);
+		safe_write(fd, &uid, sizeof(uid_t));
 		safe_write(fd, &protocol_version, sizeof(uint16_t));
 	} else
 		goto rwfail;
@@ -464,14 +460,17 @@ static int
 _sockname_regex_init(regex_t *re, const char *nodename)
 {
 	char *pattern = NULL;
+	int rc;
 
 	xstrcat(pattern, "^");
 	xstrcat(pattern, nodename);
 	xstrcat(pattern,
 		"_([[:digit:]]*)\\.([[:digit:]]*)\\.{0,1}([[:digit:]]*)$");
 
-	if (regcomp(re, pattern, REG_EXTENDED) != 0) {
-		error("sockname regex compilation failed");
+	if ((rc = regcomp(re, pattern, REG_EXTENDED))) {
+		dump_regex_error(rc, re,
+				 "sockname regex \"%s\" compilation failed",
+				 pattern);
 		return -1;
 	}
 
@@ -487,11 +486,14 @@ _sockname_regex(regex_t *re, const char *filename, slurm_step_id_t *step_id)
 	regmatch_t pmatch[5];
 	char *match;
 	size_t my_size;
+	int rc;
 
 	xassert(step_id);
 
 	memset(pmatch, 0, sizeof(regmatch_t)*nmatch);
-	if (regexec(re, filename, nmatch, pmatch, 0) == REG_NOMATCH) {
+	if ((rc = regexec(re, filename, nmatch, pmatch, 0))) {
+		if (rc != REG_NOMATCH)
+			dump_regex_error(rc, re, "regexc(%s)", filename);
 		return -1;
 	}
 
@@ -545,7 +547,7 @@ stepd_available(const char *directory, const char *nodename)
 	if (directory == NULL) {
 		slurm_conf_t *cf = slurm_conf_lock();
 		directory = slurm_conf_expand_slurmd_path(
-			cf->slurmd_spooldir, nodename);
+			cf->slurmd_spooldir, nodename, NULL);
 		slurm_conf_unlock();
 	}
 
@@ -643,7 +645,7 @@ stepd_cleanup_sockets(const char *directory, const char *nodename)
 			} else {
 				if (stepd_signal_container(
 					    fd, protocol_version, SIGKILL, 0,
-					    getuid())
+					    NULL, getuid())
 				    == -1) {
 					debug("Error sending SIGKILL to %ps",
 					      &step_id);
@@ -889,6 +891,85 @@ extern void xfree_struct_group_array(struct group **grps)
 	xfree(grps);
 }
 
+extern struct hostent *stepd_gethostbyname(int fd, uint16_t protocol_version,
+					   int mode, const char *nodename)
+{
+	int req = REQUEST_GETHOST;
+	int found = 0;
+	int len = 0;
+	int cnt = 0;
+	struct hostent *host = NULL;
+
+	safe_write(fd, &req, sizeof(int));
+
+	safe_write(fd, &mode, sizeof(int));
+
+	if (nodename) {
+		len = strlen(nodename);
+		safe_write(fd, &len, sizeof(int));
+		safe_write(fd, nodename, len);
+	} else {
+		safe_write(fd, &len, sizeof(int));
+	}
+
+	safe_read(fd, &found, sizeof(int));
+
+	if (!found)
+		return NULL;
+
+	host = xmalloc(sizeof(struct hostent));
+
+	safe_read(fd, &len, sizeof(int));
+	host->h_name = xmalloc(len + 1);
+	safe_read(fd, host->h_name, len);
+
+	safe_read(fd, &cnt, sizeof(int));
+	host->h_aliases = xcalloc(cnt + 1, sizeof(char *));
+	for (int i = 0; i < cnt; i++) {
+		safe_read(fd, &len, sizeof(int));
+		host->h_aliases[i] = xmalloc(len + 1);
+		safe_read(fd, host->h_aliases[i], len);
+	}
+	safe_read(fd, &host->h_addrtype, sizeof(int));
+	safe_read(fd, &len, sizeof(int));
+	host->h_length = len;
+
+	/*
+	 * In the current implementation, we define each host to
+	 * only have a single address.
+	 * (Since h_addr_list is a NULL terminated array, allocate
+	 * space for two elements.)
+	 */
+	host->h_addr_list = xcalloc(2, sizeof(char *));
+	host->h_addr_list[0] = xmalloc(len);
+	safe_read(fd, host->h_addr_list[0], len);
+
+	debug("Leaving %s", __func__);
+	return host;
+
+rwfail:
+	xfree_struct_hostent(host);
+	return NULL;
+
+}
+
+extern void xfree_struct_hostent(struct hostent *host)
+{
+	if (!host)
+		return;
+	xfree(host->h_name);
+	for (int i = 0; host->h_aliases && host->h_aliases[i];
+	     i++) {
+		xfree(host->h_aliases[i]);
+	}
+	xfree(host->h_aliases);
+	if (host->h_addr_list) {
+		xfree(host->h_addr_list[0]);
+		xfree(host->h_addr_list);
+	}
+	xfree(host);
+}
+
 /*
  * Return the process ID of the slurmstepd.
  */
@@ -922,14 +1003,25 @@ stepd_suspend(int fd, uint16_t protocol_version,
 	int rc = 0;
 	int errnum = 0;
 
-	if (phase == 0) {
-		safe_write(fd, &req, sizeof(int));
-		safe_write(fd, &susp_req->job_core_spec, sizeof(uint16_t));
-	} else {
-		/* Receive the return code and errno */
-		safe_read(fd, &rc, sizeof(int));
-		safe_read(fd, &errnum, sizeof(int));
-		errno = errnum;
+	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+		if (phase == 0) {
+			safe_write(fd, &req, sizeof(int));
+		} else {
+			/* Receive the return code and errno */
+			safe_read(fd, &rc, sizeof(int));
+			safe_read(fd, &errnum, sizeof(int));
+			errno = errnum;
+		}
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		if (phase == 0) {
+			safe_write(fd, &req, sizeof(int));
+			safe_write(fd, NO_VAL16, sizeof(uint16_t));
+		} else {
+			/* Receive the return code and errno */
+			safe_read(fd, &rc, sizeof(int));
+			safe_read(fd, &errnum, sizeof(int));
+			errno = errnum;
+		}
 	}
 
 	return rc;
@@ -953,14 +1045,25 @@ stepd_resume(int fd, uint16_t protocol_version,
 	int rc = 0;
 	int errnum = 0;
 
-	if (phase == 0) {
-		safe_write(fd, &req, sizeof(int));
-		safe_write(fd, &susp_req->job_core_spec, sizeof(uint16_t));
-	} else {
-		/* Receive the return code and errno */
-		safe_read(fd, &rc, sizeof(int));
-		safe_read(fd, &errnum, sizeof(int));
-		errno = errnum;
+	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+		if (phase == 0) {
+			safe_write(fd, &req, sizeof(int));
+		} else {
+			/* Receive the return code and errno */
+			safe_read(fd, &rc, sizeof(int));
+			safe_read(fd, &errnum, sizeof(int));
+			errno = errnum;
+		}
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		if (phase == 0) {
+			safe_write(fd, &req, sizeof(int));
+			safe_write(fd, NO_VAL16, sizeof(uint16_t));
+		} else {
+			/* Receive the return code and errno */
+			safe_read(fd, &rc, sizeof(int));
+			safe_read(fd, &errnum, sizeof(int));
+			errno = errnum;
+		}
 	}
 
 	return rc;
@@ -968,15 +1071,7 @@ rwfail:
 	return -1;
 }
 
-/*
- * Reconfigure the job step (Primarily to allow the stepd to refresh
- * it's log file pointer.
- *
- * Returns SLURM_SUCCESS is successful.  On error returns SLURM_ERROR
- * and sets errno.
- */
-int
-stepd_reconfig(int fd, uint16_t protocol_version)
+extern int stepd_reconfig(int fd, uint16_t protocol_version, buf_t *reconf)
 {
 	int req = REQUEST_STEP_RECONFIGURE;
 	int rc;
@@ -984,6 +1079,17 @@ stepd_reconfig(int fd, uint16_t protocol_version)
 
 	safe_write(fd, &req, sizeof(int));
 
+	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+		int len = 0;
+		if (reconf) {
+			len = get_buf_offset(reconf);
+			safe_write(fd, &len, sizeof(int));
+			safe_write(fd, get_buf_data(reconf), len);
+		} else {
+			safe_write(fd, &len, sizeof(int));
+		}
+	}
+
 	/* Receive the return code and errno */
 	safe_read(fd, &rc, sizeof(int));
 	safe_read(fd, &errnum, sizeof(int));
