@@ -47,6 +47,10 @@
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/io_hdr.h"
 
+#define GETHOST_NOT_MATCH_PID SLURM_BIT(0)
+#define GETHOST_IPV4 SLURM_BIT(1)
+#define GETHOST_IPV6 SLURM_BIT(2)
+
 typedef enum {
 	REQUEST_CONNECT = 0,
 	REQUEST_STEP_DEFUNCT_1,
@@ -75,12 +79,14 @@ typedef enum {
 	REQUEST_GETPW,
 	REQUEST_GETGR,
 	REQUEST_GET_NS_FD,
+	REQUEST_GETHOST,
 } step_msg_t;
 
 typedef enum {
 	SLURMSTEPD_NOT_RUNNING = 0,
 	SLURMSTEPD_STEP_STARTING,
 	SLURMSTEPD_STEP_RUNNING,
+	SLURMSTEPD_STEP_CANCELLED,
 	SLURMSTEPD_STEP_ENDING
 } slurmstepd_state_t;
 
@@ -164,7 +170,7 @@ int stepd_notify_job(int fd, uint16_t protocol_version, char *message);
  * Send a signal to the proctrack container of a job step.
  */
 int stepd_signal_container(int fd, uint16_t protocol_version, int signal,
-			   int flags, uid_t uid);
+			   int flags, char *details, uid_t uid);
 
 /*
  * Attach a client to a running job step.
@@ -176,10 +182,9 @@ int stepd_signal_container(int fd, uint16_t protocol_version, int signal,
  *         probably be moved into a more generic stepd_api call so that
  *         this header does not need to include slurm_protocol_defs.h.
  */
-int stepd_attach(int fd, uint16_t protocol_version,
-		 slurm_addr_t *ioaddr, slurm_addr_t *respaddr,
-		 void *job_cred_sig, uid_t uid,
-		 reattach_tasks_response_msg_t *resp);
+int stepd_attach(int fd, uint16_t protocol_version, slurm_addr_t *ioaddr,
+		 slurm_addr_t *respaddr, void *job_cred_sig, uint32_t sig_len,
+		 uid_t uid, reattach_tasks_response_msg_t *resp);
 
 /*
  * Scan for available running slurm step daemons by checking
@@ -236,6 +241,13 @@ extern struct group **stepd_getgr(int fd, uint16_t protocol_version,
 
 extern void xfree_struct_group_array(struct group **grp);
 
+/*
+ * Rerturn hostent based based off node_to_host_hashtbl for nodename.
+ */
+extern struct hostent *stepd_gethostbyname(int fd, uint16_t protocol_version,
+					   int mode, const char *nodename);
+
+extern void xfree_struct_hostent(struct hostent *host);
 /*
  * Return the process ID of the slurmstepd.
  */
@@ -263,13 +275,14 @@ extern int stepd_resume(int fd, uint16_t protocol_version,
 			suspend_int_msg_t *susp_req, int phase);
 
 /*
- * Reconfigure the job step (Primarily to allow the stepd to refresh
- * it's log file pointer.
+ * Reconfigure the job step.
+ * Allows the stepd to refresh it's log file pointer.
+ * If reconf is not NULL, update the controller addresses.
  *
  * Returns SLURM_SUCCESS if successful.  On error returns SLURM_ERROR
  * and sets errno.
  */
-int stepd_reconfig(int fd, uint16_t protocol_version);
+extern int stepd_reconfig(int fd, uint16_t protocol_version, buf_t *reconf);
 
 /*
  *
