@@ -70,6 +70,7 @@ static size_t _xstrdup_vprintf(char **str, const char *_fmt, va_list _ap);
  * for details.
  */
 strong_alias(_xstrcat,		slurm_xstrcat);
+strong_alias(_xstrcatat,	slurm_xstrcatat);
 strong_alias(_xstrncat,		slurm_xstrncat);
 strong_alias(_xstrcatchar,	slurm_xstrcatchar);
 strong_alias(_xstrftimecat,	slurm_xstrftimecat);
@@ -141,6 +142,49 @@ void _xstrcat(char **str1, const char *str2)
 	strcat(*str1, str2);
 }
 
+/*
+ * Append str2 onto str at pos, * expanding buf as needed. pos is updated to the
+ * end of the appended string.
+ *
+ * Meant to be used in loops contructing longer strings that are performance
+ * sensitive, as xstrcat() needs to re-seek to the end of str making the string
+ * construction worse by another O(log(strlen)) factor.
+ */
+void _xstrcatat(char **str, char **pos, const char *str2)
+{
+	size_t orig_len, append_len;
+
+	if (!str2)
+		return;
+
+	append_len = strlen(str2);
+
+	/* No string yet to append to, so return a copy of str2. */
+	if (!*str) {
+		*str = xstrdup(str2);
+		*pos = *str + append_len;
+		return;
+	}
+
+	if (!*pos) {
+		orig_len = strlen(*str);
+		*pos = *str + orig_len;
+	} else {
+		xassert(*pos >= *str);
+		orig_len = *pos - *str;
+	}
+
+	_makespace(str, orig_len, append_len);
+
+	memcpy(*str + orig_len, str2, append_len);
+
+	/*
+	 * Update *pos. Cannot happen earlier as _makespace() may have
+	 * changed *str to a different address.
+	 */
+	*pos = *str + orig_len + append_len;
+}
+
 /*
  * Concatenate len of str2 onto str1, expanding str1 as needed.
  *   str1 (IN/OUT)	target string (pointer to in case of expansion)
@@ -257,6 +301,35 @@ void _xrfc5424timecat(char **buf, bool msec)
 		_xstrfmtcat(buf, "%s%s", p, z);
 }
 
+extern void _xrfc3339timecat(char **buf)
+{
+	char p[64] = "";
+	char z[12] = "";
+	struct timeval tv;
+	struct tm tm;
+
+	if (gettimeofday(&tv, NULL) == -1)
+		fprintf(stderr, "gettimeofday() failed\n");
+
+	if (!localtime_r(&tv.tv_sec, &tm))
+		fprintf(stderr, "localtime_r() failed\n");
+
+	if (strftime(p, sizeof(p), "%FT%T", &tm) == 0)
+		fprintf(stderr, "strftime() returned 0\n");
+
+	/* The strftime %z format creates timezone offsets of the form
+	 * (+/-)hhmm, whereas the RFC 3339 format is (+/-)hh:mm. So
+	 * shift the minutes one step back and insert the semicolon.
+	 */
+	if (strftime(z, sizeof(z), "%z", &tm) == 0)
+		fprintf(stderr, "strftime() returned 0\n");
+	z[5] = z[4];
+	z[4] = z[3];
+	z[3] = ':';
+
+	_xstrfmtcat(buf, "%s%s", p, z);
+}
+
 /*
  * append formatted string with printf-style args to buf, expanding
  * buf as needed
@@ -587,19 +660,21 @@ bool xstring_is_whitespace(const char *str)
 	return true;
 }
 
-/*
- * If str make everything lowercase.  Should not be called on static char *'s
- */
-char *xstrtolower(char *str)
+extern bool xstrtolower(char *str)
 {
-	if (str) {
-		int j = 0;
-		while (str[j]) {
-			str[j] = tolower((int)str[j]);
-			j++;
-		}
+	bool modified = false;
+
+	if (!str)
+		return modified;
+
+	for (int j = 0; str[j]; j++) {
+		int c = tolower((int) str[j]);
+
+		if (c != str[j])
+			modified = true;
+		str[j] = c;
 	}
-	return str;
+	return modified;
 }
 
 /* safe strchr */
@@ -788,7 +863,8 @@ extern void xstrtrim(char *string)
 		memmove(string, start, (ptr - start + 1));
 }
 
-extern char *bytes_to_hex(const char *string, int len, const char *delimiter)
+extern char *xstring_bytes2hex(const unsigned char *string, int len,
+			       const char *delimiter)
 {
 	char *hex = NULL, *pos = NULL;
 
@@ -800,13 +876,14 @@ extern char *bytes_to_hex(const char *string, int len, const char *delimiter)
 			xstrfmtcatat(hex, &pos, "%s", delimiter);
 
 		/* convert each char into equiv hex */
-		xstrfmtcatat(hex, &pos, "%02x", (unsigned char) string[i]);
+		xstrfmtcatat(hex, &pos, "%02x", string[i]);
 	}
 
 	return hex;
 }
 
-extern char *bytes_to_printable(const char *string, int len, const char replace)
+extern char *xstring_bytes2printable(const unsigned char *string, int len,
+				     const char replace)
 {
 	char *str = NULL, *pos = NULL;
 
