@@ -64,11 +64,17 @@ typedef struct slurm_acct_storage_ops {
 	int  (*commit)             (void *db_conn, bool commit);
 	int  (*add_users)          (void *db_conn, uint32_t uid,
 				    List user_list);
+	char *(*add_users_cond)    (void *db_conn, uint32_t uid,
+				    slurmdb_add_assoc_cond_t *add_assoc,
+				    slurmdb_user_rec_t *user);
 	int  (*add_coord)          (void *db_conn, uint32_t uid,
 				    List acct_list,
 				    slurmdb_user_cond_t *user_cond);
 	int  (*add_accts)          (void *db_conn, uint32_t uid,
 				    List acct_list);
+	char *(*add_accts_cond)    (void *db_conn, uint32_t uid,
+				    slurmdb_add_assoc_cond_t *add_assoc,
+				    slurmdb_account_rec_t *acct);
 	int  (*add_clusters)       (void *db_conn, uint32_t uid,
 				    List cluster_list);
 	int  (*add_federations)    (void *db_conn, uint32_t uid,
@@ -150,6 +156,8 @@ typedef struct slurm_acct_storage_ops {
 				    slurmdb_assoc_cond_t *assoc_cond);
 	List (*get_events)         (void *db_conn, uint32_t uid,
 				    slurmdb_event_cond_t *event_cond);
+	List (*get_instances)	   (void *db_conn, uint32_t uid,
+				    slurmdb_instance_cond_t *instance_cond);
 	List (*get_problems)       (void *db_conn, uint32_t uid,
 				    slurmdb_assoc_cond_t *assoc_cond);
 	List (*get_qos)            (void *db_conn, uint32_t uid,
@@ -177,6 +185,7 @@ typedef struct slurm_acct_storage_ops {
 	char *(*node_inx)          (void *db_conn, char *nodes);
 	int  (*node_up)            (void *db_conn, node_record_t *node_ptr,
 				    time_t event_time);
+	int  (*node_update)	   (void *db_conn, node_record_t *node_ptr);
 	int  (*cluster_tres)       (void *db_conn, char *cluster_nodes,
 				    char *tres_str_in, time_t event_time,
 				    uint16_t rpc_version);
@@ -219,8 +228,10 @@ static const char *syms[] = {
 	"acct_storage_p_close_connection",
 	"acct_storage_p_commit",
 	"acct_storage_p_add_users",
+	"acct_storage_p_add_users_cond",
 	"acct_storage_p_add_coord",
 	"acct_storage_p_add_accts",
+	"acct_storage_p_add_accts_cond",
 	"acct_storage_p_add_clusters",
 	"acct_storage_p_add_federations",
 	"acct_storage_p_add_tres",
@@ -257,6 +268,7 @@ static const char *syms[] = {
 	"acct_storage_p_get_tres",
 	"acct_storage_p_get_assocs",
 	"acct_storage_p_get_events",
+	"acct_storage_p_get_instances",
 	"acct_storage_p_get_problems",
 	"acct_storage_p_get_qos",
 	"acct_storage_p_get_res",
@@ -269,6 +281,7 @@ static const char *syms[] = {
 	"clusteracct_storage_p_node_down",
 	"acct_storage_p_node_inx",
 	"clusteracct_storage_p_node_up",
+	"clusteracct_storage_p_node_update",
 	"clusteracct_storage_p_cluster_tres",
 	"clusteracct_storage_p_register_ctld",
 	"clusteracct_storage_p_register_disconn_ctld",
@@ -296,6 +309,7 @@ static const char *syms[] = {
 static slurm_acct_storage_ops_t ops;
 static plugin_context_t *plugin_context = NULL;
 static pthread_rwlock_t plugin_context_lock = PTHREAD_RWLOCK_INITIALIZER;
+static plugin_init_t plugin_inited = PLUGIN_NOT_INITED;
 
 /*
  * If running with slurmdbd don't run if we don't have an index, else
@@ -313,18 +327,20 @@ extern int jobacct_storage_job_start_direct(void *db_conn,
 /*
  * Initialize context for acct_storage plugin
  */
-extern int slurm_acct_storage_init(void)
+extern int acct_storage_g_init(void)
 {
 	int retval = SLURM_SUCCESS;
 	char *plugin_type = "accounting_storage";
 
 	slurm_rwlock_wrlock(&plugin_context_lock);
 
-	if (plugin_context)
+	if (plugin_inited)
 		goto done;
 
-	if (!slurm_conf.accounting_storage_type)
+	if (!slurm_conf.accounting_storage_type) {
+		plugin_inited = PLUGIN_NOOP;
 		goto done;
+	}
 
 	plugin_context = plugin_context_create(
 		plugin_type, slurm_conf.accounting_storage_type, (void **)&ops,
@@ -334,24 +350,25 @@ extern int slurm_acct_storage_init(void)
 		error("cannot create %s context for %s",
 		      plugin_type, slurm_conf.accounting_storage_type);
 		retval = SLURM_ERROR;
+		plugin_inited = PLUGIN_NOT_INITED;
 		goto done;
 	}
-
+	plugin_inited = PLUGIN_INITED;
 done:
 	slurm_rwlock_unlock(&plugin_context_lock);
 	return retval;
 }
 
-extern int slurm_acct_storage_fini(void)
+extern int acct_storage_g_fini(void)
 {
 	int rc = SLURM_SUCCESS;
 
 	slurm_rwlock_wrlock(&plugin_context_lock);
 	if (plugin_context) {
-		//(*(ops.acct_storage_fini))();
 		rc = plugin_context_destroy(plugin_context);
 		plugin_context = NULL;
 	}
+	plugin_inited = PLUGIN_NOT_INITED;
 	slurm_rwlock_unlock(&plugin_context_lock);
 	return rc;
 }
@@ -360,100 +377,179 @@ extern void *acct_storage_g_get_connection(
 	int conn_num, uint16_t *persist_conn_flags,
 	bool rollback,char *cluster_name)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_conn))(conn_num, persist_conn_flags,
 				 rollback, cluster_name);
 }
 
 extern int acct_storage_g_close_connection(void **db_conn)
 {
-	xassert(plugin_context);
-	return (*(ops.close_conn))(db_conn);
+	xassert(plugin_inited);
 
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.close_conn))(db_conn);
 }
 
 extern int acct_storage_g_commit(void *db_conn, bool commit)
 {
-	xassert(plugin_context);
-	return (*(ops.commit))(db_conn, commit);
+	xassert(plugin_inited);
 
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.commit))(db_conn, commit);
 }
 
 extern int acct_storage_g_add_users(void *db_conn, uint32_t uid,
 				    List user_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_users))(db_conn, uid, user_list);
 }
 
+extern char *acct_storage_g_add_users_cond(
+	void *db_conn, uint32_t uid,
+	slurmdb_add_assoc_cond_t *add_assoc,
+	slurmdb_user_rec_t *user)
+{
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
+	return (*(ops.add_users_cond))(db_conn, uid, add_assoc, user);
+}
+
 extern int acct_storage_g_add_coord(void *db_conn, uint32_t uid,
 				    List acct_list,
 				    slurmdb_user_cond_t *user_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_coord))(db_conn, uid, acct_list, user_cond);
 }
 
 extern int acct_storage_g_add_accounts(void *db_conn, uint32_t uid,
 				       List acct_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_accts))(db_conn, uid, acct_list);
 }
 
+extern char *acct_storage_g_add_accounts_cond(
+	void *db_conn, uint32_t uid,
+	slurmdb_add_assoc_cond_t *add_assoc,
+	slurmdb_account_rec_t *acct)
+{
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
+	return (*(ops.add_accts_cond))(db_conn, uid, add_assoc, acct);
+}
+
 extern int acct_storage_g_add_clusters(void *db_conn, uint32_t uid,
 				       List cluster_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_clusters))(db_conn, uid, cluster_list);
 }
 
 extern int acct_storage_g_add_federations(void *db_conn, uint32_t uid,
 					  List federation_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_federations))(db_conn, uid, federation_list);
 }
 
 extern int acct_storage_g_add_tres(void *db_conn, uint32_t uid,
 				   List tres_list_in)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_tres))(db_conn, uid, tres_list_in);
 }
 
 extern int acct_storage_g_add_assocs(void *db_conn, uint32_t uid,
 				     List assoc_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_assocs))(db_conn, uid, assoc_list);
 }
 
 extern int acct_storage_g_add_qos(void *db_conn, uint32_t uid,
 				  List qos_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_qos))(db_conn, uid, qos_list);
 }
 
 extern int acct_storage_g_add_res(void *db_conn, uint32_t uid,
 				  List res_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_res))(db_conn, uid, res_list);
 }
 extern int acct_storage_g_add_wckeys(void *db_conn, uint32_t uid,
 				     List wckey_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_wckeys))(db_conn, uid, wckey_list);
 }
 
 extern int acct_storage_g_add_reservation(void *db_conn,
 					  slurmdb_reservation_rec_t *resv)
 {
-	if (slurm_acct_storage_init() < 0)
-		return NO_VAL;
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.add_reservation))(db_conn, resv);
 }
 
@@ -461,7 +557,11 @@ extern List acct_storage_g_modify_users(void *db_conn, uint32_t uid,
 					slurmdb_user_cond_t *user_cond,
 					slurmdb_user_rec_t *user)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_users))(db_conn, uid, user_cond, user);
 }
 
@@ -469,7 +569,11 @@ extern List acct_storage_g_modify_accounts(void *db_conn, uint32_t uid,
 					   slurmdb_account_cond_t *acct_cond,
 					   slurmdb_account_rec_t *acct)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_accts))(db_conn, uid, acct_cond, acct);
 }
 
@@ -477,7 +581,11 @@ extern List acct_storage_g_modify_clusters(void *db_conn, uint32_t uid,
 					   slurmdb_cluster_cond_t *cluster_cond,
 					   slurmdb_cluster_rec_t *cluster)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_clusters))(db_conn, uid, cluster_cond, cluster);
 }
 
@@ -486,7 +594,11 @@ extern List acct_storage_g_modify_assocs(
 	slurmdb_assoc_cond_t *assoc_cond,
 	slurmdb_assoc_rec_t *assoc)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_assocs))(db_conn, uid, assoc_cond, assoc);
 }
 
@@ -495,7 +607,11 @@ extern List acct_storage_g_modify_federations(
 				slurmdb_federation_cond_t *fed_cond,
 				slurmdb_federation_rec_t *fed)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_federations))(db_conn, uid, fed_cond, fed);
 }
 
@@ -503,7 +619,11 @@ extern List acct_storage_g_modify_job(void *db_conn, uint32_t uid,
 				      slurmdb_job_cond_t *job_cond,
 				      slurmdb_job_rec_t *job)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 
 	return (*(ops.modify_job))(db_conn, uid, job_cond, job);
 }
@@ -512,7 +632,11 @@ extern List acct_storage_g_modify_qos(void *db_conn, uint32_t uid,
 				      slurmdb_qos_cond_t *qos_cond,
 				      slurmdb_qos_rec_t *qos)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_qos))(db_conn, uid, qos_cond, qos);
 }
 
@@ -520,7 +644,11 @@ extern List acct_storage_g_modify_res(void *db_conn, uint32_t uid,
 				      slurmdb_res_cond_t *res_cond,
 				      slurmdb_res_rec_t *res)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_res))(db_conn, uid, res_cond, res);
 }
 
@@ -528,22 +656,33 @@ extern List acct_storage_g_modify_wckeys(void *db_conn, uint32_t uid,
 					 slurmdb_wckey_cond_t *wckey_cond,
 					 slurmdb_wckey_rec_t *wckey)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.modify_wckeys))(db_conn, uid, wckey_cond, wckey);
 }
 
 extern int acct_storage_g_modify_reservation(void *db_conn,
 					     slurmdb_reservation_rec_t *resv)
 {
-	if (slurm_acct_storage_init() < 0)
-		return NO_VAL;
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.modify_reservation))(db_conn, resv);
 }
 
 extern List acct_storage_g_remove_users(void *db_conn, uint32_t uid,
 					slurmdb_user_cond_t *user_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_users))(db_conn, uid, user_cond);
 }
 
@@ -551,21 +690,33 @@ extern List acct_storage_g_remove_coord(void *db_conn, uint32_t uid,
 					List acct_list,
 					slurmdb_user_cond_t *user_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_coord))(db_conn, uid, acct_list, user_cond);
 }
 
 extern List acct_storage_g_remove_accounts(void *db_conn, uint32_t uid,
 					   slurmdb_account_cond_t *acct_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_accts))(db_conn, uid, acct_cond);
 }
 
 extern List acct_storage_g_remove_clusters(void *db_conn, uint32_t uid,
 					   slurmdb_cluster_cond_t *cluster_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_clusters))(db_conn, uid, cluster_cond);
 }
 
@@ -573,7 +724,11 @@ extern List acct_storage_g_remove_assocs(
 	void *db_conn, uint32_t uid,
 	slurmdb_assoc_cond_t *assoc_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_assocs))(db_conn, uid, assoc_cond);
 }
 
@@ -581,70 +736,109 @@ extern List acct_storage_g_remove_federations(
 					void *db_conn, uint32_t uid,
 					slurmdb_federation_cond_t *fed_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_federations))(db_conn, uid, fed_cond);
 }
 
 extern List acct_storage_g_remove_qos(void *db_conn, uint32_t uid,
 				      slurmdb_qos_cond_t *qos_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_qos))(db_conn, uid, qos_cond);
 }
 
 extern List acct_storage_g_remove_res(void *db_conn, uint32_t uid,
 				      slurmdb_res_cond_t *res_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_res))(db_conn, uid, res_cond);
 }
 
 extern List acct_storage_g_remove_wckeys(void *db_conn, uint32_t uid,
 					 slurmdb_wckey_cond_t *wckey_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.remove_wckeys))(db_conn, uid, wckey_cond);
 }
 
 extern int acct_storage_g_remove_reservation(void *db_conn,
 					     slurmdb_reservation_rec_t *resv)
 {
-	if (slurm_acct_storage_init() < 0)
-		return NO_VAL;
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.remove_reservation))(db_conn, resv);
 }
 
 extern List acct_storage_g_get_users(void *db_conn, uint32_t uid,
 				     slurmdb_user_cond_t *user_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_users))(db_conn, uid, user_cond);
 }
 
 extern List acct_storage_g_get_accounts(void *db_conn, uint32_t uid,
 					slurmdb_account_cond_t *acct_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_accts))(db_conn, uid, acct_cond);
 }
 
 extern List acct_storage_g_get_clusters(void *db_conn, uint32_t uid,
 					slurmdb_cluster_cond_t *cluster_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_clusters))(db_conn, uid, cluster_cond);
 }
 
 extern List acct_storage_g_get_federations(void *db_conn, uint32_t uid,
 					   slurmdb_federation_cond_t *fed_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_federations))(db_conn, uid, fed_cond);
 }
 
 extern List acct_storage_g_get_config(void *db_conn, char *config_name)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_config))(db_conn, config_name);
 }
 
@@ -652,7 +846,11 @@ extern List acct_storage_g_get_tres(
 	void *db_conn, uint32_t uid,
 	slurmdb_tres_cond_t *tres_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_tres))(db_conn, uid, tres_cond);
 }
 
@@ -660,56 +858,100 @@ extern List acct_storage_g_get_assocs(
 	void *db_conn, uint32_t uid,
 	slurmdb_assoc_cond_t *assoc_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_assocs))(db_conn, uid, assoc_cond);
 }
 
 extern List acct_storage_g_get_events(void *db_conn, uint32_t uid,
 				      slurmdb_event_cond_t *event_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_events))(db_conn, uid, event_cond);
 }
 
+extern List acct_storage_g_get_instances(void *db_conn,
+					 uint32_t uid,
+					 slurmdb_instance_cond_t *instance_cond)
+{
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
+	return (*(ops.get_instances))(db_conn, uid, instance_cond);
+}
+
 extern List acct_storage_g_get_problems(void *db_conn, uint32_t uid,
 					slurmdb_assoc_cond_t *assoc_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_problems))(db_conn, uid, assoc_cond);
 }
 
 extern List acct_storage_g_get_qos(void *db_conn, uint32_t uid,
 				   slurmdb_qos_cond_t *qos_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_qos))(db_conn, uid, qos_cond);
 }
 
 extern List acct_storage_g_get_res(void *db_conn, uint32_t uid,
 				   slurmdb_res_cond_t *res_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_res))(db_conn, uid, res_cond);
 }
 
 extern List acct_storage_g_get_wckeys(void *db_conn, uint32_t uid,
 				      slurmdb_wckey_cond_t *wckey_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_wckeys))(db_conn, uid, wckey_cond);
 }
 
 extern List acct_storage_g_get_reservations(
 	void *db_conn, uint32_t uid, slurmdb_reservation_cond_t *resv_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.get_resvs))(db_conn, uid, resv_cond);
 }
 
 extern List acct_storage_g_get_txn(void *db_conn,  uint32_t uid,
 				   slurmdb_txn_cond_t *txn_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.get_txn))(db_conn, uid, txn_cond);
 }
 
@@ -717,7 +959,11 @@ extern int acct_storage_g_get_usage(void *db_conn,  uint32_t uid,
 				    void *in, int type,
 				    time_t start, time_t end)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.get_usage))(db_conn, uid, in, type, start, end);
 }
 
@@ -726,7 +972,11 @@ extern int acct_storage_g_roll_usage(void *db_conn,
 				     uint16_t archive_data,
 				     List *rollup_stats_list_in)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.roll_usage))(db_conn, sent_start, sent_end, archive_data,
 				   rollup_stats_list_in);
 }
@@ -734,9 +984,12 @@ extern int acct_storage_g_roll_usage(void *db_conn,
 extern int acct_storage_g_fix_runaway_jobs(void *db_conn,
 					   uint32_t uid, List jobs)
 {
-	xassert(plugin_context);
-	return (*(ops.fix_runaway_jobs))(db_conn, uid, jobs);
+	xassert(plugin_inited);
 
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.fix_runaway_jobs))(db_conn, uid, jobs);
 }
 
 extern int clusteracct_storage_g_node_down(void *db_conn,
@@ -744,14 +997,22 @@ extern int clusteracct_storage_g_node_down(void *db_conn,
 					   time_t event_time,
 					   char *reason, uint32_t reason_uid)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.node_down))(db_conn, node_ptr, event_time,
 				  reason, reason_uid);
 }
 
 extern char *acct_storage_g_node_inx(void *db_conn, char *nodes)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
+
 	return (*(ops.node_inx))(db_conn, nodes);
 }
 
@@ -759,7 +1020,11 @@ extern int clusteracct_storage_g_node_up(void *db_conn,
 					 node_record_t *node_ptr,
 					 time_t event_time)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 
 	xfree(node_ptr->reason);
 	node_ptr->reason_time = 0;
@@ -768,6 +1033,16 @@ extern int clusteracct_storage_g_node_up(void *db_conn,
 	return (*(ops.node_up))(db_conn, node_ptr, event_time);
 }
 
+extern int clusteracct_storage_g_node_update(void *db_conn,
+					     node_record_t *node_ptr)
+{
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.node_update))(db_conn, node_ptr);
+}
 
 extern int clusteracct_storage_g_cluster_tres(void *db_conn,
 					      char *cluster_nodes,
@@ -775,29 +1050,44 @@ extern int clusteracct_storage_g_cluster_tres(void *db_conn,
 					      time_t event_time,
 					      uint16_t rpc_version)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.cluster_tres))(db_conn, cluster_nodes,
 				     tres_str_in, event_time, rpc_version);
 }
 
-
 extern int clusteracct_storage_g_register_ctld(void *db_conn, uint16_t port)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.register_ctld))(db_conn, port);
 }
 
 extern int clusteracct_storage_g_register_disconn_ctld(
 	void *db_conn, char *control_host)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.register_disconn_ctld))(db_conn, control_host);
 }
 
 extern int clusteracct_storage_g_fini_ctld(void *db_conn,
 					   slurmdb_cluster_rec_t *cluster_rec)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.fini_ctld))(db_conn, cluster_rec);
 }
 
@@ -808,7 +1098,11 @@ extern int clusteracct_storage_g_fini_ctld(void *db_conn,
 extern int jobacct_storage_g_job_start(void *db_conn,
 				       job_record_t *job_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_JOBS)
 		return SLURM_SUCCESS;
 
@@ -837,7 +1131,11 @@ extern int jobacct_storage_g_job_start(void *db_conn,
  */
 extern int jobacct_storage_g_job_heavy(void *db_conn, job_record_t *job_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_JOBS)
 		return SLURM_SUCCESS;
 	return (*(ops.job_heavy))(db_conn, job_ptr);
@@ -849,7 +1147,11 @@ extern int jobacct_storage_g_job_heavy(void *db_conn, job_record_t *job_ptr)
 extern int jobacct_storage_g_job_complete(void *db_conn,
 					  job_record_t *job_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_JOBS)
 		return SLURM_SUCCESS;
 	return (*(ops.job_complete))(db_conn, job_ptr);
@@ -860,7 +1162,11 @@ extern int jobacct_storage_g_job_complete(void *db_conn,
  */
 extern int jobacct_storage_g_step_start(void *db_conn, step_record_t *step_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_STEPS)
 		return SLURM_SUCCESS;
 	return (*(ops.step_start))(db_conn, step_ptr);
@@ -872,7 +1178,11 @@ extern int jobacct_storage_g_step_start(void *db_conn, step_record_t *step_ptr)
 extern int jobacct_storage_g_step_complete(void *db_conn,
 					   step_record_t *step_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_STEPS)
 		return SLURM_SUCCESS;
 	return (*(ops.step_complete))(db_conn, step_ptr);
@@ -884,7 +1194,11 @@ extern int jobacct_storage_g_step_complete(void *db_conn,
 extern int jobacct_storage_g_job_suspend(void *db_conn,
 					 job_record_t *job_ptr)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	if (slurm_conf.accounting_storage_enforce & ACCOUNTING_ENFORCE_NO_JOBS)
 		return SLURM_SUCCESS;
 	return (*(ops.job_suspend))(db_conn, job_ptr);
@@ -913,7 +1227,11 @@ extern List jobacct_storage_g_get_jobs_cond(void *db_conn, uint32_t uid,
 {
 	List ret_list;
 
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	ret_list = (*(ops.get_jobs_cond))(db_conn, uid, job_cond);
 
 	/* If multiple clusters were requested, the jobs are grouped together by
@@ -932,7 +1250,11 @@ extern List jobacct_storage_g_get_jobs_cond(void *db_conn, uint32_t uid,
 extern int jobacct_storage_g_archive(void *db_conn,
 				     slurmdb_archive_cond_t *arch_cond)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.archive_dump))(db_conn, arch_cond);
 }
 
@@ -942,9 +1264,12 @@ extern int jobacct_storage_g_archive(void *db_conn,
 extern int jobacct_storage_g_archive_load(void *db_conn,
 					  slurmdb_archive_rec_t *arch_rec)
 {
-	xassert(plugin_context);
-	return (*(ops.archive_load))(db_conn, arch_rec);
+	xassert(plugin_inited);
 
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.archive_load))(db_conn, arch_rec);
 }
 
 /*
@@ -954,7 +1279,11 @@ extern int jobacct_storage_g_archive_load(void *db_conn,
  */
 extern int acct_storage_g_update_shares_used(void *db_conn, List acct_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.update_shares_used))(db_conn, acct_list);
 }
 
@@ -969,7 +1298,11 @@ extern int acct_storage_g_update_shares_used(void *db_conn, List acct_list)
 extern int acct_storage_g_flush_jobs_on_cluster(
 	void *db_conn, time_t event_time)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.flush_jobs))(db_conn, event_time);
 
 }
@@ -980,7 +1313,11 @@ extern int acct_storage_g_flush_jobs_on_cluster(
  */
 extern int acct_storage_g_reconfig(void *db_conn, bool dbd)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.reconfig))(db_conn, dbd);
 
 }
@@ -992,7 +1329,11 @@ extern int acct_storage_g_reconfig(void *db_conn, bool dbd)
 extern int acct_storage_g_reset_lft_rgt(void *db_conn, uid_t uid,
 					List cluster_list)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.reset_lft_rgt))(db_conn, uid, cluster_list);
 
 }
@@ -1003,7 +1344,11 @@ extern int acct_storage_g_reset_lft_rgt(void *db_conn, uid_t uid,
  */
 extern int acct_storage_g_get_stats(void *db_conn, slurmdb_stats_rec_t **stats)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.get_stats))(db_conn, stats);
 }
 
@@ -1013,7 +1358,11 @@ extern int acct_storage_g_get_stats(void *db_conn, slurmdb_stats_rec_t **stats)
  */
 extern int acct_storage_g_clear_stats(void *db_conn)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.clear_stats))(db_conn);
 }
 
@@ -1024,11 +1373,14 @@ extern int acct_storage_g_clear_stats(void *db_conn)
 extern int acct_storage_g_get_data(void *db_conn, acct_storage_info_t dinfo,
 				    void *data)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
 	return (*(ops.get_data))(db_conn, dinfo, data);
 }
 
-
 /*
  * Send all relavant information to the DBD.
  * RET: SLURM_SUCCESS on success SLURM_ERROR else
@@ -1036,7 +1388,11 @@ extern int acct_storage_g_get_data(void *db_conn, acct_storage_info_t dinfo,
 extern void acct_storage_g_send_all(void *db_conn, time_t event_time,
 				    slurm_msg_type_t msg_type)
 {
-	xassert(plugin_context);
+	xassert(plugin_inited);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return;
+
 	(*(ops.send_all))(db_conn, event_time, msg_type);
 }
 
@@ -1046,7 +1402,10 @@ extern void acct_storage_g_send_all(void *db_conn, time_t event_time,
  */
 extern int acct_storage_g_shutdown(void *db_conn)
 {
-	xassert(plugin_context);
-	return (*(ops.shutdown))(db_conn);
+	xassert(plugin_inited);
 
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.shutdown))(db_conn);
 }
