@@ -119,7 +119,7 @@ static pthread_mutex_t profile_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t timer_thread_id = 0;
 static pthread_mutex_t timer_thread_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t timer_thread_cond = PTHREAD_COND_INITIALIZER;
-static bool init_run = false;
+static plugin_init_t plugin_inited = PLUGIN_NOT_INITED;
 
 static void _set_freq(int type, char *freq, char *freq_def)
 {
@@ -153,7 +153,8 @@ static void *_timer_thread(void *args)
 	abs.tv_sec = tvnow.tv_sec;
 	abs.tv_nsec = tvnow.tv_usec * 1000;
 
-	while (init_run && acct_gather_profile_test()) {
+	while ((plugin_inited != PLUGIN_NOT_INITED) &&
+	       acct_gather_profile_test()) {
 		slurm_mutex_lock(&g_context_lock);
 		now = time(NULL);
 
@@ -217,9 +218,14 @@ extern int acct_gather_profile_init(void)
 
 	slurm_mutex_lock(&g_context_lock);
 
-	if (g_context)
+	if (plugin_inited != PLUGIN_NOT_INITED)
 		goto done;
 
+	if (!slurm_conf.acct_gather_profile_type) {
+		plugin_inited = PLUGIN_NOOP;
+		goto done;
+	}
+
 	g_context = plugin_context_create(plugin_type,
 					  slurm_conf.acct_gather_profile_type,
 					  (void **) &ops, syms, sizeof(syms));
@@ -228,9 +234,10 @@ extern int acct_gather_profile_init(void)
 		error("cannot create %s context for %s",
 		      plugin_type, slurm_conf.acct_gather_profile_type);
 		retval = SLURM_ERROR;
+		plugin_inited = PLUGIN_NOT_INITED;
 		goto done;
 	}
-	init_run = true;
+	plugin_inited = PLUGIN_INITED;
 
 done:
 	slurm_mutex_unlock(&g_context_lock);
@@ -245,16 +252,16 @@ extern int acct_gather_profile_fini(void)
 {
 	int rc = SLURM_SUCCESS, i;
 
-	if (!g_context)
-		return SLURM_SUCCESS;
+	acct_gather_profile_endpoll();
 
+	/*
+	 * If the plugin is a NOOP, do the cleanup of jobacct gather anyways
+	 * since other plugins might have been inited in acct_gather_conf_init()
+	 * and we've started the polling mechanism in
+	 * acct_gather_profile_startpoll().
+	 */
 	slurm_mutex_lock(&g_context_lock);
 
-	if (!g_context)
-		goto done;
-
-	init_run = false;
-
 	for (i=0; i < PROFILE_CNT; i++) {
 		switch (i) {
 		case PROFILE_ENERGY:
@@ -276,16 +283,12 @@ extern int acct_gather_profile_fini(void)
 		}
 	}
 
-	if (timer_thread_id) {
-		slurm_mutex_lock(&timer_thread_mutex);
-		slurm_cond_signal(&timer_thread_cond);
-		slurm_mutex_unlock(&timer_thread_mutex);
-		pthread_join(timer_thread_id, NULL);
+	if (g_context) {
+		rc = plugin_context_destroy(g_context);
+		g_context = NULL;
 	}
 
-	rc = plugin_context_destroy(g_context);
-	g_context = NULL;
-done:
+	plugin_inited = PLUGIN_NOT_INITED;
 	slurm_mutex_unlock(&g_context_lock);
 
 	return rc;
@@ -448,7 +451,7 @@ extern int acct_gather_profile_startpoll(char *freq, char *freq_def)
 	int i;
 	uint32_t profile = ACCT_GATHER_PROFILE_NOT_SET;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
 
 	slurm_mutex_lock(&profile_running_mutex);
 	if (acct_gather_profile_running) {
@@ -459,7 +462,10 @@ extern int acct_gather_profile_startpoll(char *freq, char *freq_def)
 	acct_gather_profile_running = true;
 	slurm_mutex_unlock(&profile_running_mutex);
 
-	(*(ops.get))(ACCT_GATHER_PROFILE_RUNNING, &profile);
+	if (plugin_inited == PLUGIN_NOOP)
+		profile = ACCT_GATHER_PROFILE_NONE;
+	else
+		(*(ops.get))(ACCT_GATHER_PROFILE_RUNNING, &profile);
 	xassert(profile != ACCT_GATHER_PROFILE_NOT_SET);
 
 	for (i=0; i < PROFILE_CNT; i++) {
@@ -478,11 +484,16 @@ extern int acct_gather_profile_startpoll(char *freq, char *freq_def)
 				acct_gather_profile_timer[i].freq);
 			break;
 		case PROFILE_TASK:
-			/* Always set up the task (always first) to be
-			   done since it is used to control memory
-			   consumption and such.  It will check
-			   profile inside it's plugin.
-			*/
+			/*
+			 * Always set up the task (always first) to be
+			 * done since it is used to control memory
+			 * consumption and such.  It will check
+			 * profile inside it's plugin. Note that if this
+			 * plugin is ACCT_GATHER_PROFILE_NONE we'll still need
+			 * to call job_acct_gather_fini() at the end, in
+			 * acct_gather_profile_fini() to cleanup the cgroup
+			 * tree.
+			 */
 			_set_freq(i, freq, freq_def);
 
 			jobacct_gather_startpoll(
@@ -555,11 +566,20 @@ extern void acct_gather_profile_endpoll(void)
 			      "(acct_gather_profile_endpoll)", i);
 		}
 	}
+
+	slurm_mutex_lock(&timer_thread_mutex);
+	slurm_cond_signal(&timer_thread_cond);
+	slurm_mutex_unlock(&timer_thread_mutex);
+	pthread_join(timer_thread_id, NULL);
+	timer_thread_id = 0;
 }
 
 extern int acct_gather_profile_g_child_forked(void)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	(*(ops.child_forked))();
 	return SLURM_SUCCESS;
@@ -568,7 +588,10 @@ extern int acct_gather_profile_g_child_forked(void)
 extern int acct_gather_profile_g_conf_options(s_p_options_t **full_options,
 					       int *full_options_cnt)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	(*(ops.conf_options))(full_options, full_options_cnt);
 	return SLURM_SUCCESS;
@@ -576,7 +599,10 @@ extern int acct_gather_profile_g_conf_options(s_p_options_t **full_options,
 
 extern int acct_gather_profile_g_conf_set(s_p_hashtbl_t *tbl)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	(*(ops.conf_set))(tbl);
 	return SLURM_SUCCESS;
@@ -585,7 +611,20 @@ extern int acct_gather_profile_g_conf_set(s_p_hashtbl_t *tbl)
 extern int acct_gather_profile_g_get(enum acct_gather_profile_info info_type,
 				      void *data)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		uint32_t *uint32 = (uint32_t *) data;
+		switch (info_type) {
+		case ACCT_GATHER_PROFILE_DEFAULT:
+		case ACCT_GATHER_PROFILE_RUNNING:
+			*uint32 = ACCT_GATHER_PROFILE_NONE;
+			break;
+		default:
+			break;
+		}
+		return SLURM_SUCCESS;
+	}
 
 	(*(ops.get))(info_type, data);
 	return SLURM_SUCCESS;
@@ -593,7 +632,10 @@ extern int acct_gather_profile_g_get(enum acct_gather_profile_info info_type,
 
 extern int acct_gather_profile_g_node_step_start(stepd_step_rec_t* job)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.node_step_start))(job);
 }
@@ -602,6 +644,10 @@ extern int acct_gather_profile_g_node_step_end(void)
 {
 	int retval = SLURM_ERROR;
 
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	retval = (*(ops.node_step_end))();
 	return retval;
@@ -611,7 +657,10 @@ extern int acct_gather_profile_g_task_start(uint32_t taskid)
 {
 	int retval = SLURM_ERROR;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&profile_mutex);
 	retval = (*(ops.task_start))(taskid);
@@ -623,7 +672,10 @@ extern int acct_gather_profile_g_task_end(pid_t taskpid)
 {
 	int retval = SLURM_ERROR;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&profile_mutex);
 	retval = (*(ops.task_end))(taskpid);
@@ -635,7 +687,10 @@ extern int64_t acct_gather_profile_g_create_group(const char *name)
 {
 	int64_t retval = SLURM_ERROR;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&profile_mutex);
 	retval = (*(ops.create_group))(name);
@@ -649,7 +704,10 @@ extern int acct_gather_profile_g_create_dataset(
 {
 	int retval = SLURM_ERROR;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&profile_mutex);
 	retval = (*(ops.create_dataset))(name, parent, dataset);
@@ -662,7 +720,10 @@ extern int acct_gather_profile_g_add_sample_data(int dataset_id, void* data,
 {
 	int retval = SLURM_ERROR;
 
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&profile_mutex);
 	retval = (*(ops.add_sample_data))(dataset_id, data, sample_time);
@@ -672,14 +733,20 @@ extern int acct_gather_profile_g_add_sample_data(int dataset_id, void* data,
 
 extern void acct_gather_profile_g_conf_values(void *data)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return;
 
 	(*(ops.conf_values))(data);
 }
 
 extern bool acct_gather_profile_g_is_active(uint32_t type)
 {
-	xassert(init_run);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return false;
 
 	return (*(ops.is_active))(type);
 }
