@@ -328,6 +328,11 @@ extern int as_mysql_add_federations(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!is_user_min_admin_level(mysql_conn, uid, SLURMDB_ADMIN_SUPER_USER))
 		return ESLURM_ACCESS_DENIED;
 
+	if (!federation_list || !list_count(federation_list)) {
+		error("%s: Trying to add empty federation list", __func__);
+		return ESLURM_EMPTY_LIST;
+	}
+
 	user_name = uid_to_string((uid_t) uid);
 
 	itr = list_iterator_create(federation_list);
@@ -713,17 +718,7 @@ extern List as_mysql_remove_federations(mysql_conn_t *mysql_conn, uint32_t uid,
 
 extern int as_mysql_add_feds_to_update_list(mysql_conn_t *mysql_conn)
 {
-	int rc = SLURM_ERROR;
-	List feds = as_mysql_get_federations(mysql_conn, 0, NULL);
-
-	/* Even if there are no feds, need to send an empty list for the case
-	 * that all feds were removed. The controller needs to know that it was
-	 * removed from a federation. */
-	if (feds &&
-	    ((rc = addto_update_list(mysql_conn->update_list,
-				     SLURMDB_UPDATE_FEDS, feds))
-	     != SLURM_SUCCESS)) {
-			FREE_NULL_LIST(feds);
-	}
-	return rc;
+	/* Set the flag to get federation changes after commit has been made. */
+	mysql_conn->flags |= DB_CONN_FLAG_FEDUPDATE;
+	return SLURM_SUCCESS;
 }
