@@ -125,9 +125,12 @@ static time_t plugin_shutdown = 0;
 static char *cluster_nodes = NULL; /* Protected by node write lock */
 static char *cluster_tres = NULL; /* Protected by node write lock */
 
-static hostlist_t cluster_hl = NULL;
+static hostlist_t *cluster_hl = NULL;
 static pthread_mutex_t cluster_hl_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+static int prev_node_record_count = -1;
+static bitstr_t *total_node_bitmap = NULL;
+
 extern int jobacct_storage_p_job_start(void *db_conn, job_record_t *job_ptr);
 extern int jobacct_storage_p_job_heavy(void *db_conn, job_record_t *job_ptr);
 extern void acct_storage_p_send_all(void *db_conn, time_t event_time,
@@ -330,6 +333,7 @@ static void *_set_db_inx_thread(void *no_data)
 	ListIterator itr;
 	struct timeval tvnow;
 	struct timespec abs;
+	bool more_jobs;
 
 	/* Read lock on jobs */
 	slurmctld_lock_t job_read_lock =
@@ -359,6 +363,7 @@ static void *_set_db_inx_thread(void *no_data)
 		slurm_mutex_lock(&db_inx_lock);
 		/* info("in lock db_thread"); */
 		running_db_inx = 1;
+		more_jobs = false;
 
 		/* Here we have off loaded starting
 		 * jobs in the database out of band
@@ -410,8 +415,10 @@ static void *_set_db_inx_thread(void *no_data)
 			/* Just so we don't have a crazy
 			   amount of messages at once.
 			*/
-			if (list_count(local_job_list) > 1000)
+			if (list_count(local_job_list) > 1000) {
+				more_jobs = true;
 				break;
+			}
 		}
 		list_iterator_destroy(itr);
 		unlock_slurmctld(job_read_lock);
@@ -437,7 +444,8 @@ static void *_set_db_inx_thread(void *no_data)
 			} else if (resp.msg_type == PERSIST_RC) {
 				persist_rc_msg_t *msg = resp.data;
 				if (msg->rc == SLURM_SUCCESS) {
-					info("%s", msg->comment);
+					log_var(LOG_LEVEL_INFO, "%s",
+						msg->comment);
 				} else
 					error("%s", msg->comment);
 				slurm_persist_free_rc_msg(msg);
@@ -493,11 +501,12 @@ static void *_set_db_inx_thread(void *no_data)
 		   haven't had the start rpc come through.
 		*/
 
-		gettimeofday(&tvnow, NULL);
-		abs.tv_sec = tvnow.tv_sec + 5;
-		abs.tv_nsec = tvnow.tv_usec * 1000;
-
-		slurm_cond_timedwait(&db_inx_cond, &db_inx_lock, &abs);
+		if (!more_jobs) {
+			gettimeofday(&tvnow, NULL);
+			abs.tv_sec = tvnow.tv_sec + 5;
+			abs.tv_nsec = tvnow.tv_usec * 1000;
+			slurm_cond_timedwait(&db_inx_cond, &db_inx_lock, &abs);
+		}
 
 		slurm_mutex_unlock(&db_inx_lock);
 	}
@@ -537,8 +546,6 @@ static int _send_cluster_tres(void *db_conn,
 
 static void _update_cluster_nodes(void)
 {
-	static int prev_node_record_count = -1;
-	static bitstr_t *total_node_bitmap = NULL;
 	assoc_mgr_lock_t locks = { .tres = READ_LOCK };
 
 	xassert(verify_lock(NODE_LOCK, WRITE_LOCK));
@@ -637,7 +644,9 @@ extern int fini ( void )
 	xfree(cluster_nodes);
 	xfree(cluster_tres);
 	FREE_NULL_HOSTLIST(cluster_hl);
+	FREE_NULL_BITMAP(total_node_bitmap);
 
+	prev_node_record_count = -1;
 	first = 1;
 
 	return SLURM_SUCCESS;
@@ -716,6 +725,32 @@ extern int acct_storage_p_add_users(void *db_conn, uint32_t uid,
 	return rc;
 }
 
+extern char *acct_storage_p_add_users_cond(void *db_conn, uint32_t uid,
+					   slurmdb_add_assoc_cond_t *add_assoc,
+					   slurmdb_user_rec_t *user)
+{
+	persist_msg_t req = {0};
+	dbd_modify_msg_t msg;
+	char *ret_str = NULL;
+	int rc, resp_code = SLURM_SUCCESS;
+
+	memset(&msg, 0, sizeof(msg));
+	msg.cond = add_assoc;
+	msg.rec = user;
+
+	req.msg_type = DBD_ADD_USERS_COND;
+	req.conn = db_conn;
+	req.data = &msg;
+	rc = dbd_conn_send_recv_rc_comment_msg(SLURM_PROTOCOL_VERSION,
+					       &req, &resp_code, &ret_str);
+
+	if (resp_code != SLURM_SUCCESS)
+		rc = resp_code;
+
+	errno = rc;
+	return ret_str;
+}
+
 extern int acct_storage_p_add_coord(void *db_conn, uint32_t uid,
 				    List acct_list,
 				    slurmdb_user_cond_t *user_cond)
@@ -762,6 +797,32 @@ extern int acct_storage_p_add_accts(void *db_conn, uint32_t uid,
 	return rc;
 }
 
+extern char *acct_storage_p_add_accts_cond(void *db_conn, uint32_t uid,
+					   slurmdb_add_assoc_cond_t *add_assoc,
+					   slurmdb_account_rec_t *acct)
+{
+	persist_msg_t req = {0};
+	dbd_modify_msg_t msg;
+	char *ret_str = NULL;
+	int rc, resp_code = SLURM_SUCCESS;
+
+	memset(&msg, 0, sizeof(msg));
+	msg.cond = add_assoc;
+	msg.rec = acct;
+
+	req.msg_type = DBD_ADD_ACCOUNTS_COND;
+	req.conn = db_conn;
+	req.data = &msg;
+	rc = dbd_conn_send_recv_rc_comment_msg(SLURM_PROTOCOL_VERSION,
+					       &req, &resp_code, &ret_str);
+
+	if (resp_code != SLURM_SUCCESS)
+		rc = resp_code;
+
+	errno = rc;
+	return ret_str;
+}
+
 extern int acct_storage_p_add_clusters(void *db_conn, uint32_t uid,
 				       List cluster_list)
 {
@@ -983,7 +1044,7 @@ extern List acct_storage_p_modify_users(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1027,7 +1088,7 @@ extern List acct_storage_p_modify_accts(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1073,7 +1134,7 @@ extern List acct_storage_p_modify_clusters(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1120,7 +1181,7 @@ extern List acct_storage_p_modify_assocs(
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1167,7 +1228,7 @@ extern List acct_storage_p_modify_federations(
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1221,7 +1282,7 @@ extern List acct_storage_p_modify_job(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1265,7 +1326,7 @@ extern List acct_storage_p_modify_qos(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1309,7 +1370,7 @@ extern List acct_storage_p_modify_res(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1353,7 +1414,7 @@ extern List acct_storage_p_modify_wckeys(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -1438,7 +1499,7 @@ extern List acct_storage_p_remove_users(void *db_conn, uint32_t uid,
 	else if (resp.msg_type == PERSIST_RC) {
 		persist_rc_msg_t *msg = resp.data;
 		if (msg->rc == SLURM_SUCCESS) {
-			info("%s", msg->comment);
+			log_var(LOG_LEVEL_INFO, "%s", msg->comment);
 			ret_list = list_create(NULL);
 		} else {
 			slurm_seterrno(msg->rc);
@@ -2195,6 +2256,48 @@ extern List acct_storage_p_get_events(void *db_conn, uint32_t uid,
 	return ret_list;
 }
 
+extern List acct_storage_p_get_instances(void *db_conn,
+					 uint32_t uid,
+					 slurmdb_instance_cond_t *instance_cond)
+{
+	persist_msg_t req = {0}, resp = {0};
+	dbd_cond_msg_t get_msg = {0};
+	dbd_list_msg_t *got_msg;
+	int rc;
+	List ret_list = NULL;
+
+	get_msg.cond = instance_cond;
+
+	req.msg_type = DBD_GET_INSTANCES;
+	req.conn = db_conn;
+	req.data = &get_msg;
+	rc = dbd_conn_send_recv(SLURM_PROTOCOL_VERSION, &req, &resp);
+
+	if (rc != SLURM_SUCCESS)
+		error("DBD_GET_INSTANCES failure: %m");
+	else if (resp.msg_type == PERSIST_RC) {
+		persist_rc_msg_t *msg = resp.data;
+		if (msg->rc == SLURM_SUCCESS) {
+			info("%s", msg->comment);
+			ret_list = list_create(NULL);
+		} else {
+			slurm_seterrno(msg->rc);
+			error("%s", msg->comment);
+		}
+		slurm_persist_free_rc_msg(msg);
+	} else if (resp.msg_type != DBD_GOT_INSTANCES) {
+		error("response type not DBD_GOT_INSTANCES: %u",
+		      resp.msg_type);
+	} else {
+		got_msg = (dbd_list_msg_t *) resp.data;
+		ret_list = got_msg->my_list;
+		got_msg->my_list = NULL;
+		slurmdbd_free_list_msg(got_msg);
+	}
+
+	return ret_list;
+}
+
 extern List acct_storage_p_get_problems(void *db_conn, uid_t uid,
 					slurmdb_assoc_cond_t *assoc_cond)
 {
@@ -2670,18 +2773,25 @@ extern int clusteracct_storage_p_node_down(void *db_conn,
 extern char *acct_storage_p_node_inx(void *db_conn, char *nodes)
 {
 	char *host, *ret_str;
-	hostlist_t node_hl;
+	hostlist_t *node_hl;
 	bitstr_t *node_bitmap;
-	hostlist_iterator_t h_itr;
+	hostlist_iterator_t *h_itr;
 
-	if (!nodes || !cluster_hl)
+	if (!nodes)
 		return NULL;
-
 	node_hl = hostlist_create(nodes);
-	node_bitmap = bit_alloc(node_record_count);
 	h_itr = hostlist_iterator_create(node_hl);
 
 	slurm_mutex_lock(&cluster_hl_mutex);
+	if (!cluster_hl) {
+		slurm_mutex_unlock(&cluster_hl_mutex);
+		hostlist_iterator_destroy(h_itr);
+		FREE_NULL_HOSTLIST(node_hl);
+		return NULL;
+	}
+
+	node_bitmap = bit_alloc(hostlist_count(cluster_hl));
+
 	while ((host = hostlist_next(h_itr))) {
 		int loc;
 		if ((loc = hostlist_find(cluster_hl, host)) != -1)
@@ -2705,7 +2815,7 @@ extern int clusteracct_storage_p_node_up(void *db_conn, node_record_t *node_ptr,
 	dbd_node_state_msg_t req;
 
 	if (IS_NODE_FUTURE(node_ptr) ||
-	    (IS_NODE_CLOUD(node_ptr) && IS_NODE_POWERED_DOWN(node_ptr)))
+	    IS_NODE_POWERED_DOWN(node_ptr))
 		return SLURM_SUCCESS;
 
 	memset(&req, 0, sizeof(dbd_node_state_msg_t));
@@ -2724,6 +2834,36 @@ extern int clusteracct_storage_p_node_up(void *db_conn, node_record_t *node_ptr,
 	return SLURM_SUCCESS;
 }
 
+extern int clusteracct_storage_p_node_update(void *db_conn,
+					     node_record_t *node_ptr,
+					     time_t event_time)
+{
+	persist_msg_t msg = { 0 };
+	dbd_node_state_msg_t req;
+
+	if (IS_NODE_FUTURE(node_ptr) ||
+	    IS_NODE_POWERED_DOWN(node_ptr))
+		return SLURM_SUCCESS;
+
+	memset(&req, 0, sizeof(dbd_node_state_msg_t));
+
+	req.hostlist = node_ptr->name;
+	req.extra = node_ptr->extra;
+	req.instance_id = node_ptr->instance_id;
+	req.instance_type = node_ptr->instance_type;
+	req.new_state = DBD_NODE_STATE_UPDATE;
+	req.tres_str = node_ptr->tres_str;
+
+	msg.msg_type = DBD_NODE_STATE;
+	msg.conn = db_conn;
+	msg.data = &req;
+
+	if (slurmdbd_agent_send(SLURM_PROTOCOL_VERSION, &msg) < 0)
+		return SLURM_ERROR;
+
+	return SLURM_SUCCESS;
+}
+
 extern int clusteracct_storage_p_cluster_tres(void *db_conn,
 					      char *cluster_nodes_in,
 					      char *tres_str_in,
@@ -3443,8 +3583,8 @@ extern void acct_storage_p_send_all(void *db_conn, time_t event_time,
 		(void) send_nodes_to_accounting(event_time);
 		break;
 	default:
-		error("%s: unknown message type of %d given",
-		      __func__, msg_type);
+		error("%s: unknown message type of %s given",
+		      __func__, rpc_num2string(msg_type));
 		xassert(0);
 	}
 }
