@@ -215,7 +215,7 @@ extern void dbd_conn_close(slurm_persist_conn_t **pc)
 		log_flag(NET, "We are shutdown, not sending DB_FINI to %s:%u",
 			 (*pc)->rem_host,
 			 (*pc)->rem_port);
-		return;
+		goto destroy_conn;
 	}
 
 	/* If the connection is already gone, we don't need to send a fini. */
@@ -223,7 +223,7 @@ extern void dbd_conn_close(slurm_persist_conn_t **pc)
 		log_flag(NET, "unable to send DB_FINI msg to %s:%u",
 			 (*pc)->rem_host,
 			 (*pc)->rem_port);
-		return;
+		goto destroy_conn;
 	}
 
 	buffer = init_buf(1024);
@@ -233,12 +233,13 @@ extern void dbd_conn_close(slurm_persist_conn_t **pc)
 	slurmdbd_pack_fini_msg(&req, SLURM_PROTOCOL_VERSION, buffer);
 
 	rc = slurm_persist_send_msg(*pc, buffer);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 
 	log_flag(NET, "sent DB_FINI msg to %s:%u rc(%d):%s",
 		 (*pc)->rem_host, (*pc)->rem_port,
 		 rc, slurm_strerror(rc));
 
+destroy_conn:
 	slurm_persist_conn_destroy(*pc);
 	*pc = NULL;
 }
@@ -277,7 +278,7 @@ extern int dbd_conn_send_recv_direct(uint16_t rpc_version,
 	}
 
 	rc = slurm_persist_send_msg(use_conn, buffer);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	if (rc != SLURM_SUCCESS) {
 		error("Sending message type %s: %d: %s",
 		      slurmdbd_msg_type_2_str(req->msg_type, 1), rc,
@@ -298,7 +299,7 @@ extern int dbd_conn_send_recv_direct(uint16_t rpc_version,
 	if (rc == SLURM_SUCCESS && resp->msg_type == DBD_ID_RC)
 		rc = ((dbd_id_rc_msg_t *)resp->data)->return_code;
 
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 end_it:
 
 	log_flag(PROTOCOL, "msg_type:%s protocol_version:%hu return_code:%d response_msg_type:%s",
@@ -308,9 +309,10 @@ end_it:
 	return rc;
 }
 
-extern int dbd_conn_send_recv_rc_msg(uint16_t rpc_version,
-				     persist_msg_t *req,
-				     int *resp_code)
+extern int dbd_conn_send_recv_rc_comment_msg(uint16_t rpc_version,
+					     persist_msg_t *req,
+					     int *resp_code,
+					     char **comment)
 {
 	int rc;
 	persist_msg_t resp;
@@ -330,6 +332,7 @@ extern int dbd_conn_send_recv_rc_msg(uint16_t rpc_version,
 	} else {	/* resp.msg_type == PERSIST_RC */
 		persist_rc_msg_t *msg = resp.data;
 		*resp_code = msg->rc;
+
 		if (msg->rc != SLURM_SUCCESS &&
 		    msg->rc != ACCOUNTING_FIRST_REG &&
 		    msg->rc != ACCOUNTING_TRES_CHANGE_DB &&
@@ -358,6 +361,12 @@ extern int dbd_conn_send_recv_rc_msg(uint16_t rpc_version,
 				      msg->ret_info, msg->rc,
 				      comment);
 		}
+
+		if (comment) {
+			*comment = msg->comment;
+			msg->comment = NULL;
+		}
+
 		slurm_persist_free_rc_msg(msg);
 	}
 
@@ -368,6 +377,14 @@ extern int dbd_conn_send_recv_rc_msg(uint16_t rpc_version,
 	return rc;
 }
 
+extern int dbd_conn_send_recv_rc_msg(uint16_t rpc_version,
+				     persist_msg_t *req,
+				     int *resp_code)
+{
+	return dbd_conn_send_recv_rc_comment_msg(
+		rpc_version, req, resp_code, NULL);
+}
+
 extern int dbd_conn_send_recv(uint16_t rpc_version,
 			      persist_msg_t *req,
 			      persist_msg_t *resp)
