@@ -41,7 +41,6 @@
 
 #include "src/common/fd.h"
 #include "src/common/slurmdbd_pack.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
 #include "slurmdbd_agent.h"
@@ -60,6 +59,7 @@ slurm_persist_conn_t *slurmdbd_conn = NULL;
 
 static pthread_mutex_t agent_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  agent_cond = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t shutdown_cond = PTHREAD_COND_INITIALIZER;
 static List      agent_list     = (List) NULL;
 static pthread_t agent_tid      = 0;
 
@@ -134,7 +134,8 @@ static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 		slurm_persist_free_rc_msg(msg);
 		break;
 	default:
-		error("bad message type %d != PERSIST_RC", msg_type);
+		error("bad message type %s != PERSIST_RC",
+		      slurmdbd_msg_type_2_str(msg_type, true));
 	}
 
 	return rc;
@@ -149,7 +150,7 @@ static int _get_return_code(void)
 
 	rc = _unpack_return_code(slurmdbd_conn->version, buffer);
 
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
@@ -189,7 +190,7 @@ static int _handle_mult_rc_ret(void)
 					break;
 
 				if ((b = list_dequeue(agent_list))) {
-					free_buf(b);
+					FREE_NULL_BUFFER(b);
 				} else {
 					error("DBD_GOT_MULT_MSG "
 					      "unpack message error");
@@ -233,11 +234,12 @@ static int _handle_mult_rc_ret(void)
 			error("unpack message error");
 		break;
 	default:
-		error("bad message type %d != PERSIST_RC", msg_type);
+		error("bad message type %s != PERSIST_RC",
+		      slurmdbd_msg_type_2_str(msg_type, true));
 	}
 
 unpack_error:
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
@@ -259,7 +261,7 @@ static buf_t *_load_dbd_rec(int fd)
 		error("state recover error: %m");
 		return NULL;
 	}
-	if (msg_size > MAX_DBD_MSG_LEN) {
+	if (msg_size > MAX_BUF_SIZE) {
 		error("state recover error, msg_size=%u", msg_size);
 		return NULL;
 	}
@@ -277,7 +279,7 @@ static buf_t *_load_dbd_rec(int fd)
 			continue;
 		else {
 			error("state recover error: %m");
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return NULL;
 		}
 	}
@@ -286,7 +288,7 @@ static buf_t *_load_dbd_rec(int fd)
 	rd_size = read(fd, &magic, size);
 	if ((rd_size != size) || (magic != DBD_MAGIC)) {
 		error("state recover error");
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		return NULL;
 	}
 
@@ -323,7 +325,7 @@ static void _load_dbd_state(void)
 		safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
 		debug3("Version string in dbd_state header is %s", ver_str);
 	unpack_error:
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		buffer = NULL;
 		if (ver_str) {
 			/* get the version after VER */
@@ -350,7 +352,7 @@ static void _load_dbd_state(void)
 				set_buf_offset(buffer, 0);
 				rc = unpack_slurmdbd_msg(
 					&msg, rpc_version, buffer);
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				if (rc == SLURM_SUCCESS)
 					buffer = pack_slurmdbd_msg(
 						&msg, SLURM_PROTOCOL_VERSION);
@@ -361,8 +363,7 @@ static void _load_dbd_state(void)
 				error("no buffer given");
 				continue;
 			}
-			if (!list_enqueue(agent_list, buffer))
-				fatal("list_enqueue, no memory");
+			list_enqueue(agent_list, buffer);
 			recovered++;
 			buffer = NULL;
 		}
@@ -432,7 +433,7 @@ static void _save_dbd_state(void)
 		buffer = init_buf(strlen(curr_ver_str));
 		packstr(curr_ver_str, buffer);
 		rc = _save_dbd_rec(fd, buffer);
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		if (rc != SLURM_SUCCESS)
 			goto end_it;
 
@@ -445,19 +446,19 @@ static void _save_dbd_state(void)
 			 */
 			offset = get_buf_offset(buffer);
 			if (offset < 2) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 			set_buf_offset(buffer, 0);
 			(void) unpack16(&msg_type, buffer);  /* checked by offset */
 			set_buf_offset(buffer, offset);
 			if (msg_type == DBD_REGISTER_CTLD) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 
 			rc = _save_dbd_rec(fd, buffer);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			if (rc != SLURM_SUCCESS)
 				break;
 			wrote++;
@@ -474,65 +475,43 @@ end_it:
 	xfree(dbd_fname);
 }
 
-/* Purge queued step records from the agent queue
- * RET number of records purged */
-static int _purge_step_req(void)
+/*
+ * Purge queued records from the agent queue
+ */
+static int _purge_agent_list_req(void *x, void *arg)
 {
-	int purged = 0;
-	ListIterator iter;
 	uint16_t msg_type;
 	uint32_t offset;
-	buf_t *buffer;
+	buf_t *buffer = x;
+	uint16_t purge_type = *(uint16_t *)arg;
 
-	iter = list_iterator_create(agent_list);
-	while ((buffer = list_next(iter))) {
-		offset = get_buf_offset(buffer);
-		if (offset < 2)
-			continue;
-		set_buf_offset(buffer, 0);
-		(void) unpack16(&msg_type, buffer);	/* checked by offset */
-		set_buf_offset(buffer, offset);
+	offset = get_buf_offset(buffer);
+	if (offset < 2)
+		return 0;
+	set_buf_offset(buffer, 0);
+	(void) unpack16(&msg_type, buffer);	/* checked by offset */
+	set_buf_offset(buffer, offset);
+	switch (purge_type) {
+	case DBD_STEP_START:
 		if ((msg_type == DBD_STEP_START) ||
-		    (msg_type == DBD_STEP_COMPLETE)) {
-			list_remove(iter);
-			purged++;
-		}
+		    (msg_type == DBD_STEP_COMPLETE))
+			return 1;
+		break;
+	case DBD_JOB_START:
+		if (msg_type == DBD_JOB_START)
+			return 1;
+		break;
+	default:
+		error("unknown purge type %d", purge_type);
+		break;
 	}
-	list_iterator_destroy(iter);
-	info("purge %d step records", purged);
-	return purged;
-}
 
-/* Purge queued job start records from the agent queue
- * RET number of records purged */
-static int _purge_job_start_req(void)
-{
-	int purged = 0;
-	ListIterator iter;
-	uint16_t msg_type;
-	uint32_t offset;
-	buf_t *buffer;
-
-	iter = list_iterator_create(agent_list);
-	while ((buffer = list_next(iter))) {
-		offset = get_buf_offset(buffer);
-		if (offset < 2)
-			continue;
-		set_buf_offset(buffer, 0);
-		(void) unpack16(&msg_type, buffer);	/* checked by offset */
-		set_buf_offset(buffer, offset);
-		if (msg_type == DBD_JOB_START) {
-			list_remove(iter);
-			purged++;
-		}
-	}
-	list_iterator_destroy(iter);
-	info("purge %d job start records", purged);
-	return purged;
+	return 0;
 }
 
 static void _max_dbd_msg_action(uint32_t *msg_cnt)
 {
+	int purged = 0;
 	if (max_dbd_msg_action == MAX_DBD_ACTION_EXIT) {
 		if (*msg_cnt < slurm_conf.max_dbd_msgs)
 			return;
@@ -543,14 +522,20 @@ static void _max_dbd_msg_action(uint32_t *msg_cnt)
 	}
 
 	/* MAX_DBD_ACTION_DISCARD */
-	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1))
-		*msg_cnt -= _purge_step_req();
-	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1))
-		*msg_cnt -= _purge_job_start_req();
-}
-
-static void _sig_handler(int signal)
-{
+	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1)) {
+		uint16_t purge_type = DBD_STEP_START;
+		purged = list_delete_all(agent_list, _purge_agent_list_req,
+					 &purge_type);
+		*msg_cnt -= purged;
+		info("purge %d step records", purged);
+	}
+	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1)) {
+		uint16_t purge_type = DBD_JOB_START;
+		purged = list_delete_all(agent_list, _purge_agent_list_req,
+					 &purge_type);
+		*msg_cnt -= purged;
+		info("purge %d job start records", purged);
+	}
 }
 
 static int _print_agent_list_msg_type(void *x, void *arg)
@@ -583,7 +568,7 @@ static void _print_agent_list_msg_types(void)
 
 	if ((processed = list_for_each_max(agent_list, &max_msgs,
 					   _print_agent_list_msg_type,
-					   mlist, true)) < 0) {
+					   mlist, true, true)) < 0) {
 		error("unable to create msg type list");
 		xfree(mlist);
 		return;
@@ -606,7 +591,6 @@ static void *_agent(void *x)
 	buf_t *buffer;
 	struct timespec abs_time;
 	static time_t fail_time = 0;
-	int sigarray[] = {SIGUSR1, 0};
 	persist_msg_t list_req = {0};
 	dbd_list_msg_t list_msg;
 	DEF_TIMERS;
@@ -620,11 +604,6 @@ static void *_agent(void *x)
 	list_req.data = &list_msg;
 	memset(&list_msg, 0, sizeof(dbd_list_msg_t));
 
-	/* Prepare to catch SIGUSR1 to interrupt pending
-	 * I/O and terminate in a timely fashion. */
-	xsignal(SIGUSR1, _sig_handler);
-	xsignal_unblock(sigarray);
-
 	log_flag(AGENT, "slurmdbd agent_count=%d with msg_type=%s",
 		 list_count(agent_list),
 		 slurmdbd_msg_type_2_str(list_req.msg_type, 1));
@@ -753,7 +732,7 @@ static void *_agent(void *x)
 			} else
 				buffer = list_dequeue(agent_list);
 
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			fail_time = 0;
 		} else {
 			/* We need to free a mult_msg even on failure */
@@ -761,7 +740,7 @@ static void *_agent(void *x)
 				if (list_msg.my_list != agent_list)
 					FREE_NULL_LIST(list_msg.my_list);
 				list_msg.my_list = NULL;
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 			}
 
 			fail_time = time(NULL);
@@ -784,6 +763,7 @@ static void *_agent(void *x)
 
 	FREE_NULL_LIST(agent_list);
 	agent_running = false;
+	slurm_cond_signal(&shutdown_cond);
 	slurm_mutex_unlock(&agent_lock);
 	return NULL;
 }
@@ -808,39 +788,40 @@ static void _create_agent(void)
 
 static void _shutdown_agent(void)
 {
-	int i;
+	struct timespec ts = {0, 0};
+	int rc;
 
-	if (agent_tid) {
-		slurmdbd_shutdown = time(NULL);
-		for (i=0; i<50; i++) {	/* up to 5 secs total */
-			slurm_mutex_lock(&agent_lock);
-			if (!agent_running) {
-				slurm_mutex_unlock(&agent_lock);
-				goto fini;
-			}
-			slurm_cond_broadcast(&agent_cond);
-			slurm_mutex_unlock(&agent_lock);
+	if (!agent_tid)
+		return;
 
-			usleep(100000);	/* 0.1 sec per try */
-			if (pthread_kill(agent_tid, SIGUSR1))
-				break;
+	slurmdbd_shutdown = time(NULL);
+	slurm_mutex_lock(&agent_lock);
+	if (!agent_running) {
+		slurm_mutex_unlock(&agent_lock);
+		goto fini;
+	}
 
-		}
+	slurm_cond_broadcast(&agent_cond);
+	ts.tv_sec = time(NULL) + 5;
+	rc = pthread_cond_timedwait(&shutdown_cond, &agent_lock, &ts);
+	slurm_mutex_unlock(&agent_lock);
 
-		/* On rare occasions agent thread may not end quickly,
+	if (rc == ETIMEDOUT) {
+		/*
+		 * On rare occasions agent thread may not end quickly,
 		 * perhaps due to communication problems with slurmdbd.
 		 * Cancel it and join before returning or we could remove
-		 * and leave the agent without valid data */
-		if (pthread_kill(agent_tid, 0) == 0) {
-			error("agent failed to shutdown gracefully");
-			error("unable to save pending requests");
-			pthread_cancel(agent_tid);
-		}
+		 * and leave the agent without valid data.
+		 */
+		error("agent failed to shutdown gracefully");
+		error("unable to save pending requests");
+		/* FIXME: this is not safe! */
+		pthread_cancel(agent_tid);
+	}
 
 fini:
-		pthread_join(agent_tid,  NULL);
-		agent_tid = 0;
-	}
+	pthread_join(agent_tid,  NULL);
+	agent_tid = 0;
 }
 
 /****************************************************************************
@@ -891,12 +872,6 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 
 	xassert(req);
 	xassert(resp);
-	xassert(slurmdbd_conn);
-
-	if (req->conn && (req->conn != slurmdbd_conn))
-		error("We are overriding the connection!!!!!");
-
-	req->conn = slurmdbd_conn;
 
 	/*
 	 * To make sure we can get this to send instead of the agent
@@ -905,8 +880,20 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 	 */
 	halt_agent = 1;
 	slurm_mutex_lock(&slurmdbd_lock);
+
 	halt_agent = 0;
 
+	if (!slurmdbd_conn) {
+		slurm_cond_signal(&slurmdbd_cond);
+		slurm_mutex_unlock(&slurmdbd_lock);
+		return ESLURM_DB_CONNECTION_INVALID;
+	}
+
+	if (req->conn && (req->conn != slurmdbd_conn))
+		error("We are overriding the connection!!!!!");
+
+	req->conn = slurmdbd_conn;
+
 	rc = dbd_conn_send_recv_direct(rpc_version, req, resp);
 
 	slurm_cond_signal(&slurmdbd_cond);
@@ -942,7 +929,7 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 		_create_agent();
 		if ((agent_tid == 0) || (agent_list == NULL)) {
 			slurm_mutex_unlock(&agent_lock);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return SLURM_ERROR;
 		}
 	}
@@ -961,15 +948,14 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 	_max_dbd_msg_action(&cnt);
 
 	if (cnt < slurm_conf.max_dbd_msgs) {
-		if (list_enqueue(agent_list, buffer) == NULL)
-			fatal("list_enqueue: memory allocation failure");
+		list_enqueue(agent_list, buffer);
 	} else {
 		error("agent queue is full (%u), discarding %s:%u request",
 		      cnt,
 		      slurmdbd_msg_type_2_str(req->msg_type, 1),
 		      req->msg_type);
 		(slurmdbd_conn->trigger_callbacks.acct_full)();
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		rc = SLURM_ERROR;
 	}
 
