@@ -48,11 +48,11 @@
 
 #include "src/common/slurm_xlator.h"
 #include "src/common/assoc_mgr.h"
-#include "src/common/slurm_acct_gather_interconnect.h"
+#include "src/interfaces/acct_gather_interconnect.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
-#include "src/slurmd/common/proctrack.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/proctrack.h"
+#include "src/interfaces/acct_gather_profile.h"
 
 #include "src/slurmd/slurmd/slurmd.h"
 #include "acct_gather_interconnect_ofed.h"
