@@ -42,34 +42,29 @@
 
 #include "src/common/slurm_xlator.h"
 
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 #include "src/common/list.h"
-#include "src/common/cgroup.h"
+#include "src/interfaces/cgroup.h"
 
-/*
- * Common validation for what was read in from the gres.conf.
- * IN gres_conf_list
- * IN gres_name
- * IN config
- * OUT gres_devices
- */
-extern int common_node_config_load(List gres_conf_list, char *gres_name,
-				   node_config_load_t *config,
-				   List *gres_devices);
+typedef struct {
+	bitstr_t *bit_alloc;
+	char ***env_ptr;
+	gres_internal_flags_t flags;
+	int global_id;
+	char *global_list;
+	uint64_t gres_cnt;
+	uint32_t gres_conf_flags;
+	List gres_devices;
+	bool is_job;
+	bool is_task;
+	char *local_list;
+	char *prefix;
+	bitstr_t *usable_gres;
+	bool use_dev_num;
+} common_gres_env_t;
 
 /* set the environment for a job/step with the appropriate values */
-extern void common_gres_set_env(List gres_devices, char ***env_ptr,
-				bitstr_t *usable_gres, char *prefix,
-				int *local_inx, bitstr_t *bit_alloc,
-				char **local_list, char **global_list,
-				bool reset, bool is_job, int *global_id,
-				gres_internal_flags_t flags, bool use_dev_num);
-
-/* Send GRES information to slurmstepd via a buffer */
-extern void common_send_stepd(buf_t *buffer, List gres_devices);
-
-/* Receive GRES information from slurmd via a buffer */
-extern void common_recv_stepd(buf_t *buffer, List *gres_devices);
+extern void common_gres_set_env(common_gres_env_t *gres_env);
 
 /*
  * A one-liner version of _print_gres_conf_full()
@@ -90,12 +85,7 @@ extern void print_gres_list_parsable(List gres_list);
 /*
  * Set the appropriate env variables for all gpu like gres.
  */
-extern void gres_common_gpu_set_env(char ***env_ptr, bitstr_t *gres_bit_alloc,
-				    bitstr_t *usable_gres, bool *already_seen,
-				    int *local_inx, bool is_task, bool is_job,
-				    gres_internal_flags_t flags,
-				    uint32_t gres_conf_flags,
-				    List gres_devices);
+extern void gres_common_gpu_set_env(common_gres_env_t *gres_env);
 
 /*
  * Set environment variables as appropriate for a job's prolog or epilog based
@@ -103,10 +93,10 @@ extern void gres_common_gpu_set_env(char ***env_ptr, bitstr_t *gres_bit_alloc,
  *
  * RETURN: 1 if nothing was done, 0 otherwise.
  */
-extern bool gres_common_epilog_set_env(char ***epilog_env_ptr,
-				       gres_epilog_info_t *gres_ei,
-				       int node_inx, uint32_t gres_conf_flags,
-				       List gres_devices);
+extern bool gres_common_prep_set_env(char ***prep_env_ptr,
+				     gres_prep_t *gres_prep,
+				     int node_inx, uint32_t gres_conf_flags,
+				     List gres_devices);
 
 extern int gres_common_set_env_types_on_node_flags(void *x, void *arg);
 
