@@ -47,7 +47,6 @@
 #include <sched.h>
 #include <fcntl.h>
 #include <sys/mount.h>
-#include <linux/limits.h>
 #include <semaphore.h>
 
 #include "src/common/slurm_xlator.h"
@@ -80,12 +79,11 @@ const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
 
 static slurm_jc_conf_t *jc_conf = NULL;
 static int step_ns_fd = -1;
+static bool plugin_disabled = false;
 
 static void _create_paths(uint32_t job_id, char **job_mount, char **ns_holder,
 			  char **src_bind)
 {
-	jc_conf = get_slurm_jc_conf();
-	xassert(jc_conf);
 	xassert(job_mount);
 
 	xstrfmtcat(*job_mount, "%s/%u", jc_conf->basepath, job_id);
@@ -102,6 +100,11 @@ static int _find_step_in_list(step_loc_t *stepd, uint32_t *job_id)
 	return (stepd->step_id.job_id == *job_id);
 }
 
+static bool _is_plugin_disabled(char *basepath)
+{
+	return ((!basepath) || (!xstrncasecmp(basepath, "none", 4)));
+}
+
 static int _restore_ns(List steps, const char *d_name)
 {
 	char *endptr;
@@ -117,7 +120,7 @@ static int _restore_ns(List steps, const char *d_name)
 	}
 
 	/* here we think this is a job container */
-	debug3("determine if job %lu is still running", job_id);
+	log_flag(JOB_CONT, "determine if job %lu is still running", job_id);
 	stepd = list_find_first(steps, (ListFindF)_find_step_in_list, &job_id);
 	if (!stepd) {
 		debug("%s: Job %lu not found, deleting the namespace",
@@ -149,20 +152,17 @@ extern void container_p_reconfig(void)
  */
 extern int init(void)
 {
-#if defined(__APPLE__) || defined(__FreeBSD__)
-	fatal("%s is not available on this system. (mount bind limitation)",
-	      plugin_name);
-#endif
 	if (running_in_slurmd()) {
 		/*
 		 * Only init the config here for the slurmd. It will be sent by
 		 * the slurmd to the slurmstepd at launch time.
 		 */
-		if (!init_slurm_jc_conf()) {
+		if (!(jc_conf = init_slurm_jc_conf())) {
 			error("%s: Configuration not read correctly: Does '%s' not exist?",
 			      plugin_type, tmpfs_conf_file);
 			return SLURM_ERROR;
 		}
+		plugin_disabled = _is_plugin_disabled(jc_conf->basepath);
 		debug("job_container.conf read successfully");
 	}
 
@@ -206,8 +206,8 @@ extern int container_p_restore(char *dir_name, bool recover)
 	return SLURM_SUCCESS;
 #endif
 
-	jc_conf = get_slurm_jc_conf();
-	xassert(jc_conf);
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
 
 	if (jc_conf->auto_basepath) {
 		int fstatus;
@@ -270,7 +270,6 @@ static int _mount_private_dirs(char *path, uid_t uid)
 		      __func__);
 		return -1;
 	}
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 	buffer = xstrdup(jc_conf->dirs);
 	token = strtok_r(buffer, ",", &save_ptr);
 	while (token) {
@@ -304,7 +303,6 @@ static int _mount_private_dirs(char *path, uid_t uid)
 		token = strtok_r(NULL, ",", &save_ptr);
 		xfree(mount_path);
 	}
-#endif
 
 private_mounts_exit:
 	xfree(buffer);
@@ -323,7 +321,6 @@ static int _mount_private_shm(void)
 	if (!((loc[8] == ',') || (loc[8] == 0)))
 		return rc;
 
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 	/* handle mounting a new /dev/shm */
 	if (!jc_conf->shared) {
 		/*
@@ -341,7 +338,6 @@ static int _mount_private_shm(void)
 		error("%s: /dev/shm mount failed: %m", __func__);
 		return -1;
 	}
-#endif
 	return rc;
 }
 
@@ -365,8 +361,8 @@ static int _clean_job_basepath(uint32_t job_id)
 				   jc_conf->basepath, ep->d_name);
 			/* it is not important if this fails */
 			if (umount2(path, MNT_DETACH))
-				debug2("failed to unmount %s for job %u",
-				       path, job_id);
+				log_flag(JOB_CONT, "failed to unmount %s for job %u",
+					 path, job_id);
 			xfree(path);
 		}
 	}
@@ -381,7 +377,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 	char *result = NULL;
 	int fd;
 	int rc = 0;
-	bool user_name_set = 0;
 	sem_t *sem1 = NULL;
 	sem_t *sem2 = NULL;
 	pid_t cpid;
@@ -398,7 +393,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		goto end_it;
 	}
 
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 	/*
 	 * MS_BIND mountflag would make mount() ignore all other mountflags
 	 * except MS_REC. We need MS_PRIVATE mountflag as well to make the
@@ -416,7 +410,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		rc = SLURM_ERROR;
 		goto end_it;
 	}
-#endif
 
 	fd = open(ns_holder, O_CREAT|O_RDWR, S_IRWXU);
 	if (fd == -1) {
@@ -451,15 +444,9 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_JOB_UID", "%u",
 					step->uid);
-		if (!step->user_name) {
-			step->user_name = uid_to_string(step->uid);
-			user_name_set = true;
-		}
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_JOB_USER", "%s",
 					step->user_name);
-		if (user_name_set)
-			xfree(step->user_name);
 		if (step->cwd)
 			env_array_overwrite_fmt(&run_command_args.env,
 						"SLURM_JOB_WORK_DIR", "%s",
@@ -467,9 +454,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURM_CONF", "%s",
 					slurm_conf.slurm_conf);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_NODE_ALIASES", "%s",
-					step->alias_list);
 		env_array_overwrite_fmt(&run_command_args.env,
 					"SLURMD_NODENAME", "%s",
 					conf->node_name);
@@ -481,7 +465,7 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 			xfree(result);
 			goto exit2;
 		} else {
-			debug3("initscript stdout: %s", result);
+			log_flag(JOB_CONT, "initscript stdout: %s", result);
 		}
 		xfree(result);
 	}
@@ -545,7 +529,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 			rc = -1;
 			goto child_exit;
 		}
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 		if (!jc_conf->shared) {
 			/* Set root filesystem to private */
 			if (mount(NULL, "/", NULL, MS_PRIVATE|MS_REC, NULL)) {
@@ -570,7 +553,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 				goto child_exit;
 			}
 		}
-#endif
 
 		/*
 		 * Now we have a persistent mount namespace.
@@ -636,7 +618,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		 * Bind mount /proc/pid/ns/mnt to hold namespace active
 		 * without a process attached to it
 		 */
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
 		rc = mount(proc_path, ns_holder, NULL, MS_BIND, NULL);
 		xfree(proc_path);
 		if (rc) {
@@ -646,7 +627,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 				      __func__);
 			goto exit1;
 		}
-#endif
 		if (sem_post(sem2) < 0) {
 			error("%s: sem_post failed: %m", __func__);
 			goto exit1;
@@ -677,8 +657,11 @@ exit2:
 			rc = SLURM_ERROR;
 			goto end_it;
 		}
-		umount2(job_mount, MNT_DETACH);
-		rmdir(job_mount);
+		if (umount2(job_mount, MNT_DETACH))
+			error("%s: umount2 %s failed: %m",
+			      __func__, job_mount);
+		if (rmdir(job_mount))
+			error("rmdir %s failed: %m", job_mount);
 	}
 
 end_it:
@@ -698,6 +681,9 @@ extern int container_p_join_external(uint32_t job_id)
 {
 	char *job_mount = NULL, *ns_holder = NULL;
 
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
 	_create_paths(job_id, &job_mount, &ns_holder, NULL);
 
 	if (step_ns_fd == -1) {
@@ -727,6 +713,9 @@ extern int container_p_join(uint32_t job_id, uid_t uid)
 	return SLURM_SUCCESS;
 #endif
 
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
 	/*
 	 * Jobid 0 means we are not a real job, but a script running instead we
 	 * do not need to handle this request.
@@ -754,7 +743,7 @@ extern int container_p_join(uint32_t job_id, uid_t uid)
 		xfree(ns_holder);
 		return SLURM_ERROR;
 	} else {
-		debug3("job entered namespace");
+		log_flag(JOB_CONT, "job %u entered namespace", job_id);
 	}
 
 	close(fd);
@@ -784,8 +773,9 @@ static int _delete_ns(uint32_t job_id)
 	 */
 	if (step_ns_fd != -1) {
 		if (close(step_ns_fd))
-			log_flag(JOB_CONT, "close step_ns_fd(%d) failed: %m",
-				 step_ns_fd);
+			log_flag(JOB_CONT, "job %u close step_ns_fd(%d) failed: %m",
+				 job_id, step_ns_fd);
+
 		else
 			step_ns_fd = -1;
 	}
@@ -798,8 +788,8 @@ static int _delete_ns(uint32_t job_id)
 	rc = umount2(ns_holder, MNT_DETACH);
 	if (rc) {
 		if ((errno == EINVAL) || (errno == ENOENT)) {
-			debug2("%s: umount2 %s failed: %m",
-			       __func__, ns_holder);
+			log_flag(JOB_CONT, "%s: umount2 %s failed: %m",
+				 __func__, ns_holder);
 		} else {
 			error("%s: umount2 %s failed: %m",
 			      __func__, ns_holder);
@@ -813,8 +803,9 @@ static int _delete_ns(uint32_t job_id)
 		error("%s: failed to remove %d files from %s",
 		      __func__, failures, job_mount);
 	if (umount2(job_mount, MNT_DETACH))
-		debug2("umount2: %s failed: %m", job_mount);
-	rmdir(job_mount);
+		log_flag(JOB_CONT, "umount2: %s failed: %m", job_mount);
+	if (rmdir(job_mount))
+		error("rmdir %s failed: %m", job_mount);
 
 	xfree(job_mount);
 	xfree(ns_holder);
@@ -829,11 +820,17 @@ extern int container_p_delete(uint32_t job_id)
 
 extern int container_p_stepd_create(uint32_t job_id, stepd_step_rec_t *step)
 {
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
 	return _create_ns(job_id, step);
 }
 
 extern int container_p_stepd_delete(uint32_t job_id)
 {
+	if (plugin_disabled)
+		return SLURM_SUCCESS;
+
 	return _delete_ns(job_id);
 }
 
@@ -867,9 +864,11 @@ extern int container_p_recv_stepd(int fd)
 	buf = init_buf(len);
 	safe_read(fd, buf->head, len);
 
-	if(!set_slurm_jc_conf(buf))
+	if (!(jc_conf = set_slurm_jc_conf(buf)))
 		goto rwfail;
 
+	plugin_disabled = _is_plugin_disabled(jc_conf->basepath);
+
 	return SLURM_SUCCESS;
 rwfail:
 	error("%s: failed", __func__);
