@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  mpi_cray_shasta.c - Cray Shasta MPI plugin
  *****************************************************************************
- *  Copyright 2019 Hewlett Packard Enterprise Development LP
+ *  Copyright 2019,2022-2023 Hewlett Packard Enterprise Development LP
  *  Written by David Gloe <dgloe@cray.com>
  *
  *  This file is part of Slurm, a resource management program.
@@ -34,9 +34,15 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+#include "config.h"
+
 #include <fcntl.h>
+#include <inttypes.h>
 #include <limits.h>
 #include <signal.h>
+#ifdef HAVE_GETRANDOM
+#include <sys/random.h>
+#endif
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -45,9 +51,10 @@
 #include "src/common/slurm_xlator.h"
 
 #include "src/common/env.h"
+#include "src/common/fd.h"
 #include "src/common/parse_config.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_mpi.h"
+#include "src/interfaces/mpi.h"
 #include "src/common/xstring.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
@@ -82,16 +89,22 @@ const char plugin_type[] = "mpi/cray_shasta";
 const uint32_t plugin_id = MPI_PLUGIN_CRAY_SHASTA;
 const uint32_t plugin_version = SLURM_VERSION_NUMBER;
 
-/* Name of the directory to store Cray MPI data */
-#define MPI_CRAY_DIR "mpi_cray_shasta"
-
 /* Environment variables available for applications */
 #define PALS_APID_ENV "PALS_APID"
 #define PALS_APINFO_ENV "PALS_APINFO"
-#define PALS_RANKID_ENV "PALS_RANKID"
+#define PALS_LOCAL_RANKID_ENV "PALS_LOCAL_RANKID"
 #define PALS_NODEID_ENV "PALS_NODEID"
+#define PALS_RANKID_ENV "PALS_RANKID"
 #define PALS_SPOOL_DIR_ENV "PALS_SPOOL_DIR"
 
+#define PMI_JOBID_ENV "PMI_JOBID"
+#define PMI_LOCAL_RANK_ENV "PMI_LOCAL_RANK"
+#define PMI_LOCAL_SIZE_ENV "PMI_LOCAL_SIZE"
+#define PMI_RANK_ENV "PMI_RANK"
+#define PMI_SIZE_ENV "PMI_SIZE"
+#define PMI_UNIVERSE_SIZE_ENV "PMI_UNIVERSE_SIZE"
+#define PMI_SHARED_SECRET_ENV "PMI_SHARED_SECRET"
+
 /* GLOBAL vars */
 char *appdir = NULL; // Application-specific spool directory
 char *apinfo = NULL; // Application PMI file
@@ -118,13 +131,13 @@ static int _create_mpi_dir(const char *spool)
 /*
  * Create the application-specific directory under the Cray MPI directory
  */
-static int _create_app_dir(const stepd_step_rec_t *job, const char *spool)
+static int _create_app_dir(const stepd_step_rec_t *step, const char *spool)
 {
 	xfree(appdir);
 	// Format the directory name
 	appdir = xstrdup_printf("%s/%s/%u.%u",
 				spool, MPI_CRAY_DIR,
-				job->step_id.job_id, job->step_id.step_id);
+				step->step_id.job_id, step->step_id.step_id);
 
 	// Create the directory
 	if ((mkdir(appdir, 0700) == -1) && (errno != EEXIST)) {
@@ -134,7 +147,7 @@ static int _create_app_dir(const stepd_step_rec_t *job, const char *spool)
 	}
 
 	// Change directory owner
-	if (chown(appdir, job->uid, job->gid) == -1) {
+	if (chown(appdir, step->uid, step->gid) == -1) {
 		error("%s: Couldn't change directory %s owner: %m",
 		      plugin_type, appdir);
 		goto error;
@@ -175,74 +188,17 @@ static void _set_pmi_port(char ***env)
 	env_array_overwrite_fmt(env, "PMI_CONTROL_PORT", "%lu", pmi_port);
 }
 
-/*
- * Determine whether the given path is a directory
- */
-static int _is_dir(char *path)
-{
-	struct stat stat_buf;
-
-	if (stat(path, &stat_buf)) {
-		error("%s: Cannot stat %s: %m", plugin_type, path);
-		return 1;
-	} else if (!S_ISDIR(stat_buf.st_mode)) {
-		return 0;
-	}
-	return 1;
-}
-
-/*
- * Recursively remove a directory
- */
-static int _rmdir_recursive(char *path)
-{
-	char nested_path[PATH_MAX];
-	DIR *dp;
-	struct dirent *ent;
-
-	if (!(dp = opendir(path))) {
-		error("%s: Can't open directory %s: %m", plugin_type, path);
-		return SLURM_ERROR;
-	}
-
-	while ((ent = readdir(dp))) {
-		if (!xstrcmp(ent->d_name, ".") ||
-		    !xstrcmp(ent->d_name, "..")) {
-			/* skip special dir's */
-			continue;
-		}
-		snprintf(nested_path, sizeof(nested_path), "%s/%s", path,
-			 ent->d_name);
-		if (_is_dir(nested_path)) {
-			_rmdir_recursive(nested_path);
-		} else {
-			debug("%s: Removed file %s", plugin_type, nested_path);
-			unlink(nested_path);
-		}
-	}
-	closedir(dp);
-
-	if (rmdir(path) == -1) {
-		error("%s: Can't remove directory %s: %m",
-		      plugin_type, path);
-		return SLURM_ERROR;
-	}
-
-	debug("%s: Removed directory %s", plugin_type, path);
-	return SLURM_SUCCESS;
-}
-
-extern int mpi_p_slurmstepd_prefork(const stepd_step_rec_t *job, char ***env)
+extern int mpi_p_slurmstepd_prefork(const stepd_step_rec_t *step, char ***env)
 {
 	/* do the node_name substitution once */
 	char *spool = slurm_conf_expand_slurmd_path(slurm_conf.slurmd_spooldir,
-						    job->node_name,
-						    job->node_name);
+						    step->node_name,
+						    step->node_name);
 
 	// Set up spool directory and apinfo
 	if (_create_mpi_dir(spool) == SLURM_ERROR ||
-	    _create_app_dir(job, spool) == SLURM_ERROR ||
-	    create_apinfo(job) == SLURM_ERROR) {
+	    _create_app_dir(step, spool) == SLURM_ERROR ||
+	    create_apinfo(step, spool) == SLURM_ERROR) {
 		xfree(spool);
 		return SLURM_ERROR;
 	}
@@ -252,15 +208,29 @@ extern int mpi_p_slurmstepd_prefork(const stepd_step_rec_t *job, char ***env)
 	return SLURM_SUCCESS;
 }
 
-extern int mpi_p_slurmstepd_task(const mpi_plugin_task_info_t *job, char ***env)
+extern int mpi_p_slurmstepd_task(const mpi_task_info_t *mpi_task, char ***env)
 {
 	// Set environment variables
 	env_array_overwrite_fmt(env, PALS_APID_ENV, "%u.%u",
-				job->step_id.job_id, job->step_id.step_id);
-	env_array_overwrite_fmt(env, PALS_RANKID_ENV, "%u", job->gtaskid);
-	env_array_overwrite_fmt(env, PALS_NODEID_ENV, "%u", job->nodeid);
-	env_array_overwrite_fmt(env, PALS_SPOOL_DIR_ENV, "%s", appdir);
+				mpi_task->step_id.job_id,
+				mpi_task->step_id.step_id);
 	env_array_overwrite_fmt(env, PALS_APINFO_ENV, "%s", apinfo);
+	env_array_overwrite_fmt(env, PALS_LOCAL_RANKID_ENV, "%u",
+				mpi_task->ltaskid);
+	env_array_overwrite_fmt(env, PALS_NODEID_ENV, "%u", mpi_task->nodeid);
+	env_array_overwrite_fmt(env, PALS_RANKID_ENV, "%u", mpi_task->gtaskid);
+	env_array_overwrite_fmt(env, PALS_SPOOL_DIR_ENV, "%s", appdir);
+
+	env_array_overwrite_fmt(env, PMI_JOBID_ENV, "%u",
+				mpi_task->step_id.job_id);
+	env_array_overwrite_fmt(env, PMI_LOCAL_RANK_ENV, "%u",
+				mpi_task->ltaskid);
+	env_array_overwrite_fmt(env, PMI_LOCAL_SIZE_ENV, "%u",
+				mpi_task->ltasks);
+	env_array_overwrite_fmt(env, PMI_RANK_ENV, "%u", mpi_task->gtaskid);
+	env_array_overwrite_fmt(env, PMI_SIZE_ENV, "%u", mpi_task->ntasks);
+	env_array_overwrite_fmt(env, PMI_UNIVERSE_SIZE_ENV, "%u",
+				mpi_task->ntasks);
 
 	_set_pmi_port(env);
 
@@ -268,8 +238,35 @@ extern int mpi_p_slurmstepd_task(const mpi_plugin_task_info_t *job, char ***env)
 }
 
 extern mpi_plugin_client_state_t *
-mpi_p_client_prelaunch(const mpi_plugin_client_info_t *job, char ***env)
+mpi_p_client_prelaunch(const mpi_step_info_t *mpi_step, char ***env)
 {
+#ifdef HAVE_GETRANDOM
+	static uint64_t shared_secret = 0;
+	static pthread_mutex_t shared_secret_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+	slurm_mutex_lock(&shared_secret_mutex);
+
+	/*
+	 * Get a non-zero pseudo-random value. getrandom() is guaranteed to
+	 * return up to 256 bytes uninturrupted. The only error we might expect
+	 * here is ENOSYS, indicating that the kernel does not implement the
+	 * getrandom() system call. getrandom() should be present on all
+	 * supported cray systems.
+	 */
+
+	if (!shared_secret &&
+	    getrandom(&shared_secret, sizeof(shared_secret), 0) < 0) {
+		error("%s: getrandom() failed: %m", __func__);
+		slurm_mutex_unlock(&shared_secret_mutex);
+		return NULL;
+	}
+
+	/* Set PMI_SHARED_SECRET for PMI authentication */
+	env_array_overwrite_fmt(env, PMI_SHARED_SECRET_ENV, "%"PRIu64,
+				shared_secret);
+
+	slurm_mutex_unlock(&shared_secret_mutex);
+#endif
 	/* only return NULL on error */
 	return (void *)0xdeadbeef;
 }
@@ -291,7 +288,7 @@ extern int fini(void)
 {
 	// Remove application spool directory
 	if (appdir)
-		_rmdir_recursive(appdir);
+		rmdir_recursive(appdir, true);
 
 	// Free allocated storage
 	xfree(appdir);
