@@ -159,7 +159,7 @@ static int pmix_stepd_width = 16;
 static int pmix_stepd_children = 0;
 
 /* we allocate a hostlist in init and destroy it in finalize */
-static hostlist_t pmix_stepd_hostlist = NULL;
+static hostlist_t *pmix_stepd_hostlist = NULL;
 
 /* return rank of our parent in stepd tree,
  * returns -1 if we're the root */
@@ -232,7 +232,7 @@ static int pmix_stepd_send(const char* buf, uint32_t size, int rank)
 			/* cancel the step to avoid tasks hang */
 			slurm_kill_job_step(job_info.step_id.job_id,
 					    job_info.step_id.step_id,
-					    SIGKILL);
+					    SIGKILL, 0);
 		}
 
 		/* didn't succeeded, but we'll retry again,
@@ -339,9 +339,7 @@ int pmix_ring_finalize()
 	}
 
 	/* free host list */
-	if (pmix_stepd_hostlist != NULL) {
-		hostlist_destroy(pmix_stepd_hostlist);
-        }
+	FREE_NULL_HOSTLIST(pmix_stepd_hostlist);
 
 	return rc;
 }
