@@ -142,9 +142,9 @@ pmixp_coll_t *pmixp_coll_ring_from_cbdata(void *cbdata)
 
 int pmixp_coll_ring_unpack(buf_t *buf, pmixp_coll_type_t *type,
 			   pmixp_coll_ring_msg_hdr_t *ring_hdr,
-			   pmixp_proc_t **r, size_t *nr)
+			   pmix_proc_t **r, size_t *nr)
 {
-	pmixp_proc_t *procs = NULL;
+	pmix_proc_t *procs = NULL;
 	uint32_t nprocs = 0;
 	uint32_t tmp;
 	int rc, i;
@@ -164,14 +164,14 @@ int pmixp_coll_ring_unpack(buf_t *buf, pmixp_coll_type_t *type,
 	}
 	*nr = nprocs;
 
-	procs = xmalloc(sizeof(pmixp_proc_t) * nprocs);
+	procs = xmalloc(sizeof(pmix_proc_t) * nprocs);
 	*r = procs;
 
 	/* 3. get namespace/rank of particular process */
 	for (i = 0; i < (int)nprocs; i++) {
 		if ((rc = unpackmem_ptr(&temp_ptr, &tmp, buf)) ||
 		    (strlcpy(procs[i].nspace, temp_ptr,
-			     PMIXP_MAX_NSLEN + 1) > PMIXP_MAX_NSLEN)) {
+			     sizeof(procs[i].nspace)) > PMIX_MAX_NSLEN)) {
 			PMIXP_ERROR("Cannot unpack namespace for process #%d",
 				    i);
 			return rc;
@@ -202,7 +202,7 @@ static int _pack_coll_ring_info(pmixp_coll_t *coll,
 				pmixp_coll_ring_msg_hdr_t *ring_hdr,
 				buf_t *buf)
 {
-	pmixp_proc_t *procs = coll->pset.procs;
+	pmix_proc_t *procs = coll->pset.procs;
 	size_t nprocs = coll->pset.nprocs;
 	uint32_t type = PMIXP_COLL_TYPE_FENCE_RING;
 	int i;
@@ -475,7 +475,7 @@ pmixp_coll_ring_ctx_t *pmixp_coll_ring_ctx_select(pmixp_coll_t *coll,
 	return ret;
 }
 
-int pmixp_coll_ring_init(pmixp_coll_t *coll, hostlist_t *hl)
+int pmixp_coll_ring_init(pmixp_coll_t *coll, hostlist_t **hl)
 {
 #ifdef PMIXP_COLL_DEBUG
 	PMIXP_DEBUG("called");
@@ -519,8 +519,8 @@ void pmixp_coll_ring_free(pmixp_coll_ring_t *ring)
 		FREE_NULL_BUFFER(coll_ctx->ring_buf);
 		xfree(coll_ctx->contrib_map);
 	}
-	list_destroy(ring->fwrd_buf_pool);
-	list_destroy(ring->ring_buf_pool);
+	FREE_NULL_LIST(ring->fwrd_buf_pool);
+	FREE_NULL_LIST(ring->ring_buf_pool);
 }
 
 inline static int _pmixp_coll_contrib(pmixp_coll_ring_ctx_t *coll_ctx,
@@ -629,7 +629,7 @@ int pmixp_coll_ring_check(pmixp_coll_t *coll, pmixp_coll_ring_msg_hdr_t *hdr)
 			    hdr->seq, nodename, hdr->nodeid, coll->seq);
 		pmixp_debug_hang(0); /* enable hang to debug this! */
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 		xfree(nodename);
 		return SLURM_SUCCESS;
 	} else if (PMIXP_COLL_REQ_SKIP == rc) {
@@ -735,7 +735,7 @@ void pmixp_coll_ring_reset_if_to(pmixp_coll_t *coll, time_t ts) {
 		}
 		if (ts - coll->ts > pmixp_info_timeout()) {
 			/* respond to the libpmix */
-			pmixp_coll_localcb_nodata(coll, PMIXP_ERR_TIMEOUT);
+			pmixp_coll_localcb_nodata(coll, PMIX_ERR_TIMEOUT);
 
 			/* report the timeout event */
 			PMIXP_ERROR("%p: collective timeout seq=%d",
@@ -781,8 +781,8 @@ void pmixp_coll_ring_log(pmixp_coll_t *coll)
 		if (coll_ctx->in_use) {
 			int id;
 			char *done_contrib = NULL, *wait_contrib = NULL;
-			hostlist_t hl_done_contrib = NULL,
-				hl_wait_contrib = NULL, *tmp_list;
+			hostlist_t *hl_done_contrib = NULL,
+				*hl_wait_contrib = NULL, **tmp_list;
 
 			PMIXP_ERROR("\t seq=%d contribs: loc=%d/prev=%d/fwd=%d",
 				    coll_ctx->seq, coll_ctx->contrib_local,
