@@ -49,7 +49,7 @@ static void _reset_coll(pmixp_coll_t *coll);
 
 static int _pack_coll_info(pmixp_coll_t *coll, buf_t *buf)
 {
-	pmixp_proc_t *procs = coll->pset.procs;
+	pmix_proc_t *procs = coll->pset.procs;
 	size_t nprocs = coll->pset.nprocs;
 	uint32_t size;
 	int i;
@@ -70,9 +70,9 @@ static int _pack_coll_info(pmixp_coll_t *coll, buf_t *buf)
 }
 
 int pmixp_coll_tree_unpack(buf_t *buf, pmixp_coll_type_t *type,
-			   int *nodeid, pmixp_proc_t **r, size_t *nr)
+			   int *nodeid, pmix_proc_t **r, size_t *nr)
 {
-	pmixp_proc_t *procs = NULL;
+	pmix_proc_t *procs = NULL;
 	uint32_t nprocs = 0;
 	uint32_t tmp;
 	int i, rc;
@@ -92,14 +92,14 @@ int pmixp_coll_tree_unpack(buf_t *buf, pmixp_coll_type_t *type,
 	}
 	*nr = nprocs;
 
-	procs = xmalloc(sizeof(pmixp_proc_t) * nprocs);
+	procs = xmalloc(sizeof(pmix_proc_t) * nprocs);
 	*r = procs;
 
 	for (i = 0; i < (int)nprocs; i++) {
 		/* 3. get namespace/rank of particular process */
 		if ((rc = unpackmem_ptr(&temp_ptr, &tmp, buf)) ||
 		    (strlcpy(procs[i].nspace, temp_ptr,
-			     PMIXP_MAX_NSLEN + 1) > PMIXP_MAX_NSLEN)) {
+			     sizeof(procs[i].nspace)) > PMIX_MAX_NSLEN)) {
 			PMIXP_ERROR("Cannot unpack namespace for process #%d",
 				    i);
 			return rc;
@@ -190,14 +190,14 @@ static void _reset_coll(pmixp_coll_t *coll)
 		/* collective is spoiled, reset state */
 		tree->state = PMIXP_COLL_TREE_SYNC;
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 	}
 }
 
 /*
  * Based on ideas provided by Hongjia Cao <hjcao@nudt.edu.cn> in PMI2 plugin
  */
-int pmixp_coll_tree_init(pmixp_coll_t *coll, hostlist_t *hl)
+int pmixp_coll_tree_init(pmixp_coll_t *coll, hostlist_t **hl)
 {
 	int max_depth, width, depth, i;
 	char *p;
@@ -610,7 +610,7 @@ static int _progress_ufwd(pmixp_coll_t *coll)
 		/* collective is spoiled, reset state */
 		tree->state = PMIXP_COLL_TREE_SYNC;
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 		return false;
 	}
 
@@ -742,7 +742,7 @@ static int _progress_ufwd_sc(pmixp_coll_t *coll)
 		/* collective is spoiled, reset state */
 		tree->state = PMIXP_COLL_TREE_SYNC;
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 		return false;
 	}
 
@@ -836,7 +836,7 @@ static int _progress_dfwd(pmixp_coll_t *coll)
 		/* collective is spoiled, reset state */
 		tree->state = PMIXP_COLL_TREE_SYNC;
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 		return false;
 	}
 #ifdef PMIXP_COLL_DEBUG
@@ -944,7 +944,7 @@ int pmixp_coll_tree_local(pmixp_coll_t *coll, char *data, size_t size,
 		/* collective is spoiled, reset state */
 		tree->state = PMIXP_COLL_TREE_SYNC;
 		slurm_kill_job_step(pmixp_info_jobid(),
-				    pmixp_info_stepid(), SIGKILL);
+				    pmixp_info_stepid(), SIGKILL, 0);
 		ret = SLURM_ERROR;
 		goto exit;
 	}
@@ -1143,7 +1143,7 @@ error:
 	_reset_coll(coll);
 error2:
 	slurm_kill_job_step(pmixp_info_jobid(),
-			    pmixp_info_stepid(), SIGKILL);
+			    pmixp_info_stepid(), SIGKILL, 0);
 	/* unlock the structure */
 	slurm_mutex_unlock(&coll->lock);
 
@@ -1292,7 +1292,7 @@ error:
 	_reset_coll(coll);
 error2:
 	slurm_kill_job_step(pmixp_info_jobid(),
-			    pmixp_info_stepid(), SIGKILL);
+			    pmixp_info_stepid(), SIGKILL, 0);
 	slurm_mutex_unlock(&coll->lock);
 
 	return SLURM_ERROR;
@@ -1312,7 +1312,7 @@ void pmixp_coll_tree_reset_if_to(pmixp_coll_t *coll, time_t ts)
 
 	if (ts - coll->ts > pmixp_info_timeout()) {
 		/* respond to the libpmix */
-		pmixp_coll_localcb_nodata(coll, PMIXP_ERR_TIMEOUT);
+		pmixp_coll_localcb_nodata(coll, PMIX_ERR_TIMEOUT);
 
 		/* report the timeout event */
 		PMIXP_ERROR("%p: collective timeout seq=%d", coll, coll->seq);
@@ -1350,8 +1350,8 @@ void pmixp_coll_tree_log(pmixp_coll_t *coll)
 	}
 	if (tree->chldrn_cnt) {
 		char *done_contrib = NULL, *wait_contrib = NULL;
-		hostlist_t hl_done_contrib = NULL,
-			hl_wait_contrib = NULL, *tmp_list;
+		hostlist_t *hl_done_contrib = NULL,
+			*hl_wait_contrib = NULL, **tmp_list;
 
 		PMIXP_ERROR("child contribs [%d]:", tree->chldrn_cnt);
 		for (i = 0; i < tree->chldrn_cnt; i++) {
