@@ -43,14 +43,14 @@
 #define _GNU_SOURCE	/* For POLLRDHUP */
 #include <fcntl.h>
 #include <poll.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
-#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
+#ifndef POLLRDHUP
 #define POLLRDHUP POLLHUP
-#include <signal.h>
 #endif
 
 #include "slurm/slurm.h"
