@@ -47,8 +47,10 @@
 #include "src/common/log.h"
 #include "src/common/plugin.h"
 #include "src/common/xstring.h"
-#include "src/common/slurm_accounting_storage.h"
-#include "src/slurmctld/preempt.h"
+
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/preempt.h"
+
 #include "src/slurmctld/slurmctld.h"
 #include "src/slurmctld/job_scheduler.h"
 #include "src/slurmctld/acct_policy.h"
