@@ -44,15 +44,15 @@
 #include "src/common/fd.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
-#include "src/common/plugstack.h"
-#include "src/common/prep.h"
+#include "src/interfaces/prep.h"
 #include "src/common/run_command.h"
+#include "src/common/spank.h"
 #include "src/common/track_script.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-#include "src/slurmd/common/job_container_plugin.h"
+#include "src/interfaces/job_container.h"
 #include "src/slurmd/slurmd/req.h"
 
 #include "prep_script.h"
@@ -65,8 +65,8 @@ slurmd_conf_t *conf = NULL;
 
 static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 			 bool is_epilog);
-static int _run_spank_job_script(const char *mode, char **env, uint32_t job_id);
-
+static int _run_spank_job_script(const char *mode, char **env, uint32_t job_id,
+				 int (*container_join)(uint32_t , uid_t));
 
 static int _ef(const char *p, int errnum)
 {
@@ -172,7 +172,8 @@ extern int slurmd_script(job_env_t *job_env, slurm_cred_t *cred,
 	    (!is_epilog && spank_has_prolog())) {
 		if (!env)
 			env = _build_env(job_env, cred, is_epilog);
-		rc = _run_spank_job_script(name, env, jobid);
+		rc = _run_spank_job_script(name, env, jobid,
+					   job_env->container_join);
 	}
 
 	if (path) {
@@ -219,7 +220,7 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 			 bool is_epilog)
 {
 	char **env = env_array_create();
-	bool user_name_set = 0;
+	bool user_set = false;
 
 	env[0] = NULL;
 	if (!valid_spank_job_env(job_env->spank_job_env,
@@ -247,19 +248,6 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 	setenvf(&env, "SLURM_JOB_UID", "%u", job_env->uid);
 	setenvf(&env, "SLURM_JOB_GID", "%u", job_env->gid);
 	setenvf(&env, "SLURM_JOB_WORK_DIR", "%s", job_env->work_dir);
-
-#ifndef HAVE_NATIVE_CRAY
-	/* uid_to_string on a cray is a heavy call, so try to avoid it */
-	if (!job_env->user_name) {
-		job_env->user_name = uid_to_string(job_env->uid);
-		user_name_set = true;
-	}
-#endif
-
-	setenvf(&env, "SLURM_JOB_USER", "%s", job_env->user_name);
-	if (user_name_set)
-		xfree(job_env->user_name);
-
 	setenvf(&env, "SLURM_JOBID", "%u", job_env->jobid);
 
 	if (job_env->het_job_id && (job_env->het_job_id != NO_VAL)) {
@@ -270,19 +258,10 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 
 	setenvf(&env, "SLURM_UID", "%u", job_env->uid);
 
-	if (job_env->node_aliases)
-		setenvf(&env, "SLURM_NODE_ALIASES", "%s",
-			job_env->node_aliases);
-
-	if (job_env->node_list)
+	if (job_env->node_list) {
 		setenvf(&env, "SLURM_NODELIST", "%s", job_env->node_list);
-
-	/*
-	 * Overridden by the credential version if available.
-	 * Remove two versions after 22.05.
-	 */
-	if (job_env->partition)
-		setenvf(&env, "SLURM_JOB_PARTITION", "%s", job_env->partition);
+		setenvf(&env, "SLURM_JOB_NODELIST", "%s", job_env->node_list);
+	}
 
 	if (is_epilog)
 		setenvf(&env, "SLURM_SCRIPT_CONTEXT", "epilog_slurmd");
@@ -309,9 +288,27 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 		if (cred_arg->job_comment)
 			setenvf(&env, "SLURM_JOB_COMMENT", "%s",
 				cred_arg->job_comment);
+		if (cred_arg->job_core_spec == NO_VAL16) {
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_COUNT", "0");
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_TYPE", "cores");
+		} else if (cred_arg->job_core_spec & CORE_SPEC_THREAD) {
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_COUNT", "%u",
+				cred_arg->job_core_spec & (~CORE_SPEC_THREAD));
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_TYPE", "threads");
+		} else {
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_COUNT", "%u",
+				cred_arg->job_core_spec);
+			setenvf(&env, "SLURM_JOB_CORE_SPEC_TYPE", "cores");
+		}
 		if (cred_arg->job_constraints)
 			setenvf(&env, "SLURM_JOB_CONSTRAINTS", "%s",
 				cred_arg->job_constraints);
+		if (cred_arg->job_end_time)
+			setenvf(&env, "SLURM_JOB_END_TIME", "%lu",
+				cred_arg->job_end_time);
+		if (cred_arg->job_extra)
+			setenvf(&env, "SLURM_JOB_EXTRA", "%s",
+				cred_arg->job_extra);
 		if (cred_arg->cpu_array_count) {
 			char *tmp = uint32_compressed_to_str(
 				cred_arg->cpu_array_count,
@@ -320,12 +317,17 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 			setenvf(&env, "SLURM_JOB_CPUS_PER_NODE", "%s", tmp);
 			xfree(tmp);
 		}
+		if (cred_arg->job_licenses)
+			setenvf(&env, "SLURM_JOB_LICENSES", "%s",
+				cred_arg->job_licenses);
 		if (cred_arg->job_ntasks)
 			setenvf(&env, "SLURM_JOB_NTASKS", "%u",
 				cred_arg->job_ntasks);
 		if (cred_arg->job_nhosts)
 			setenvf(&env, "SLURM_JOB_NUM_NODES", "%u",
 				cred_arg->job_nhosts);
+		setenvf(&env, "SLURM_JOB_OVERSUBSCRIBE", "%s",
+			job_share_string(cred_arg->job_oversubscribe));
 		if (cred_arg->job_partition)
 			setenvf(&env, "SLURM_JOB_PARTITION", "%s",
 				cred_arg->job_partition);
@@ -335,6 +337,9 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 		if (cred_arg->job_restart_cnt != INFINITE16)
 			setenvf(&env, "SLURM_JOB_RESTART_COUNT", "%u",
 				cred_arg->job_restart_cnt);
+		if (cred_arg->job_start_time)
+			setenvf(&env, "SLURM_JOB_START_TIME", "%lu",
+				cred_arg->job_start_time);
 		if (cred_arg->job_std_err)
 			setenvf(&env, "SLURM_JOB_STDERR", "%s",
 				cred_arg->job_std_err);
@@ -344,14 +349,25 @@ static char **_build_env(job_env_t *job_env, slurm_cred_t *cred,
 		if (cred_arg->job_std_out)
 			setenvf(&env, "SLURM_JOB_STDOUT", "%s",
 				cred_arg->job_std_out);
-
+		if (cred_arg->id->pw_name) {
+			user_set = true;
+			setenvf(&env, "SLURM_JOB_USER", "%s",
+				cred_arg->id->pw_name);
+		}
 		slurm_cred_unlock_args(cred);
 	}
 
+	if (!user_set) {
+		char *user_name = uid_to_string(job_env->uid);
+		setenvf(&env, "SLURM_JOB_USER", "%s", user_name);
+		xfree(user_name);
+	}
+
 	return env;
 }
 
-static int _run_spank_job_script(const char *mode, char **env, uint32_t job_id)
+static int _run_spank_job_script(const char *mode, char **env, uint32_t job_id,
+				 int (*container_join)(uint32_t , uid_t))
 {
 	pid_t cpid;
 	int status = 0, timeout;
@@ -384,7 +400,8 @@ static int _run_spank_job_script(const char *mode, char **env, uint32_t job_id)
 		 * to avoid a race condition if this process makes a file
 		 * before we add the pid to the container in the parent.
 		 */
-		if (container_g_join(job_id, getuid()) != SLURM_SUCCESS)
+		if (container_join &&
+		    (container_join(job_id, getuid()) != SLURM_SUCCESS))
 			error("container_g_join(%u): %m", job_id);
 
 		if (dup2(pfds[0], STDIN_FILENO) < 0)
