@@ -51,14 +51,14 @@
 #include "other_select.h"
 #include "src/common/plugin.h"
 #include "src/common/plugrack.h"
+#include "src/interfaces/select.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/xstring.h"
-#include "src/common/node_select.h"
 
 uint16_t other_select_type_param = 0;
 
 /*
- * Must be synchronized with slurm_select_ops_t in node_select.h.
+ * Must be synchronized with slurm_select_ops_t in select.h.
  */
 const char *node_select_syms[] = {
 	"plugin_id",
@@ -71,7 +71,6 @@ const char *node_select_syms[] = {
 	"select_p_job_ready",
 	"select_p_job_expand",
 	"select_p_job_resized",
-	"select_p_job_signal",
 	"select_p_job_fini",
 	"select_p_job_suspend",
 	"select_p_job_resume",
@@ -92,12 +91,8 @@ const char *node_select_syms[] = {
 	"select_p_select_jobinfo_copy",
 	"select_p_select_jobinfo_pack",
 	"select_p_select_jobinfo_unpack",
-	"select_p_select_jobinfo_sprint",
-	"select_p_select_jobinfo_xstrdup",
 	"select_p_get_info_from_plugin",
-	"select_p_update_node_config",
 	"select_p_reconfigure",
-	"select_p_resv_test",
 };
 
 static slurm_select_ops_t ops;
@@ -126,9 +121,7 @@ extern int other_select_init(void)
 	if (!other_select_type_param)
 		other_select_type_param = slurm_conf.select_type_param;
 
-	if (other_select_type_param & CR_OTHER_CONS_RES)
-		type = "select/cons_res";
-	else if (other_select_type_param & CR_OTHER_CONS_TRES)
+	if (other_select_type_param & CR_OTHER_CONS_TRES)
 		type = "select/cons_tres";
 	else
 		type = "select/linear";
@@ -137,7 +130,7 @@ extern int other_select_init(void)
 	if (n_syms != sizeof(ops))
 		fatal("For some reason node_select_syms in "
 		      "src/plugins/select/other/other_select.c differs from "
-		      "slurm_select_ops_t found in src/common/node_select.h.  "
+		      "slurm_select_ops_t found in src/interfaces/select.h.  "
 		      "node_select_syms should match what is in "
 		      "src/common/node_select.c");
 
@@ -213,12 +206,12 @@ extern int other_job_init(List job_list)
  * IN node_ptr - current node data
  * IN node_count - number of node entries
  */
-extern int other_node_init(node_record_t *node_ptr, int node_cnt)
+extern int other_node_init(void)
 {
 	if (other_select_init() < 0)
 		return SLURM_ERROR;
 
-	return (*(ops.node_init))(node_ptr, node_cnt);
+	return (*(ops.node_init))();
 }
 
 /*
@@ -238,14 +231,14 @@ extern int other_node_init(node_record_t *node_ptr, int node_cnt)
  *		jobs to be preempted to initiate the pending job. Not set
  *		if mode=SELECT_MODE_TEST_ONLY or input pointer is NULL.
  *		Existing list is appended to.
- * IN exc_core_bitmap - bitmap of cores being reserved.
+ * IN resv_exc_ptr - Various TRES which the job can NOT use.
  * RET zero on success, EINVAL otherwise
  */
 extern int other_job_test(job_record_t *job_ptr, bitstr_t *bitmap,
 			  uint32_t min_nodes, uint32_t max_nodes,
 			  uint32_t req_nodes, uint16_t mode,
 			  List preemptee_candidates, List *preemptee_job_list,
-			  bitstr_t *exc_core_bitmap)
+			  resv_exc_t *resv_exc_ptr)
 {
 	if (other_select_init() < 0)
 		return SLURM_ERROR;
@@ -255,7 +248,7 @@ extern int other_job_test(job_record_t *job_ptr, bitstr_t *bitmap,
 		 min_nodes, max_nodes,
 		 req_nodes, mode,
 		 preemptee_candidates, preemptee_job_list,
-		 exc_core_bitmap);
+		 resv_exc_ptr);
 }
 
 /*
@@ -313,19 +306,6 @@ extern int other_job_resized(job_record_t *job_ptr, node_record_t *node_ptr)
 	return (*(ops.job_resized))(job_ptr, node_ptr);
 }
 
-/*
- * Pass job-step signal to other plugin.
- * IN job_ptr - job to be signaled
- * IN signal  - signal(7) number
- */
-extern int other_job_signal(job_record_t *job_ptr, int signal)
-{
-	if (other_select_init() < 0)
-		return SLURM_ERROR;
-
-	return (*(ops.job_signal))(job_ptr, signal);
-}
-
 /*
  * Note termination of job is starting. Executed from slurmctld.
  * IN job_ptr - pointer to job being terminated
@@ -569,35 +549,6 @@ extern int other_select_jobinfo_unpack(select_jobinfo_t **jobinfo,
 	return (*(ops.jobinfo_unpack))(jobinfo, buffer, protocol_version);
 }
 
-/* write select job credential to a string
- * IN jobinfo - a select job credential
- * OUT buf    - location to write job credential contents
- * IN size    - byte size of buf
- * IN mode    - print mode, see enum select_print_mode
- * RET        - the string, same as buf
- */
-extern char *other_select_jobinfo_sprint(select_jobinfo_t *jobinfo,
-					    char *buf, size_t size, int mode)
-{
-	if (other_select_init() < 0)
-		return NULL;
-
-	return (*(ops.jobinfo_sprint))(jobinfo, buf, size, mode);
-}
-/* write select job info to a string
- * IN jobinfo - a select job credential
- * IN mode    - print mode, see enum select_print_mode
- * RET        - char * containing string of request
- */
-extern char *other_select_jobinfo_xstrdup(
-	select_jobinfo_t *jobinfo, int mode)
-{
-	if (other_select_init() < 0)
-		return NULL;
-
-	return (*(ops.jobinfo_xstrdup))(jobinfo, mode);
-}
-
 /*
  * Get select data from a plugin
  * IN dinfo     - type of data to get from the node record
@@ -613,21 +564,6 @@ extern int other_get_info_from_plugin(enum select_plugindata_info dinfo,
 	return (*(ops.get_info_from_plugin))(dinfo, job_ptr, data);
 }
 
-/*
- * Updated a node configuration. This happens when a node registers with
- *     more resources than originally configured (e.g. memory).
- * IN index  - index into the node record list
- * RETURN SLURM_SUCCESS on success || SLURM_ERROR else wise
- */
-extern int other_update_node_config (int index)
-{
-	if (other_select_init() < 0)
-		return SLURM_ERROR;
-
-	return (*(ops.
-		  update_node_config))(index);
-}
-
 /*
  * Note reconfiguration or change in partition configuration
  */
@@ -638,15 +574,3 @@ extern int other_reconfigure (void)
 
 	return (*(ops.reconfigure))();
 }
-
-extern bitstr_t * other_resv_test(resv_desc_msg_t *resv_desc_ptr,
-				  uint32_t node_cnt,
-				  bitstr_t *avail_bitmap,
-				  bitstr_t **core_bitmap)
-{
-	if (other_select_init() < 0)
-		return NULL;
-
-	return (*(ops.resv_test))(resv_desc_ptr, node_cnt,
-				  avail_bitmap, core_bitmap);
-}
