@@ -83,7 +83,7 @@ extern int other_job_init(List job_list);
  * IN node_ptr - current node data
  * IN node_count - number of node entries
  */
-extern int other_node_init();
+extern int other_node_init(void);
 
 /*
  * Get select data from a plugin
@@ -113,14 +113,14 @@ extern int other_get_info_from_plugin(enum select_plugindata_info dinfo,
  *		jobs to be preempted to initiate the pending job. Not set
  *		if mode=SELECT_MODE_TEST_ONLY or input pointer is NULL.
  *		Existing list is appended to.
- * IN exc_core_bitmap - bitmap of cores being reserved.
+ * IN resv_exc_ptr - Various TRES which the job can NOT use.
  * RET zero on success, EINVAL otherwise
  */
 extern int other_job_test(job_record_t *job_ptr, bitstr_t *bitmap,
 			  uint32_t min_nodes, uint32_t max_nodes,
 			  uint32_t req_nodes, uint16_t mode,
 			  List preemptee_candidates, List *preemptee_job_list,
-			  bitstr_t *exc_core_bitmap);
+			  resv_exc_t *resv_exc_ptr);
 
 /*
  * Note initiation of job is about to begin. Called immediately
@@ -294,9 +294,4 @@ extern int other_select_nodeinfo_get(select_nodeinfo_t *nodeinfo,
 /* Note reconfiguration or change in partition configuration */
 extern int other_reconfigure(void);
 
-extern bitstr_t * other_resv_test(resv_desc_msg_t *resv_desc_ptr,
-				  uint32_t node_cnt,
-				  bitstr_t *avail_bitmap,
-				  bitstr_t **core_bitmap);
-
 #endif /* _CRAY_OTHER_SELECT_H */
