@@ -45,6 +45,9 @@
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+#include "src/interfaces/serializer.h"
+
+#include "src/interfaces/serializer.h"
 
 /*
  * These variables are required by the generic plugin interface.  If they
@@ -86,8 +89,9 @@ static bool _is_char_hex(char buffer)
 	       (buffer >= 'A' && buffer <= 'F');
 }
 
-extern int serializer_p_serialize(char **dest, const data_t *data,
-				  data_serializer_flags_t flags)
+extern int serialize_p_data_to_string(char **dest, size_t *length,
+				      const data_t *src,
+				      serializer_flags_t flags)
 {
 	return ESLURM_NOT_SUPPORTED;
 }
@@ -99,11 +103,19 @@ static int _handle_new_key_char(data_t *d, char **key, char **buffer,
 		/* example: &test=value */
 	} else if (*key == NULL && *buffer != NULL) {
 		/*
-		 * example: test&test=value
-		 * existing buffer, assume null value.
+		 * example: test1&test2=value
+		 * only buffer given but not key value. Assume that the buffer
+		 * is instead the key and this is the user providing a flag
+		 * input which will be parsed as being true.
+		 *
+		 * The behavior is not yet standardised by OpenAPI:
+		 *  https://github.com/OAI/OpenAPI-Specification/issues/1782
+		 *
+		 * RFC3986 provides an example of "key=value" but leaves
+		 * the flag values ambiguous.
 		 */
 		data_t *c = data_key_set(d, *buffer);
-		data_set_null(c);
+		data_set_bool(c, true);
 		xfree(*buffer);
 		*buffer = NULL;
 	} else if (*key != NULL && *buffer == NULL) {
@@ -183,11 +195,10 @@ static unsigned char _decode_seq(const char *ptr)
  * 	breaks key=value&key2=value2&...
  * 	into a data_t dictionary
  * 	dup keys will override existing keys
- * IN len - not used
  * RET SLURM_SUCCESS or error
  */
-extern int serializer_p_deserialize(data_t **dest, const char *src,
-				    size_t len)
+extern int serialize_p_string_to_data(data_t **dest, const char *src,
+				      size_t length)
 {
 	int rc = SLURM_SUCCESS;
 	data_t *d = data_set_dict(data_new());
