@@ -373,7 +373,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)));
+				      data_get_type_string(d));
 				rc = SLURM_ERROR;
 			} else {
 				rc = _yaml_parse_block(depth, parser, d, mode,
@@ -392,8 +392,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 					error("%s: data (0x%"PRIXPTR") depth=%d unexpected block entry type:%s for data_type:%s without key",
 					      __func__, (uintptr_t) d, depth,
 					      data_type_to_string(type),
-					      data_type_to_string(
-						      data_get_type(d)));
+					      data_get_type_string(d));
 					rc = SLURM_ERROR;
 				} else {
 					rc = _yaml_parse_block(depth, parser, d,
@@ -409,8 +408,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 					error("%s: data (0x%"PRIXPTR") depth=%d unexpected block entry type:%s for data_type:%s without key",
 					      __func__, (uintptr_t)d, depth,
 					      data_type_to_string(type),
-					      data_type_to_string(
-						      data_get_type(d)));
+					      data_get_type_string(d));
 					rc = SLURM_ERROR;
 				} else
 					debug5("%s: data (0x%"PRIXPTR") depth=%d ignoring YAML_BLOCK_ENTRY_TOKEN (already in list)",
@@ -420,7 +418,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				      __func__, (uintptr_t)d, depth,
 				      _yaml_parse_mode_string(mode),
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)),
+				      data_get_type_string(d),
 				      key);
 				rc = SLURM_ERROR;
 			}
@@ -439,7 +437,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s mode:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)),
+				      data_get_type_string(d),
 				      _yaml_parse_mode_string(mode));
 				rc = SLURM_ERROR;
 			} else {
@@ -456,7 +454,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)));
+				      data_get_type_string(d));
 				rc = SLURM_ERROR;
 			} else {
 				xassert(key);
@@ -472,7 +470,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)));
+				      data_get_type_string(d));
 				rc = SLURM_ERROR;
 			} else {
 				rc = _yaml_parse_block(depth, parser, d, mode,
@@ -525,7 +523,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)));
+				      data_get_type_string(d));
 				rc = SLURM_ERROR;
 			} else {
 				rc = _yaml_parse_block(depth, parser, d, mode,
@@ -545,7 +543,7 @@ static int _yaml_to_data(int depth, yaml_parser_t *parser, data_t *d,
 				error("%s: data (0x%"PRIXPTR") depth=%d unexpected suffix type:%s for data_type:%s",
 				      __func__, (uintptr_t)d, depth,
 				      data_type_to_string(type),
-				      data_type_to_string(data_get_type(d)));
+				      data_get_type_string(d));
 				rc = SLURM_ERROR;
 			} else {
 				rc = _yaml_parse_block(depth, parser, d, mode,
@@ -859,7 +857,8 @@ static int _yaml_write_handler(void *data, unsigned char *buffer, size_t size)
 	return 1;
 }
 
-static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter, buf_t *buf)
+static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter, buf_t *buf,
+		      serializer_flags_t flags)
 {
 	yaml_event_t event;
 
@@ -872,6 +871,12 @@ static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter, buf_t *buf)
 	if (!yaml_emitter_initialize(emitter))
 		_yaml_emitter_error;
 
+	if (flags == SER_FLAGS_COMPACT) {
+		yaml_emitter_set_indent(emitter, 0);
+		yaml_emitter_set_width(emitter, -1);
+		yaml_emitter_set_break(emitter, YAML_ANY_BREAK);
+	}
+
 	yaml_emitter_set_output(emitter, _yaml_write_handler, buf);
 
 	//TODO defaulted to UTF8 but maybe this should be a flag?
@@ -917,7 +922,7 @@ extern int serialize_p_data_to_string(char **dest, size_t *length,
 	yaml_emitter_t emitter;
 	buf_t *buf = init_buf(0);
 
-	if (_dump_yaml(src, &emitter, buf)) {
+	if (_dump_yaml(src, &emitter, buf, flags)) {
 		error("%s: dump yaml failed", __func__);
 
 		FREE_NULL_BUFFER(buf);
