@@ -43,7 +43,6 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <limits.h>
-#include <linux/limits.h>
 #include <math.h>
 #include <sched.h>
 #include <signal.h>
@@ -56,8 +55,9 @@
 #include "switch_cray_aries.h"
 #include "slurm/slurm.h"
 #include "slurm/slurm_errno.h"
+#include "src/interfaces/gres.h"
 #include "src/common/pack.h"
-#include "src/common/gres.h"
+#include "src/interfaces/select.h"
 
 #ifdef HAVE_NATIVE_CRAY
 #include <job.h> /* Cray's job module component */
@@ -216,6 +216,9 @@ extern int switch_p_build_jobinfo(switch_jobinfo_t *switch_job,
 	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 
+	select_g_select_jobinfo_get(step_ptr->select_jobinfo,
+				    SELECT_JOBDATA_NETWORK,
+				    &job->access);
 	xfree(nodes);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
@@ -328,7 +331,14 @@ extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, buf_t *buffer,
 		print_jobinfo(job);
 	}
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		pack32(job->magic, buffer);
+		pack32(job->num_cookies, buffer);
+		packstr_array(job->cookies, job->num_cookies, buffer);
+		pack32_array(job->cookie_ids, job->num_cookies, buffer);
+		pack64(job->apid, buffer);
+		pack16(job->access, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(job->magic, buffer);
 		pack32(job->num_cookies, buffer);
 		packstr_array(job->cookies, job->num_cookies, buffer);
@@ -355,7 +365,33 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 	job = xmalloc(sizeof(slurm_cray_jobinfo_t));
 	*switch_job = (switch_jobinfo_t *)job;
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		safe_unpack32(&job->magic, buffer);
+
+		if (job->magic == CRAY_NULL_JOBINFO_MAGIC) {
+			CRAY_DEBUG("Nothing to unpack");
+			return SLURM_SUCCESS;
+		}
+
+		xassert(job->magic == CRAY_JOBINFO_MAGIC);
+		safe_unpack32(&(job->num_cookies), buffer);
+		safe_unpackstr_array(&(job->cookies), &num_cookies, buffer);
+		if (num_cookies != job->num_cookies) {
+			CRAY_ERR("Wrong number of cookies received."
+				 " Expected: %" PRIu32 "Received: %" PRIu32,
+				 job->num_cookies, num_cookies);
+			goto unpack_error;
+		}
+		safe_unpack32_array(&(job->cookie_ids), &num_cookies, buffer);
+		if (num_cookies != job->num_cookies) {
+			CRAY_ERR("Wrong number of cookie IDs received."
+				 " Expected: %" PRIu32 "Received: %" PRIu32,
+				 job->num_cookies, num_cookies);
+			goto unpack_error;
+		}
+		safe_unpack64(&job->apid, buffer);
+		safe_unpack16(&job->access, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&job->magic, buffer);
 
 		if (job->magic == CRAY_NULL_JOBINFO_MAGIC) {
@@ -411,22 +447,22 @@ unpack_error:
  * switch functions for job initiation
  */
 
-extern int switch_p_job_preinit(stepd_step_rec_t *job)
+extern int switch_p_job_preinit(stepd_step_rec_t *step)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_init(stepd_step_rec_t *job)
+extern int switch_p_job_init(stepd_step_rec_t *step)
 {
 
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *sw_job = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *sw_job = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	int rc, num_ptags;
 	int exclusive = 0, mem_scaling = 100, cpu_scaling = 100;
 	int *ptags = NULL;
 	char *err_msg = NULL;
-	uint64_t cont_id = job->cont_id;
+	uint64_t cont_id = step->cont_id;
 	alpsc_peInfo_t alpsc_pe_info = {-1, -1, -1, -1, NULL, NULL, NULL};
 	int cmd_index = 0;
 #ifdef HAVE_NATIVE_CRAY
@@ -437,41 +473,37 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	uint32_t jobid;
 #endif
 
-#if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	char *npc = "none";
-	int access = ALPSC_NET_PERF_CTR_NONE;
-#endif
 	DEF_TIMERS;
 
 	START_TIMER;
 
 #ifdef HAVE_CRAY_NETWORK
 	/* No PAGG job containers; uid used instead to configure network */
-	cont_id = (uint64_t)job->uid;
+	cont_id = (uint64_t)step->uid;
 #endif
 
 	if (!sw_job || (sw_job->magic == CRAY_NULL_JOBINFO_MAGIC)) {
-		CRAY_DEBUG("job->switch_job was NULL");
+		CRAY_DEBUG("step->switch_job was NULL");
 		return SLURM_SUCCESS;
 	}
 
-	xassert(job->msg);
+	xassert(step->msg);
 	xassert(sw_job->magic == CRAY_JOBINFO_MAGIC);
 
 #ifdef HAVE_NATIVE_CRAY
 	// Attach to the cncu container
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->step_id.job_id;
-	rc = alpsc_attach_cncu_container(&err_msg, jobid, job->cont_id);
+		jobid = step->step_id.job_id;
+	rc = alpsc_attach_cncu_container(&err_msg, jobid, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_attach_cncu_container");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 
 	// Create the apid directory
-	rc = create_apid_dir(sw_job->apid, job->uid, job->gid);
+	rc = create_apid_dir(sw_job->apid, step->uid, step->gid);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
 	}
@@ -488,7 +520,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	/*
 	 * Fill in the alpsc_pe_info structure
 	 */
-	rc = build_alpsc_pe_info(job, &alpsc_pe_info, &cmd_index);
+	rc = build_alpsc_pe_info(step, &alpsc_pe_info, &cmd_index);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
 	}
@@ -523,11 +555,11 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 		 * non-exclusive jobs.
 		 */
 
-		cpu_scaling = get_cpu_scaling(job);
+		cpu_scaling = get_cpu_scaling(step);
 		if (cpu_scaling == -1)
 			return SLURM_ERROR;
 
-		mem_scaling = get_mem_scaling(job);
+		mem_scaling = get_mem_scaling(step);
 		if (mem_scaling == -1)
 			return SLURM_ERROR;
 	}
@@ -560,7 +592,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	// Write the IAA file
-	rc = write_iaa_file(job, sw_job, sw_job->ptags, sw_job->num_ptags,
+	rc = write_iaa_file(step, sw_job, sw_job->ptags, sw_job->num_ptags,
 			    &alpsc_pe_info);
 	if (rc != SLURM_SUCCESS) {
 		free_alpsc_pe_info(&alpsc_pe_info);
@@ -573,18 +605,13 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	 * If there is reserved access to network performance counters,
 	 * configure the appropriate access permission in the kernel.
 	 */
-	access = ALPSC_NET_PERF_CTR_NONE;
-	select_g_select_jobinfo_get(job->msg->select_jobinfo,
-		SELECT_JOBDATA_NETWORK, &npc);
-	CRAY_DEBUG("network performance counters SELECT_JOBDATA_NETWORK %s",
-		npc);
-	if (xstrcasecmp(npc, "system") == 0) {
-		access = ALPSC_NET_PERF_CTR_SYSTEM;
-	} else if (xstrcasecmp(npc, "blade") == 0) {
-		access = ALPSC_NET_PERF_CTR_BLADE;
-	}
-	if (access != ALPSC_NET_PERF_CTR_NONE) {
-		rc = alpsc_set_perf_ctr_perms(&err_msg, job->cont_id, access);
+	CRAY_DEBUG("network performance counters SELECT_JOBDATA_NETWORK %u",
+		   sw_job->access);
+
+	if ((sw_job->access != ALPSC_NET_PERF_CTR_NONE) &&
+	    (sw_job->access != NO_VAL16)) {
+		rc = alpsc_set_perf_ctr_perms(&err_msg, step->cont_id,
+					      (int)sw_job->access);
 		ALPSC_CN_DEBUG("alpsc_set_perf_ctr_perms");
 		if (rc != 1) {
 			free_alpsc_pe_info(&alpsc_pe_info);
@@ -642,7 +669,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	/*
 	 * Write some environment variables used by LLI and PMI
 	 */
-	rc = set_job_env(job, sw_job);
+	rc = set_job_env(step, sw_job);
 	if (rc != SLURM_SUCCESS)
 		return rc;
 
@@ -651,11 +678,11 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	 * Query the generic resources to see if the GPU should be allocated
 	 */
 
-	rc = gres_get_step_info(job->step_gres_list, "gpu", 0,
+	rc = gres_get_step_info(step->step_gres_list, "gpu", 0,
 				GRES_STEP_DATA_COUNT, &gpu_cnt);
 	CRAY_INFO("gres_cnt: %d %"PRIu64, rc, gpu_cnt);
 	if (gpu_cnt > 0)
-		setup_gpu(job);
+		setup_gpu(step);
 
 	/*
 	 * Set the Job's APID
@@ -671,43 +698,6 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_suspend_test(switch_jobinfo_t *jobinfo)
-{
-	return SLURM_SUCCESS;
-}
-
-extern void switch_p_job_suspend_info_get(switch_jobinfo_t *jobinfo,
-					  void **suspend_info)
-{
-	return;
-}
-extern void switch_p_job_suspend_info_pack(void *suspend_info, buf_t *buffer,
-					   uint16_t protocol_version)
-{
-	return;
-}
-
-extern int switch_p_job_suspend_info_unpack(void **suspend_info, buf_t *buffer,
-					    uint16_t protocol_version)
-{
-	return SLURM_SUCCESS;
-}
-
-extern void switch_p_job_suspend_info_free(void *suspend_info)
-{
-	return;
-}
-
-extern int switch_p_job_suspend(void *suspend_info, int max_wait)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_job_resume(void *suspend_info, int max_wait)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int switch_p_job_fini(switch_jobinfo_t *jobinfo)
 {
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
@@ -743,12 +733,12 @@ extern int switch_p_job_fini(switch_jobinfo_t *jobinfo)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_postfini(stepd_step_rec_t *job)
+extern int switch_p_job_postfini(stepd_step_rec_t *step)
 {
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	int rc;
 	char *err_msg = NULL;
-	uid_t pgid = job->jmgr_pid;
+	uid_t pgid = step->jmgr_pid;
 #ifdef HAVE_NATIVE_CRAY
         uint64_t gpu_cnt = 0;
 #endif
@@ -756,8 +746,8 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	if (!job->switch_job) {
-		CRAY_DEBUG("job->switch_job was NULL");
+	if (!step->switch_job) {
+		CRAY_DEBUG("step->switch_job was NULL");
 	}
 
 	/*
@@ -768,7 +758,7 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 		kill(-pgid, SIGKILL);
 	} else
 		CRAY_INFO("%ps: Bad pid value %lu",
-			  &job->step_id, (unsigned long) pgid);
+			  &step->step_id, (unsigned long) pgid);
 	/*
 	 * Clean-up
 	 *
@@ -780,10 +770,10 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 
 #ifdef HAVE_NATIVE_CRAY
 	// Set the proxy back to the default state.
-	rc = gres_get_step_info(job->step_gres_list, "gpu", 0,
+	rc = gres_get_step_info(step->step_gres_list, "gpu", 0,
 				GRES_STEP_DATA_COUNT, &gpu_cnt);
 	if (gpu_cnt > 0) {
-		reset_gpu(job);
+		reset_gpu(step);
 	}
 #endif
 	if (!lustre_no_flush) {
@@ -820,49 +810,6 @@ extern int switch_p_job_attach(switch_jobinfo_t *jobinfo, char ***env,
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_get_jobinfo(switch_jobinfo_t *switch_job, int key,
-				void *resulting_data)
-{
-	slurm_seterrno(EINVAL);
-	return SLURM_ERROR;
-}
-
-/*
- * node switch state monitoring functions
- * required for IBM Federation switch
- */
-extern int switch_p_clear_node_state(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_alloc_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_build_node_info(switch_node_info_t *switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_pack_node_info(switch_node_info_t *switch_node, buf_t *buffer,
-				   uint16_t protocol_version)
-{
-	return 0;
-}
-
-extern int switch_p_unpack_node_info(switch_node_info_t **switch_node,
-				     buf_t *buffer, uint16_t protocol_version)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_free_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 				      char *nodelist)
 {
@@ -903,21 +850,21 @@ extern int switch_p_job_step_allocated(switch_jobinfo_t *jobinfo,
 /*
  * Functions for suspend/resume support
  */
-extern int switch_p_job_step_pre_suspend(stepd_step_rec_t *job)
+extern int switch_p_job_step_pre_suspend(stepd_step_rec_t *step)
 {
 #if _DEBUG
 	info("switch_p_job_step_pre_suspend(%ps)", &job->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *jobinfo = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *jobinfo = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	char *err_msg = NULL;
 	int rc;
 	DEF_TIMERS;
 
 	START_TIMER;
 
-	rc = alpsc_pre_suspend(&err_msg, job->cont_id, jobinfo->ptags,
+	rc = alpsc_pre_suspend(&err_msg, step->cont_id, jobinfo->ptags,
 			       jobinfo->num_ptags, SUSPEND_TIMEOUT_MSEC);
 	ALPSC_CN_DEBUG("alpsc_pre_suspend");
 	if (rc != 1) {
@@ -930,10 +877,10 @@ extern int switch_p_job_step_pre_suspend(stepd_step_rec_t *job)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_post_suspend(stepd_step_rec_t *job)
+extern int switch_p_job_step_post_suspend(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_post_suspend(%ps)", &job->step_id);
+	info("switch_p_job_step_post_suspend(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
 	char *err_msg = NULL;
@@ -942,7 +889,7 @@ extern int switch_p_job_step_post_suspend(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	rc = alpsc_post_suspend(&err_msg, job->cont_id);
+	rc = alpsc_post_suspend(&err_msg, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_post_suspend");
 	if (rc != 1) {
 		return SLURM_ERROR;
@@ -954,21 +901,21 @@ extern int switch_p_job_step_post_suspend(stepd_step_rec_t *job)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_pre_resume(stepd_step_rec_t *job)
+extern int switch_p_job_step_pre_resume(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_pre_resume(%ps)", &job->step_id);
+	info("switch_p_job_step_pre_resume(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *jobinfo = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *jobinfo = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	char *err_msg = NULL;
 	int rc;
 	DEF_TIMERS;
 
 	START_TIMER;
 
-	rc = alpsc_pre_resume(&err_msg, job->cont_id, jobinfo->ptags,
+	rc = alpsc_pre_resume(&err_msg, step->cont_id, jobinfo->ptags,
 			       jobinfo->num_ptags);
 	ALPSC_CN_DEBUG("alpsc_pre_resume");
 	if (rc != 1) {
@@ -981,10 +928,10 @@ extern int switch_p_job_step_pre_resume(stepd_step_rec_t *job)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_post_resume(stepd_step_rec_t *job)
+extern int switch_p_job_step_post_resume(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_post_resume(%ps)", &job->step_id);
+	info("switch_p_job_step_post_resume(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
 	char *err_msg = NULL;
@@ -993,7 +940,7 @@ extern int switch_p_job_step_post_resume(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	rc = alpsc_post_resume(&err_msg, job->cont_id);
+	rc = alpsc_post_resume(&err_msg, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_post_resume");
 	if (rc != 1) {
 		return SLURM_ERROR;
@@ -1004,3 +951,8 @@ extern int switch_p_job_step_post_resume(stepd_step_rec_t *job)
 #endif
 	return SLURM_SUCCESS;
 }
+
+extern void switch_p_job_complete(uint32_t job_id)
+{
+	return;
+}
