@@ -3,6 +3,7 @@
  *	switch topology
  *****************************************************************************
  *  Copyright (C) 2009 Lawrence Livermore National Security.
+ *  Copyright (C) 2023 NVIDIA CORPORATION.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -47,10 +48,14 @@
 #include "src/common/bitstring.h"
 #include "src/common/log.h"
 #include "src/common/node_conf.h"
-#include "src/common/slurm_topology.h"
+#include "src/interfaces/topology.h"
 #include "src/common/xstring.h"
+
+#include "src/slurmctld/locks.h"
 #include "src/slurmctld/slurmctld.h"
 
+#include "../common/common_topo.h"
+
 /* These are defined here so when we link with something other than
  * the slurmctld we will have these symbols defined.  They will get
  * overwritten when linking with the slurmctld.
@@ -61,12 +66,14 @@ extern int node_record_count __attribute__((weak_import));
 extern switch_record_t *switch_record_table __attribute__((weak_import));
 extern int switch_record_cnt __attribute__((weak_import));
 extern int switch_levels __attribute__((weak_import));
+extern int active_node_record_count __attribute__((weak_import));
 #else
 node_record_t **node_record_table_ptr;
 int node_record_count;
 switch_record_t *switch_record_table;
 int switch_record_cnt;
 int switch_levels;
+int active_node_record_count;
 #endif
 
 /*
@@ -95,6 +102,7 @@ int switch_levels;
  */
 const char plugin_name[]        = "topology tree plugin";
 const char plugin_type[]        = "topology/tree";
+const uint32_t plugin_id = TOPOLOGY_PLUGIN_TREE;
 const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
 
 typedef struct slurm_conf_switches {
@@ -105,8 +113,15 @@ typedef struct slurm_conf_switches {
 	char *switches;		/* names if child switches directly
 				 * connected to this switch, if any */
 } slurm_conf_switches_t;
+
+typedef topo_info_t topoinfo_switch_t;
+
+typedef struct topoinfo_tree {
+	uint32_t record_count;		/* number of records */
+	topoinfo_switch_t *topo_array;	/* the switch topology records */
+} topoinfo_tree_t;
+
 static s_p_hashtbl_t *conf_hashtbl = NULL;
-static char* topo_conf = NULL;
 
 static void _check_better_path(int i, int j ,int k);
 static void _destroy_switches(void *ptr);
@@ -114,11 +129,11 @@ static void _free_switch_record_table(void);
 static int  _get_switch_inx(const char *name);
 static void _log_switches(void);
 static int  _node_name2bitmap(char *node_names, bitstr_t **bitmap,
-			      hostlist_t *invalid_hostlist);
+			      hostlist_t **invalid_hostlist);
 static int  _parse_switches(void **dest, slurm_parser_enum_t type,
 			    const char *key, const char *value,
 			    const char *line, char **leftover);
-extern int  _read_topo_file(slurm_conf_switches_t **ptr_array[]);
+static int  _read_topo_file(slurm_conf_switches_t **ptr_array[]);
 static void _find_child_switches(int sw);
 static void _validate_switches(void);
 
@@ -140,7 +155,6 @@ extern int init(void)
 extern int fini(void)
 {
 	_free_switch_record_table();
-	xfree(topo_conf);
 	return SLURM_SUCCESS;
 }
 
@@ -148,7 +162,7 @@ extern int fini(void)
  * topo_build_config - build or rebuild system topology information
  *	after a system startup or reconfiguration.
  */
-extern int topo_build_config(void)
+extern int topology_p_build_config(void)
 {
 	if (node_record_count)
 		_validate_switches();
@@ -156,11 +170,43 @@ extern int topo_build_config(void)
 }
 
 /*
- * topo_generate_node_ranking  -  this plugin does not set any node_rank fields
+ * When TopologyParam=SwitchAsNodeRank is set, this plugin assigns a unique
+ * node_rank for all nodes belonging to the same leaf switch.
  */
-extern bool topo_generate_node_ranking(void)
+extern bool topology_p_generate_node_ranking(void)
 {
-	return false;
+	/* By default, node_rank is 0, so start at 1 */
+	int switch_rank = 1;
+
+	if (!xstrcasestr(slurm_conf.topology_param, "SwitchAsNodeRank"))
+		return false;
+
+	/* Build a temporary topology to be able to find the leaf switches. */
+	_validate_switches();
+
+	if (switch_record_cnt == 0)
+		return false;
+
+	for (int sw = 0; sw < switch_record_cnt; sw++) {
+		/* skip if not a leaf switch */
+		if (switch_record_table[sw].level != 0)
+			continue;
+
+		for (int n = 0; n < node_record_count; n++) {
+			if (!bit_test(switch_record_table[sw].node_bitmap, n))
+				continue;
+			node_record_table_ptr[n]->node_rank = switch_rank;
+			debug("node=%s rank=%d",
+			      node_record_table_ptr[n]->name, switch_rank);
+		}
+
+		switch_rank++;
+	}
+
+	/* Discard the temporary topology since it is using node bitmaps */
+	_free_switch_record_table();
+
+	return true;
 }
 
 /*
@@ -171,10 +217,11 @@ extern bool topo_generate_node_ranking(void)
  *      address : s0.s4.s8.tux1
  *      pattern : switch.switch.switch.node
  */
-extern int topo_get_node_addr(char* node_name, char** paddr, char** ppattern)
+extern int topology_p_get_node_addr(char *node_name, char **paddr,
+				    char **ppattern)
 {
 	node_record_t *node_ptr;
-	hostlist_t sl = NULL;
+	hostlist_t *sl = NULL;
 
 	int s_max_level = 0;
 	int i, j;
@@ -236,6 +283,201 @@ extern int topo_get_node_addr(char* node_name, char** paddr, char** ppattern)
 	return SLURM_SUCCESS;
 }
 
+/*
+ * _subtree_split_hostlist() split a hostlist into topology aware subhostlists
+ *
+ * IN/OUT nodes_bitmap - bitmap of all hosts that need to be sent
+ * IN parent - location in switch_record_table
+ * IN/OUT msg_count - running count of how many messages we need to send
+ * IN/OUT sp_hl - array of subhostlists
+ * IN/OUT count - position in sp_hl array
+ */
+static int _subtree_split_hostlist(bitstr_t *nodes_bitmap, int parent,
+				   int *msg_count, hostlist_t ***sp_hl,
+				   int *count)
+{
+	int lst_count = 0, sw_count;
+	bitstr_t *fwd_bitmap = NULL;		/* nodes in forward list */
+
+	for (int i = 0; i < switch_record_table[parent].num_switches; i++) {
+		int k = switch_record_table[parent].switch_index[i];
+
+		if (!fwd_bitmap)
+			fwd_bitmap = bit_copy(
+				switch_record_table[k].node_bitmap);
+		else
+			bit_copybits(fwd_bitmap,
+				     switch_record_table[k].node_bitmap);
+		bit_and(fwd_bitmap, nodes_bitmap);
+		sw_count = bit_set_count(fwd_bitmap);
+		if (sw_count == 0) {
+			continue; /* no nodes on this switch in message list */
+		}
+		(*sp_hl)[*count] = bitmap2hostlist(fwd_bitmap);
+		/* Now remove nodes from this switch from message list */
+		bit_and_not(nodes_bitmap, fwd_bitmap);
+		if (slurm_conf.debug_flags & DEBUG_FLAG_ROUTE) {
+			char *buf;
+			buf = hostlist_ranged_string_xmalloc((*sp_hl)[*count]);
+			debug("ROUTE: ... sublist[%d] switch=%s :: %s",
+			      i, switch_record_table[i].name, buf);
+			xfree(buf);
+		}
+		(*count)++;
+		lst_count += sw_count;
+		if (lst_count == *msg_count)
+			break; /* all nodes in message are in a child list */
+	}
+	*msg_count -= lst_count;
+
+	FREE_NULL_BITMAP(fwd_bitmap);
+	return lst_count;
+}
+
+extern int topology_p_split_hostlist(hostlist_t *hl, hostlist_t ***sp_hl,
+				     int *count, uint16_t tree_width)
+{
+	int i, j, k, msg_count, switch_count;
+	int s_first, s_last;
+	char *buf;
+	bitstr_t *nodes_bitmap = NULL;		/* nodes in message list */
+	bitstr_t *switch_bitmap = NULL;		/* switches  */
+	slurmctld_lock_t node_read_lock = { .node = READ_LOCK };
+	static pthread_mutex_t init_lock = PTHREAD_MUTEX_INITIALIZER;
+
+	if (!common_topo_route_tree()) {
+		return common_topo_split_hostlist_treewidth(
+			hl, sp_hl, count, tree_width);
+	}
+
+	slurm_mutex_lock(&init_lock);
+	if (switch_record_cnt == 0) {
+		if (running_in_slurmctld())
+			fatal_abort("%s: Somehow we have 0 for switch_record_cnt and we are here in the slurmctld.  This should never happen.", __func__);
+		/* configs have not already been processed */
+		init_node_conf();
+		build_all_nodeline_info(false, 0);
+		rehash_node();
+
+		if (topology_g_build_config() != SLURM_SUCCESS) {
+			fatal("ROUTE: Failed to build topology config");
+		}
+	}
+	slurm_mutex_unlock(&init_lock);
+
+	/* Only acquire the slurmctld lock if running as the slurmctld. */
+	if (running_in_slurmctld())
+		lock_slurmctld(node_read_lock);
+
+	/* create bitmap of nodes to send message too */
+	if (hostlist2bitmap(hl, false, &nodes_bitmap) != SLURM_SUCCESS) {
+		buf = hostlist_ranged_string_xmalloc(hl);
+		fatal("ROUTE: Failed to make bitmap from hostlist=%s.", buf);
+	}
+
+	/* Find lowest level switches containing all the nodes in the list */
+	switch_bitmap = bit_alloc(switch_record_cnt);
+	for (j = 0; j < switch_record_cnt; j++) {
+		if ((switch_record_table[j].level == 0 ) &&
+		    bit_overlap_any(switch_record_table[j].node_bitmap,
+				    nodes_bitmap)) {
+				bit_set(switch_bitmap, j);
+		}
+	}
+
+	switch_count = bit_set_count(switch_bitmap);
+
+	for (i = 1; i <= switch_levels; i++) {
+		/* All nodes in message list are in one switch */
+		if (switch_count < 2)
+			break;
+		for (j = 0; j < switch_record_cnt; j++) {
+			if (switch_count < 2)
+				break;
+			if (switch_record_table[j].level == i) {
+				int first_child = -1, child_cnt = 0, num_desc;
+				num_desc = switch_record_table[j].
+						num_desc_switches;
+				for (k = 0; k < num_desc; k++) {
+					int index = switch_record_table[j].
+						switch_desc_index[k];
+					if (bit_test(switch_bitmap, index)) {
+						child_cnt++;
+						if (child_cnt > 1) {
+							bit_clear(switch_bitmap,
+								  index);
+						} else {
+							first_child = index;
+						}
+					}
+				}
+				if (child_cnt > 1) {
+					bit_clear(switch_bitmap, first_child);
+					bit_set(switch_bitmap, j);
+					switch_count -= (child_cnt - 1);
+				}
+			}
+		}
+	}
+
+	s_first = bit_ffs(switch_bitmap);
+	if (s_first != -1)
+		s_last = bit_fls(switch_bitmap);
+	else
+		s_last = -2;
+
+	if (switch_count == 1 && switch_record_table[s_first].level == 0 &&
+	    bit_super_set(nodes_bitmap,
+			  switch_record_table[s_first].node_bitmap)) {
+		/* This is a leaf switch. Construct list based on TreeWidth */
+		if (running_in_slurmctld())
+			unlock_slurmctld(node_read_lock);
+		FREE_NULL_BITMAP(nodes_bitmap);
+		FREE_NULL_BITMAP(switch_bitmap);
+		return common_topo_split_hostlist_treewidth(hl, sp_hl, count,
+							    tree_width);
+	}
+	*sp_hl = xcalloc(switch_record_cnt, sizeof(hostlist_t *));
+	msg_count = hostlist_count(hl);
+	*count = 0;
+	for (j = s_first; j <= s_last; j++) {
+		xassert(msg_count);
+
+		if (!bit_test(switch_bitmap, j))
+			continue;
+		_subtree_split_hostlist(nodes_bitmap, j, &msg_count, sp_hl,
+					count);
+	}
+	xassert(msg_count == bit_set_count(nodes_bitmap));
+	if (msg_count) {
+		size_t new_size = xsize(*sp_hl);
+		node_record_t *node_ptr;
+
+		if (slurm_conf.debug_flags & DEBUG_FLAG_ROUTE) {
+			buf = bitmap2node_name(nodes_bitmap);
+			debug("ROUTE: didn't find switch containing nodes=%s",
+			      buf);
+			xfree(buf);
+		}
+		new_size += msg_count * sizeof(hostlist_t *);
+		xrealloc(*sp_hl, new_size);
+
+		for (j = 0; (node_ptr = next_node_bitmap(nodes_bitmap, &j));
+		     j++) {
+			(*sp_hl)[*count] = hostlist_create(NULL);
+			hostlist_push_host((*sp_hl)[*count], node_ptr->name);
+			(*count)++;
+		}
+	}
+
+	if (running_in_slurmctld())
+		unlock_slurmctld(node_read_lock);
+	FREE_NULL_BITMAP(nodes_bitmap);
+	FREE_NULL_BITMAP(switch_bitmap);
+
+	return SLURM_SUCCESS;
+}
+
 /*
  * _find_child_switches creates an array of indexes to the
  * immediate descendants of switch sw.
@@ -244,8 +486,8 @@ static void _find_child_switches(int sw)
 {
 	int i;
 	int cldx; /* index into array of child switches */
-	hostlist_iterator_t hi;
-	hostlist_t swlist;
+	hostlist_iterator_t *hi;
+	hostlist_t *swlist;
 	char *swname;
 
 	swlist = hostlist_create(switch_record_table[sw].switches);
@@ -316,9 +558,9 @@ static void _find_desc_switches(int sw)
 static void _validate_switches(void)
 {
 	slurm_conf_switches_t *ptr, **ptr_array;
-	int depth, i, j;
+	int depth, i, j, node_count;
 	switch_record_t *switch_ptr, *prior_ptr;
-	hostlist_t hl, invalid_hl = NULL;
+	hostlist_t *hl, *invalid_hl = NULL;
 	char *child, *buf;
 	bool  have_root = false;
 	bitstr_t *multi_homed_bitmap = NULL;	/* nodes on >1 leaf switch */
@@ -444,8 +686,8 @@ static void _validate_switches(void)
 		i = bit_set_count(switches_bitmap);
 		if (i > 0) {
 			child = bitmap2node_name(switches_bitmap);
-			error("WARNING: switches lack access to %d nodes: %s",
-			      i, child);
+			warning("switches lack access to %d nodes: %s",
+				i, child);
 			xfree(child);
 		}
 		FREE_NULL_BITMAP(switches_bitmap);
@@ -454,8 +696,7 @@ static void _validate_switches(void)
 
 	if (invalid_hl) {
 		buf = hostlist_ranged_string_xmalloc(invalid_hl);
-		error("WARNING: Invalid hostnames in switch configuration: %s",
-		      buf);
+		warning("Invalid hostnames in switch configuration: %s", buf);
 		xfree(buf);
 		hostlist_destroy(invalid_hl);
 	}
@@ -465,19 +706,19 @@ static void _validate_switches(void)
 	i = bit_set_count(multi_homed_bitmap);
 	if (i > 0) {
 		child = bitmap2node_name(multi_homed_bitmap);
-		error("WARNING: Multiple leaf switches contain nodes: %s",
-		      child);
+		warning("Multiple leaf switches contain nodes: %s", child);
 		xfree(child);
 	}
 	FREE_NULL_BITMAP(multi_homed_bitmap);
 
+	node_count = active_node_record_count;
 	/* Create array of indexes of children of each switch,
 	 * and see if any switch can reach all nodes */
 	for (i = 0; i < switch_record_cnt; i++) {
 		if (switch_record_table[i].level != 0) {
 			_find_child_switches(i);
 		}
-		if (node_record_count ==
+		if (node_count ==
 			bit_set_count(switch_record_table[i].node_bitmap)) {
 			have_root = true;
 		}
@@ -518,7 +759,7 @@ static void _validate_switches(void)
 		}
 	}
 	if (!have_root && running_in_daemon())
-		info("TOPOLOGY: warning -- no switch can reach all nodes through its descendants. If this is not intentional, fix the topology.conf file.");
+		warning("TOPOLOGY: no switch can reach all nodes through its descendants. If this is not intentional, fix the topology.conf file.");
 
 	s_p_hashtbl_destroy(conf_hashtbl);
 	_log_switches();
@@ -599,7 +840,7 @@ static void _free_switch_record_table(void)
 }
 
 /* Return count of switch configuration entries read */
-extern int  _read_topo_file(slurm_conf_switches_t **ptr_array[])
+static int  _read_topo_file(slurm_conf_switches_t **ptr_array[])
 {
 	static s_p_options_t switch_options[] = {
 		{"SwitchName", S_P_ARRAY, _parse_switches, _destroy_switches},
@@ -608,12 +849,11 @@ extern int  _read_topo_file(slurm_conf_switches_t **ptr_array[])
 	int count;
 	slurm_conf_switches_t **ptr;
 
-	debug("Reading the topology.conf file");
-	if (!topo_conf)
-		topo_conf = get_extra_conf_path("topology.conf");
+	xassert(topo_conf);
+	debug("Reading the %s file", topo_conf);
 
 	conf_hashtbl = s_p_hashtbl_create(switch_options);
-	if (s_p_parse_file(conf_hashtbl, NULL, topo_conf, false, NULL) ==
+	if (s_p_parse_file(conf_hashtbl, NULL, topo_conf, 0, NULL) ==
 	    SLURM_ERROR) {
 		s_p_hashtbl_destroy(conf_hashtbl);
 		fatal("something wrong with opening/reading %s: %m",
@@ -691,11 +931,11 @@ static void _destroy_switches(void *ptr)
  *       to free memory when variables are no longer required
  */
 static int _node_name2bitmap(char *node_names, bitstr_t **bitmap,
-			     hostlist_t *invalid_hostlist)
+			     hostlist_t **invalid_hostlist)
 {
 	char *this_node_name;
 	bitstr_t *my_bitmap;
-	hostlist_t host_list;
+	hostlist_t *host_list;
 
 	my_bitmap = (bitstr_t *) bit_alloc(node_record_count);
 	*bitmap = my_bitmap;
@@ -750,3 +990,167 @@ static void _check_better_path(int i, int j ,int k)
 	if (switch_record_table[j].switches_dist[k] > tmp)
 		switch_record_table[j].switches_dist[k] = tmp;
 }
+
+extern int topology_p_topology_free(void *topoinfo_ptr)
+{
+	int i = 0;
+	topoinfo_tree_t *topoinfo = topoinfo_ptr;
+	if (topoinfo) {
+		if (topoinfo->topo_array) {
+			for (i = 0; i < topoinfo->record_count; i++) {
+				xfree(topoinfo->topo_array[i].name);
+				xfree(topoinfo->topo_array[i].nodes);
+				xfree(topoinfo->topo_array[i].switches);
+			}
+			xfree(topoinfo->topo_array);
+		}
+		xfree(topoinfo);
+	}
+	return SLURM_SUCCESS;
+}
+
+extern int topology_p_topology_get(void **topoinfo_pptr)
+{
+	int i = 0;
+	topoinfo_tree_t *topoinfo_ptr =
+		xmalloc(sizeof(topoinfo_tree_t));
+
+	*topoinfo_pptr = topoinfo_ptr;
+	topoinfo_ptr->record_count = switch_record_cnt;
+	topoinfo_ptr->topo_array = xcalloc(topoinfo_ptr->record_count,
+					   sizeof(topoinfo_switch_t));
+
+	for (i = 0; i < topoinfo_ptr->record_count; i++) {
+		topoinfo_ptr->topo_array[i].level =
+			switch_record_table[i].level;
+		topoinfo_ptr->topo_array[i].link_speed =
+			switch_record_table[i].link_speed;
+		topoinfo_ptr->topo_array[i].name =
+			xstrdup(switch_record_table[i].name);
+		topoinfo_ptr->topo_array[i].nodes =
+			xstrdup(switch_record_table[i].nodes);
+		topoinfo_ptr->topo_array[i].switches =
+			xstrdup(switch_record_table[i].switches);
+	}
+
+	return SLURM_SUCCESS;
+}
+
+extern int topology_p_topology_pack(void *topoinfo_ptr, buf_t *buffer,
+				    uint16_t protocol_version)
+{
+	int i;
+	topoinfo_tree_t *topoinfo = topoinfo_ptr;
+
+	pack32(topoinfo->record_count, buffer);
+	for (i = 0; i < topoinfo->record_count; i++) {
+		pack16(topoinfo->topo_array[i].level, buffer);
+		pack32(topoinfo->topo_array[i].link_speed, buffer);
+		packstr(topoinfo->topo_array[i].name, buffer);
+		packstr(topoinfo->topo_array[i].nodes, buffer);
+		packstr(topoinfo->topo_array[i].switches, buffer);
+	}
+	return SLURM_SUCCESS;
+}
+void _print_topo_record(topoinfo_switch_t * topo_ptr, char **out)
+{
+	char *env, *line = NULL, *pos = NULL;
+
+	/****** Line 1 ******/
+	xstrfmtcatat(line, &pos, "SwitchName=%s Level=%u LinkSpeed=%u",
+		     topo_ptr->name, topo_ptr->level, topo_ptr->link_speed);
+
+	if (topo_ptr->nodes)
+		xstrfmtcatat(line, &pos, " Nodes=%s", topo_ptr->nodes);
+
+	if (topo_ptr->switches)
+		xstrfmtcatat(line, &pos, " Switches=%s", topo_ptr->switches);
+
+	if ((env = getenv("SLURM_TOPO_LEN")))
+		xstrfmtcat(*out, "%.*s\n", atoi(env), line);
+	else
+		xstrfmtcat(*out, "%s\n", line);
+
+	xfree(line);
+
+}
+
+extern int topology_p_topology_print(void *topoinfo_ptr, char *nodes_list,
+				     char **out)
+{
+	int i, match, match_cnt = 0;;
+	topoinfo_tree_t *topoinfo = topoinfo_ptr;
+
+	*out = NULL;
+
+	if ((nodes_list == NULL) || (nodes_list[0] == '\0')) {
+		if (topoinfo->record_count == 0) {
+			error("No topology information available");
+			return SLURM_SUCCESS;
+		}
+
+		for (i = 0; i < topoinfo->record_count; i++)
+			_print_topo_record(&topoinfo->topo_array[i], out);
+
+		return SLURM_SUCCESS;
+	}
+
+	/* Search for matching switch name */
+	for (i = 0; i < topoinfo->record_count; i++) {
+		if (xstrcmp(topoinfo->topo_array[i].name, nodes_list))
+			continue;
+		_print_topo_record(&topoinfo->topo_array[i], out);
+		return SLURM_SUCCESS;
+	}
+
+	/* Search for matching node name */
+	for (i = 0; i < topoinfo->record_count; i++) {
+		hostset_t *hs;
+
+		if ((topoinfo->topo_array[i].nodes == NULL) ||
+		    (topoinfo->topo_array[i].nodes[0] == '\0'))
+			continue;
+		hs = hostset_create(topoinfo->topo_array[i].nodes);
+		if (hs == NULL)
+			fatal("hostset_create: memory allocation failure");
+		match = hostset_within(hs, nodes_list);
+		hostset_destroy(hs);
+		if (!match)
+			continue;
+		match_cnt++;
+		_print_topo_record(&topoinfo->topo_array[i], out);
+	}
+
+	if (match_cnt == 0) {
+		error("Topology information contains no switch or "
+		      "node named %s", nodes_list);
+	}
+	return SLURM_SUCCESS;
+}
+
+extern int topology_p_topology_unpack(void **topoinfo_pptr, buf_t *buffer,
+				      uint16_t protocol_version)
+{
+	int i = 0;
+	topoinfo_tree_t *topoinfo_ptr =
+		xmalloc(sizeof(topoinfo_tree_t));
+
+	*topoinfo_pptr = topoinfo_ptr;
+	safe_unpack32(&topoinfo_ptr->record_count, buffer);
+	safe_xcalloc(topoinfo_ptr->topo_array, topoinfo_ptr->record_count,
+		     sizeof(topoinfo_switch_t));
+	for (i = 0; i < topoinfo_ptr->record_count; i++) {
+		safe_unpack16(&topoinfo_ptr->topo_array[i].level, buffer);
+		safe_unpack32(&topoinfo_ptr->topo_array[i].link_speed, buffer);
+		safe_unpackstr(&topoinfo_ptr->topo_array[i].name, buffer);
+		safe_unpackstr(&topoinfo_ptr->topo_array[i].nodes, buffer);
+		safe_unpackstr(&topoinfo_ptr->topo_array[i].switches, buffer);
+	}
+
+	return SLURM_SUCCESS;
+
+unpack_error:
+	topology_p_topology_free(topoinfo_ptr);
+	*topoinfo_pptr = NULL;
+	return SLURM_ERROR;
+}
