@@ -39,6 +39,7 @@
 \*****************************************************************************/
 
 #include "src/sacctmgr/sacctmgr.h"
+#include "src/interfaces/data_parser.h"
 
 static int _set_cond(int *start, int argc, char **argv,
 		     slurmdb_assoc_cond_t *assoc_cond,
@@ -131,103 +132,6 @@ static int _set_cond(int *start, int argc, char **argv,
 	return set;
 }
 
-extern bool sacctmgr_check_default_qos(uint32_t qos_id,
-				       slurmdb_assoc_cond_t *assoc_cond)
-{
-	char *object = NULL;
-	ListIterator itr;
-	slurmdb_assoc_rec_t *assoc;
-	List no_access_list = NULL;
-	List assoc_list = NULL;
-
-	if (qos_id == NO_VAL)
-		return true;
-
-	assoc_list = slurmdb_associations_get(
-		db_conn, assoc_cond);
-	if (!assoc_list) {
-		fprintf(stderr, "Couldn't get a list back for checking qos.\n");
-		return false;
-	}
-
-	if (!g_qos_list)
-		g_qos_list = slurmdb_qos_get(db_conn, NULL);
-
-	itr = list_iterator_create(assoc_list);
-	while ((assoc = list_next(itr))) {
-		char *qos = NULL;
-
-		if (assoc->qos_list) {
-			int check_qos = qos_id;
-			if (check_qos == -1)
-				check_qos = assoc->def_qos_id;
-			if ((int)check_qos > 0) {
-				ListIterator qos_itr =
-					list_iterator_create(assoc->qos_list);
-				while ((qos = list_next(qos_itr))) {
-					if (qos[0] == '-')
-						continue;
-					else if (qos[0] == '+')
-						qos++;
-					/* info("looking for %u ?= %u", */
-					/*      check_qos, slurm_atoul(qos)); */
-					if (check_qos == slurm_atoul(qos))
-						break;
-				}
-				list_iterator_destroy(qos_itr);
-			} else
-				qos = "";
-		}
-
-		if (!qos) {
-			char *name = slurmdb_qos_str(g_qos_list,
-						     assoc->def_qos_id);
-			if (!assoc->user) {
-				// see if this isn't a user
-				object = xstrdup_printf(
-					"  DefQOS = %-10s C = %-10s A = %-20s ",
-					name, assoc->cluster, assoc->acct);
-			} else if (assoc->partition) {
-				// see if there is a partition name
-				object = xstrdup_printf(
-					"  DefQOS = %-10s C = %-10s A = %-20s "
-					"U = %-9s P = %s",
-					name, assoc->cluster, assoc->acct,
-					assoc->user, assoc->partition);
-			} else {
-				object = xstrdup_printf(
-					"  DefQOS = %-10s C = %-10s A = %-20s "
-					"U = %-9s",
-					name, assoc->cluster,
-					assoc->acct, assoc->user);
-			}
-
-			if (!no_access_list)
-				no_access_list = list_create(xfree_ptr);
-			list_append(no_access_list, object);
-		}
-	}
-	list_iterator_destroy(itr);
-	FREE_NULL_LIST(assoc_list);
-
-	if (!no_access_list)
-		return true;
-	fprintf(stderr,
-		" These associations don't have access to their "
-		"default qos.\n");
-	fprintf(stderr,
-		" Please give them access before they the default "
-		"can be set to this.\n");
-	itr = list_iterator_create(no_access_list);
-	while ((object = list_next(itr)))
-		fprintf(stderr, "%s\n", object);
-	list_iterator_destroy(itr);
-	FREE_NULL_LIST(no_access_list);
-
-	return 0;
-}
-
-
 extern int sacctmgr_set_assoc_cond(slurmdb_assoc_cond_t *assoc_cond,
 					 char *type, char *value,
 					 int command_len, int option)
@@ -327,7 +231,10 @@ extern int sacctmgr_set_assoc_rec(slurmdb_assoc_rec_t *assoc,
 	if (!assoc)
 		return set;
 
-	if (!xstrncasecmp(type, "DefaultQOS", MAX(command_len, 8))) {
+	if (!xstrncasecmp(type, "Comment", MAX(command_len, 2))) {
+		assoc->comment = strip_quotes(value, NULL, 1);
+		set = 1;
+	} else if (!xstrncasecmp(type, "DefaultQOS", MAX(command_len, 8))) {
 		if (!g_qos_list)
 			g_qos_list = slurmdb_qos_get(
 				db_conn, NULL);
@@ -624,6 +531,7 @@ extern void sacctmgr_print_assoc_rec(slurmdb_assoc_rec_t *assoc,
 {
 	char *print_acct = NULL;
 	char *tmp_char = NULL;
+	uint64_t tmp_uint64 = NO_VAL64;
 
 	xassert(field);
 
@@ -657,6 +565,9 @@ extern void sacctmgr_print_assoc_rec(slurmdb_assoc_rec_t *assoc,
 	case PRINT_CLUSTER:
 		field->print_routine(field, assoc->cluster, last);
 		break;
+	case PRINT_COMMENT:
+		field->print_routine(field, assoc->comment, last);
+		break;
 	case PRINT_DQOS:
 		if (!g_qos_list)
 			g_qos_list = slurmdb_qos_get(
@@ -677,121 +588,127 @@ extern void sacctmgr_print_assoc_rec(slurmdb_assoc_rec_t *assoc,
 		if (assoc->shares_raw == SLURMDB_FS_USE_PARENT)
 			print_fields_str(field, "parent", last);
 		else
-			field->print_routine(field, assoc->shares_raw, last);
+			field->print_routine(field, &assoc->shares_raw, last);
 		break;
 	case PRINT_GRPCM:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->grp_tres_mins, TRES_CPU),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->grp_tres_mins, TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_GRPCRM:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->grp_tres_run_mins, TRES_CPU),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->grp_tres_run_mins,
-					     TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_GRPC:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->grp_tres, TRES_CPU),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->grp_tres,
-					     TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_GRPTM:
-		field->print_routine(field, assoc->grp_tres_mins, last);
+		field->print_routine(field, &assoc->grp_tres_mins, last);
 		break;
 	case PRINT_GRPTRM:
-		field->print_routine(field, assoc->grp_tres_run_mins, last);
+		field->print_routine(field, &assoc->grp_tres_run_mins, last);
 		break;
 	case PRINT_GRPT:
-		field->print_routine(field, assoc->grp_tres, last);
+		field->print_routine(field, &assoc->grp_tres, last);
 		break;
 	case PRINT_GRPJ:
-		field->print_routine(field, assoc->grp_jobs, last);
+		field->print_routine(field, &assoc->grp_jobs, last);
 		break;
 	case PRINT_GRPJA:
-		field->print_routine(field, assoc->grp_jobs_accrue, last);
+		field->print_routine(field, &assoc->grp_jobs_accrue, last);
 		break;
 	case PRINT_GRPMEM:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->grp_tres, TRES_MEM),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->grp_tres, TRES_MEM),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_GRPN:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->grp_tres, TRES_NODE),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->grp_tres, TRES_NODE),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_GRPS:
-		field->print_routine(field, assoc->grp_submit_jobs, last);
+		field->print_routine(field, &assoc->grp_submit_jobs, last);
 		break;
 	case PRINT_GRPW:
-		field->print_routine(field, assoc->grp_wall, last);
+		field->print_routine(field, &assoc->grp_wall, last);
 		break;
 	case PRINT_ID:
-		field->print_routine(field, assoc->id, last);
+		field->print_routine(field, &assoc->id, last);
 		break;
-	case PRINT_LFT:
-		field->print_routine(field, assoc->lft, last);
+	case PRINT_LINEAGE:
+		field->print_routine(field, assoc->lineage, last);
 		break;
 	case PRINT_MAXCM:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->max_tres_mins_pj, TRES_CPU),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->max_tres_mins_pj, TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_MAXCRM:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->max_tres_run_mins, TRES_CPU);
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->max_tres_run_mins,
-					     TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_MAXC:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->max_tres_pj, TRES_CPU);
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->max_tres_pj, TRES_CPU),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_MAXTM:
-		field->print_routine(field, assoc->max_tres_mins_pj, last);
+		field->print_routine(field, &assoc->max_tres_mins_pj, last);
 		break;
 	case PRINT_MAXTRM:
-		field->print_routine(field, assoc->max_tres_run_mins, last);
+		field->print_routine(field, &assoc->max_tres_run_mins, last);
 		break;
 	case PRINT_MAXT:
-		field->print_routine(field, assoc->max_tres_pj, last);
+		field->print_routine(field, &assoc->max_tres_pj, last);
 		break;
 	case PRINT_MAXTN:
-		field->print_routine(field, assoc->max_tres_pn, last);
+		field->print_routine(field, &assoc->max_tres_pn, last);
 		break;
 	case PRINT_MAXJ:
-		field->print_routine(field, assoc->max_jobs, last);
+		field->print_routine(field, &assoc->max_jobs, last);
 		break;
 	case PRINT_MAXJA:
-		field->print_routine(field, assoc->max_jobs_accrue, last);
+		field->print_routine(field, &assoc->max_jobs_accrue, last);
 		break;
 	case PRINT_MINPT:
-		field->print_routine(field, assoc->min_prio_thresh, last);
+		field->print_routine(field, &assoc->min_prio_thresh, last);
 		break;
 	case PRINT_MAXN:
+		tmp_uint64 = slurmdb_find_tres_count_in_string(
+					assoc->max_tres_pj, TRES_NODE),
 		field->print_routine(field,
-				     slurmdb_find_tres_count_in_string(
-					     assoc->max_tres_pj, TRES_NODE),
+				     &tmp_uint64,
 				     last);
 		break;
 	case PRINT_MAXS:
-		field->print_routine(field, assoc->max_submit_jobs, last);
+		field->print_routine(field, &assoc->max_submit_jobs, last);
 		break;
 	case PRINT_MAXW:
-		field->print_routine(field, assoc->max_wall_pj,	last);
+		field->print_routine(field, &assoc->max_wall_pj, last);
 		break;
 	case PRINT_PID:
-		field->print_routine(field, assoc->parent_id, last);
+		field->print_routine(field, &assoc->parent_id, last);
 		break;
 	case PRINT_PNAME:
 		field->print_routine(field, assoc->parent_acct, last);
@@ -800,20 +717,21 @@ extern void sacctmgr_print_assoc_rec(slurmdb_assoc_rec_t *assoc,
 		field->print_routine(field, assoc->partition, last);
 		break;
 	case PRINT_PRIO:
-		field->print_routine(field, assoc->priority, last);
+		field->print_routine(field, &assoc->priority, last);
 		break;
 	case PRINT_QOS:
 		if (!g_qos_list)
 			g_qos_list = slurmdb_qos_get(
 				db_conn, NULL);
-
-		field->print_routine(field, g_qos_list, assoc->qos_list, last);
+		tmp_char = get_qos_complete_str(g_qos_list, assoc->qos_list);
+		field->print_routine(field, tmp_char, last);
+		xfree(tmp_char);
 		break;
 	case PRINT_QOS_RAW:
-		field->print_routine(field, assoc->qos_list, last);
+		field->print_routine(field, &assoc->qos_list, last);
 		break;
 	case PRINT_RGT:
-		field->print_routine(field, assoc->rgt, last);
+		field->print_routine(field, &assoc->rgt, last);
 		break;
 	case PRINT_USER:
 		field->print_routine(field, assoc->user, last);
@@ -878,6 +796,14 @@ extern int sacctmgr_list_assoc(int argc, char **argv)
 	assoc_list = slurmdb_associations_get(db_conn, assoc_cond);
 	slurmdb_destroy_assoc_cond(assoc_cond);
 
+	if (mime_type) {
+		DATA_DUMP_CLI_SINGLE(OPENAPI_ASSOCS_RESP, assoc_list, argc,
+				     argv, db_conn, mime_type, data_parser, rc);
+		FREE_NULL_LIST(print_fields_list);
+		FREE_NULL_LIST(assoc_list);
+		return rc;
+	}
+
 	if (!assoc_list) {
 		exit_code=1;
 		fprintf(stderr, " Error with request: %s\n",
@@ -886,7 +812,7 @@ extern int sacctmgr_list_assoc(int argc, char **argv)
 		return SLURM_ERROR;
 	}
 
-	slurmdb_sort_hierarchical_assoc_list(assoc_list, true);
+	slurmdb_sort_hierarchical_assoc_list(assoc_list);
 
 	itr = list_iterator_create(assoc_list);
 	itr2 = list_iterator_create(print_fields_list);
