@@ -40,6 +40,7 @@
 
 #include "src/sacctmgr/sacctmgr.h"
 #include "src/common/uid.h"
+#include "src/interfaces/data_parser.h"
 
 static bool with_deleted = 0;
 static bool with_fed = 0;
@@ -118,15 +119,6 @@ static int _set_cond(int *start, int argc, char **argv,
 					 MAX(command_len, 2))) {
 			if (format_list)
 				slurm_addto_char_list(format_list, argv[i]+end);
-		} else if (!end || !xstrncasecmp(argv[i], "PluginIDSelect",
-						MAX(command_len, 1))) {
-			if (!cluster_cond->plugin_id_select_list)
-				cluster_cond->plugin_id_select_list =
-					list_create(xfree_ptr);
-			if (slurm_addto_char_list(
-				   cluster_cond->plugin_id_select_list,
-				   argv[i]+end))
-				cond_set |= SA_SET_CLUST;
 		} else if (!end || !xstrncasecmp(argv[i], "RPCVersions",
 						MAX(command_len, 1))) {
 			if (!cluster_cond->rpc_version_list)
@@ -505,6 +497,14 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 	cluster_list = slurmdb_clusters_get(db_conn, cluster_cond);
 	slurmdb_destroy_cluster_cond(cluster_cond);
 
+	if (mime_type) {
+		DATA_DUMP_CLI_SINGLE(OPENAPI_CLUSTERS_RESP, cluster_list, argc,
+				     argv, db_conn, mime_type, data_parser, rc);
+		FREE_NULL_LIST(print_fields_list);
+		FREE_NULL_LIST(cluster_list);
+		return rc;
+	}
+
 	if (!cluster_list) {
 		exit_code=1;
 		fprintf(stderr, " Problem with query.\n");
@@ -537,18 +537,20 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 				break;
 			case PRINT_CPORT:
 				field->print_routine(field,
-						     cluster->control_port,
+						     &cluster->control_port,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_CLASS:
+				tmp_char = get_classification_str(
+						cluster->classification);
 				field->print_routine(field,
-						     get_classification_str(
-						     cluster->classification),
+						     tmp_char,
 						     (curr_inx == field_count));
+				tmp_char = NULL;
 				break;
 			case PRINT_FEATURES:
 				field->print_routine(field,
-						     cluster->fed.feature_list,
+						     &cluster->fed.feature_list,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_FEDERATION:
@@ -564,11 +566,11 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 				break;
 			}
 			case PRINT_FEDSTATERAW:
-				field->print_routine(field, cluster->fed.state,
+				field->print_routine(field, &cluster->fed.state,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_ID:
-				field->print_routine(field, cluster->fed.id,
+				field->print_routine(field, &cluster->fed.id,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_TRES:
@@ -582,17 +584,15 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 				xfree(tmp_char);
 				break;
 			case PRINT_FLAGS:
-			{
-				char *tmp_char = slurmdb_cluster_flags_2_str(
-							     cluster->flags);
+				tmp_char = slurmdb_cluster_flags_2_str(
+							cluster->flags);
 				field->print_routine(field, tmp_char,
 						     (curr_inx == field_count));
 				xfree(tmp_char);
 				break;
-			}
 			case PRINT_NODECNT:
 			{
-				hostlist_t hl = hostlist_create(cluster->nodes);
+				hostlist_t *hl = hostlist_create(cluster->nodes);
 				int cnt = 0;
 				if (hl) {
 					cnt = hostlist_count(hl);
@@ -600,7 +600,7 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 				}
 				field->print_routine(
 					field,
-					cnt,
+					&cnt,
 					(curr_inx == field_count));
 				break;
 			}
@@ -613,13 +613,7 @@ extern int sacctmgr_list_cluster(int argc, char **argv)
 			case PRINT_RPC_VERSION:
 				field->print_routine(
 					field,
-					cluster->rpc_version,
-					(curr_inx == field_count));
-				break;
-			case PRINT_SELECT:
-				field->print_routine(
-					field,
-					cluster->plugin_id_select,
+					&cluster->rpc_version,
 					(curr_inx == field_count));
 				break;
 			default:
@@ -789,30 +783,6 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 		}
 	}
 
-	if (cond_set & SA_SET_USER) {
-		List temp_list = NULL;
-
-		temp_list = slurmdb_clusters_get(db_conn, &cluster_cond);
-		if (!temp_list) {
-			exit_code=1;
-			fprintf(stderr,
-				" Problem getting clusters from database.  "
-				"Contact your admin.\n");
-			rc = SLURM_ERROR;
-			goto end_it;
-		} else if (!list_count(temp_list)) {
-			fprintf(stderr,
-				" Query didn't return any clusters.\n");
-			rc = SLURM_ERROR;
-			goto end_it;
-		}
-		/* we are only looking for the clusters returned from
-		   this query, so we free the cluster_list and replace
-		   it */
-		FREE_NULL_LIST(assoc_cond->cluster_list);
-		assoc_cond->cluster_list = temp_list;
-	}
-
 	printf(" Setting\n");
 	if (rec_set & SA_SET_CLUST)
 		sacctmgr_print_cluster(cluster);
