@@ -39,18 +39,31 @@
 \*****************************************************************************/
 
 #include "src/sacctmgr/sacctmgr.h"
+#include "src/common/macros.h"
 #include "src/common/slurmdbd_defs.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
+#include "src/common/slurm_protocol_defs.h"
 
 #include <unistd.h>
 #include <termios.h>
 
-static pthread_t lock_warning_thread;
+static bool warn_needed = false;
+static pthread_mutex_t warn_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t warn_cond = PTHREAD_COND_INITIALIZER;
 
 static void *_print_lock_warn(void *no_data)
 {
-	sleep(5);
-	printf(" Database is busy or waiting for lock from other user.\n");
+	struct timespec ts = { 0, 0 };
+	ts.tv_sec = time(NULL) + 5;
+
+	slurm_mutex_lock(&warn_mutex);
+	if (warn_needed) {
+		slurm_cond_timedwait(&warn_cond, &warn_mutex, &ts);
+		if (warn_needed)
+			printf(" Database is busy or waiting for lock from other user.\n");
+		warn_needed = false;
+	}
+	slurm_mutex_unlock(&warn_mutex);
 
 	return NULL;
 }
@@ -191,9 +204,9 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("Allowed", object, MAX(command_len, 2))) {
 		field->type = PRINT_ALLOWED;
-		field->name = xstrdup("% Allowed");
-		field->len = 10;
-		field->print_routine = print_fields_uint16;
+		field->name = xstrdup("Allowed");
+		field->len = 8;
+		field->print_routine = print_fields_uint32;
 	} else if (!xstrncasecmp("Associations", object, MAX(command_len, 2))) {
 		field->type = PRINT_ASSOC_NAME;
 		field->name = xstrdup("Assocs");
@@ -220,11 +233,16 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Cluster");
 		field->len = 10;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("Coordinators", object, MAX(command_len, 2))) {
+	} else if (!xstrncasecmp("Coordinators", object, MAX(command_len, 3))) {
 		field->type = PRINT_COORDS;
 		field->name = xstrdup("Coord Accounts");
 		field->len = 20;
 		field->print_routine = sacctmgr_print_coord_list;
+	} else if (!xstrncasecmp("Comment", object, MAX(command_len, 3))) {
+		field->type = PRINT_COMMENT;
+		field->name = xstrdup("Comment");
+		field->len = 20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("ControlHost", object, MAX(command_len, 8))) {
 		field->type = PRINT_CHOST;
 		field->name = xstrdup("ControlHost");
@@ -292,6 +310,11 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Event");
 		field->len = 7;
 		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("Extra", object, MAX(command_len, 2))) {
+		field->type = PRINT_EXTRA;
+		field->name = xstrdup("Extra");
+		field->len = 20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("Features", object, MAX(command_len, 3))) {
 		field->type = PRINT_FEATURES;
 		field->name = xstrdup("Features");
@@ -389,16 +412,26 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("ID");
 		field->len = 6;
 		field->print_routine = print_fields_uint;
-	} else if (!xstrncasecmp("Info", object, MAX(command_len, 2))) {
+	} else if (!xstrncasecmp("Info", object, MAX(command_len, 3))) {
 		field->type = PRINT_INFO;
 		field->name = xstrdup("Info");
 		field->len = 20;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("LFT", object, MAX(command_len, 1))) {
-		field->type = PRINT_LFT;
-		field->name = xstrdup("LFT");
-		field->len = 6;
-		field->print_routine = print_fields_uint;
+	} else if (!xstrncasecmp("InstanceId", object, MAX(command_len, 9))) {
+		field->type = PRINT_INSTANCE_ID;
+		field->name = xstrdup("InstanceId");
+		field->len = 20;
+		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("InstanceType", object, MAX(command_len, 9))) {
+		field->type = PRINT_INSTANCE_TYPE;
+		field->name = xstrdup("InstanceType");
+		field->len = 20;
+		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("Lineage", object, MAX(command_len, 1))) {
+		field->type = PRINT_LINEAGE;
+		field->name = xstrdup("Lineage");
+		field->len = -20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("servertype", object, MAX(command_len, 10))) {
 		field->type = PRINT_SERVERTYPE;
 		field->name = xstrdup("ServerType");
@@ -641,12 +674,6 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Partition");
 		field->len = 10;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("PluginIDSelect", object,
-				 MAX(command_len, 2))) {
-		field->type = PRINT_SELECT;
-		field->name = xstrdup("PluginIDSelect");
-		field->len = 14;
-		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("PreemptMode", object, MAX(command_len, 8))) {
 		field->type = PRINT_PREEM;
 		field->name = xstrdup("PreemptMode");
@@ -657,7 +684,7 @@ static print_field_t *_get_print_field(char *object)
 		field->type = PRINT_PREE;
 		field->name = xstrdup("Preempt");
 		field->len = 10;
-		field->print_routine = sacctmgr_print_qos_bitstr;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("PreemptExemptTime", object,
 				 MAX(command_len, 8))) {
 		field->type = PRINT_PRXMPT;
@@ -678,7 +705,7 @@ static print_field_t *_get_print_field(char *object)
 		field->type = PRINT_QOS;
 		field->name = xstrdup("QOS");
 		field->len = 20;
-		field->print_routine = sacctmgr_print_qos_list;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("QOSRAWLevel", object, MAX(command_len, 4))) {
 		field->type = PRINT_QOS_RAW;
 		field->name = xstrdup("QOS_RAW");
@@ -785,9 +812,14 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_double;
 	} else if (!xstrncasecmp("Allocated", object, MAX(command_len, 7))) {
 		field->type = PRINT_ALLOCATED;
-		field->name = xstrdup("% Allocated");
-		field->len = 11;
-		field->print_routine = print_fields_uint16;
+		field->name = xstrdup("Allocated");
+		field->len = 9;
+		field->print_routine = print_fields_uint32;
+	} else if (!xstrncasecmp("LastConsumed", object, MAX(command_len, 8))) {
+		field->type = PRINT_LAST_CONSUMED;
+		field->name = xstrdup("LastConsumed");
+		field->len = 12;
+		field->print_routine = print_fields_uint32;
 	} else if (!xstrncasecmp("User", object, MAX(command_len, 1))) {
 		field->type = PRINT_USER;
 		field->name = xstrdup("User");
@@ -814,15 +846,20 @@ static print_field_t *_get_print_field(char *object)
 	return field;
 }
 
-extern void notice_thread_init()
+extern void notice_thread_init(void)
 {
-	slurm_thread_create_detached(&lock_warning_thread,
-				     _print_lock_warn, NULL);
+	slurm_mutex_lock(&warn_mutex);
+	warn_needed = true;
+	slurm_thread_create_detached(_print_lock_warn, NULL);
+	slurm_mutex_unlock(&warn_mutex);
 }
 
-extern void notice_thread_fini()
+extern void notice_thread_fini(void)
 {
-	pthread_cancel(lock_warning_thread);
+	slurm_mutex_lock(&warn_mutex);
+	warn_needed = false;
+	slurm_cond_broadcast(&warn_cond);
+	slurm_mutex_unlock(&warn_mutex);
 }
 
 extern int commit_check(char *warning)
@@ -998,7 +1035,8 @@ end_it:
 	return rc;
 }
 
-extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
+extern int sacctmgr_update_qos_usage(slurmdb_qos_cond_t *qos_cond,
+				     long double new_raw_usage)
 {
 	List update_list = NULL;
 	List cluster_list;
@@ -1024,7 +1062,7 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 		list_append(cluster_list, xstrdup(slurm_conf.cluster_name));
 	}
 
-	if (!commit_check("Would you like to reset usage?")) {
+	if (!commit_check("Would you like to update usage?")) {
 		printf(" Changes Discarded\n");
 		return rc;
 	}
@@ -1049,7 +1087,7 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 
 		update_list = list_create(slurmdb_destroy_update_object);
 		update_obj = xmalloc(sizeof(slurmdb_update_object_t));
-		update_obj->type = SLURMDB_REMOVE_QOS_USAGE;
+		update_obj->type = SLURMDB_UPDATE_QOS_USAGE;
 		update_obj->objects = list_create(NULL);
 
 		while ((qos_name = list_next(itr2))) {
@@ -1061,6 +1099,9 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 				slurmdb_destroy_update_object(update_obj);
 				goto end_it;
 			}
+			if (!rec->usage)
+				rec->usage = xmalloc(sizeof(*rec->usage));
+			rec->usage->usage_raw = new_raw_usage;
 			list_append(update_obj->objects, rec);
 		}
 		list_iterator_reset(itr2);
@@ -1509,113 +1550,52 @@ extern int get_double(char *in_value, double *out_value, char *type)
 	return SLURM_SUCCESS;
 }
 
+static int _addto_action_char_list_internal(List char_list, char *name, void *x)
+{
+	uint32_t id = 0;
+	char *tmp_name = NULL;
+
+	id = str_2_slurmdbd_msg_type(name);
+	if (id == NO_VAL) {
+		error("You gave a bad action '%s'.", name);
+		list_flush(char_list);
+		return SLURM_ERROR;
+	}
+
+	tmp_name = xstrdup_printf("%u", id);
+
+	if (!list_find_first(char_list, slurm_find_char_in_list, tmp_name)) {
+		list_append(char_list, tmp_name);
+		return 1;
+	} else {
+		xfree(tmp_name);
+		return 0;
+	}
+}
+
 extern int addto_action_char_list(List char_list, char *names)
 {
-	int i=0, start=0;
-	char *name = NULL, *tmp_char = NULL;
-	ListIterator itr = NULL;
-	char quote_c = '\0';
-	int quote = 0;
-	uint32_t id=0;
-	int count = 0;
-
 	if (!char_list) {
 		error("No list was given to fill in");
 		return 0;
 	}
 
-	itr = list_iterator_create(char_list);
-	if (names) {
-		if (names[i] == '\"' || names[i] == '\'') {
-			quote_c = names[i];
-			quote = 1;
-			i++;
-		}
-		start = i;
-		while(names[i]) {
-			if (quote && names[i] == quote_c)
-				break;
-			else if (names[i] == '\"' || names[i] == '\'')
-				names[i] = '`';
-			else if (names[i] == ',') {
-				if ((i-start) > 0) {
-					name = xmalloc((i-start+1));
-					memcpy(name, names+start, (i-start));
-
-					id = str_2_slurmdbd_msg_type(name);
-					if (id == NO_VAL) {
-						error("You gave a bad action "
-						      "'%s'.", name);
-						xfree(name);
-						break;
-					}
-					xfree(name);
-
-					name = xstrdup_printf("%u", id);
-					while((tmp_char = list_next(itr))) {
-						if (!xstrcasecmp(tmp_char,
-								 name))
-							break;
-					}
-					list_iterator_reset(itr);
-
-					if (!tmp_char) {
-						list_append(char_list, name);
-						count++;
-					} else
-						xfree(name);
-				}
-
-				i++;
-				start = i;
-				if (names[i] == ' ') {
-					error("There is a problem with "
-					      "your request.  It appears you "
-					      "have spaces inside your list.");
-					break;
-				}
-			}
-			i++;
-		}
-		if ((i-start) > 0) {
-			name = xmalloc((i-start)+1);
-			memcpy(name, names+start, (i-start));
-
-			id = str_2_slurmdbd_msg_type(name);
-			if (id == NO_VAL)  {
-				error("You gave a bad action '%s'.",
-				      name);
-				xfree(name);
-				goto end_it;
-			}
-			xfree(name);
-
-			name = xstrdup_printf("%u", id);
-			while((tmp_char = list_next(itr))) {
-				if (!xstrcasecmp(tmp_char, name))
-					break;
-			}
-
-			if (!tmp_char) {
-				list_append(char_list, name);
-				count++;
-			} else
-				xfree(name);
-		}
-	}
-end_it:
-	list_iterator_destroy(itr);
-	return count;
+	return slurm_parse_char_list(char_list, names, NULL,
+				     _addto_action_char_list_internal);
 }
 
 extern void sacctmgr_print_coord_list(
-	print_field_t *field, List value, int last)
+	print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
 	ListIterator itr = NULL;
 	char *print_this = NULL;
+	List value = NULL;
 	slurmdb_coord_rec_t *object = NULL;
 
+	if (input)
+		value = *(List *)input;
+
 	if (!value || !list_count(value)) {
 		if (print_fields_parsable_print)
 			print_this = xstrdup("");
@@ -1651,68 +1631,19 @@ extern void sacctmgr_print_coord_list(
 	xfree(print_this);
 }
 
-extern void sacctmgr_print_qos_list(print_field_t *field, List qos_list,
-				    List value, int last)
-{
-	int abs_len = abs(field->len);
-	char *print_this = NULL;
-
-	print_this = get_qos_complete_str(qos_list, value);
-
-	if (print_fields_parsable_print == PRINT_FIELDS_PARSABLE_NO_ENDING
-	    && last)
-		printf("%s", print_this);
-	else if (print_fields_parsable_print)
-		printf("%s|", print_this);
-	else {
-		if (strlen(print_this) > abs_len)
-			print_this[abs_len-1] = '+';
-
-		if (field->len == abs_len)
-			printf("%*.*s ", abs_len, abs_len, print_this);
-		else
-			printf("%-*.*s ", abs_len, abs_len, print_this);
-	}
-	xfree(print_this);
-}
-
-extern void sacctmgr_print_qos_bitstr(print_field_t *field, List qos_list,
-				      bitstr_t *value, int last)
-{
-	int abs_len = abs(field->len);
-	char *print_this = NULL;
-
-	print_this = get_qos_complete_str_bitstr(qos_list, value);
-
-	if (print_fields_parsable_print == PRINT_FIELDS_PARSABLE_NO_ENDING
-	    && last)
-		printf("%s", print_this);
-	else if (print_fields_parsable_print)
-		printf("%s|", print_this);
-	else {
-		if (strlen(print_this) > abs_len)
-			print_this[abs_len-1] = '+';
-
-		if (field->len == abs_len)
-			printf("%*.*s ", abs_len, abs_len, print_this);
-		else
-			printf("%-*.*s ", abs_len, abs_len, print_this);
-	}
-	xfree(print_this);
-}
-
-extern void sacctmgr_print_tres(print_field_t *field, char *tres_simple_str,
-				int last)
+extern void sacctmgr_print_tres(print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
 	char *print_this;
+	char *value = NULL;
+
+	if (input)
+		value = *(char **) input;
 
 	sacctmgr_initialize_g_tres_list();
 
 	print_this = slurmdb_make_tres_string_from_simple(
-		tres_simple_str, g_tres_list, NO_VAL, CONVERT_NUM_UNIT_EXACT, 0,
-		NULL);
-
+		value, g_tres_list, NO_VAL, CONVERT_NUM_UNIT_EXACT, 0, NULL);
 
 	if (!print_this)
 		print_this = xstrdup("");
@@ -1890,6 +1821,10 @@ extern void sacctmgr_print_assoc_limits(slurmdb_assoc_rec_t *assoc)
 		       slurmdb_qos_str(g_qos_list, assoc->def_qos_id));
 	}
 
+	/* This should be last because it might be long */
+	if (assoc->comment)
+		printf("  Comment       = %s\n", assoc->comment);
+
 }
 
 static int _print_cluster_features(void *object, void *arg)
