@@ -84,93 +84,58 @@ static uint32_t _decode_node_state(char *val)
 	return NO_VAL;
 }
 
+static int _addto_state_char_list_internal(List char_list, char *name, void *x)
+{
+	uint32_t c;
+	char *tmp_name = NULL;
+
+	c = _decode_node_state(name);
+	if (c == NO_VAL)
+		fatal("unrecognized job state value");
+	tmp_name = xstrdup_printf("%u", c);
+
+	if (!list_find_first(char_list, slurm_find_char_in_list, tmp_name)) {
+		list_append(char_list, tmp_name);
+		return 1;
+	} else {
+		xfree(tmp_name);
+		return 0;
+	}
+}
+
 static int _addto_state_char_list(List char_list, char *names)
 {
-	int i=0, start=0;
-	uint32_t c;
-	char *name = NULL, *tmp_char = NULL;
-	ListIterator itr = NULL;
-	char quote_c = '\0';
-	int quote = 0;
-	int count = 0;
-
 	if (!char_list) {
 		error("No list was given to fill in");
 		return 0;
 	}
 
-	itr = list_iterator_create(char_list);
-	if (names) {
-		if (names[i] == '\"' || names[i] == '\'') {
-			quote_c = names[i];
-			quote = 1;
-			i++;
+	return slurm_parse_char_list(char_list, names, NULL,
+				     _addto_state_char_list_internal);
+}
+
+static uint32_t _parse_cond_flags(const char *flags_str)
+{
+	uint32_t flags = 0;
+	char *tmp_flags, *flag, *save_ptr = NULL;
+
+	tmp_flags = xstrdup(flags_str);
+
+	flag = strtok_r(tmp_flags, ",", &save_ptr);
+	while (flag) {
+		if (!xstrcasecmp(flag, "OPEN")) {
+			flags |= SLURMDB_EVENT_COND_OPEN;
+		} else {
+			error("Unknown condition flag %s", flag);
+			exit_code = 1;
 		}
-		start = i;
-		while(names[i]) {
-			//info("got %d - %d = %d", i, start, i-start);
-			if (quote && names[i] == quote_c)
-				break;
-			else if (names[i] == '\"' || names[i] == '\'')
-				names[i] = '`';
-			else if (names[i] == ',') {
-				if ((i-start) > 0) {
-					name = xmalloc((i-start+1));
-					memcpy(name, names+start, (i-start));
-					c = _decode_node_state(name);
-					if (c == NO_VAL)
-						fatal("unrecognized job "
-						      "state value");
-					xfree(name);
-					name = xstrdup_printf("%u", c);
 
-					while((tmp_char = list_next(itr))) {
-						if (!xstrcasecmp(tmp_char,
-								 name))
-							break;
-					}
-
-					if (!tmp_char) {
-						list_append(char_list, name);
-						count++;
-					} else
-						xfree(name);
-					list_iterator_reset(itr);
-				}
-				i++;
-				start = i;
-				if (names[i] == ' ') {
-					info("There is a problem with "
-					     "your request.  It appears you "
-					     "have spaces inside your list.");
-					break;
-				}
-			}
-			i++;
-		}
-		if ((i-start) > 0) {
-			name = xmalloc((i-start)+1);
-			memcpy(name, names+start, (i-start));
-			c = _decode_node_state(name);
-			if (c == NO_VAL)
-				fatal("unrecognized job state value");
-			xfree(name);
-			name = xstrdup_printf("%u", c);
-
-			while((tmp_char = list_next(itr))) {
-				if (!xstrcasecmp(tmp_char, name))
-					break;
-			}
-
-			if (!tmp_char) {
-				list_append(char_list, name);
-				count++;
-			} else
-				xfree(name);
-		}
+		flag = strtok_r(NULL, ",", &save_ptr);
 	}
-	list_iterator_destroy(itr);
-	return count;
+
+	xfree(tmp_flags);
+
+	return flags;
 }
 
 static int _set_cond(int *start, int argc, char **argv,
@@ -247,13 +212,17 @@ static int _set_cond(int *start, int argc, char **argv,
 			list_iterator_destroy(itr);
 			FREE_NULL_LIST(tmp_list);
 		} else if (!xstrncasecmp(argv[i], "Clusters",
-					 MAX(command_len, 1))) {
+					 MAX(command_len, 2))) {
 			if (!event_cond->cluster_list)
 				event_cond->cluster_list =
 					list_create(xfree_ptr);
 			if (slurm_addto_char_list(event_cond->cluster_list,
 						 argv[i]+end))
 				set = 1;
+		} else if (!xstrncasecmp(argv[i], "CondFlags",
+					 MAX(command_len, 2))) {
+			event_cond->cond_flags = _parse_cond_flags(argv[i]+end);
+			set = 1;
 		} else if (!xstrncasecmp(argv[i], "End", MAX(command_len, 1))) {
 			event_cond->period_end = parse_time(argv[i]+end, 1);
 			set = 1;
@@ -293,7 +262,7 @@ static int _set_cond(int *start, int argc, char **argv,
 			if (!event_cond->state_list)
 				event_cond->state_list = list_create(xfree_ptr);
 			if (_addto_state_char_list(event_cond->state_list,
-						  argv[i]+end)) {
+						  argv[i]+end) > 0) {
 				event_cond->event_type = SLURMDB_EVENT_NODE;
 				set = 1;
 			}
@@ -304,7 +273,7 @@ static int _set_cond(int *start, int argc, char **argv,
 					list_create(xfree_ptr);
 			if (slurm_addto_id_char_list(
 				event_cond->reason_uid_list,
-				argv[i]+end, 0)) {
+				argv[i]+end, 0) > 0) {
 				event_cond->event_type = SLURMDB_EVENT_NODE;
 				set = 1;
 			} else {
@@ -437,6 +406,7 @@ extern int sacctmgr_list_event(int argc, char **argv)
 	while((event = list_next(itr))) {
 		int curr_inx = 1;
 		char tmp[20], *tmp_char;
+		uint64_t tmp_uint64;
 		time_t newend = event->period_end;
 
 		while((field = list_next(itr2))) {
@@ -466,19 +436,19 @@ extern int sacctmgr_list_event(int argc, char **argv)
 			case PRINT_DURATION:
 				if (!newend)
 					newend = time(NULL);
+				tmp_uint64 = newend - event->period_start;
 				field->print_routine(
 					field,
-					(uint64_t)(newend
-						   - event->period_start),
+					&tmp_uint64,
 					(curr_inx == field_count));
 				break;
 			case PRINT_TIMEEND:
 				field->print_routine(field,
-						     event->period_end,
+						     &event->period_end,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_EVENTRAW:
-				field->print_routine(field, event->event_type,
+				field->print_routine(field, &event->event_type,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_EVENT:
@@ -500,7 +470,7 @@ extern int sacctmgr_list_event(int argc, char **argv)
 				break;
 			case PRINT_TIMESTART:
 				field->print_routine(field,
-						     event->period_start,
+						     &event->period_start,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_REASON:
@@ -508,7 +478,7 @@ extern int sacctmgr_list_event(int argc, char **argv)
 						     (curr_inx == field_count));
 				break;
 			case PRINT_STATERAW:
-				field->print_routine(field, event->state,
+				field->print_routine(field, &event->state,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_STATE:
