@@ -48,7 +48,7 @@ static void _print_overcommit(slurmdb_res_rec_t *res,
 	slurmdb_clus_res_rec_t *clus_res = NULL;
 	char *cluster;
 
-	if (res->percent_used == NO_VAL16)
+	if (res->allocated == NO_VAL)
 		return;
 
 	/* Don't use the global g_res_list since we are going to
@@ -71,7 +71,12 @@ static void _print_overcommit(slurmdb_res_rec_t *res,
 
 	itr = list_iterator_create(res_list);
 	while ((found_res = list_next(itr))) {
-		int total = 0, percent_allowed;
+		int total = 0, allowed;
+		char *percent_str = "%";
+
+		if (found_res->flags & SLURMDB_RES_FLAG_ABSOLUTE)
+			percent_str = "";
+
 		fprintf(stderr, "  %s@%s\n",
 			found_res->name, found_res->server);
 		if (cluster_list)
@@ -92,35 +97,35 @@ static void _print_overcommit(slurmdb_res_rec_t *res,
 					 * This means we didn't specify any
 					 * clusters (All clusters are
 					 * overwritten with the requested
-					 * percentage) so just put something
-					 * there to get the correct
-					 * percent_allowed.
+					 * percentage/count) so just put
+					 * something there to get the correct
+					 * allowed.
 					 */
 					cluster = "nothing";
 				}
-				percent_allowed = cluster ? res->percent_used :
-					clus_res->percent_allowed;
-				total += percent_allowed;
+				allowed = cluster ? res->allocated :
+					clus_res->allowed;
+				total += allowed;
 
 				fprintf(stderr,
-					"   Cluster - %s\t %u%%\n",
+					"   Cluster - %s\t %u%s\n",
 					clus_res->cluster,
-					percent_allowed);
+					allowed, percent_str);
 			}
 		} else if (clus_itr) {
 			while ((cluster = list_next(clus_itr))) {
-				total += res->percent_used;
+				total += res->allocated;
 				fprintf(stderr,
-					"   Cluster - %s\t %u%%\n",
-					cluster,
-					res->percent_used);
+					"   Cluster - %s\t %u%s\n",
+					cluster, res->allocated,
+					percent_str);
 			}
 		}
 		if (clus_itr)
 			list_iterator_destroy(clus_itr);
 		if (found_clus_itr)
 			list_iterator_destroy(found_clus_itr);
-		fprintf(stderr, "   total\t\t%u%%\n", total);
+		fprintf(stderr, "   total\t\t%u%s\n", total, percent_str);
 	}
 	list_iterator_destroy(itr);
 
@@ -229,12 +234,13 @@ static int _set_res_cond(int *start, int argc, char **argv,
 				}
 			}
 			list_iterator_destroy(itr);
-		} else if (!xstrncasecmp(argv[i], "PercentAllowed",
+		} else if (!xstrncasecmp(argv[i], "Allowed",
+					 MAX(command_len, 1)) ||
+			   !xstrncasecmp(argv[i], "PercentAllowed",
 					 MAX(command_len, 1))) {
-			if (!res_cond->percent_list) {
-				res_cond->percent_list = list_create(xfree_ptr);
-			}
-			if (slurm_addto_char_list(res_cond->percent_list,
+			if (!res_cond->allowed_list)
+				res_cond->allowed_list = list_create(xfree_ptr);
+			if (slurm_addto_char_list(res_cond->allowed_list,
 						  argv[i]+end))
 				set = 1;
 		} else if (!xstrncasecmp(argv[i], "ServerType",
@@ -360,11 +366,19 @@ static int _set_res_rec(int *start, int argc, char **argv,
 				res->manager =
 					strip_quotes(argv[i]+end, NULL, 1);
 			set = 1;
-		} else if (!xstrncasecmp(argv[i], "PercentAllowed",
+		} else if (!xstrncasecmp(argv[i], "Allowed",
+					 MAX(command_len, 1)) ||
+			   !xstrncasecmp(argv[i], "PercentAllowed",
 					 MAX(command_len, 1))) {
-			/* overload percent_used here */
-			if (get_uint16(argv[i]+end, &res->percent_used,
-				       "PercentAllowed") == SLURM_SUCCESS) {
+			/* overload allocated here */
+			if (get_uint(argv[i]+end, &res->allocated,
+				     "Allowed") == SLURM_SUCCESS) {
+				set = 1;
+			}
+		} else if (!xstrncasecmp(argv[i], "LastConsumed",
+					 MAX(command_len, 8))) {
+			if (get_uint(argv[i]+end, &res->last_consumed,
+				     "LastConsumed") == SLURM_SUCCESS) {
 				set = 1;
 			}
 		} else if (!xstrncasecmp(argv[i], "Type",
@@ -403,35 +417,44 @@ static void _print_res_format(slurmdb_res_rec_t *res,
 	int curr_inx = 1;
 	char *tmp_char;
 	print_field_t *field = NULL;
-	uint32_t count;
+	uint32_t tmp_uint32;
 
 	xassert(itr);
 	xassert(res);
 
 	while ((field = list_next(itr))) {
 		switch(field->type) {
+		case PRINT_LAST_CONSUMED:
+			field->print_routine(field, &res->last_consumed,
+					     (curr_inx == field_count));
+			break;
 		case PRINT_ALLOWED:
+			tmp_uint32 = clus_res ? clus_res->allowed : 0;
 			field->print_routine(
-				field, clus_res ? clus_res->percent_allowed : 0,
+				field, &tmp_uint32,
 				(curr_inx == field_count));
 			break;
 		case PRINT_CLUSTER:
+			tmp_char = clus_res ? clus_res->cluster : NULL;
 			field->print_routine(
-				field, clus_res ? clus_res->cluster : NULL,
+				field, tmp_char,
 				(curr_inx == field_count));
 			break;
 		case PRINT_CALLOWED:
-			if (clus_res)
-				count = (res->count *
-					 clus_res->percent_allowed) / 100;
-			else
-				count = 0;
-			field->print_routine(field, count,
+			if (clus_res) {
+				if (res->flags & SLURMDB_RES_FLAG_ABSOLUTE)
+					tmp_uint32 = res->count;
+				else
+					tmp_uint32 = (res->count *
+						clus_res->allowed) / 100;
+			} else
+				tmp_uint32 = 0;
+			field->print_routine(field, &tmp_uint32,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_COUNT:
 			field->print_routine(field,
-					     res->count,
+					     &res->count,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_DESC:
@@ -441,7 +464,7 @@ static void _print_res_format(slurmdb_res_rec_t *res,
 			break;
 		case PRINT_ID:
 			field->print_routine(
-				field, res->id,
+				field, &res->id,
 				(curr_inx == field_count));
 			break;
 		case PRINT_FLAGS:
@@ -468,14 +491,15 @@ static void _print_res_format(slurmdb_res_rec_t *res,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_TYPE:
+			tmp_char = slurmdb_res_type_str(res->type);
 			field->print_routine(field,
-					     slurmdb_res_type_str(
-						     res->type),
+					     tmp_char,
 					     (curr_inx == field_count));
+			tmp_char = NULL;
 			break;
 		case PRINT_ALLOCATED:
 			field->print_routine(
-				field, res->percent_used,
+				field, &res->allocated,
 				(curr_inx == field_count));
 			break;
 		default:
@@ -567,14 +591,7 @@ extern int sacctmgr_add_res(int argc, char **argv)
 		found_res = sacctmgr_find_res_from_list(
 			g_res_list, NO_VAL, name, start_res->server);
 		if (!found_res) {
-			if (start_res->type == SLURMDB_RESOURCE_NOTSET) {
-				exit_code=1;
-				fprintf(stderr,
-					" Need to designate a resource "
-					"type to initially add '%s'.\n", name);
-				break;
-
-			} else if (start_res->count == NO_VAL) {
+			if (start_res->count == NO_VAL) {
 				exit_code=1;
 				fprintf(stderr,
 					" Need to designate a resource "
@@ -590,9 +607,10 @@ extern int sacctmgr_add_res(int argc, char **argv)
 			res->manager = xstrdup(start_res->manager);
 			res->server = xstrdup(start_res->server);
 			res->count = start_res->count;
+			res->last_consumed = start_res->last_consumed;
 			res->flags = start_res->flags;
 			res->type = start_res->type;
-			res->percent_used = 0;
+			res->allocated = 0;
 
 			xstrfmtcat(res_str, "  %s@%s\n",
 				   res->name, res->server);
@@ -604,7 +622,6 @@ extern int sacctmgr_add_res(int argc, char **argv)
 			ListIterator found_itr = NULL;
 			slurmdb_clus_res_rec_t *clus_res;
 			char *cluster;
-			uint16_t start_used = 0;
 
 			if (found_res) {
 				if (found_res->clus_res_list)
@@ -612,11 +629,16 @@ extern int sacctmgr_add_res(int argc, char **argv)
 						found_res->clus_res_list);
 				res = xmalloc(sizeof(slurmdb_res_rec_t));
 				slurmdb_init_res_rec(res, 0);
+				res->count = found_res->count;
+				res->last_consumed = found_res->last_consumed;
 				res->id = found_res->id;
+				xfree(res->name);
+				res->name = xstrdup(found_res->name);
 				res->type = found_res->type;
+				xfree(res->server);
 				res->server = xstrdup(found_res->server);
-				start_used = res->percent_used =
-					found_res->percent_used;
+				res->flags = found_res->flags;
+				res->allocated = found_res->allocated;
 			}
 
 			res->clus_res_list = list_create(
@@ -642,40 +664,17 @@ extern int sacctmgr_add_res(int argc, char **argv)
 						list_append(res_list, res);
 						added = true;
 					}
-					/* make sure we don't overcommit */
-					res->percent_used +=
-						start_res->percent_used;
-					if (res->percent_used > 100) {
-						exit_code = 1;
-						fprintf(stderr,
-							" Adding this %d "
-							"clusters to resource "
-							"%s@%s at %u%% each "
-							", with %u%% already "
-							"used,  would go over "
-							"100%%.  Please redo "
-							"your math and "
-							"resubmit.\n",
-							list_count(
-								cluster_list),
-							res->name, res->server,
-							start_res->percent_used,
-							start_used);
-						break;
-					}
 					clus_res = xmalloc(
 						sizeof(slurmdb_clus_res_rec_t));
 					list_append(res->clus_res_list,
 						    clus_res);
 					clus_res->cluster = xstrdup(cluster);
-					clus_res->percent_allowed =
-						start_res->percent_used;
+					clus_res->allowed =
+						start_res->allocated;
 					xstrfmtcat(res_str,
-						   "   Cluster - %s\t%u%%\n",
+						   "   Cluster - %s\t%u\n",
 						   cluster,
-						   clus_res->percent_allowed);
-					/* FIXME: make sure we don't
-					   overcommit */
+						   clus_res->allowed);
 				}
 			}
 
@@ -685,9 +684,6 @@ extern int sacctmgr_add_res(int argc, char **argv)
 			if (found_itr)
 				list_iterator_destroy(found_itr);
 
-			if (added && (res->percent_used > 100))
-				break;
-
 			list_iterator_reset(clus_itr);
 		}
 	}
@@ -732,6 +728,13 @@ extern int sacctmgr_add_res(int argc, char **argv)
 			printf("  ServerType     = %s\n", res->manager);
 		if (res->count != NO_VAL)
 			printf("  Count          = %u\n", res->count);
+		if (res->last_consumed != NO_VAL)
+			printf("  LastConsumed   = %u\n", res->last_consumed);
+		if (!(res->flags & SLURMDB_RES_FLAG_NOTSET)) {
+			char *res_tmp_str = slurmdb_res_flags_str(res->flags);
+			printf("  Flags          = %s\n", res_tmp_str);
+			xfree(res_tmp_str);
+		}
 		printf("  Type           = %s\n", tmp_str);
 
 		xfree(res_str);
@@ -795,10 +798,11 @@ extern int sacctmgr_list_res(int argc, char **argv)
 	} else if (!list_count(format_list)) {
 		slurm_addto_char_list(
 			format_list,
-			"Name,Server,Type,Count,Allocated,ServerType");
+			"Name,Server,Type,Count,LastConsumed,Allocated,ServerType");
 		if (res_cond->with_clusters)
 			slurm_addto_char_list(
 				format_list, "Cluster,Allowed");
+		slurm_addto_char_list(format_list, "Flags");
 	}
 
 	print_fields_list = sacctmgr_process_format_list(format_list);
@@ -902,9 +906,12 @@ extern int sacctmgr_modify_res(int argc, char **argv)
 		fprintf(stderr, "Can't change \"count\" on a cluster-based "
 			"resource. Remove cluster selection.\n");
 		return SLURM_ERROR;
-	} else if (res->percent_used != NO_VAL16 &&
-			!res_cond->cluster_list) {
-		fprintf(stderr, "Can't change \"percentallowed\" without "
+	} else if (res->last_consumed != NO_VAL && res_cond->cluster_list &&
+		   list_count(res_cond->cluster_list)) {
+		fprintf(stderr, "Can't change \"lastconsumed\" on a cluster-based resource. Remove cluster selection.\n");
+		return SLURM_ERROR;
+	} else if ((res->allocated != NO_VAL) && !res_cond->cluster_list) {
+		fprintf(stderr, "Can't change \"allowed\" without "
 			"specifying a cluster.\n");
 		return SLURM_ERROR;
 	}
