@@ -790,7 +790,7 @@ static void _usage(void)
 "              [--delay-boot=mins] [--use-min-nodes]\n"
 "              [--cpus-per-gpu=n] [--gpus=n] [--gpu-bind=...] [--gpu-freq=...]\n"
 "              [--gpus-per-node=n] [--gpus-per-socket=n] [--gpus-per-task=n]\n"
-"              [--mem-per-gpu=MB] [--tres-per-task=list]\n"
+"              [--mem-per-gpu=MB] [--tres-bind=...] [--tres-per-task=list]\n"
 "              [command [args...]]\n");
 }
 
@@ -860,6 +860,7 @@ static void _help(void)
 "      --thread-spec=threads   count of reserved threads\n"
 "  -t, --time=minutes          time limit\n"
 "      --time-min=minutes      minimum time limit (if distinct)\n"
+"      --tres-bind=...         task to tres binding options\n"
 "      --tres-per-task=list    list of tres required per task\n"
 "      --use-min-nodes         if a range of node counts is given, prefer the\n"
 "                              smaller count\n"
