@@ -139,7 +139,7 @@ int sattach(int argc, char **argv)
 		log_alter(logopt, 0, NULL);
 	}
 
-	if (slurm_cred_init() != SLURM_SUCCESS) {
+	if (cred_g_init() != SLURM_SUCCESS) {
 		error("failed to initialize cred plugin");
 		exit(error_exit);
 	}
@@ -258,7 +258,7 @@ _nodeid_from_layout(slurm_step_layout_t *layout, uint32_t taskid)
 
 static void print_layout_info(slurm_step_layout_t *layout)
 {
-	hostlist_t nl;
+	hostlist_t *nl;
 	int i, j;
 
 	printf("Job step layout:\n");
@@ -414,6 +414,7 @@ static int _attach_to_tasks(slurm_step_id_t stepid,
 		hosts = layout->front_end;
 	else
 		hosts = layout->node_list;
+	fwd_set_alias_addrs(layout->alias_addrs);
 	nodes_resp = slurm_send_recv_msgs(hosts, &msg, timeout);
 	if (nodes_resp == NULL) {
 		error("slurm_send_recv_msgs failed: %m");
@@ -526,7 +527,7 @@ _launch_handler(message_thread_state_t *mts, slurm_msg_t *resp)
 static void
 _exit_handler(message_thread_state_t *mts, slurm_msg_t *exit_msg)
 {
-	task_exit_msg_t *msg = (task_exit_msg_t *) exit_msg->data;
+	task_exit_msg_t *msg = exit_msg->data;
 	int i;
 	int rc;
 
@@ -597,8 +598,8 @@ _handle_msg(void *arg, slurm_msg_t *msg)
 		/* FIXME - does nothing yet */
 		break;
 	default:
-		error("received spurious message type: %d",
-		      msg->msg_type);
+		error("received spurious message type: %s",
+		      rpc_num2string(msg->msg_type));
 		break;
 	}
 	return;
