@@ -39,29 +39,28 @@
 \*****************************************************************************/
 
 #include <fcntl.h>
+#include <limits.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/param.h>               /* MAXPATHLEN */
 #include <sys/resource.h> /* for RLIMIT_NOFILE */
 
 #include "slurm/slurm.h"
 
-#include "src/common/cli_filter.h"
+#include "src/interfaces/cli_filter.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/env.h"
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 #include "src/common/pack.h"
-#include "src/common/plugstack.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
 #include "src/common/run_command.h"
-#include "src/common/select.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_rlimits_info.h"
+#include "src/common/spank.h"
 #include "src/common/xstring.h"
 #include "src/common/xmalloc.h"
 
@@ -102,9 +101,12 @@ int main(int argc, char **argv)
 	if (!isatty(STDERR_FILENO))
 		setvbuf(stderr, NULL, _IOLBF, 0);
 
-	slurm_conf_init(NULL);
+	slurm_init(NULL);
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
 
+	if (cli_filter_init() != SLURM_SUCCESS)
+		fatal("failed to initialize cli_filter plugin");
+
 	_set_exit_code();
 	if (spank_init_allocator() < 0) {
 		error("Failed to initialize plugin stack");
@@ -137,7 +139,7 @@ int main(int argc, char **argv)
 	if (script_body == NULL)
 		exit(error_exit);
 
-	het_job_argc = argc - sbopt.script_argc;
+	het_job_argc = argc - opt.argc;
 	het_job_argv = argv;
 	for (het_job_inx = 0; !het_job_fini; het_job_inx++) {
 		bool more_het_comps = false;
@@ -171,7 +173,7 @@ int main(int argc, char **argv)
 			}
 			run_command_add_to_script(&script_body,
 						  get_buf_data(buf));
-			free_buf(buf);
+			FREE_NULL_BUFFER(buf);
 		}
 
 		if (spank_init_post_opt() < 0) {
@@ -186,13 +188,6 @@ int main(int argc, char **argv)
 			(void) _set_rlimit_env();
 		}
 
-		/*
-		 * if the environment is coming from a file, the
-		 * environment at execution startup, must be unset.
-		 */
-		if (sbopt.export_file != NULL)
-			env_unset_environment();
-
 		_set_prio_process_env();
 		_set_spank_env();
 		_set_submit_dir_env();
@@ -344,9 +339,9 @@ int main(int argc, char **argv)
 		rc = _job_wait(resp->job_id);
 
 #ifdef MEMORY_LEAK_DEBUG
-	select_g_fini();
+	cli_filter_fini();
 	slurm_reset_all_options(&opt, false);
-	slurm_auth_fini();
+	auth_g_fini();
 	slurm_conf_destroy();
 	log_fini();
 #endif /* MEMORY_LEAK_DEBUG */
@@ -379,8 +374,7 @@ static int _job_wait(uint32_t job_id)
 		rc = slurm_load_job(&resp, job_id, SHOW_ALL);
 		if (rc == SLURM_SUCCESS) {
 			for (i = 0, job_ptr = resp->job_array;
-			     (i < resp->record_count) && complete;
-			     i++, job_ptr++) {
+			     (i < resp->record_count); i++, job_ptr++) {
 				if (IS_JOB_FINISHED(job_ptr)) {
 					if (WIFEXITED(job_ptr->exit_code)) {
 						ec2 = WEXITSTATUS(job_ptr->
@@ -423,23 +417,24 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 	desc->array_inx = sbopt.array_inx;
 	desc->batch_features = sbopt.batch_features;
 	desc->container = xstrdup(opt.container);
+	xfree(desc->container_id);
+	desc->container_id = xstrdup(opt.container_id);
 
 	desc->wait_all_nodes = sbopt.wait_all_nodes;
 
 	desc->environment = NULL;
-	if (sbopt.export_file) {
-		desc->environment = env_array_from_file(sbopt.export_file);
-		if (desc->environment == NULL)
-			exit(1);
-	}
 	if (opt.export_env == NULL) {
 		env_array_merge(&desc->environment, (const char **) environ);
 	} else if (!xstrcasecmp(opt.export_env, "ALL")) {
 		env_array_merge(&desc->environment, (const char **) environ);
+	} else if (!xstrcasecmp(opt.export_env, "NIL")) {
+		desc->environment = env_array_create();
+		env_array_merge_slurm_spank(&desc->environment,
+					    (const char **) environ);
 	} else if (!xstrcasecmp(opt.export_env, "NONE")) {
 		desc->environment = env_array_create();
-		env_array_merge_slurm(&desc->environment,
-				      (const char **)environ);
+		env_array_merge_slurm_spank(&desc->environment,
+					    (const char **) environ);
 		opt.get_user_env_time = 0;
 	} else {
 		env_merge_filter(&opt, desc);
@@ -458,8 +453,8 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 
 	desc->env_size = envcount(desc->environment);
 
-	desc->argc     = sbopt.script_argc;
-	desc->argv     = sbopt.script_argv;
+	desc->argc     = opt.argc;
+	desc->argv     = opt.argv;
 	desc->std_err  = xstrdup(opt.efname);
 	desc->std_in   = xstrdup(opt.ifname);
 	desc->std_out  = xstrdup(opt.ofname);
@@ -501,9 +496,9 @@ static void _set_spank_env(void)
  * current state */
 static void _set_submit_dir_env(void)
 {
-	char buf[MAXPATHLEN + 1], host[256];
+	char buf[PATH_MAX], host[256];
 
-	if ((getcwd(buf, MAXPATHLEN)) == NULL)
+	if ((getcwd(buf, PATH_MAX)) == NULL)
 		error("getcwd failed: %m");
 	else if (setenvf(NULL, "SLURM_SUBMIT_DIR", "%s", buf) < 0)
 		error("unable to set SLURM_SUBMIT_DIR in environment");
@@ -643,10 +638,20 @@ static void *_get_script_buffer(const char *filename, int *size)
 			error("Unable to open file %s", filename);
 			goto fail;
 		}
+		/* Quick check against absurdly large file sizes */
+		if (fd != STDIN_FILENO) {
+			struct stat st;
+			if (fstat(fd, &st) == -1)
+				fatal("Cannot stat %s: %m", filename);
+			if (st.st_size > MAX_BATCH_SCRIPT_SIZE)
+				fatal("Script file %s is too large",
+				      filename);
+			buf_size = st.st_size + 1;
+		}
 	}
 
 	/*
-	 * Then read in the script.
+	 * Then read in the script, in chunks of BUFSIZE bytes.
 	 */
 	buf = ptr = xmalloc(buf_size);
 	buf_left = buf_size;
@@ -654,7 +659,24 @@ static void *_get_script_buffer(const char *filename, int *size)
 		buf_left -= tmp_size;
 		script_size += tmp_size;
 		if (buf_left == 0) {
-			buf_size += BUFSIZ;
+			if (buf_size >= MAX_BATCH_SCRIPT_SIZE) {
+				if (filename)
+					close(fd);
+				xfree(buf);
+				if (filename)
+					fatal("Script %s is too big, read %d > %d bytes.",
+					      filename, script_size,
+					      MAX_BATCH_SCRIPT_SIZE);
+				else
+					fatal("Script from STDIN is too big, read %d > %d bytes.",
+					      script_size,
+					      MAX_BATCH_SCRIPT_SIZE);
+			}
+
+			if (buf_size < (MAX_BATCH_SCRIPT_SIZE - BUFSIZ))
+				buf_size += BUFSIZ;
+			else
+				buf_size = MAX_BATCH_SCRIPT_SIZE;
 			xrealloc(buf, buf_size);
 		}
 		ptr = buf + script_size;
