@@ -34,7 +34,7 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
-#include "limits.h"
+#include <limits.h>
 
 #include "slurm/slurm.h"
 
@@ -51,7 +51,7 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-#include "scrun.h"
+#include "src/scrun/scrun.h"
 
 /* max number of seconds to delay while waiting for job */
 #define MAX_DELAY 60
@@ -125,6 +125,7 @@ static const env_vars_t env_vars[] = {
 	{ "SCRUN_THREADS_PER_CORE", LONG_OPT_THREADSPERCORE },
 	{ "SCRUN_THREADS", 'T' },
 	{ "SCRUN_TIMELIMIT", 't' },
+	{ "SCRUN_TRES_BIND", LONG_OPT_TRES_BIND },
 	{ "SCRUN_TRES_PER_TASK", LONG_OPT_TRES_PER_TASK },
 	{ "SCRUN_UNBUFFEREDIO", 'u' },
 	{ "SCRUN_USE_MIN_NODES", LONG_OPT_USE_MIN_NODES },
@@ -189,7 +190,7 @@ static void _script_env(void)
 	if ((state.group_id != NO_VAL) &&
 	    (state.group_id != SLURM_AUTH_NOBODY)) {
 		/* set group if we know it but we may not in a user namespace */
-		char *u = gid_to_string_or_null(state.group_id);
+		char *u = gid_to_string(state.group_id);
 		_set_env(state, "SCRUN_GROUP", u);
 		xfree(u);
 		_set_env_args(state, "SCRUN_GROUP_ID", "%u", state.group_id);
@@ -243,15 +244,16 @@ static int _stage_in()
 	return rc;
 }
 
-static void *_on_connection(con_mgr_fd_t *con, void *arg)
+static void *_on_connection(conmgr_fd_t *con, void *arg)
 {
-	debug("%s:[%s] new srun connection", __func__, con->name);
+	debug("%s:[%s] new srun connection",
+	      __func__, conmgr_fd_get_name(con));
 
 	/* must return !NULL or connection will be closed */
 	return con;
 }
 
-static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
+static int _on_msg(conmgr_fd_t *con, slurm_msg_t *msg, void *arg)
 {
 	int rc = SLURM_SUCCESS;
 	xassert(arg == con);
@@ -269,10 +271,11 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		response_init(&resp_msg, msg, RESPONSE_SLURM_RC, &rc_msg);
 		resp_msg.data_size = sizeof(rc_msg);
 
-		rc = con_mgr_queue_write_msg(con, &resp_msg);
+		rc = conmgr_queue_write_msg(con, &resp_msg);
 		/* nothing to xfree() */
 
-		debug("%s:[%s] srun RPC PING has been PONGED", __func__, con->name);
+		debug("%s:[%s] srun RPC PING has been PONGED",
+		      __func__, conmgr_fd_get_name(con));
 		break;
 	}
 	case SRUN_JOB_COMPLETE:
@@ -280,7 +283,8 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		xassert(sizeof(srun_job_complete_msg_t) ==
 			sizeof(slurm_step_id_t));
 		slurm_step_id_t *step = msg->data;
-		debug("%s:[%s] %pS complete srun RPC", __func__, con->name, step);
+		debug("%s:[%s] %pS complete srun RPC",
+		      __func__, conmgr_fd_get_name(con), step);
 		stop_anchor(SLURM_SUCCESS);
 		break;
 	}
@@ -288,7 +292,8 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 	{
 		srun_timeout_msg_t *to = msg->data;
 		debug("%s:[%s] srun RPC %pS timeout at %ld RPC",
-		      __func__, con->name, &to->step_id, to->timeout);
+		      __func__, conmgr_fd_get_name(con), &to->step_id,
+		      to->timeout);
 		stop_anchor(ESLURM_JOB_TIMEOUT_KILLED);
 		break;
 	}
@@ -297,7 +302,7 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		srun_user_msg_t *um = msg->data;
 
 		debug("%s:[%s] JobId=%u srun user message RPC",
-		      __func__, con->name, um->job_id);
+		      __func__, conmgr_fd_get_name(con), um->job_id);
 
 		print_multi_line_string(um->msg, -1, LOG_LEVEL_INFO);
 		break;
@@ -306,7 +311,8 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 	{
 		srun_node_fail_msg_t *nf = msg->data;
 		debug("%s:[%s] srun RPC %pS nodes failed: %s",
-		      __func__, con->name, &nf->step_id, nf->nodelist);
+		      __func__, conmgr_fd_get_name(con), &nf->step_id,
+		      nf->nodelist);
 		stop_anchor(ESLURM_JOB_NODE_FAIL_KILLED);
 		break;
 	}
@@ -315,7 +321,7 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		suspend_msg_t *sus_msg = msg->data;
 		rc = SLURM_UNEXPECTED_MSG_ERROR;
 		error("%s:[%s] rejecting srun suspend RPC for %s",
-		      __func__, con->name, sus_msg->job_id_str);
+		      __func__, conmgr_fd_get_name(con), sus_msg->job_id_str);
 		break;
 	}
 	case SRUN_NET_FORWARD:
@@ -323,13 +329,14 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 		net_forward_msg_t *net = msg->data;
 		rc = SLURM_UNEXPECTED_MSG_ERROR;
 		error("%s:[%s] rejecting srun net forward RPC for %s",
-		      __func__, con->name, net->target);
+		      __func__, conmgr_fd_get_name(con), net->target);
 		break;
 	}
 	default:
 		rc = SLURM_UNEXPECTED_MSG_ERROR;
-		error("%s:[%s] received spurious srun message type: %u",
-		      __func__, con->name, msg->msg_type);
+		error("%s:[%s] received spurious srun message type: %s",
+		      __func__, conmgr_fd_get_name(con),
+		      rpc_num2string(msg->msg_type));
 	}
 
 	return rc;
@@ -337,12 +344,12 @@ static int _on_msg(con_mgr_fd_t *con, slurm_msg_t *msg, void *arg)
 
 static void _on_finish(void *arg)
 {
-	con_mgr_fd_t *con = arg;
+	conmgr_fd_t *con = arg;
 
 	if (get_log_level() > LOG_LEVEL_DEBUG) {
 		read_lock_state();
 		debug("%s: [%s] closed srun connection state=%s",
-		      __func__, con->name,
+		      __func__, conmgr_fd_get_name(con),
 		      slurm_container_status_to_str(state.status));
 		unlock_state();
 	}
@@ -352,9 +359,9 @@ static void _on_finish(void *arg)
  * Listen on srun port to make sure that slurmctld doesn't mark job as dead
  * RET port listening on
  */
-static uint32_t _setup_listener(con_mgr_t *conmgr)
+static uint32_t _setup_listener(void)
 {
-	static const con_mgr_events_t events = {
+	static const conmgr_events_t events = {
 		.on_connection = _on_connection,
 		.on_msg = _on_msg,
 		.on_finish = _on_finish,
@@ -379,9 +386,9 @@ static uint32_t _setup_listener(con_mgr_t *conmgr)
 	xassert(port > 0);
 	debug("%s: listening for srun RPCs on port=%hu", __func__, port);
 
-	if ((rc = con_mgr_process_fd(conmgr, CON_TYPE_RPC, fd, fd, events, NULL,
-				     0, NULL)))
-		fatal("%s: conmgr refuesed fd=%d: %s",
+	if ((rc = conmgr_process_fd(CON_TYPE_RPC, fd, fd, events, NULL, 0,
+				    NULL)))
+		fatal("%s: conmgr refused fd=%d: %s",
 		      __func__, fd, slurm_strerror(rc));
 
 	return port;
@@ -393,9 +400,8 @@ static void _pending_callback(uint32_t job_id)
 }
 
 /* check allocation has all nodes ready */
-extern void check_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
-			     con_mgr_work_type_t type,
-			     con_mgr_work_status_t status, const char *tag,
+extern void check_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
+			     conmgr_work_status_t status, const char *tag,
 			     void *arg)
 {
 	/* there must be only 1 thread that will call this at any one time */
@@ -423,7 +429,7 @@ extern void check_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
 
 	if (status != CONMGR_WORK_STATUS_RUN) {
 		debug("%s: bailing due to callback status %s",
-		      __func__, con_mgr_work_status_string(status));
+		      __func__, conmgr_work_status_string(status));
 		stop_anchor(ESLURM_ALREADY_DONE);
 		return;
 	}
@@ -446,8 +452,8 @@ extern void check_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
 			      __func__, state.jobid, delay);
 			unlock_state();
 		}
-		con_mgr_add_delayed_work(conmgr, NULL, check_allocation, delay,
-					 0, NULL, "check_allocation");
+		conmgr_add_delayed_work(NULL, check_allocation, delay, 0, NULL,
+					"check_allocation");
 	} else if ((rc == READY_JOB_FATAL) || !(rc & READY_JOB_STATE)) {
 		/* job failed! */
 		if (get_log_level() >= LOG_LEVEL_DEBUG) {
@@ -470,13 +476,13 @@ extern void check_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
 			stop_anchor(rc);
 		} else {
 			/* we have a job now. see if creating is done */
-			con_mgr_add_work(conmgr, NULL, on_allocation,
+			conmgr_add_work(NULL, on_allocation,
 					 CONMGR_WORK_TYPE_FIFO, NULL, __func__);
 		}
 	}
 }
 
-static void _alloc_job(con_mgr_t *conmgr)
+static void _alloc_job(void)
 {
 	int rc;
 	resource_allocation_response_msg_t *alloc = NULL;
@@ -524,7 +530,7 @@ static void _alloc_job(con_mgr_t *conmgr)
 	desc->user_id = SLURM_AUTH_NOBODY;
 	desc->group_id = SLURM_AUTH_NOBODY;
 	desc->name = xstrdup("scrun");
-	desc->other_port = _setup_listener(conmgr);
+	desc->other_port = _setup_listener();
 
 	debug("%s: requesting allocation with %u tasks and %u hosts",
 	      __func__, (desc->num_tasks == NO_VAL ? 1 : desc->num_tasks),
@@ -541,8 +547,8 @@ static void _alloc_job(con_mgr_t *conmgr)
 	}
 
 	if (get_log_level() >= LOG_LEVEL_DEBUG) {
-		char *user = uid_to_string_or_null(alloc->uid);
-		char *group = gid_to_string_or_null(alloc->gid);
+		char *user = uid_to_string(alloc->uid);
+		char *group = gid_to_string(alloc->gid);
 
 		debug("allocated jobId=%u user[%u]=%s group[%u]=%s",
 		      alloc->job_id, alloc->uid, user, alloc->uid, group);
@@ -586,9 +592,8 @@ static void _alloc_job(con_mgr_t *conmgr)
 	slurm_free_resource_allocation_response_msg(alloc);
 }
 
-extern void get_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
-			   con_mgr_work_type_t type,
-			   con_mgr_work_status_t status, const char *tag,
+extern void get_allocation(conmgr_fd_t *con, conmgr_work_type_t type,
+			   conmgr_work_status_t status, const char *tag,
 			   void *arg)
 {
 	int rc;
@@ -613,12 +618,13 @@ extern void get_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
 
 		/* scrape SLURM_* from calling env */
 		state.job_env = env_array_create();
-		env_array_merge_slurm(&state.job_env, (const char **) environ);
+		env_array_merge_slurm_spank(&state.job_env,
+					    (const char **) environ);
 		unlock_state();
 
 		debug("Running under existing JobId=%u", job_id);
 	} else {
-		_alloc_job(conmgr);
+		_alloc_job();
 
 		read_lock_state();
 		job_id = state.jobid;
@@ -672,10 +678,10 @@ extern void get_allocation(con_mgr_t *conmgr, con_mgr_fd_t *con,
 		if ((rc = _stage_in()))
 			stop_anchor(rc);
 		else
-			con_mgr_add_work(conmgr, NULL, on_allocation,
+			conmgr_add_work(NULL, on_allocation,
 					 CONMGR_WORK_TYPE_FIFO, NULL, __func__);
 	} else {
-		con_mgr_add_delayed_work(conmgr, NULL, check_allocation, 0, 1,
-					 NULL, "check_allocation");
+		conmgr_add_delayed_work(NULL, check_allocation, 0, 1, NULL,
+					 "check_allocation");
 	}
 }
