@@ -53,8 +53,13 @@ typedef struct {
 	 * this field.
 	 */
 	pthread_rwlock_t lock; /* lock must be used for access */
+
+#ifndef NDEBUG
+	/* lock must be used for access needs_lock and locked*/
+	pthread_mutex_t debug_lock;
 	bool needs_lock;
 	int locked;
+#endif
 
 	/* every field required by OCI runtime-spec v1.0.2 state query. */
 	char *oci_version;
@@ -111,9 +116,11 @@ extern void check_state(void);
 extern void init_state(void);
 extern void destroy_state(void);
 
+#ifndef NDEBUG
 #define read_lock_state()                                                    \
 	do {                                                                 \
 		slurm_rwlock_rdlock(&state.lock);                            \
+		slurm_mutex_lock(&state.debug_lock);                         \
 		xassert(state.needs_lock);                                   \
 		xassert(state.locked >= 0);                                  \
 		state.locked++;                                              \
@@ -121,15 +128,18 @@ extern void destroy_state(void);
 		debug3("%s: taking state read lock needs_lock=%c locked=%d", \
 		       __func__, (state.needs_lock ? 'T' : 'F'),             \
 		       state.locked);                                        \
+		slurm_mutex_unlock(&state.debug_lock);                         \
 	} while (false)
 
 #define write_lock_state()                                                    \
 	do {                                                                  \
 		slurm_rwlock_wrlock(&state.lock);                             \
+		slurm_mutex_lock(&state.debug_lock);                          \
 		xassert(state.needs_lock);                                    \
 		xassert(state.locked >= 0);                                   \
 		state.locked++;                                               \
 		check_state();                                                \
+		slurm_mutex_unlock(&state.debug_lock);                        \
 		debug3("%s: taking state write lock needs_lock=%c locked=%d", \
 		       __func__, (state.needs_lock ? 'T' : 'F'),              \
 		       state.locked);                                         \
@@ -137,6 +147,7 @@ extern void destroy_state(void);
 
 #define unlock_state()                                                       \
 	do {                                                                 \
+		slurm_mutex_lock(&state.debug_lock);                         \
 		debug3("%s: unlock state needs_lock=%c locked=%d",           \
 		       __func__, (state.needs_lock ? 'T' : 'F'),             \
 		       state.locked);                                        \
@@ -144,8 +155,14 @@ extern void destroy_state(void);
 		xassert(state.needs_lock);                                   \
 		xassert(state.locked > 0);                                   \
 		state.locked--;                                              \
+		slurm_mutex_unlock(&state.debug_lock);                       \
 		slurm_rwlock_unlock(&state.lock);                            \
 	} while (false)
+#else /* NDEBUG */
+#define read_lock_state() slurm_rwlock_rdlock(&state.lock)
+#define write_lock_state() slurm_rwlock_wrlock(&state.lock)
+#define unlock_state() slurm_rwlock_unlock(&state.lock)
+#endif /* NDEBUG */
 
 extern int command_create(void);
 extern int command_start(void);
