@@ -50,6 +50,7 @@
 #include "src/common/list.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
+#include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
@@ -150,7 +151,7 @@ static int _resv_name_width(reserve_info_t *resv_ptr)
 
 static void _print_reservation(reserve_info_t *resv_ptr, int width)
 {
-	char format[64], tmp1[32], tmp2[32], tmp3[32];
+	char format[64], tmp1[256], tmp2[256], tmp3[32];
 	char *state = "INACTIVE";
 	uint32_t duration;
 	time_t now = time(NULL);
@@ -983,6 +984,21 @@ int _print_reason(sinfo_data_t * sinfo_data, int width,
 	return SLURM_SUCCESS;
 }
 
+int _print_resv_name(sinfo_data_t *sinfo_data, int width,
+		     bool right_justify, char *suffix)
+{
+	if (sinfo_data) {
+		char *resv_name = sinfo_data->resv_name ?
+			sinfo_data->resv_name : "";
+		_print_str(resv_name, width, right_justify, true);
+	} else
+		_print_str("RESERVATION", width, right_justify, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_root(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
@@ -1081,7 +1097,8 @@ int _print_state_complete(sinfo_data_t * sinfo_data, int width,
 		uint32_t my_state;
 
 		my_state = sinfo_data->node_state;
-		state = xstrtolower(node_state_string_complete(my_state));
+		state = node_state_string_complete(my_state);
+		xstrtolower(state);
 		_print_str(state, width, right_justify, true);
 		xfree(state);
 	} else if (sinfo_data)
@@ -1140,7 +1157,7 @@ int _print_timestamp(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	if (sinfo_data && sinfo_data->reason_time) {
-		char time_str[32];
+		char time_str[256];
 		slurm_make_time_str(&sinfo_data->reason_time,
 				    time_str, sizeof(time_str));
 		_print_str(time_str, width, right_justify, true);
@@ -1158,15 +1175,13 @@ int _print_user(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	if (sinfo_data && (sinfo_data->reason_uid != NO_VAL)) {
-		char user[FORMAT_STRING_SIZE];
-		struct passwd *pw = NULL;
+		char *user = uid_to_string_or_null(sinfo_data->reason_uid);
 
-		if ((pw=getpwuid(sinfo_data->reason_uid)))
-			snprintf(user, sizeof(user), "%s", pw->pw_name);
-		else
-			snprintf(user, sizeof(user), "Unk(%u)",
-				 sinfo_data->reason_uid);
+		if (!user)
+			xstrfmtcat(user, "Unk(%u)", sinfo_data->reason_uid);
 		_print_str(user, width, right_justify, true);
+
+		xfree(user);
 	} else if (sinfo_data)
 		_print_str("Unknown", width, right_justify, true);
 	else
@@ -1181,16 +1196,14 @@ int _print_user_long(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	if (sinfo_data && (sinfo_data->reason_uid != NO_VAL)) {
-		char user[FORMAT_STRING_SIZE];
-		struct passwd *pw = NULL;
+		char *user = uid_to_string_or_null(sinfo_data->reason_uid);
 
-		if ((pw=getpwuid(sinfo_data->reason_uid)))
-			snprintf(user, sizeof(user), "%s(%u)", pw->pw_name,
-				 sinfo_data->reason_uid);
-		else
-			snprintf(user, sizeof(user), "Unk(%u)",
-				 sinfo_data->reason_uid);
+		if (!user)
+			user = xstrdup("Unk");
+		xstrfmtcat(user, "(%u)", sinfo_data->reason_uid);
 		_print_str(user, width, right_justify, true);
+
+		xfree(user);
 	} else if (sinfo_data)
 		_print_str("Unknown", width, right_justify, true);
 	else
