@@ -259,8 +259,9 @@ static int _get_info(bool clear_old, slurmdb_federation_rec_t *fed,
 
 	sort_sinfo_list(sinfo_list);
 	if (params.mimetype)
-		rc = DATA_DUMP_CLI(SINFO_DATA_LIST, sinfo_list, "sinfo", argc,
-				   argv, NULL, params.mimetype);
+		DATA_DUMP_CLI_SINGLE(OPENAPI_SINFO_RESP, sinfo_list, argc, argv,
+				     NULL, params.mimetype, params.data_parser,
+				     rc);
 	else
 		rc = print_sinfo_list(sinfo_list);
 
@@ -335,9 +336,7 @@ static List _query_server(bool clear_old)
 	static partition_info_msg_t *old_part_ptr = NULL, *new_part_ptr;
 	static node_info_msg_t *old_node_ptr = NULL, *new_node_ptr;
 	int error_code;
-	uint16_t show_flags = 0;
-	int cc;
-	node_info_t *node_ptr;
+	uint16_t show_flags = SHOW_MIXED;
 	List sinfo_list = NULL;
 
 	if (params.all_flag)
@@ -400,28 +399,6 @@ static List _query_server(bool clear_old)
 	}
 	old_node_ptr = new_node_ptr;
 
-	/* Set the node state as NODE_STATE_MIXED. */
-	for (cc = 0; cc < new_node_ptr->record_count; cc++) {
-		node_ptr = &(new_node_ptr->node_array[cc]);
-		if (IS_NODE_DRAIN(node_ptr)) {
-			/* don't worry about mixed since the
-			 * whole node is being drained. */
-		} else {
-			uint16_t alloc_cpus = 0, idle_cpus;
-
-			select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-						     SELECT_NODEDATA_SUBCNT,
-						     NODE_STATE_ALLOCATED,
-						     &alloc_cpus);
-			idle_cpus = node_ptr->cpus_efctv - alloc_cpus;
-
-			if (idle_cpus && (idle_cpus != node_ptr->cpus_efctv)) {
-				node_ptr->node_state &= NODE_STATE_FLAGS;
-				node_ptr->node_state |= NODE_STATE_MIXED;
-			}
-		}
-	}
-
 	sinfo_list = list_create(_sinfo_list_delete);
 	_build_sinfo_data(sinfo_list, new_part_ptr, new_node_ptr);
 
@@ -431,14 +408,12 @@ static List _query_server(bool clear_old)
 static void *_load_job_prio_thread(void *args)
 {
 	load_info_struct_t *load_args = (load_info_struct_t *) args;
-	uint16_t show_flags = 0;
+	uint16_t show_flags = SHOW_MIXED;
 	char *node_name = NULL;
 	slurmdb_cluster_rec_t *cluster = load_args->cluster;
 	int error_code;
 	partition_info_msg_t *new_part_ptr;
 	node_info_msg_t *new_node_ptr;
-	int cc;
-	node_info_t *node_ptr;
 	List sinfo_list = NULL;
 
 	if (params.node_name_single)
@@ -453,7 +428,7 @@ static void *_load_job_prio_thread(void *args)
 	if (error_code) {
 		slurm_perror("slurm_load_partitions");
 		xfree(args);
-		return (void *) NULL;
+		return NULL;
 	}
 	list_append(load_args->part_info_msg_list, new_part_ptr);
 
@@ -467,32 +442,10 @@ static void *_load_job_prio_thread(void *args)
 	if (error_code) {
 		slurm_perror("slurm_load_node");
 		xfree(args);
-		return (void *) NULL;
+		return NULL;
 	}
 	list_append(load_args->node_info_msg_list, new_node_ptr);
 
-	/* Set the node state as NODE_STATE_MIXED. */
-	for (cc = 0; cc < new_node_ptr->record_count; cc++) {
-		node_ptr = &(new_node_ptr->node_array[cc]);
-		if (IS_NODE_DRAIN(node_ptr)) {
-			/* don't worry about mixed since the
-			 * whole node is being drained. */
-		} else {
-			uint16_t alloc_cpus = 0, idle_cpus;
-
-			select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-						     SELECT_NODEDATA_SUBCNT,
-						     NODE_STATE_ALLOCATED,
-						     &alloc_cpus);
-			idle_cpus = node_ptr->cpus_efctv - alloc_cpus;
-
-			if (idle_cpus && (idle_cpus != node_ptr->cpus_efctv)) {
-				node_ptr->node_state &= NODE_STATE_FLAGS;
-				node_ptr->node_state |= NODE_STATE_MIXED;
-			}
-		}
-	}
-
 	sinfo_list = list_create(_sinfo_list_delete);
 	_build_sinfo_data(sinfo_list, new_part_ptr, new_node_ptr);
 	if (sinfo_list) {
@@ -507,7 +460,7 @@ static void *_load_job_prio_thread(void *args)
 	}
 
 	xfree(args);
-	return (void *) NULL;
+	return NULL;
 }
 
 /*
@@ -660,7 +613,7 @@ static int _build_sinfo_data(List sinfo_list,
 
 		if (node_msg->record_count == 1) { /* node_name_single */
 			int pos = -1;
-			hostlist_t hl;
+			hostlist_t *hl;
 
 			node_ptr = &(node_msg->node_array[0]);
 			if ((node_ptr->name == NULL) ||
@@ -687,7 +640,7 @@ static int _build_sinfo_data(List sinfo_list,
 		sinfo_cnt++;
 		slurm_mutex_unlock(&sinfo_cnt_mutex);
 
-		slurm_thread_create_detached(NULL, _build_part_info,
+		slurm_thread_create_detached(_build_part_info,
 					     build_struct_ptr);
 	}
 
@@ -709,7 +662,7 @@ static int _build_sinfo_data(List sinfo_list,
  */
 static bool _filter_out(node_info_t *node_ptr)
 {
-	static hostlist_t host_list = NULL;
+	static hostlist_t *host_list = NULL;
 
 	if (params.nodes) {
 		if (host_list == NULL)
