@@ -84,7 +84,6 @@
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
 #include "src/common/parse_time.h"
 #include "src/common/run_command.h"
 #include "src/common/slurm_protocol_api.h"
@@ -94,12 +93,17 @@
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+
+#include "src/interfaces/select.h"
+
 #include "src/slurmctld/agent.h"
 #include "src/slurmctld/front_end.h"
 #include "src/slurmctld/job_scheduler.h"
 #include "src/slurmctld/locks.h"
 #include "src/slurmctld/ping_nodes.h"
+#include "src/slurmctld/sackd_mgr.h"
 #include "src/slurmctld/slurmctld.h"
+#include "src/slurmctld/slurmscriptd.h"
 #include "src/slurmctld/state_save.h"
 #include "src/slurmctld/srun_comm.h"
 
@@ -108,7 +112,7 @@
 #define RPC_PACK_MAX_AGE	30	/* Rebuild data over 30 seconds old */
 #define DUMP_RPC_COUNT 		25
 #define HOSTLIST_MAX_SIZE 	80
-#define MAIL_PROG_TIMEOUT 120*1000
+#define MAIL_PROG_TIMEOUT 120 /* Timeout in seconds */
 
 typedef enum {
 	DSH_NEW,        /* Request not yet started */
@@ -136,7 +140,8 @@ typedef struct thd {
 					 * upon termination */
 	slurm_addr_t *addr;		/* specific addr to send to
 					 * will not do nodelist if set */
-	char *nodelist;			/* list of nodes to send to */
+	hostlist_t *nodelist;		/* list of nodes to send to */
+	char *nodename;			/* node to send to */
 	List ret_list;
 } thd_t;
 
@@ -148,8 +153,10 @@ typedef struct agent_info {
 	uint16_t retry;			/* if set, keep trying */
 	thd_t *thread_struct;		/* thread structures */
 	bool get_reply;			/* flag if reply expected */
+	uid_t r_uid;			/* receiver UID */
 	slurm_msg_type_t msg_type;	/* RPC to be issued */
 	void **msg_args_pptr;		/* RPC data to be used */
+	uint16_t msg_flags;		/* Flags to be added to msg*/
 	uint16_t protocol_version;	/* if set, use this version */
 } agent_info_t;
 
@@ -161,8 +168,10 @@ typedef struct task_info {
 	uint32_t *threads_active_ptr;	/* currently active thread ptr */
 	thd_t *thread_struct_ptr;	/* thread structures ptr */
 	bool get_reply;			/* flag if reply expected */
+	uid_t r_uid;			/* receiver UID */
 	slurm_msg_type_t msg_type;	/* RPC to be issued */
 	void *msg_args_ptr;		/* ptr to RPC data to be used */
+	uint16_t msg_flags;		/* Flags to be added to msg*/
 	uint16_t protocol_version;	/* if set, use this version */
 } task_info_t;
 
@@ -193,6 +202,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		int no_resp_cnt, int retry_cnt);
 static void _purge_agent_args(agent_arg_t *agent_arg_ptr);
 static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count);
+static void _queue_update_node(char *node_name);
 static int  _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 			   int *count, int *spot);
 static void _sig_handler(int dummy);
@@ -214,6 +224,9 @@ static List defer_list = NULL;		/* agent_arg_t list for requests
 static List mail_list = NULL;		/* pending e-mail requests */
 static List retry_list = NULL;		/* agent_arg_t list for retry */
 
+static list_t *update_node_list = NULL;	/* node list for update */
+static pthread_mutex_t update_nodes_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t update_nodes_cond = PTHREAD_COND_INITIALIZER;
 
 static pthread_mutex_t agent_cnt_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  agent_cnt_cond  = PTHREAD_COND_INITIALIZER;
@@ -227,6 +240,7 @@ static pthread_cond_t  pending_cond = PTHREAD_COND_INITIALIZER;
 static int pending_wait_time = NO_VAL16;
 static bool pending_mail = false;
 static bool pending_thread_running = false;
+static bool pending_check_defer = false;
 
 static bool run_scheduler    = false;
 
@@ -317,11 +331,12 @@ void *agent(void *args)
 	/* start the watchdog thread */
 	slurm_thread_create(&thread_wdog, _wdog, agent_info_ptr);
 
-	log_flag(AGENT, "%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
+	log_flag(AGENT, "%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c r_uid:%u msg_type:%s protocol_version:%hu",
 		 __func__, agent_info_ptr->thread_count,
 		 agent_info_ptr->threads_active,
 		 agent_info_ptr->retry ? 'T' : 'F',
 		 agent_info_ptr->get_reply ? 'T' : 'F',
+		 agent_info_ptr->r_uid,
 		 rpc_num2string(agent_arg_ptr->msg_type),
 		 agent_info_ptr->protocol_version);
 
@@ -341,9 +356,9 @@ void *agent(void *args)
 		 */
 		task_specific_ptr = _make_task_data(agent_info_ptr, i);
 
-		slurm_thread_create_detached(&thread_ptr[i].thread,
-					     _thread_per_group_rpc,
-					     task_specific_ptr);
+		slurm_thread_create(&thread_ptr[i].thread,
+				    _thread_per_group_rpc,
+				    task_specific_ptr);
 		agent_info_ptr->threads_active++;
 		slurm_mutex_unlock(&agent_info_ptr->thread_mutex);
 	}
@@ -352,14 +367,16 @@ void *agent(void *args)
 	pthread_join(thread_wdog, NULL);
 	delay = (int) difftime(time(NULL), begin_time);
 	if (delay > (slurm_conf.msg_timeout * 2)) {
-		info("agent msg_type=%u ran for %d seconds",
-			agent_arg_ptr->msg_type,  delay);
+		info("agent msg_type=%s ran for %d seconds",
+		     rpc_num2string(agent_arg_ptr->msg_type),  delay);
 	}
 	slurm_mutex_lock(&agent_info_ptr->thread_mutex);
 	while (agent_info_ptr->threads_active != 0) {
 		slurm_cond_wait(&agent_info_ptr->thread_cond,
 				&agent_info_ptr->thread_mutex);
 	}
+	for (i = 0; i < agent_info_ptr->thread_count; i++)
+		pthread_join(thread_ptr[i].thread, NULL);
 	slurm_mutex_unlock(&agent_info_ptr->thread_mutex);
 
 	log_flag(AGENT, "%s: end agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
@@ -399,7 +416,7 @@ cleanup:
 	slurm_mutex_unlock(&agent_cnt_mutex);
 
 	if (spawn_retry_agent)
-		agent_trigger(RPC_RETRY_INTERVAL, true);
+		agent_trigger(RPC_RETRY_INTERVAL, true, false);
 
 	return NULL;
 }
@@ -420,18 +437,21 @@ static int _valid_agent_arg(agent_arg_t *agent_arg_ptr)
 		     __func__, agent_arg_ptr->node_count, hostlist_cnt);
 		return SLURM_ERROR;	/* no messages to be sent */
 	}
+	if (!agent_arg_ptr->r_uid_set) {
+		error("%s: r_uid not set for message:%u ",
+		      __func__, agent_arg_ptr->msg_type);
+		return SLURM_ERROR;
+	}
 	return SLURM_SUCCESS;
 }
 
 static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 {
-	int i = 0, j = 0;
 	agent_info_t *agent_info_ptr = NULL;
 	thd_t *thread_ptr = NULL;
-	int *span = NULL;
 	int thr_count = 0;
-	hostlist_t hl = NULL;
 	char *name = NULL;
+	bool split;
 
 	agent_info_ptr = xmalloc(sizeof(agent_info_t));
 	slurm_mutex_init(&agent_info_ptr->thread_mutex);
@@ -440,18 +460,25 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 	agent_info_ptr->retry          = agent_arg_ptr->retry;
 	agent_info_ptr->threads_active = 0;
 	thread_ptr = xcalloc(agent_info_ptr->thread_count, sizeof(thd_t));
-	memset(thread_ptr, 0, (agent_info_ptr->thread_count * sizeof(thd_t)));
 	agent_info_ptr->thread_struct  = thread_ptr;
+	agent_info_ptr->r_uid = agent_arg_ptr->r_uid;
 	agent_info_ptr->msg_type       = agent_arg_ptr->msg_type;
 	agent_info_ptr->msg_args_pptr  = &agent_arg_ptr->msg_args;
+	agent_info_ptr->msg_flags = agent_arg_ptr->msg_flags;
 	agent_info_ptr->protocol_version = agent_arg_ptr->protocol_version;
 
+	if (!agent_info_ptr->thread_count)
+		return agent_info_ptr;
+
+	xassert(agent_arg_ptr->node_count ==
+		hostlist_count(agent_arg_ptr->hostlist));
+
 	if ((agent_arg_ptr->msg_type != REQUEST_JOB_NOTIFY)	&&
 	    (agent_arg_ptr->msg_type != REQUEST_REBOOT_NODES)	&&
 	    (agent_arg_ptr->msg_type != REQUEST_RECONFIGURE)	&&
+	    (agent_arg_ptr->msg_type != REQUEST_RECONFIGURE_SACKD) &&
 	    (agent_arg_ptr->msg_type != REQUEST_RECONFIGURE_WITH_CONFIG) &&
 	    (agent_arg_ptr->msg_type != REQUEST_SHUTDOWN)	&&
-	    (agent_arg_ptr->msg_type != SRUN_EXEC)		&&
 	    (agent_arg_ptr->msg_type != SRUN_TIMEOUT)		&&
 	    (agent_arg_ptr->msg_type != SRUN_NODE_FAIL)		&&
 	    (agent_arg_ptr->msg_type != SRUN_REQUEST_SUSPEND)	&&
@@ -460,57 +487,68 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 	    (agent_arg_ptr->msg_type != SRUN_STEP_SIGNAL)	&&
 	    (agent_arg_ptr->msg_type != SRUN_JOB_COMPLETE)) {
 #ifdef HAVE_FRONT_END
-		span = set_span(agent_arg_ptr->node_count,
-				agent_arg_ptr->node_count);
+		split = true;
 #else
 		/* Sending message to a possibly large number of slurmd.
 		 * Push all message forwarding to slurmd in order to
 		 * offload as much work from slurmctld as possible. */
-		span = set_span(agent_arg_ptr->node_count, 1);
+		split = false;
 #endif
 		agent_info_ptr->get_reply = true;
 	} else {
 		/* Message is going to one node (for srun) or we want
 		 * it to get processed ASAP (SHUTDOWN or RECONFIGURE).
 		 * Send the message directly to each node. */
-		span = set_span(agent_arg_ptr->node_count,
-				agent_arg_ptr->node_count);
+		split = true;
 	}
-	i = 0;
-	while (i < agent_info_ptr->thread_count) {
-		thread_ptr[thr_count].state      = DSH_NEW;
-		thread_ptr[thr_count].addr = agent_arg_ptr->addr;
+	if (agent_arg_ptr->addr || !split) {
+		thread_ptr[0].state = DSH_NEW;
+		if (agent_arg_ptr->addr) {
+			name = hostlist_shift(agent_arg_ptr->hostlist);
+			thread_ptr[0].addr = agent_arg_ptr->addr;
+			thread_ptr[0].nodename = xstrdup(name);
+			if (agent_arg_ptr->node_count > 1)
+				error("%s: you will only be sending this to %s",
+				      __func__, name);
+			free(name);
+			log_flag(AGENT, "%s: sending msg_type %s to node %s",
+				 __func__,
+				 rpc_num2string(agent_arg_ptr->msg_type),
+				 thread_ptr[0].nodename);
+		} else {
+			thread_ptr[0].nodelist = agent_arg_ptr->hostlist;
+			thread_ptr[0].addr = NULL;
+			if (slurm_conf.debug_flags & DEBUG_FLAG_AGENT) {
+				char *buf;
+				buf = hostlist_ranged_string_xmalloc(
+						agent_arg_ptr->hostlist);
+				debug("%s: sending msg_type %s to nodes %s",
+				      __func__,
+				      rpc_num2string(agent_arg_ptr->msg_type),
+				      buf);
+				xfree(buf);
+			}
+		}
+		agent_info_ptr->thread_count = 1;
+		return agent_info_ptr;
+	}
+
+	hostlist_uniq(agent_arg_ptr->hostlist);
+	while (thr_count < agent_info_ptr->thread_count) {
 		name = hostlist_shift(agent_arg_ptr->hostlist);
 		if (!name) {
 			debug3("no more nodes to send to");
 			break;
 		}
-		hl = hostlist_create(name);
-		if (thread_ptr[thr_count].addr && span[thr_count]) {
-			debug("warning: you will only be sending this to %s",
-			      name);
-			span[thr_count] = 0;
-		}
-		free(name);
-		i++;
-		for (j = 0; j < span[thr_count]; j++) {
-			name = hostlist_shift(agent_arg_ptr->hostlist);
-			if (!name)
-				break;
-			hostlist_push_host(hl, name);
-			free(name);
-			i++;
-		}
-		hostlist_uniq(hl);
-		thread_ptr[thr_count].nodelist =
-			hostlist_ranged_string_xmalloc(hl);
-		hostlist_destroy(hl);
-		log_flag(AGENT, "%s: sending msg_type %s to nodes %s",
+		thread_ptr[thr_count].state = DSH_NEW;
+		thread_ptr[thr_count].addr = NULL;
+		thread_ptr[thr_count].nodename = xstrdup(name);
+		log_flag(AGENT, "%s: sending msg_type %s to node %s",
 			 __func__, rpc_num2string(agent_arg_ptr->msg_type),
-			 thread_ptr[thr_count].nodelist);
+			 thread_ptr[thr_count].nodename);
+		free(name);
 		thr_count++;
 	}
-	xfree(span);
 	agent_info_ptr->thread_count = thr_count;
 	return agent_info_ptr;
 }
@@ -525,8 +563,10 @@ static task_info_t *_make_task_data(agent_info_t *agent_info_ptr, int inx)
 	task_info_ptr->threads_active_ptr= &agent_info_ptr->threads_active;
 	task_info_ptr->thread_struct_ptr = &agent_info_ptr->thread_struct[inx];
 	task_info_ptr->get_reply         = agent_info_ptr->get_reply;
+	task_info_ptr->r_uid = agent_info_ptr->r_uid;
 	task_info_ptr->msg_type          = agent_info_ptr->msg_type;
 	task_info_ptr->msg_args_ptr      = *agent_info_ptr->msg_args_pptr;
+	task_info_ptr->msg_flags = agent_info_ptr->msg_flags;
 	task_info_ptr->protocol_version  = agent_info_ptr->protocol_version;
 
 	return task_info_ptr;
@@ -542,10 +582,8 @@ static void _update_wdog_state(thd_t *thread_ptr,
 		if (thread_ptr->end_time <= thd_comp->now) {
 			log_flag(AGENT, "%s: agent thread %lu timed out",
 				 __func__, (unsigned long) thread_ptr->thread);
-			if (pthread_kill(thread_ptr->thread, SIGUSR1) == ESRCH)
-				*state = DSH_NO_RESP;
-			else
-				thread_ptr->end_time += message_timeout;
+			(void) pthread_kill(thread_ptr->thread, SIGUSR1);
+			thread_ptr->end_time += message_timeout;
 		}
 		break;
 	case DSH_NEW:
@@ -574,7 +612,7 @@ static void _update_wdog_state(thd_t *thread_ptr,
  */
 static void *_wdog(void *args)
 {
-	bool srun_agent = false;
+	bool srun_agent = false, sack_agent = false;
 	int i;
 	agent_info_t *agent_ptr = (agent_info_t *) args;
 	thd_t *thread_ptr = agent_ptr->thread_struct;
@@ -587,7 +625,6 @@ static void *_wdog(void *args)
 	     (agent_ptr->msg_type == SRUN_REQUEST_SUSPEND)		||
 	     (agent_ptr->msg_type == SRUN_STEP_MISSING)			||
 	     (agent_ptr->msg_type == SRUN_STEP_SIGNAL)			||
-	     (agent_ptr->msg_type == SRUN_EXEC)				||
 	     (agent_ptr->msg_type == SRUN_NODE_FAIL)			||
 	     (agent_ptr->msg_type == SRUN_PING)				||
 	     (agent_ptr->msg_type == SRUN_TIMEOUT)			||
@@ -595,6 +632,8 @@ static void *_wdog(void *args)
 	     (agent_ptr->msg_type == RESPONSE_RESOURCE_ALLOCATION)	||
 	     (agent_ptr->msg_type == RESPONSE_HET_JOB_ALLOCATION) )
 		srun_agent = true;
+	if (agent_ptr->msg_type == REQUEST_RECONFIGURE_SACKD)
+		sack_agent = true;
 
 	thd_comp.max_delay = 0;
 
@@ -632,9 +671,12 @@ static void *_wdog(void *args)
 		slurm_mutex_unlock(&agent_ptr->thread_mutex);
 	}
 
-	if (srun_agent) {
+	if (sack_agent) {
+		if (thread_ptr[0].state != DSH_DONE)
+			sackd_mgr_remove_node(thread_ptr[0].nodename);
+	} else if (srun_agent) {
 		_notify_slurmctld_jobs(agent_ptr);
-	} else {
+	} else if (agent_ptr->msg_type != REQUEST_SHUTDOWN) {
 		_notify_slurmctld_nodes(agent_ptr,
 					thd_comp.no_resp_cnt,
 					thd_comp.retry_cnt);
@@ -642,7 +684,7 @@ static void *_wdog(void *args)
 
 	for (i = 0; i < agent_ptr->thread_count; i++) {
 		FREE_NULL_LIST(thread_ptr[i].ret_list);
-		xfree(thread_ptr[i].nodelist);
+		xfree(thread_ptr[i].nodename);
 	}
 
 	if (thd_comp.max_delay)
@@ -650,7 +692,7 @@ static void *_wdog(void *args)
 			 __func__, thd_comp.max_delay);
 
 	slurm_mutex_unlock(&agent_ptr->thread_mutex);
-	return (void *) NULL;
+	return NULL;
 }
 
 static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
@@ -686,7 +728,6 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 		   (agent_ptr->msg_type == SRUN_REQUEST_SUSPEND)	||
 		   (agent_ptr->msg_type == SRUN_STEP_MISSING)		||
 		   (agent_ptr->msg_type == SRUN_STEP_SIGNAL)		||
-		   (agent_ptr->msg_type == SRUN_EXEC)			||
 		   (agent_ptr->msg_type == SRUN_USER_MSG)) {
 		return;		/* no need to note srun response */
 	} else if (agent_ptr->msg_type == SRUN_NODE_FAIL) {
@@ -715,6 +756,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		{ .conf = READ_LOCK, .node = WRITE_LOCK };
 	thd_t *thread_ptr = agent_ptr->thread_struct;
 	int i;
+	bool locked = false;
 
 	/* Notify slurmctld of non-responding nodes */
 	if (no_resp_cnt) {
@@ -740,9 +782,8 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		_queue_agent_retry(agent_ptr, retry_cnt);
 
 	/* Update last_response on responding nodes */
-	lock_slurmctld(node_write_lock);
 	for (i = 0; i < agent_ptr->thread_count; i++) {
-		char *down_msg, *node_names;
+		char *down_msg, **node_names;
 		slurm_msg_type_t resp_type = RESPONSE_SLURM_RC;
 
 		if (!thread_ptr[i].ret_list) {
@@ -757,45 +798,70 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 			state = ret_data_info->err;
 		switch_on_state:
 			if (is_ret_list) {
-				node_names = ret_data_info->node_name;
+				node_names = &ret_data_info->node_name;
 				resp_type = ret_data_info->type;
 			} else
-				node_names = thread_ptr[i].nodelist;
+				node_names = &thread_ptr[i].nodename;
+
+			if (!is_ret_list && !(*node_names))
+				goto finished;
 
 			switch (state) {
 			case DSH_NO_RESP:
-				node_not_resp(node_names,
+				if (!locked) {
+					locked = true;
+					lock_slurmctld(node_write_lock);
+				}
+				node_not_resp(*node_names,
 					      thread_ptr[i].start_time,
 					      resp_type);
 				break;
 			case DSH_FAILED:
+				if (!locked) {
+					locked = true;
+					lock_slurmctld(node_write_lock);
+				}
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names, "Prolog/Epilog failure",
+				drain_nodes(*node_names, "Prolog/Epilog failure",
 				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Prolog/Epilog failure on nodes %s%s",
-				      node_names, down_msg);
+				      *node_names, down_msg);
 				break;
 			case DSH_DUP_JOBID:
+				if (!locked) {
+					locked = true;
+					lock_slurmctld(node_write_lock);
+				}
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names, "Duplicate jobid",
+				drain_nodes(*node_names, "Duplicate jobid",
 				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Duplicate jobid on nodes %s%s",
-				      node_names, down_msg);
+				      *node_names, down_msg);
 				break;
 			case DSH_DONE:
-				node_did_resp(node_names);
+				/*
+				 * Process now if we've already obtained the
+				 * lock. Otherwise delegate to the dedicated
+				 * processing thread.
+				 */
+				if (locked) {
+					node_did_resp(*node_names);
+				} else {
+					_queue_update_node(*node_names);
+					*node_names = NULL;
+				}
 				break;
 			default:
 				error("unknown state returned for %s",
-				      node_names);
+				      *node_names);
 				break;
 			}
 			if (!is_ret_list)
@@ -804,11 +870,13 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		list_iterator_destroy(itr);
 finished:	;
 	}
-	unlock_slurmctld(node_write_lock);
+	if (locked)
+		unlock_slurmctld(node_write_lock);
+
 	if (run_scheduler) {
 		run_scheduler = false;
 		/* below functions all have their own locking */
-		schedule(false);
+		queue_job_scheduler();
 	}
 	if ((agent_ptr->msg_type == REQUEST_PING) ||
 	    (agent_ptr->msg_type == REQUEST_HEALTH_CHECK) ||
@@ -871,7 +939,7 @@ static void *_thread_per_group_rpc(void *args)
 	thd_t           *thread_ptr         = task_ptr->thread_struct_ptr;
 	state_t thread_state = DSH_NO_RESP;
 	slurm_msg_type_t msg_type = task_ptr->msg_type;
-	bool is_kill_msg, srun_agent;
+	bool is_kill_msg, srun_agent, sack_agent;
 	List ret_list = NULL;
 	ListIterator itr;
 	ret_data_info_t *ret_data_info = NULL;
@@ -894,7 +962,6 @@ static void *_thread_per_group_rpc(void *args)
 			(msg_type == REQUEST_KILL_PREEMPTED)	||
 			(msg_type == REQUEST_TERMINATE_JOB) );
 	srun_agent = (	(msg_type == SRUN_PING)			||
-			(msg_type == SRUN_EXEC)			||
 			(msg_type == SRUN_JOB_COMPLETE)		||
 			(msg_type == SRUN_STEP_MISSING)		||
 			(msg_type == SRUN_STEP_SIGNAL)		||
@@ -902,6 +969,7 @@ static void *_thread_per_group_rpc(void *args)
 			(msg_type == SRUN_USER_MSG)		||
 			(msg_type == RESPONSE_RESOURCE_ALLOCATION) ||
 			(msg_type == SRUN_NODE_FAIL) );
+	sack_agent = (msg_type == REQUEST_RECONFIGURE_SACKD);
 
 	thread_ptr->start_time = time(NULL);
 
@@ -918,22 +986,38 @@ static void *_thread_per_group_rpc(void *args)
 
 	msg.msg_type = msg_type;
 	msg.data     = task_ptr->msg_args_ptr;
+	slurm_msg_set_r_uid(&msg, task_ptr->r_uid);
+	msg.flags |= task_ptr->msg_flags;
 
-	log_flag(AGENT, "%s: sending %s to %s",
-		 __func__, rpc_num2string(msg_type), thread_ptr->nodelist);
+	if (thread_ptr->nodename)
+		log_flag(AGENT, "%s: sending %s to %s", __func__,
+			 rpc_num2string(msg_type), thread_ptr->nodename);
+	else if (slurm_conf.debug_flags & DEBUG_FLAG_AGENT) {
+		char *tmp_str;
+		tmp_str = hostlist_ranged_string_xmalloc(thread_ptr->nodelist);
+		debug("%s: sending %s to %s", __func__,
+		      rpc_num2string(msg_type), tmp_str);
+		xfree(tmp_str);
+	}
 
 	if (task_ptr->get_reply) {
 		if (thread_ptr->addr) {
 			msg.address = *thread_ptr->addr;
 
 			if (!(ret_list = slurm_send_addr_recv_msgs(
-				     &msg, thread_ptr->nodelist, 0))) {
+				     &msg, thread_ptr->nodename, 0))) {
+				error("%s: no ret_list given", __func__);
+				goto cleanup;
+			}
+		} else if (thread_ptr->nodelist) {
+			if (!(ret_list = start_msg_tree(thread_ptr->nodelist,
+							&msg, 0))) {
 				error("%s: no ret_list given", __func__);
 				goto cleanup;
 			}
 		} else {
 			if (!(ret_list = slurm_send_recv_msgs(
-				     thread_ptr->nodelist, &msg, 0))) {
+				thread_ptr->nodename, &msg, 0))) {
 				error("%s: no ret_list given", __func__);
 				goto cleanup;
 			}
@@ -944,15 +1028,16 @@ static void *_thread_per_group_rpc(void *args)
 			msg.address = *thread_ptr->addr;
 		} else {
 			//info("no address given");
-			if (slurm_conf_get_addr(thread_ptr->nodelist,
+			xassert(thread_ptr->nodename);
+			if (slurm_conf_get_addr(thread_ptr->nodename,
 					        &msg.address, msg.flags)
 			    == SLURM_ERROR) {
 				error("%s: can't find address for host %s, check slurm.conf",
-				      __func__, thread_ptr->nodelist);
+				      __func__, thread_ptr->nodename);
 				goto cleanup;
 			}
 		}
-		//info("sending %u to %s", msg_type, thread_ptr->nodelist);
+		//info("sending %u to %s", msg_type, thread_ptr->nodename);
 		if (msg_type == SRUN_JOB_COMPLETE) {
 			/*
 			 * The srun runs as a single thread, while the kernel
@@ -971,9 +1056,9 @@ static void *_thread_per_group_rpc(void *args)
 		} else if (slurm_send_only_node_msg(&msg) == SLURM_SUCCESS) {
 			thread_state = DSH_DONE;
 		} else {
-			if (!srun_agent) {
+			if (!sack_agent && !srun_agent) {
 				lock_slurmctld(node_read_lock);
-				_comm_err(thread_ptr->nodelist, msg_type);
+				_comm_err(thread_ptr->nodename, msg_type);
 				unlock_slurmctld(node_read_lock);
 			}
 		}
@@ -1147,7 +1232,7 @@ static void *_thread_per_group_rpc(void *args)
 			thread_state = DSH_DONE;
 			break;
 		default:
-			if (!srun_agent) {
+			if (!sack_agent && !srun_agent) {
 				if (ret_data_info->err)
 					errno = ret_data_info->err;
 				else
@@ -1158,7 +1243,7 @@ static void *_thread_per_group_rpc(void *args)
 				unlock_slurmctld(node_read_lock);
 			}
 
-			if (srun_agent)
+			if (sack_agent || srun_agent)
 				thread_state = DSH_FAILED;
 			else if (rc || (ret_data_info->type ==
 					RESPONSE_FORWARD_FAILED))
@@ -1176,7 +1261,6 @@ static void *_thread_per_group_rpc(void *args)
 	list_iterator_destroy(itr);
 
 cleanup:
-	xfree(args);
 	if (!ret_list && (msg_type == REQUEST_SIGNAL_TASKS)) {
 		job_record_t *job_ptr;
 		signal_tasks_msg_t *msg_ptr =
@@ -1191,6 +1275,7 @@ cleanup:
 			unlock_slurmctld(job_write_lock);
 		}
 	}
+	xfree(args);
 	/* handled at end of thread just in case resend is needed */
 	destroy_forward(&msg.forward);
 	slurm_mutex_lock(thread_mutex_ptr);
@@ -1202,7 +1287,7 @@ cleanup:
 	(*threads_active_ptr)--;
 	slurm_cond_signal(thread_cond_ptr);
 	slurm_mutex_unlock(thread_mutex_ptr);
-	return (void *) NULL;
+	return NULL;
 }
 
 /*
@@ -1289,6 +1374,8 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 	agent_arg_ptr->msg_args = *(agent_info_ptr->msg_args_pptr);
 	*(agent_info_ptr->msg_args_pptr) = NULL;
 
+	set_agent_arg_r_uid(agent_arg_ptr, agent_info_ptr->r_uid);
+
 	j = 0;
 	for (i = 0; i < agent_info_ptr->thread_count; i++) {
 		if (!thread_ptr[i].ret_list) {
@@ -1296,12 +1383,12 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 				continue;
 
 			debug("got the name %s to resend",
-			      thread_ptr[i].nodelist);
+			      thread_ptr[i].nodename);
 #ifdef HAVE_FRONT_END
 			node_ptr = find_front_end_record(
-						thread_ptr[i].nodelist);
+						thread_ptr[i].nodename);
 #else
-			node_ptr = find_node_record(thread_ptr[i].nodelist);
+			node_ptr = find_node_record(thread_ptr[i].nodename);
 #endif
 			if (node_ptr &&
 			    (IS_NODE_DOWN(node_ptr) ||
@@ -1312,7 +1399,7 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 					count--;
 			} else {
 				hostlist_push_host(agent_arg_ptr->hostlist,
-						   thread_ptr[i].nodelist);
+						   thread_ptr[i].nodename);
 				j++;
 			}
 			if (j == count)
@@ -1336,17 +1423,17 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 			count, j);
 		agent_arg_ptr->node_count = j;
 	}
-	debug2("Queue RPC msg_type=%u, nodes=%d for retry",
-	       agent_arg_ptr->msg_type, j);
+	debug2("Queue RPC msg_type=%s, nodes=%d for retry",
+	       rpc_num2string(agent_arg_ptr->msg_type), j);
 
-	/* add the requeust to a list */
+	/* add the request to a list */
 	queued_req_ptr = xmalloc(sizeof(queued_request_t));
 	queued_req_ptr->agent_arg_ptr = agent_arg_ptr;
 	queued_req_ptr->last_attempt  = time(NULL);
 	slurm_mutex_lock(&retry_mutex);
 	if (retry_list == NULL)
 		retry_list = list_create(_list_delete_retry);
-	(void) list_append(retry_list, (void *) queued_req_ptr);
+	list_append(retry_list, queued_req_ptr);
 	slurm_mutex_unlock(&retry_mutex);
 }
 
@@ -1377,7 +1464,8 @@ static void *_agent_init(void *arg)
 	while (true) {
 		slurm_mutex_lock(&pending_mutex);
 		while (!slurmctld_config.shutdown_time &&
-		       !pending_mail && (pending_wait_time == NO_VAL16)) {
+		       !pending_mail && !pending_check_defer &&
+		       (pending_wait_time == NO_VAL16)) {
 			ts.tv_sec  = time(NULL) + 2;
 			slurm_cond_timedwait(&pending_cond, &pending_mutex,
 					     &ts);
@@ -1392,8 +1480,10 @@ static void *_agent_init(void *arg)
 		pending_wait_time = NO_VAL16;
 		slurm_mutex_unlock(&pending_mutex);
 
-		if (last_defer_attempt + 2 < last_job_update) {
+		if ((last_defer_attempt + 2 < last_job_update) ||
+		    pending_check_defer) {
 			last_defer_attempt = time(NULL);
+			pending_check_defer = false;
 			_agent_defer();
 		}
 
@@ -1406,6 +1496,49 @@ static void *_agent_init(void *arg)
 	return NULL;
 }
 
+static int _foreach_node_did_resp(void *x, void *arg)
+{
+	node_did_resp((char *) x);
+	return 1;
+}
+
+/* Start a thread to manage queued agent requests */
+static void *_agent_nodes_update(void *arg)
+{
+	struct timespec ts = {0, 0};
+	slurmctld_lock_t node_write_lock =
+		{ .conf = READ_LOCK, .node = WRITE_LOCK };
+
+	slurm_mutex_lock(&update_nodes_mutex);
+	while (true) {
+		ts.tv_sec = time(NULL) + 2;
+		slurm_cond_timedwait(&update_nodes_cond, &update_nodes_mutex,
+				     &ts);
+
+		if (slurmctld_config.shutdown_time) {
+			slurm_mutex_unlock(&update_nodes_mutex);
+			break;
+		}
+
+		if (!list_count(update_node_list))
+			continue;
+		lock_slurmctld(node_write_lock);
+		list_delete_all(update_node_list, _foreach_node_did_resp, NULL);
+		unlock_slurmctld(node_write_lock);
+	}
+
+	return NULL;
+}
+
+static void _queue_update_node(char *node_name)
+{
+	slurm_mutex_lock(&update_nodes_mutex);
+	if (!update_node_list)
+		update_node_list = list_create(xfree_ptr);
+	list_append(update_node_list, node_name);
+	slurm_mutex_unlock(&update_nodes_mutex);
+}
+
 extern void agent_init(void)
 {
 	slurm_mutex_lock(&pending_mutex);
@@ -1415,7 +1548,8 @@ extern void agent_init(void)
 		return;
 	}
 
-	slurm_thread_create_detached(NULL, _agent_init, NULL);
+	slurm_thread_create_detached(_agent_init, NULL);
+	slurm_thread_create_detached(_agent_nodes_update, NULL);
 	pending_thread_running = true;
 	slurm_mutex_unlock(&pending_mutex);
 }
@@ -1426,8 +1560,9 @@ extern void agent_init(void)
  * IN mail_too - Send pending email too, note this performed using a
  *	fork/waitpid, so it can take longer than just creating a pthread
  *	to send RPCs
+ * IN check_defer - force defer_list check
  */
-extern void agent_trigger(int min_wait, bool mail_too)
+extern void agent_trigger(int min_wait, bool mail_too, bool check_defer)
 {
 	log_flag(AGENT, "%s: pending_wait_time=%d->%d mail_too=%c->%c Agent_cnt=%d agent_thread_cnt=%d retry_list_size=%d",
 		 __func__, pending_wait_time, min_wait,
@@ -1440,6 +1575,8 @@ extern void agent_trigger(int min_wait, bool mail_too)
 		pending_wait_time = min_wait;
 	if (mail_too)
 		pending_mail = mail_too;
+	if (check_defer)
+		pending_check_defer = check_defer;
 	slurm_cond_broadcast(&pending_cond);
 	slurm_mutex_unlock(&pending_mutex);
 }
@@ -1538,8 +1675,9 @@ static void _agent_defer(void)
 				 REQUEST_SIGNAL_TASKS)
 				rc = _signal_defer(queued_req_ptr);
 			else
-				fatal("%s: Invalid message type (%u)",
-				      __func__, agent_arg_ptr->msg_type);
+				fatal("%s: Invalid message type (%s)",
+				      __func__,
+				      rpc_num2string(agent_arg_ptr->msg_type));
 
 			if (rc == -1) {   /* abort request */
 				_purge_agent_args(
@@ -1573,6 +1711,19 @@ static void _agent_defer(void)
 	return;
 }
 
+static int _find_request(void *x, void *key)
+{
+	queued_request_t *queued_req_ptr = x;
+	double *before = key;
+
+	if (!(*before) && queued_req_ptr->last_attempt == 0)
+		return 1;
+	else if (queued_req_ptr->last_attempt < *before)
+		return 1;
+
+	return 0;
+}
+
 /* Do the work requested by agent_retry (retry pending RPCs).
  * This is a separate thread so the job records can be locked */
 static void _agent_retry(int min_wait, bool mail_too)
@@ -1580,19 +1731,22 @@ static void _agent_retry(int min_wait, bool mail_too)
 	time_t now = time(NULL);
 	queued_request_t *queued_req_ptr = NULL;
 	agent_arg_t *agent_arg_ptr = NULL;
-	ListIterator retry_iter;
 	mail_info_t *mi = NULL;
+	int list_size = 0, agent_started = 0;
 
+next:
 	slurm_mutex_lock(&retry_mutex);
-	if (retry_list) {
+	if (retry_list && !list_size) {
 		static time_t last_msg_time = (time_t) 0;
 		uint32_t msg_type[5] = {0, 0, 0, 0, 0};
-		int i = 0, list_size = list_count(retry_list);
+		int i = 0;
+		list_size = list_count(retry_list);
 		if (((list_size > 100) &&
 		     (difftime(now, last_msg_time) > 300)) ||
 		    ((list_size > 0) &&
 		     (slurm_conf.debug_flags & DEBUG_FLAG_AGENT))) {
 			/* Note sizable backlog (retry_list_size()) of work */
+			ListIterator retry_iter;
 			retry_iter = list_iterator_create(retry_list);
 			while ((queued_req_ptr = list_next(retry_iter))) {
 				agent_arg_ptr = queued_req_ptr->agent_arg_ptr;
@@ -1611,42 +1765,28 @@ static void _agent_retry(int min_wait, bool mail_too)
 		}
 	}
 
-	slurm_mutex_lock(&agent_cnt_mutex);
-	if (agent_thread_cnt + AGENT_THREAD_COUNT + 2 > MAX_SERVER_THREADS) {
+	if (get_agent_thread_count() + AGENT_THREAD_COUNT + 2 >
+	    MAX_SERVER_THREADS) {
 		/* too much work already */
-		slurm_mutex_unlock(&agent_cnt_mutex);
 		slurm_mutex_unlock(&retry_mutex);
 		return;
 	}
-	slurm_mutex_unlock(&agent_cnt_mutex);
 
 	if (retry_list) {
 		/* first try to find a new (never tried) record */
-		retry_iter = list_iterator_create(retry_list);
-		while ((queued_req_ptr = list_next(retry_iter))) {
- 			if (queued_req_ptr->last_attempt == 0) {
-				list_remove(retry_iter);
-				break;		/* Process this request now */
-			}
-		}
-		list_iterator_destroy(retry_iter);
+		double key = 0;
+
+		queued_req_ptr = list_remove_first(retry_list, _find_request,
+						   &key);
 	}
 
 	if (retry_list && (queued_req_ptr == NULL)) {
 		/* now try to find a requeue request that is
 		 * relatively old */
-		double age = 0;
+		double before = difftime(now, min_wait);
 
-		retry_iter = list_iterator_create(retry_list);
-		/* next try to find an older record to retry */
-		while ((queued_req_ptr = list_next(retry_iter))) {
-			age = difftime(now, queued_req_ptr->last_attempt);
-			if (age > min_wait) {
-				list_remove(retry_iter);
-				break;
-			}
-		}
-		list_iterator_destroy(retry_iter);
+		queued_req_ptr = list_remove_first(retry_list, _find_request,
+						   &before);
 	}
 	slurm_mutex_unlock(&retry_mutex);
 
@@ -1656,9 +1796,16 @@ static void _agent_retry(int min_wait, bool mail_too)
 		if (agent_arg_ptr) {
 			debug2("Spawning RPC agent for msg_type %s",
 			       rpc_num2string(agent_arg_ptr->msg_type));
-			slurm_thread_create_detached(NULL, agent, agent_arg_ptr);
+			slurm_thread_create_detached(agent, agent_arg_ptr);
+			agent_started++;
 		} else
 			error("agent_retry found record with no agent_args");
+
+		if ((list_size > agent_started) && !LOTS_OF_AGENTS) {
+			log_flag(AGENT, "%s: created %d agent, try to start more",
+				 __func__, agent_started);
+			goto next;
+		}
 	} else if (mail_too) {
 		slurm_mutex_lock(&agent_cnt_mutex);
 		slurm_mutex_lock(&mail_mutex);
@@ -1670,7 +1817,7 @@ static void _agent_retry(int min_wait, bool mail_too)
 
 			mail_thread_cnt++;
 			agent_thread_cnt++;
-			slurm_thread_create_detached(NULL, _mail_proc, mi);
+			slurm_thread_create_detached(_mail_proc, mi);
 		}
 		slurm_mutex_unlock(&mail_mutex);
 		slurm_mutex_unlock(&agent_cnt_mutex);
@@ -1696,10 +1843,9 @@ void agent_queue_request(agent_arg_t *agent_arg_ptr)
 	}
 
 	if (agent_arg_ptr->msg_type == REQUEST_SHUTDOWN) {
-		/* execute now */
-		slurm_thread_create_detached(NULL, agent, agent_arg_ptr);
-		/* give agent a chance to start */
-		usleep(10000);
+		pthread_t agent_thread = 0;
+		slurm_thread_create(&agent_thread, agent, agent_arg_ptr);
+		pthread_join(agent_thread, NULL);
 		return;
 	}
 
@@ -1725,7 +1871,7 @@ void agent_queue_request(agent_arg_t *agent_arg_ptr)
 	}
 	/* now process the request in a separate pthread
 	 * (if we can create another pthread to do so) */
-	agent_trigger(999, false);
+	agent_trigger(999, false, false);
 }
 
 /* agent_purge - purge all pending RPC requests */
@@ -1748,6 +1894,9 @@ extern void agent_purge(void)
 		FREE_NULL_LIST(mail_list);
 		slurm_mutex_unlock(&mail_mutex);
 	}
+	slurm_mutex_lock(&update_nodes_mutex);
+	FREE_NULL_LIST(update_node_list);
+	slurm_mutex_unlock(&update_nodes_mutex);
 
 	xfree(rpc_stat_counts);
 	xfree(rpc_stat_types);
@@ -1811,8 +1960,6 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 			slurm_free_kill_job_msg(agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == SRUN_USER_MSG)
 			slurm_free_srun_user_msg(agent_arg_ptr->msg_args);
-		else if (agent_arg_ptr->msg_type == SRUN_EXEC)
-			slurm_free_srun_exec_msg(agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == SRUN_NODE_FAIL)
 			slurm_free_srun_node_fail_msg(agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == SRUN_STEP_MISSING)
@@ -1829,6 +1976,10 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 			slurm_free_prolog_launch_msg(agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == REQUEST_REBOOT_NODES)
 			slurm_free_reboot_msg(agent_arg_ptr->msg_args);
+		else if (agent_arg_ptr->msg_type == REQUEST_RECONFIGURE_SACKD)
+			slurm_free_config_response_msg(agent_arg_ptr->msg_args);
+		else if (agent_arg_ptr->msg_type == REQUEST_RECONFIGURE_WITH_CONFIG)
+			slurm_free_config_response_msg(agent_arg_ptr->msg_args);
 		else
 			xfree(agent_arg_ptr->msg_args);
 	}
@@ -1881,8 +2032,9 @@ static void *_mail_proc(void *arg)
 	char *argv[5] = {
 		slurm_conf.mail_prog, "-s", mi->message, mi->user_name, NULL};
 
-	result = run_command("MailProg", slurm_conf.mail_prog, argv,
-			     mi->environment, MAIL_PROG_TIMEOUT, 0, &status);
+	status = slurmscriptd_run_mail(slurm_conf.mail_prog, 5, argv,
+				       mi->environment, MAIL_PROG_TIMEOUT,
+				       &result);
 	if (status)
 		error("MailProg returned error, it's output was '%s'", result);
 	else if (result && (strlen(result) > 0))
@@ -1904,7 +2056,7 @@ static void *_mail_proc(void *arg)
 	slurm_mutex_unlock(&mail_mutex);
 	slurm_mutex_unlock(&agent_cnt_mutex);
 
-	return (void *) NULL;
+	return NULL;
 }
 
 static char *_mail_type_str(uint16_t mail_type)
@@ -2128,7 +2280,7 @@ extern void mail_job_info(job_record_t *job_ptr, uint16_t mail_type)
 	slurm_mutex_lock(&mail_mutex);
 	if (!mail_list)
 		mail_list = list_create(_mail_free);
-	(void) list_enqueue(mail_list, (void *) mi);
+	list_enqueue(mail_list, mi);
 	slurm_mutex_unlock(&mail_mutex);
 	return;
 }
@@ -2207,6 +2359,23 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 #endif
 	}
 
+	if ((slurm_conf.prolog_flags & PROLOG_FLAG_DEFER_BATCH) &&
+	    (job_ptr->state_reason == WAIT_PROLOG)) {
+		if (job_ptr->node_bitmap_pr &&
+		    (slurm_conf.debug_flags &
+		     (DEBUG_FLAG_TRACE_JOBS | DEBUG_FLAG_AGENT))) {
+			char *tmp_pr;
+			tmp_pr = bitmap2node_name(job_ptr->node_bitmap_pr);
+			verbose("%s: JobId=%u still waiting on prologs on %s",
+			       __func__, job_ptr->job_id, tmp_pr);
+			xfree(tmp_pr);
+		} else {
+			debug2("%s: JobId=%u still waiting on node prologs",
+			       __func__, job_ptr->job_id);
+		}
+		return 1;
+	}
+
 	if (nodes_ready) {
 		if (IS_JOB_CONFIGURING(job_ptr))
 			job_config_fini(job_ptr);
@@ -2276,9 +2445,10 @@ extern int retry_list_size(void)
 
 static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 {
-	char *argv[3], *pname;
-	pid_t child;
+	char *argv[4], *pname;
+	uint32_t argc;
 	int rc, status = 0;
+	reboot_msg_t *reboot_msg = agent_arg_ptr->msg_args;
 
 	if (!agent_arg_ptr->hostlist) {
 		error("%s: hostlist is NULL", __func__);
@@ -2295,28 +2465,33 @@ static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 	else
 		argv[0] = slurm_conf.reboot_program;
 	argv[1] = hostlist_deranged_string_xmalloc(agent_arg_ptr->hostlist);
-	argv[2] = NULL;
-
-	child = fork();
-	if (child == 0) {
-		closeall(0);
-		(void) setpgid(0, 0);
-		(void) execv(slurm_conf.reboot_program, argv);
-		_exit(1);
-	} else if (child < 0) {
-		error("fork: %m");
+	if (reboot_msg && reboot_msg->features) {
+		argc = 4;
+		argv[2] = reboot_msg->features;
+		argv[3] = NULL;
 	} else {
-		(void) waitpid(child, &status, 0);
-		if (WIFEXITED(status)) {
-			rc = WEXITSTATUS(status);
-			if (rc != 0) {
-				error("RebootProgram exit status of %d",
-				      rc);
-			}
-		} else if (WIFSIGNALED(status)) {
-			error("RebootProgram signaled: %s",
-			      strsignal(WTERMSIG(status)));
-		}
+		argc = 3;
+		argv[2] = NULL;
 	}
+
+	status = slurmscriptd_run_reboot(slurm_conf.reboot_program, argc, argv);
+	if (WIFEXITED(status)) {
+		rc = WEXITSTATUS(status);
+		if (rc != 0) {
+			error("RebootProgram exit status of %d",
+			      rc);
+		}
+	} else if (WIFSIGNALED(status)) {
+		error("RebootProgram signaled: %s",
+		      strsignal(WTERMSIG(status)));
+	}
+
 	xfree(argv[1]);
 }
+
+/* Set r_uid of agent_arg */
+extern void set_agent_arg_r_uid(agent_arg_t *agent_arg_ptr, uid_t r_uid)
+{
+	agent_arg_ptr->r_uid = r_uid;
+	agent_arg_ptr->r_uid_set = true;
+}
