@@ -48,6 +48,9 @@ typedef struct {
 	uint32_t	used;		/* used licenses */
 	uint32_t	reserved;	/* currently reserved licenses */
 	uint8_t         remote;	        /* non-zero if remote (from database) */
+	uint32_t last_deficit;		/* last calculated deficit */
+	uint32_t last_consumed;		/* consumed count (for remote) */
+	time_t last_update;		/* last updated timestamp (for remote) */
 } licenses_t;
 
 /*
@@ -61,8 +64,7 @@ typedef struct {
 	slurmctld_resv_t *resv_ptr;
 } bf_license_t;
 
-extern List license_list;
-extern List clus_license_list;
+extern list_t *cluster_license_list;
 extern time_t last_license_update;
 
 /* Initialize licenses on this system based upon slurm.conf */
@@ -75,7 +77,7 @@ extern int license_update(char *licenses);
 extern void license_add_remote(slurmdb_res_rec_t *rec);
 extern void license_update_remote(slurmdb_res_rec_t *rec);
 extern void license_remove_remote(slurmdb_res_rec_t *rec);
-extern void license_sync_remote(List res_list);
+extern void license_sync_remote(list_t *res_list);
 
 /* Free memory associated with licenses on this system */
 extern void license_free(void);
@@ -88,14 +90,15 @@ extern void license_free_rec(void *x);
  * IN license_list_src - job license list to be copied
  * RET a copy of the license list
  */
-extern List license_copy(List license_list_src);
+extern list_t *license_copy(list_t *license_list_src);
 
 /*
  * license_job_get - Get the licenses required for a job
  * IN job_ptr - job identification
+ * IN restore - is this a new allocation, or are we loading state from disk
  * RET SLURM_SUCCESS or failure code
  */
-extern int license_job_get(job_record_t *job_ptr);
+extern int license_job_get(job_record_t *job_ptr, bool restore);
 
 /*
  * license_job_merge - The licenses from one job have just been merged into
@@ -105,6 +108,14 @@ extern int license_job_get(job_record_t *job_ptr);
  */
 extern void license_job_merge(job_record_t *job_ptr);
 
+/*
+ * license_job_return - Return the licenses allocated to a job
+ * IN job_ptr - job identification
+ * RET SLURM_SUCCESS or failure code
+ */
+extern int license_job_return_to_list(job_record_t *job_ptr,
+				      list_t *license_list);
+
 /*
  * license_job_return - Return the licenses allocated to a job
  * IN job_ptr - job identification
@@ -112,6 +123,17 @@ extern void license_job_merge(job_record_t *job_ptr);
  */
 extern int license_job_return(job_record_t *job_ptr);
 
+/*
+ * license_job_test_with_list - Test if the licenses required for a job are
+ *	available in provided list
+ * IN job_ptr - job identification
+ * IN when    - time to check
+ * IN reboot    - true if node reboot required to start job
+ * RET: SLURM_SUCCESS, EAGAIN (not available now), SLURM_ERROR (never runnable)
+ */
+extern int license_job_test_with_list(job_record_t *job_ptr, time_t when,
+				      bool reboot, list_t *license_list);
+
 /*
  * license_job_test - Test if the licenses required for a job are available
  * IN job_ptr - job identification
@@ -134,15 +156,15 @@ extern int license_job_test(job_record_t *job_ptr, time_t when,
  *             are configured (though not necessarily available now)
  * RET license_list, must be destroyed by caller
  */
-extern List license_validate(char *licenses, bool validate_configured,
-			     bool validate_existing,
-			     uint64_t *tres_req_cnt, bool *valid);
+extern list_t *license_validate(char *licenses, bool validate_configured,
+				bool validate_existing,
+				uint64_t *tres_req_cnt, bool *valid);
 
 /*
  * license_list_overlap - test if there is any overlap in licenses
  *	names found in the two lists
  */
-extern bool license_list_overlap(List list_1, List list_2);
+extern bool license_list_overlap(list_t *list_1, list_t *list_2);
 
 /*
  * Given a list of license_t records, return a license string.
@@ -154,7 +176,7 @@ extern bool license_list_overlap(List list_1, List list_2);
  *
  * RET string representation of licenses. Must be destroyed by caller.
  */
-extern char *license_list_to_string(List license_list);
+extern char *license_list_to_string(list_t *license_list);
 
 /* pack_all_licenses()
  *
@@ -176,14 +198,14 @@ extern uint32_t get_total_license_cnt(char *name);
 /* node_read should be locked before coming in here
  * returns tres_str of the license_list.
  */
-extern char *licenses_2_tres_str(List license_list);
+extern char *licenses_2_tres_str(list_t *license_list);
 
 
 /* node_read should be locked before coming in here
  * fills in tres_cnt of the license_list.
  * locked if assoc_mgr tres read lock is locked or not.
  */
-extern void license_set_job_tres_cnt(List license_list,
+extern void license_set_job_tres_cnt(list_t *license_list,
 				     uint64_t *tres_cnt,
 				     bool locked);
 
