@@ -453,10 +453,13 @@ extern void run_health_check(void)
 		if (run_cyclic &&
 		    (node_test_cnt++ >= node_limit))
 				break;
-		if (IS_NODE_NO_RESPOND(node_ptr) ||
-		    IS_NODE_FUTURE(node_ptr) ||
+		if (IS_NODE_FUTURE(node_ptr) ||
+		    IS_NODE_INVALID_REG(node_ptr) ||
+		    IS_NODE_NO_RESPOND(node_ptr) ||
+		    IS_NODE_POWERED_DOWN(node_ptr) ||
 		    IS_NODE_POWERING_DOWN(node_ptr) ||
-		    IS_NODE_POWERED_DOWN(node_ptr))
+		    IS_NODE_POWERING_UP(node_ptr) ||
+		    IS_NODE_REBOOT_ISSUED(node_ptr))
 			continue;
 		if (node_states != HEALTH_CHECK_NODE_ANY) {
 			uint16_t cpus_total, cpus_used = 0;
@@ -502,7 +505,7 @@ extern void run_health_check(void)
 		if (PACK_FANOUT_ADDRS(node_ptr))
 			check_agent_args->msg_flags |= SLURM_PACK_ADDRS;
 	}
-	if (base_node_loc >= node_record_count)
+	if (!node_ptr)
 		base_node_loc = 0;
 #endif
 
@@ -554,10 +557,13 @@ extern void update_nodes_acct_gather_data(void)
 	}
 #else
 	for (i = 0; (node_ptr = next_node(&i)); i++) {
-		if (IS_NODE_NO_RESPOND(node_ptr) ||
-		    IS_NODE_FUTURE(node_ptr) ||
+		if (IS_NODE_FUTURE(node_ptr) ||
+		    IS_NODE_INVALID_REG(node_ptr) ||
+		    IS_NODE_NO_RESPOND(node_ptr) ||
+		    IS_NODE_POWERED_DOWN(node_ptr) ||
 		    IS_NODE_POWERING_DOWN(node_ptr) ||
-		    IS_NODE_POWERED_DOWN(node_ptr))
+		    IS_NODE_POWERING_UP(node_ptr) ||
+		    IS_NODE_REBOOT_ISSUED(node_ptr))
 			continue;
 		if (agent_args->protocol_version > node_ptr->protocol_version)
 			agent_args->protocol_version =
