@@ -41,10 +41,12 @@
 
 #include <string.h>
 
-#include "src/common/node_select.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+
+#include "src/interfaces/select.h"
+
 #include "src/slurmctld/agent.h"
 #include "src/slurmctld/fed_mgr.h"
 #include "src/slurmctld/proc_req.h"
@@ -57,7 +59,7 @@
  */
 static void _srun_agent_launch(slurm_addr_t *addr, char *host,
 			       slurm_msg_type_t type, void *msg_args,
-			       uint16_t protocol_version)
+			       uid_t r_uid, uint16_t protocol_version)
 {
 	agent_arg_t *agent_args = xmalloc(sizeof(agent_arg_t));
 
@@ -67,6 +69,7 @@ static void _srun_agent_launch(slurm_addr_t *addr, char *host,
 	agent_args->hostlist   = hostlist_create(host);
 	agent_args->msg_type   = type;
 	agent_args->msg_args   = msg_args;
+	set_agent_arg_r_uid(agent_args, r_uid);
 	agent_args->protocol_version = protocol_version;
 
 	agent_queue_request(agent_args);
@@ -145,6 +148,7 @@ extern void srun_allocate(job_record_t *job_ptr)
 		msg_arg = build_alloc_msg(job_ptr, SLURM_SUCCESS, NULL);
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   RESPONSE_RESOURCE_ALLOCATION, msg_arg,
+				   job_ptr->user_id,
 				   job_ptr->start_protocol_ver);
 	} else if (_pending_het_jobs(job_ptr)) {
 		return;
@@ -169,6 +173,7 @@ extern void srun_allocate(job_record_t *job_ptr)
 		list_iterator_destroy(iter);
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   RESPONSE_HET_JOB_ALLOCATION, job_resp_list,
+				   job_ptr->user_id,
 				   job_ptr->start_protocol_ver);
 	} else {
 		error("%s: Can not find hetjob leader %pJ",
@@ -195,7 +200,7 @@ extern void srun_allocate_abort(job_record_t *job_ptr)
 		msg_arg->step_het_comp = NO_VAL;
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   SRUN_JOB_COMPLETE,
-				   msg_arg,
+				   msg_arg, job_ptr->user_id,
 				   job_ptr->start_protocol_ver);
 	}
 }
@@ -205,6 +210,20 @@ typedef struct {
 	char *node_name;
 } srun_node_fail_args_t;
 
+slurm_addr_t *_srun_set_addr(step_record_t *step_ptr)
+{
+	char *nodeaddr;
+	slurm_addr_t *addr = xmalloc(sizeof(*addr));
+
+	if ((nodeaddr = slurm_conf_get_nodeaddr(step_ptr->host))) {
+		slurm_set_addr(addr, step_ptr->port, nodeaddr);
+		xfree(nodeaddr);
+	} else
+		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+
+	return addr;
+}
+
 static int _srun_node_fail(void *x, void *arg)
 {
 	step_record_t *step_ptr = (step_record_t *) x;
@@ -222,13 +241,14 @@ static int _srun_node_fail(void *x, void *arg)
 	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
 		return 0;
 
-	addr = xmalloc(sizeof(*addr));
-	slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+	addr = _srun_set_addr(step_ptr);
+
 	msg_arg = xmalloc(sizeof(*msg_arg));
 	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
 	msg_arg->nodelist = xstrdup(args->node_name);
 	_srun_agent_launch(addr, step_ptr->host, SRUN_NODE_FAIL,
-			   msg_arg, step_ptr->start_protocol_ver);
+			   msg_arg, step_ptr->job_ptr->user_id,
+			   step_ptr->start_protocol_ver);
 
 	return 0;
 }
@@ -260,7 +280,7 @@ extern void srun_node_fail(job_record_t *job_ptr, char *node_name)
 #else
 	if (!node_name || (node_ptr = find_node_record(node_name)) == NULL)
 		return;
-	args.bit_position = node_ptr - node_record_table_ptr;
+	args.bit_position = node_ptr->index;
 #endif
 
 	list_for_each(job_ptr->step_list, _srun_node_fail, &args);
@@ -274,7 +294,8 @@ extern void srun_node_fail(job_record_t *job_ptr, char *node_name)
 		msg_arg->step_id.step_het_comp = NO_VAL;
 		msg_arg->nodelist = xstrdup(node_name);
 		_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_NODE_FAIL,
-				   msg_arg, job_ptr->start_protocol_ver);
+				   msg_arg, job_ptr->user_id,
+				   job_ptr->start_protocol_ver);
 	}
 }
 
@@ -300,7 +321,7 @@ static int _srun_ping(void *x, void *arg)
 	msg_arg->job_id = job_ptr->job_id;
 
 	_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_PING, msg_arg,
-			   job_ptr->start_protocol_ver);
+			   job_ptr->user_id, job_ptr->start_protocol_ver);
 
 	return 0;
 }
@@ -317,7 +338,7 @@ extern void srun_ping (void)
 	if (slurm_conf.inactive_limit == 0)
 		return;		/* No limit, don't bother pinging */
 
-	list_for_each(job_list, _srun_ping, &old);
+	list_for_each_ro(job_list, _srun_ping, &old);
 }
 
 static int _srun_step_timeout(void *x, void *arg)
@@ -334,14 +355,15 @@ static int _srun_step_timeout(void *x, void *arg)
 	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
 		return 0;
 
-	addr = xmalloc(sizeof(*addr));
 	msg_arg = xmalloc(sizeof(*msg_arg));
 
-	slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+	addr = _srun_set_addr(step_ptr);
+
 	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
 	msg_arg->timeout = step_ptr->job_ptr->end_time;
 
 	_srun_agent_launch(addr, step_ptr->host, SRUN_TIMEOUT, msg_arg,
+			   step_ptr->job_ptr->user_id,
 			   step_ptr->start_protocol_ver);
 
 	return 0;
@@ -369,7 +391,8 @@ extern void srun_timeout(job_record_t *job_ptr)
 		msg_arg->step_id.step_het_comp = NO_VAL;
 		msg_arg->timeout  = job_ptr->end_time;
 		_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_TIMEOUT,
-				   msg_arg, job_ptr->start_protocol_ver);
+				   msg_arg, job_ptr->user_id,
+				   job_ptr->start_protocol_ver);
 	}
 
 	list_for_each(job_ptr->step_list, _srun_step_timeout, NULL);
@@ -395,7 +418,8 @@ extern int srun_user_message(job_record_t *job_ptr, char *msg)
 		msg_arg->job_id = job_ptr->job_id;
 		msg_arg->msg    = xstrdup(msg);
 		_srun_agent_launch(addr, job_ptr->resp_host, SRUN_USER_MSG,
-				   msg_arg, job_ptr->start_protocol_ver);
+				   msg_arg, job_ptr->user_id,
+				   job_ptr->start_protocol_ver);
 		return SLURM_SUCCESS;
 	} else if (job_ptr->batch_flag && IS_JOB_RUNNING(job_ptr)) {
 #ifndef HAVE_FRONT_END
@@ -436,6 +460,7 @@ extern int srun_user_message(job_record_t *job_ptr, char *msg)
 		agent_arg_ptr->msg_type = REQUEST_JOB_NOTIFY;
 		agent_arg_ptr->msg_args = (void *) notify_msg_ptr;
 		/* Launch the RPC via agent */
+		set_agent_arg_r_uid(agent_arg_ptr, SLURM_AUTH_UID_ANY);
 		agent_queue_request(agent_arg_ptr);
 		return SLURM_SUCCESS;
 	}
@@ -472,6 +497,7 @@ extern void srun_job_complete(job_record_t *job_ptr)
 		msg_arg->step_het_comp = NO_VAL;
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   SRUN_JOB_COMPLETE, msg_arg,
+				   job_ptr->user_id,
 				   job_ptr->start_protocol_ver);
 	}
 
@@ -500,6 +526,7 @@ extern bool srun_job_suspend(job_record_t *job_ptr, uint16_t op)
 		msg_arg->op     = op;
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   SRUN_REQUEST_SUSPEND, msg_arg,
+				   job_ptr->user_id,
 				   job_ptr->start_protocol_ver);
 		msg_sent = true;
 	}
@@ -517,13 +544,13 @@ extern void srun_step_complete(step_record_t *step_ptr)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(slurm_addr_t));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+		addr = _srun_set_addr(step_ptr);
+
 		msg_arg = xmalloc(sizeof(srun_job_complete_msg_t));
-		memcpy(&msg_arg->step_id, &step_ptr->step_id,
-		       sizeof(msg_arg->step_id));
+		memcpy(msg_arg, &step_ptr->step_id, sizeof(*msg_arg));
 		_srun_agent_launch(addr, step_ptr->host, SRUN_JOB_COMPLETE,
-				   msg_arg, step_ptr->start_protocol_ver);
+				   msg_arg, step_ptr->job_ptr->user_id,
+				   step_ptr->start_protocol_ver);
 	}
 }
 
@@ -540,14 +567,15 @@ extern void srun_step_missing(step_record_t *step_ptr, char *node_list)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(slurm_addr_t));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+		addr = _srun_set_addr(step_ptr);
+
 		msg_arg = xmalloc(sizeof(srun_step_missing_msg_t));
 		memcpy(&msg_arg->step_id, &step_ptr->step_id,
 		       sizeof(msg_arg->step_id));
 		msg_arg->nodelist = xstrdup(node_list);
 		_srun_agent_launch(addr, step_ptr->host, SRUN_STEP_MISSING,
-				   msg_arg, step_ptr->start_protocol_ver);
+				   msg_arg, step_ptr->job_ptr->user_id,
+				   step_ptr->start_protocol_ver);
 	}
 }
 
@@ -564,48 +592,15 @@ extern void srun_step_signal(step_record_t *step_ptr, uint16_t signal)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(slurm_addr_t));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+		addr = _srun_set_addr(step_ptr);
+
 		msg_arg = xmalloc(sizeof(job_step_kill_msg_t));
 		memcpy(&msg_arg->step_id, &step_ptr->step_id,
 		       sizeof(msg_arg->step_id));
 		msg_arg->signal      = signal;
 		_srun_agent_launch(addr, step_ptr->host, SRUN_STEP_SIGNAL,
-				   msg_arg, step_ptr->start_protocol_ver);
-	}
-}
-
-/*
- * srun_exec - request that srun execute a specific command
- *	and route it's output to stdout
- * IN step_ptr - pointer to the slurmctld job step record
- * IN argv - command and arguments to execute
- */
-extern void srun_exec(step_record_t *step_ptr, char **argv)
-{
-	slurm_addr_t * addr;
-	srun_exec_msg_t *msg_arg;
-	int cnt = 1, i;
-
-	xassert(step_ptr);
-
-	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		for (i=0; argv[i]; i++)
-			cnt++;	/* start at 1 to include trailing NULL */
-		addr = xmalloc(sizeof(slurm_addr_t));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
-		msg_arg = xmalloc(sizeof(srun_exec_msg_t));
-		memcpy(&msg_arg->step_id, &step_ptr->step_id,
-		       sizeof(msg_arg->step_id));
-		msg_arg->argc    = cnt;
-		msg_arg->argv    = xmalloc(sizeof(char *) * cnt);
-		for (i=0; i<cnt ; i++)
-			msg_arg->argv[i] = xstrdup(argv[i]);
-		_srun_agent_launch(addr, step_ptr->host, SRUN_EXEC,
-				   msg_arg, step_ptr->start_protocol_ver);
-	} else {
-		error("srun_exec %pS lacks communication channel",
-		      step_ptr);
+				   msg_arg, step_ptr->job_ptr->user_id,
+				   step_ptr->start_protocol_ver);
 	}
 }
 
