@@ -77,7 +77,6 @@ static int _chk_cpuinfo_uint32(char *buffer, char *keyword, uint32_t *val);
 
 static int _range_to_map(char* range, uint16_t *map, uint16_t map_size,
 			 int add_threads);
-static int _map_to_range(uint16_t *map, uint16_t map_size, char** prange);
 
 bool     initialized = false;
 uint16_t procs, boards, sockets, cores, threads=1;
@@ -85,6 +84,8 @@ uint16_t block_map_size;
 uint16_t *block_map, *block_map_inv;
 extern slurmd_conf_t *conf;
 
+static bool refresh_hwloc = false;
+
 /*
  * get_procs - Return the count of procs on this system
  * Input: procs - buffer for the CPU count
@@ -166,6 +167,68 @@ static inline int _internal_hwloc_topology_export_xml(
 #endif
 }
 
+static void _remove_ecores(hwloc_topology_t *topology)
+{
+#if HWLOC_API_VERSION > 0x00020401
+	int type_cnt;
+	hwloc_bitmap_t cpuset, cpuset_tot = NULL;
+
+	if (xstrcasestr(slurm_conf.slurmd_params, "allow_ecores"))
+		return;
+
+	if (!(type_cnt = hwloc_cpukinds_get_nr(*topology, 0)))
+		return;
+
+	/*
+	 * Handle the removal of Intel E-Cores here.
+	 *
+	 * At the time of writing this Intel Gen 12+ procs have introduced what
+	 * are known as 'P' (performance) and 'E' (efficiency) cores. The
+	 * former can have hyperthreads, where the latter are only single
+	 * threaded, thus creating a situation where we could get a
+	 * heterogeneous socket (which Slurm doesn't like). Here we can restrict
+	 * to only  "IntelCore" (P-Cores) and disregard the "IntelAtom"
+	 * (E-Cores).
+	 *
+	 * In the future, if desired, we should probably figure out a way to
+	 * handle these E-Cores through a core spec instead.
+	 *
+	 * This logic should do nothing on any other existing processor.
+	 */
+	cpuset = hwloc_bitmap_alloc();
+	for (int i = 0; i < type_cnt; i++) {
+		unsigned nr_infos = 0;
+		struct hwloc_info_s *infos;
+		if (hwloc_cpukinds_get_info(
+			    *topology, i, cpuset, NULL, &nr_infos, &infos, 0))
+			fatal("Error getting info from hwloc_cpukinds_get_info() %m");
+
+		for (int j = 0; j < nr_infos; j++) {
+			if (!xstrcasecmp(infos[j].name, "CoreType") &&
+			    !xstrcasecmp(infos[j].value, "IntelCore")) {
+				/* Restrict the node to only IntelCores */
+				if (!cpuset_tot)
+					cpuset_tot = hwloc_bitmap_alloc();
+				hwloc_bitmap_or(cpuset_tot, cpuset_tot, cpuset);
+			}
+		}
+
+		/*
+		 * If we have a cpuset_tot it means we are on a system with
+		 * IntelCore cpus. We will restrict to only those and be done
+		 * here.
+		 */
+		if (cpuset_tot) {
+			hwloc_topology_restrict(*topology, cpuset_tot, 0);
+			hwloc_bitmap_free(cpuset_tot);
+			break;
+		}
+	}
+	hwloc_bitmap_free(cpuset);
+
+#endif
+}
+
 /* read or load topology and write if needed
  * init and destroy topology must be outside this function */
 extern int xcpuinfo_hwloc_topo_load(
@@ -187,7 +250,7 @@ extern int xcpuinfo_hwloc_topo_load(
 
 	if (full && first_full) {
 		/* Always regenerate file on slurmd startup */
-		if (running_in_slurmd())
+		if (refresh_hwloc)
 			check_file = false;
 		first_full = false;
 	}
@@ -240,7 +303,12 @@ handle_write:
 		/* error in load hardware topology */
 		debug("hwloc_topology_load() failed.");
 		ret = SLURM_ERROR;
-	} else if (!conf->def_config) {
+		goto end_it;
+	}
+
+	_remove_ecores(topology);
+
+	if (!conf->def_config) {
 		debug2("hwloc_topology_export_xml");
 		if (_internal_hwloc_topology_export_xml(*topology, topo_file)) {
 			/* error in export hardware topology */
@@ -248,6 +316,7 @@ handle_write:
 		}
 	}
 
+end_it:
 	if (!topology_in)
 		hwloc_topology_destroy(tmp_topo);
 
@@ -261,7 +330,7 @@ handle_write:
  *         p_sockets - number of physical processor sockets
  *         p_cores - total number of physical CPU cores
  *         p_threads - total number of hardware execution threads
- *         block_map - asbtract->physical block distribution map
+ *         block_map - abstract->physical block distribution map
  *         block_map_inv - physical->abstract block distribution map (inverse)
  *         return code - 0 if no error, otherwise errno
  * NOTE: User must xfree block_map and block_map_inv
@@ -319,6 +388,11 @@ extern int xcpuinfo_hwloc_topo_get(
 	objtype[SOCKET] = HWLOC_OBJ_SOCKET;
 	objtype[CORE]   = HWLOC_OBJ_CORE;
 	objtype[PU]     = HWLOC_OBJ_PU;
+#if HWLOC_API_VERSION >= 0x00020000
+	if (xstrcasestr(slurm_conf.sched_params, "Ignore_NUMA")) {
+		info("SchedulerParamaters=Ignore_NUMA not supported by hwloc v2");
+	}
+#else
 	if (hwloc_get_type_depth(topology, HWLOC_OBJ_NODE) >
 	    hwloc_get_type_depth(topology, HWLOC_OBJ_SOCKET)) {
 		if (xstrcasestr(slurm_conf.sched_params, "Ignore_NUMA")) {
@@ -328,12 +402,32 @@ extern int xcpuinfo_hwloc_topo_get(
 			objtype[SOCKET] = HWLOC_OBJ_NODE;
 		}
 	}
+#endif
 
 	if (xstrcasestr(slurm_conf.slurmd_params, "l3cache_as_socket")) {
 #if HWLOC_API_VERSION >= 0x00020000
 		objtype[SOCKET] = HWLOC_OBJ_L3CACHE;
 #else
 		error("SlurmdParameters=l3cache_as_socket requires hwloc v2");
+#endif
+	} else if (xstrcasestr(slurm_conf.slurmd_params,
+			       "numa_node_as_socket")) {
+#if HWLOC_API_VERSION >= 0x00020000
+		hwloc_obj_t numa_obj = hwloc_get_next_obj_by_type(
+			topology, HWLOC_OBJ_NODE, NULL);
+
+		if (numa_obj && numa_obj->parent) {
+			objtype[SOCKET] = numa_obj->parent->type;
+			if (get_log_level() >= LOG_LEVEL_DEBUG2) {
+				char tmp[128];
+				hwloc_obj_type_snprintf(tmp, sizeof(tmp),
+							numa_obj->parent, 0);
+				debug2("%s: numa_node_as_socket mapped to '%s'",
+				       __func__, tmp);
+			}
+		}
+#else
+		error("SlurmdParameters=numa_node_as_socket requires hwloc v2");
 #endif
 	}
 
@@ -679,8 +773,7 @@ extern int xcpuinfo_hwloc_topo_get(
 		if (cores == 0) {
 			cores = numcpu / sockets;	/* assume multi-core */
 			if (cores > 1) {
-				debug3("Warning: cpuinfo missing 'core id' or "
-				       "'cpu cores' but assuming multi-core");
+				debug3("cpuinfo missing 'core id' or 'cpu cores' but assuming multi-core");
 			}
 		}
 		if (cores == 0)
@@ -949,22 +1042,27 @@ int
 xcpuinfo_init(void)
 {
 	if ( initialized )
-		return XCPUINFO_SUCCESS;
+		return SLURM_SUCCESS;
 
 	if (xcpuinfo_hwloc_topo_get(&procs,&boards,&sockets,&cores,&threads,
 				    &block_map_size,&block_map,&block_map_inv))
-		return XCPUINFO_ERROR;
+		return SLURM_ERROR;
 
 	initialized = true ;
 
-	return XCPUINFO_SUCCESS;
+	return SLURM_SUCCESS;
+}
+
+extern void xcpuinfo_refresh_hwloc(bool refresh)
+{
+	refresh_hwloc = refresh;
 }
 
 int
 xcpuinfo_fini(void)
 {
 	if ( ! initialized )
-		return XCPUINFO_SUCCESS;
+		return SLURM_SUCCESS;
 
 	initialized = false ;
 	procs = sockets = cores = threads = 0;
@@ -984,7 +1082,7 @@ xcpuinfo_fini(void)
 		xfree(hwloc_xml_whole);
 	}
 #endif
-	return XCPUINFO_SUCCESS;
+	return SLURM_SUCCESS;
 }
 
 /*
@@ -1031,7 +1129,11 @@ int xcpuinfo_abs_to_mac(char *lrange, char **prange)
 	for (icore = 0; icore < total_cores; icore++) {
 		if (bit_test(absmap, icore)) {
 			for (ithread = 0; ithread < conf->threads; ithread++) {
-				absid = icore * conf->threads + ithread;
+				/*
+				 * Use actual hardware thread count to get the
+				 * correct offset for the CPU ID.
+				 */
+				absid = icore * conf->actual_threads + ithread;
 				absid %= total_cpus;
 
 				macid = conf->block_map[absid];
@@ -1102,7 +1204,11 @@ int xcpuinfo_mac_to_abs(char *in_range, char **out_range)
 	for (int icore = 0; icore < total_cores; icore++) {
 		for (int ithread = 0; ithread < conf->threads; ithread++) {
 			int absid, macid;
-			macid = (icore * conf->threads) + ithread;
+			/*
+			 * Use actual hardware thread count to get the
+			 * correct offset for the CPU ID.
+			 */
+			macid = (icore * conf->actual_threads) + ithread;
 			macid %= total_cpus;
 
 			/* Skip this machine CPU id if not in in_range */
@@ -1119,7 +1225,11 @@ int xcpuinfo_mac_to_abs(char *in_range, char **out_range)
 	/* condense abstract CPU bitmap into an abstract core bitmap */
 	for (int icore = 0; icore < total_cores; icore++) {
 		for (int ithread = 0; ithread < conf->threads; ithread++) {
-			int icpu = (icore * conf->threads) + ithread;
+			/*
+			 * Use actual hardware thread count to get the
+			 * correct offset for the CPU ID.
+			 */
+			int icpu = (icore * conf->actual_threads) + ithread;
 			icpu %= total_cpus;
 
 			if (bit_test(absmap, icpu)) {
@@ -1154,57 +1264,6 @@ xcpuinfo_abs_to_map(char* lrange,uint16_t **map,uint16_t *map_size)
 	return _range_to_map(lrange,*map,*map_size,1);
 }
 
-int
-xcpuinfo_map_to_mac(uint16_t *map,uint16_t map_size,char** range)
-{
-	return _map_to_range(map,map_size,range);
-}
-
-int
-xcpuinfo_mac_to_map(char* lrange,uint16_t **map,uint16_t *map_size)
-{
-	*map_size = block_map_size;
-	*map = (uint16_t*) xmalloc(block_map_size*sizeof(uint16_t));
-	/* machine range already includes the hyperthreads */
-	return _range_to_map(lrange,*map,*map_size,0);
-}
-
-int
-xcpuinfo_absmap_to_macmap(uint16_t *amap,uint16_t amap_size,
-			  uint16_t **bmap,uint16_t *bmap_size)
-{
-	/* int i; */
-
-	/* abstract to machine conversion using block map */
-	uint16_t *map_out;
-
-	*bmap_size = amap_size;
-	map_out = (uint16_t*) xmalloc(amap_size*sizeof(uint16_t));
-	*bmap = map_out;
-
-	return XCPUINFO_SUCCESS;
-}
-
-int
-xcpuinfo_macmap_to_absmap(uint16_t *amap,uint16_t amap_size,
-			  uint16_t **bmap,uint16_t *bmap_size)
-{
-	int i;
-
-	/* machine to abstract conversion using inverted block map */
-	uint16_t *cmap;
-	cmap = block_map_inv;
-	*bmap_size = amap_size;
-	*bmap = (uint16_t*) xmalloc(amap_size*sizeof(uint16_t));
-	for( i = 0 ; i < amap_size ; i++) {
-		if ( amap[i] )
-			(*bmap)[cmap[i]]=1;
-		else
-			(*bmap)[cmap[i]]=0;
-	}
-	return XCPUINFO_SUCCESS;
-}
-
 /*
  * set to 1 each element of already allocated map of size
  * map_size if they are present in the input range
@@ -1279,71 +1338,8 @@ _range_to_map(char* range,uint16_t *map,uint16_t map_size,int add_threads)
 
 	if ( bad_nb > 0 ) {
 		/* bad format for input range */
-		return XCPUINFO_ERROR;
+		return SLURM_ERROR;
 	}
 
-	return XCPUINFO_SUCCESS;
-}
-
-
-/*
- * allocate and build a range of ids using an input map
- * having printable element set to 1
- */
-static int
-_map_to_range(uint16_t *map,uint16_t map_size,char** prange)
-{
-	size_t len;
-	int num_fl=0;
-	int con_fl=0;
-
-	char *str;
-
-	uint16_t start=0,end=0,i;
-
-	str = xstrdup("");
-	for ( i = 0 ; i < map_size ; i++ ) {
-
-		if ( map[i] ) {
-			num_fl=1;
-			end=i;
-			if ( !con_fl ) {
-				start=end;
-				con_fl=1;
-			}
-		}
-		else if ( num_fl ) {
-			if ( start < end ) {
-				xstrfmtcat(str, "%u-%u,", start, end);
-			}
-			else {
-				xstrfmtcat(str, "%u,", start);
-			}
-			con_fl = num_fl = 0;
-		}
-	}
-	if ( num_fl ) {
-		if ( start < end ) {
-			xstrfmtcat(str, "%u-%u,", start, end);
-		}
-		else {
-			xstrfmtcat(str, "%u,", start);
-		}
-	}
-
-	len = strlen(str);
-	if ( len > 0 ) {
-		str[len-1]='\0';
-	}
-	else {
-		xfree(str);
-		return XCPUINFO_ERROR;
-	}
-
-	if ( prange != NULL )
-		*prange = str;
-	else
-		xfree(str);
-
-	return XCPUINFO_SUCCESS;
+	return SLURM_SUCCESS;
 }
