@@ -48,38 +48,36 @@
 
 #include "src/common/assoc_mgr.h"
 #include "src/common/cpu_frequency.h"
-#include "src/interfaces/gres.h"
-#include "src/interfaces/hash.h"
 #include "src/common/run_command.h"
-#include "src/interfaces/route.h"
-#include "src/interfaces/select.h"
 #include "src/interfaces/topology.h"
 #include "src/common/setproctitle.h"
-#include "src/interfaces/auth.h"
-#include "src/interfaces/jobacct_gather.h"
-#include "src/interfaces/acct_gather_profile.h"
-#include "src/interfaces/mpi.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/spank.h"
 #include "src/common/stepd_api.h"
-#include "src/interfaces/switch.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
-#include "src/interfaces/cgroup.h"
 
+#include "src/interfaces/acct_gather_energy.h"
+#include "src/interfaces/acct_gather_profile.h"
+#include "src/interfaces/auth.h"
+#include "src/interfaces/cgroup.h"
 #include "src/interfaces/core_spec.h"
 #include "src/interfaces/gpu.h"
+#include "src/interfaces/gres.h"
+#include "src/interfaces/hash.h"
 #include "src/interfaces/job_container.h"
+#include "src/interfaces/jobacct_gather.h"
+#include "src/interfaces/mpi.h"
+#include "src/interfaces/proctrack.h"
+#include "src/interfaces/switch.h"
+#include "src/interfaces/task.h"
 
+#include "src/slurmd/common/privileges.h"
 #include "src/slurmd/common/set_oomadj.h"
 #include "src/slurmd/common/slurmstepd_init.h"
-#include "src/interfaces/task.h"
-#include "src/interfaces/acct_gather_energy.h"
-#include "src/interfaces/proctrack.h"
-#include "src/slurmd/common/xcpuinfo.h"
 #include "src/slurmd/slurmd/slurmd.h"
 #include "src/slurmd/slurmstepd/container.h"
 #include "src/slurmd/slurmstepd/mgr.h"
@@ -88,18 +86,16 @@
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 static int _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
-			     slurm_addr_t **_self, slurm_msg_t **_msg);
+			    slurm_msg_t **_msg);
 
-static void _dump_user_env(void);
 static void _send_ok_to_slurmd(int sock);
-static void _send_fail_to_slurmd(int sock);
+static void _send_fail_to_slurmd(int sock, int rc);
 static void _got_ack_from_slurmd(int);
-static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_addr_t *self,
-				     slurm_msg_t *msg);
+static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_msg_t *msg);
 #ifdef MEMORY_LEAK_DEBUG
 static void _step_cleanup(stepd_step_rec_t *step, slurm_msg_t *msg, int rc);
 #endif
-static int _process_cmdline (int argc, char **argv);
+static void _process_cmdline(int argc, char **argv);
 
 static pthread_mutex_t cleanup_mutex = PTHREAD_MUTEX_INITIALIZER;
 static bool cleanup = false;
@@ -122,32 +118,30 @@ main (int argc, char **argv)
 {
 	log_options_t lopts = LOG_OPTS_INITIALIZER;
 	slurm_addr_t *cli;
-	slurm_addr_t *self;
 	slurm_msg_t *msg;
 	stepd_step_rec_t *step;
 	int rc = 0;
 
-	if (_process_cmdline (argc, argv) < 0)
-		fatal ("Error in slurmstepd command line");
+	_process_cmdline(argc, argv);
 
 	run_command_init();
 
 	xsignal_block(slurmstepd_blocked_signals);
 	conf = xmalloc(sizeof(*conf));
-	conf->argv = &argv;
-	conf->argc = &argc;
+	conf->argv = argv;
+	conf->argc = argc;
 	init_setproctitle(argc, argv);
 
 	log_init(argv[0], lopts, LOG_DAEMON, NULL);
 
 	/* Receive job parameters from the slurmd */
-	_init_from_slurmd(STDIN_FILENO, argv, &cli, &self, &msg);
+	_init_from_slurmd(STDIN_FILENO, argv, &cli, &msg);
 
 	/* Create the stepd_step_rec_t, mostly from info in a
 	 * launch_tasks_request_msg_t or a batch_job_launch_msg_t */
-	if (!(step = _step_setup(cli, self, msg))) {
-		_send_fail_to_slurmd(STDOUT_FILENO);
+	if (!(step = _step_setup(cli, msg))) {
 		rc = SLURM_ERROR;
+		_send_fail_to_slurmd(STDOUT_FILENO, rc);
 		goto ending;
 	}
 
@@ -157,20 +151,19 @@ main (int argc, char **argv)
 
 	/* sets step->msg_handle and step->msgid */
 	if (msg_thr_create(step) == SLURM_ERROR) {
-		_send_fail_to_slurmd(STDOUT_FILENO);
 		rc = SLURM_ERROR;
+		_send_fail_to_slurmd(STDOUT_FILENO, rc);
 		goto ending;
 	}
 
 	if (step->step_id.step_id != SLURM_EXTERN_CONT)
-		close_slurmd_conn();
+		close_slurmd_conn(rc);
 
 	/* slurmstepd is the only daemon that should survive upgrade. If it
 	 * had been swapped out before upgrade happened it could easily lead
 	 * to SIGBUS at any time after upgrade. Avoid that by locking it
 	 * in-memory. */
 	if (xstrstr(slurm_conf.launch_params, "slurmstepd_memlock")) {
-#ifdef _POSIX_MEMLOCK
 		int flags = MCL_CURRENT;
 		if (xstrstr(slurm_conf.launch_params, "slurmstepd_memlock_all"))
 			flags |= MCL_FUTURE;
@@ -178,9 +171,6 @@ main (int argc, char **argv)
 			info("failed to mlock() slurmstepd pages: %m");
 		else
 			debug("slurmstepd locked in memory");
-#else
-		info("mlockall() system call does not appear to be available");
-#endif
 	}
 
 	acct_gather_energy_g_set_data(ENERGY_DATA_STEP_PTR, step);
@@ -189,14 +179,13 @@ main (int argc, char **argv)
 	 * and blocks until the step is complete */
 	rc = job_manager(step);
 
-	return stepd_cleanup(msg, step, cli, self, rc, 0);
+	return stepd_cleanup(msg, step, cli, rc, false);
 ending:
-	return stepd_cleanup(msg, step, cli, self, rc, 1);
+	return stepd_cleanup(msg, step, cli, rc, true);
 }
 
 extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
-			 slurm_addr_t *cli, slurm_addr_t *self,
-			 int rc, bool only_mem)
+			 slurm_addr_t *cli, int rc, bool only_mem)
 {
 	slurm_mutex_lock(&cleanup_mutex);
 
@@ -258,7 +247,6 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	cgroup_conf_destroy();
 
 	xfree(cli);
-	xfree(self);
 	xfree(conf->block_map);
 	xfree(conf->block_map_inv);
 	xfree(conf->hostname);
@@ -274,13 +262,27 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	cleanup = true;
 done:
 	slurm_mutex_unlock(&cleanup_mutex);
-	info("done with job");
+	/* skipping lock of step_complete.lock */
+	if (rc || step_complete.step_rc) {
+		info("%s: done with step (rc[0x%x]:%s, cleanup_rc[0x%x]:%s)",
+		     __func__, step_complete.step_rc,
+		     slurm_strerror(step_complete.step_rc), rc,
+		     slurm_strerror(rc));
+	} else {
+		info("done with step");
+	}
 	return rc;
 }
 
-extern void close_slurmd_conn(void)
+extern void close_slurmd_conn(int rc)
 {
-	_send_ok_to_slurmd(STDOUT_FILENO);
+	debug("%s: sending %d: %s", __func__, rc, slurm_strerror(rc));
+
+	if (rc)
+		_send_fail_to_slurmd(STDOUT_FILENO, rc);
+	else
+		_send_ok_to_slurmd(STDOUT_FILENO);
+
 	_got_ack_from_slurmd(STDIN_FILENO);
 
 	/* Fancy way of closing stdin that keeps STDIN_FILENO from being
@@ -415,7 +417,7 @@ static int _get_jobid_uid_gid_from_env(uint32_t *jobid, uid_t *uid, gid_t *gid)
 	return SLURM_SUCCESS;
 }
 
-static int _handle_spank_mode (int argc, char **argv)
+static int _handle_spank_mode(int argc, char **argv)
 {
 	char *prefix = NULL;
 	const char *mode = argv[2];
@@ -444,8 +446,8 @@ static int _handle_spank_mode (int argc, char **argv)
 	 *   This could happen if slurmstepd is run standalone for
 	 *   testing.
 	 */
-	conf = _read_slurmd_conf_lite (STDIN_FILENO);
-	close (STDIN_FILENO);
+	conf = _read_slurmd_conf_lite(STDIN_FILENO);
+	close(STDIN_FILENO);
 
 	if (_get_jobid_uid_gid_from_env(&jobid, &uid, &gid))
 		return error("spank environment invalid");
@@ -453,45 +455,43 @@ static int _handle_spank_mode (int argc, char **argv)
 	debug("Running spank/%s for jobid [%u] uid [%u] gid [%u]",
 	      mode, jobid, uid, gid);
 
-	if (xstrcmp (mode, "prolog") == 0) {
+	if (!xstrcmp(mode, "prolog")) {
 		if (spank_job_prolog(jobid, uid, gid) < 0)
-			return (-1);
-	}
-	else if (xstrcmp (mode, "epilog") == 0) {
+			return -1;
+	} else if (!xstrcmp(mode, "epilog")) {
 		if (spank_job_epilog(jobid, uid, gid) < 0)
-			return (-1);
+			return -1;
+	} else {
+		error("Invalid mode %s specified!", mode);
+		return -1;
 	}
-	else {
-		error ("Invalid mode %s specified!", mode);
-		return (-1);
-	}
-	return (0);
+
+	return 0;
 }
 
 /*
  *  Process special "modes" of slurmstepd passed as cmdline arguments.
  */
-static int _process_cmdline (int argc, char **argv)
+static void _process_cmdline(int argc, char **argv)
 {
-	if ((argc == 2) && (xstrcmp(argv[1], "getenv") == 0)) {
+	if ((argc == 2) && !xstrcmp(argv[1], "getenv")) {
 		print_rlimits();
-		_dump_user_env();
+		for (int i = 0; environ[i]; i++)
+			printf("%s\n", environ[i]);
 		exit(0);
 	}
-	if ((argc == 2) && (xstrcmp(argv[1], "infinity") == 0)) {
+	if ((argc == 2) && !xstrcmp(argv[1], "infinity")) {
 		set_oom_adj(-1000);
 		(void) poll(NULL, 0, -1);
 		exit(0);
 	}
-	if ((argc == 3) && (xstrcmp(argv[1], "spank") == 0)) {
+	if ((argc == 3) && !xstrcmp(argv[1], "spank")) {
 		if (_handle_spank_mode(argc, argv) < 0)
-			exit (1);
-		exit (0);
+			exit(1);
+		exit(0);
 	}
-	return (0);
 }
 
-
 static void
 _send_ok_to_slurmd(int sock)
 {
@@ -506,18 +506,12 @@ rwfail:
 #endif
 }
 
-static void
-_send_fail_to_slurmd(int sock)
+static void _send_fail_to_slurmd(int sock, int rc)
 {
 	/* If running under valgrind/memcheck, this pipe doesn't work correctly
 	 * so just skip it. */
 #if (SLURMSTEPD_MEMCHECK == 0)
-	int fail = SLURM_ERROR;
-
-	if (errno)
-		fail = errno;
-
-	safe_write(sock, &fail, sizeof(int));
+	safe_write(sock, &rc, sizeof(int));
 	return;
 rwfail:
 	error("Unable to send \"fail\" to slurmd");
@@ -558,8 +552,8 @@ static void _set_job_log_prefix(slurm_step_id_t *step_id)
  *  sent by _send_slurmstepd_init() in src/slurmd/slurmd/req.c.
  */
 static int
-_init_from_slurmd(int sock, char **argv,
-		  slurm_addr_t **_cli, slurm_addr_t **_self, slurm_msg_t **_msg)
+_init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
+		  slurm_msg_t **_msg)
 {
 	char *incoming_buffer = NULL;
 	buf_t *buffer;
@@ -567,7 +561,6 @@ _init_from_slurmd(int sock, char **argv,
 	int len;
 	uint16_t proto;
 	slurm_addr_t *cli = NULL;
-	slurm_addr_t *self = NULL;
 	slurm_msg_t *msg = NULL;
 	slurm_step_id_t step_id = {
 		.job_id = 0,
@@ -581,6 +574,11 @@ _init_from_slurmd(int sock, char **argv,
 
 	slurm_conf.slurmd_port = conf->port;
 	slurm_conf.slurmd_syslog_debug = conf->syslog_debug;
+	/*
+	 * max_node_cnt is not sent over from slurmd and will be 0 unless we set
+	 * it here to be consistent with the way it's used elsewhere.
+	 */
+	slurm_conf.max_node_cnt = NO_VAL;
 
 	setenvf(NULL, "SLURMD_NODENAME", "%s", conf->node_name);
 
@@ -598,14 +596,19 @@ _init_from_slurmd(int sock, char **argv,
 	safe_read(sock, &step_complete.children, sizeof(int));
 	safe_read(sock, &step_complete.depth, sizeof(int));
 	safe_read(sock, &step_complete.max_depth, sizeof(int));
-	safe_read(sock, &step_complete.parent_addr, sizeof(slurm_addr_t));
+	safe_read(sock, &len, sizeof(int));
+	if (len) {
+		step_complete.parent_name = xmalloc(len + 1);
+		safe_read(sock, step_complete.parent_name, len);
+	}
+
 	if (step_complete.children)
 		step_complete.bits = bit_alloc(step_complete.children);
 	step_complete.jobacct = jobacctinfo_create(NULL);
 	slurm_mutex_unlock(&step_complete.lock);
 
-	debug3("slurmstepd rank %d, parent = %pA",
-	       step_complete.rank, &step_complete.parent_addr);
+	debug3("slurmstepd rank %d, parent = %s",
+	       step_complete.rank, step_complete.parent_name);
 
 	/* receive cli from slurmd */
 	safe_read(sock, &len, sizeof(int));
@@ -617,22 +620,6 @@ _init_from_slurmd(int sock, char **argv,
 		fatal("slurmstepd: problem with unpack of slurmd_conf");
 	FREE_NULL_BUFFER(buffer);
 
-	/* receive self from slurmd */
-	safe_read(sock, &len, sizeof(int));
-	if (len > 0) {
-		/* receive packed self from main slurmd */
-		incoming_buffer = xmalloc(len);
-		safe_read(sock, incoming_buffer, len);
-		buffer = create_buf(incoming_buffer,len);
-		self = xmalloc(sizeof(slurm_addr_t));
-		if (slurm_unpack_addr_no_alloc(self, buffer)
-		    == SLURM_ERROR) {
-			fatal("slurmstepd: problem with unpack of "
-			      "slurmd_conf");
-		}
-		FREE_NULL_BUFFER(buffer);
-	}
-
 	/* Grab the slurmd's spooldir. Has %n expanded. */
 	cpu_freq_init(conf);
 
@@ -665,13 +652,13 @@ _init_from_slurmd(int sock, char **argv,
 		break;
 	}
 
-	/* Init select and switch before unpack_msg to only init the default */
-	if (select_g_init(1) != SLURM_SUCCESS )
-		fatal( "failed to initialize node selection plugin" );
-
+	/* Init switch before unpack_msg to only init the default */
 	if (switch_init(1) != SLURM_SUCCESS)
 		fatal( "failed to initialize authentication plugin" );
 
+	if (cred_g_init() != SLURM_SUCCESS)
+		fatal("failed to initialize credential plugin");
+
 	if (gres_init() != SLURM_SUCCESS)
 		fatal("failed to initialize gres plugins");
 
@@ -700,19 +687,17 @@ _init_from_slurmd(int sock, char **argv,
 	/*
 	 * Init all plugins after receiving the slurm.conf from the slurmd.
 	 */
-	if ((slurm_auth_init(NULL) != SLURM_SUCCESS) ||
+	if ((auth_g_init() != SLURM_SUCCESS) ||
 	    (cgroup_g_init() != SLURM_SUCCESS) ||
 	    (hash_g_init() != SLURM_SUCCESS) ||
 	    (acct_gather_conf_init() != SLURM_SUCCESS) ||
 	    (core_spec_g_init() != SLURM_SUCCESS) ||
-	    (slurm_proctrack_init() != SLURM_SUCCESS) ||
+	    (proctrack_g_init() != SLURM_SUCCESS) ||
 	    (slurmd_task_init() != SLURM_SUCCESS) ||
 	    (jobacct_gather_init() != SLURM_SUCCESS) ||
 	    (acct_gather_profile_init() != SLURM_SUCCESS) ||
-	    (slurm_cred_init() != SLURM_SUCCESS) ||
 	    (job_container_init() != SLURM_SUCCESS) ||
-	    (route_init() != SLURM_SUCCESS) ||
-	    (slurm_topo_init() != SLURM_SUCCESS))
+	    (topology_g_init() != SLURM_SUCCESS))
 		fatal("Couldn't load all plugins");
 
 	/*
@@ -761,7 +746,6 @@ _init_from_slurmd(int sock, char **argv,
 	msg->protocol_version = proto;
 
 	*_cli = cli;
-	*_self = self;
 	*_msg = msg;
 
 	return 1;
@@ -771,8 +755,7 @@ rwfail:
 	exit(1);
 }
 
-static stepd_step_rec_t *
-_step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
+static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_msg_t *msg)
 {
 	stepd_step_rec_t *step = NULL;
 
@@ -783,7 +766,7 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 		break;
 	case REQUEST_LAUNCH_TASKS:
 		debug2("setup for a launch_task");
-		step = mgr_launch_tasks_setup(msg->data, cli, self,
+		step = mgr_launch_tasks_setup(msg->data, cli,
 					      msg->protocol_version);
 		break;
 	default:
@@ -835,7 +818,9 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 				    step->step_id.job_id,
 				    step->step_id.step_id);
 	}
-	if (step->batch || (step->step_id.step_id == SLURM_INTERACTIVE_STEP)) {
+	if (step->batch ||
+	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (step->flags & LAUNCH_EXT_LAUNCHER)) {
 		gres_g_job_set_env(step, 0);
 	} else if (msg->msg_type == REQUEST_LAUNCH_TASKS) {
 		gres_g_step_set_env(step);
@@ -848,11 +833,11 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 			    conf->node_topo_addr);
 	env_array_overwrite(&step->env,"SLURM_TOPOLOGY_ADDR_PATTERN",
 			    conf->node_topo_pattern);
-	/*
-	 * Reset address for cloud nodes
-	 */
-	if (step->alias_list && set_nodes_alias(step->alias_list)) {
-		error("%s: set_nodes_alias failed: %s", __func__,
+
+	/* Reset addrs for dyanmic/cloud nodes to hash tables*/
+	if (step->node_addrs &&
+	    add_remote_nodes_to_conf_tbls(step->node_list, step->node_addrs)) {
+		error("%s: failed to add node addrs: %s", __func__,
 		      step->alias_list);
 		stepd_step_rec_destroy(step);
 		return NULL;
@@ -895,11 +880,3 @@ _step_cleanup(stepd_step_rec_t *step, slurm_msg_t *msg, int rc)
 	jobacctinfo_destroy(step_complete.jobacct);
 }
 #endif
-
-static void _dump_user_env(void)
-{
-	int i;
-
-	for (i=0; environ[i]; i++)
-		printf("%s\n",environ[i]);
-}
