@@ -42,13 +42,13 @@
 
 #include "src/common/list.h"
 #include "src/common/log.h"
-#include "src/interfaces/openapi.h"
 #include "src/common/plugin.h"
 #include "src/interfaces/auth.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+#include "src/slurmrestd/openapi.h"
 #include "src/slurmrestd/rest_auth.h"
 
 static pthread_mutex_t init_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -169,7 +169,7 @@ extern int rest_authenticate_http_request(on_http_request_args_t *args)
 
 	if (context) {
 		fatal("%s: authentication context already set for connection: %s",
-		      __func__, args->context->con->name);
+		      __func__, conmgr_fd_get_name(args->context->con));
 	}
 
 	args->context->auth = context = rest_auth_g_new();
