@@ -51,7 +51,9 @@
 #include "src/common/read_config.h"
 #include "src/common/xstring.h"
 #include "src/common/proc_args.h"
+#include "src/common/ref.h"
 #include "src/common/uid.h"
+#include "src/interfaces/serializer.h"
 
 #include "src/squeue/squeue.h"
 
@@ -61,21 +63,20 @@
 #define OPT_LONG_HIDE         0x102
 #define OPT_LONG_START        0x103
 #define OPT_LONG_NOCONVERT    0x104
-#define OPT_LONG_ARRAY_UNIQUE 0x105
 #define OPT_LONG_LOCAL        0x106
 #define OPT_LONG_SIBLING      0x107
 #define OPT_LONG_FEDR         0x108
 #define OPT_LONG_ME           0x109
 #define OPT_LONG_JSON         0x110
 #define OPT_LONG_YAML         0x111
+#define OPT_LONG_AUTOCOMP     0x112
 
 /* FUNCTIONS */
-static List  _build_job_list( char* str );
-static List  _build_str_list( char* str );
-static List  _build_state_list( char* str );
-static List  _build_all_states_list( void );
-static List  _build_step_list( char* str );
-static List  _build_user_list( char* str );
+static list_t *_build_job_list(char *str);
+static list_t *_build_str_list(char *str);
+static list_t *_build_state_list(char *str);
+static list_t *_build_step_list(char *str);
+static list_t *_build_user_list(char *str);
 static char *_get_prefix(char *token);
 static void  _help( void );
 static int   _parse_state( char* str, uint32_t* states );
@@ -86,9 +87,12 @@ static void _parse_long_token( char *token, char *sep, int *field_size,
 static void  _print_options( void );
 static void  _usage( void );
 static void _filter_nodes(void);
-static List _load_clusters_nodes(void);
+static list_t *_load_clusters_nodes(void);
 static void _node_info_list_del(void *data);
-static char *_map_node_name(List clusters_node_info, char *name);
+static char *_map_node_name(list_t *clusters_node_info, char *name);
+
+decl_static_data(help_txt);
+decl_static_data(usage_txt);
 
 /*
  * parse_command_line
@@ -101,10 +105,10 @@ parse_command_line( int argc, char* *argv )
 	int opt_char;
 	int option_index;
 	static struct option long_options[] = {
+		{"autocomplete", required_argument, 0, OPT_LONG_AUTOCOMP},
 		{"accounts",   required_argument, 0, 'A'},
 		{"all",        no_argument,       0, 'a'},
 		{"array",      no_argument,       0, 'r'},
-		{"array-unique",no_argument,      0, OPT_LONG_ARRAY_UNIQUE},
 		{"Format",     required_argument, 0, 'O'},
 		{"format",     required_argument, 0, 'o'},
 		{"federation", no_argument,       0, OPT_LONG_FEDR},
@@ -139,8 +143,8 @@ parse_command_line( int argc, char* *argv )
 		{"users",      required_argument, 0, 'u'},
 		{"verbose",    no_argument,       0, 'v'},
 		{"version",    no_argument,       0, 'V'},
-		{"json", no_argument, 0, OPT_LONG_JSON},
-		{"yaml", no_argument, 0, OPT_LONG_YAML},
+		{"json", optional_argument, 0, OPT_LONG_JSON},
+		{"yaml", optional_argument, 0, OPT_LONG_YAML},
 		{NULL,         0,                 0, 0}
 	};
 
@@ -155,8 +159,6 @@ parse_command_line( int argc, char* *argv )
 		params.array_flag = true;
 	if ( ( env_val = getenv("SQUEUE_SORT") ) )
 		params.sort = xstrdup(env_val);
-	if (getenv("SQUEUE_ARRAY_UNIQUE"))
-		params.array_unique_flag = true;
 	if ( ( env_val = getenv("SLURM_CLUSTERS") ) ) {
 		if (!(params.clusters = slurmdb_get_info_cluster(env_val))) {
 			print_db_notok(env_val, 1);
@@ -323,9 +325,6 @@ parse_command_line( int argc, char* *argv )
 				exit(1);
 			}
 			break;
-		case OPT_LONG_ARRAY_UNIQUE:
-			params.array_unique_flag = true;
-			break;
 		case OPT_LONG_HELP:
 			_help();
 			exit(0);
@@ -358,11 +357,21 @@ parse_command_line( int argc, char* *argv )
 			exit(0);
 		case OPT_LONG_JSON:
 			params.mimetype = MIME_TYPE_JSON;
-			data_init(MIME_TYPE_JSON_PLUGIN, NULL);
+			params.data_parser = optarg;
+			params.detail_flag = true;
+			if (serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL))
+				fatal("JSON plugin load failure");
 			break;
 		case OPT_LONG_YAML:
 			params.mimetype = MIME_TYPE_YAML;
-			data_init(MIME_TYPE_YAML_PLUGIN, NULL);
+			params.data_parser = optarg;
+			params.detail_flag = true;
+			if (serializer_g_init(MIME_TYPE_YAML_PLUGIN, NULL))
+				fatal("YAML plugin load failure");
+			break;
+		case OPT_LONG_AUTOCOMP:
+			suggest_completion(long_options, optarg);
+			exit(0);
 			break;
 		}
 	}
@@ -466,7 +475,7 @@ parse_command_line( int argc, char* *argv )
 		params.job_id = job_step_ptr->step_id.job_id;
 	}
 	if (params.user_list && (list_count(params.user_list) == 1)) {
-		ListIterator iterator;
+		list_itr_t *iterator;
 		uint32_t *uid_ptr;
 		iterator = list_iterator_create(params.user_list);
 		while ((uid_ptr = list_next(iterator))) {
@@ -836,12 +845,6 @@ extern int parse_format( char* format )
 							    field_size,
 							    right_justify,
 							    suffix );
-			else if (field[0] == 's')
-				job_format_add_select_jobinfo(
-					params.format_list,
-					field_size,
-					right_justify,
-					suffix );
 			else if (field[0] == 'S')
 				job_format_add_time_start( params.format_list,
 							   field_size,
@@ -973,6 +976,12 @@ extern int parse_long_format( char* format_long )
 							  field_size,
 							  right_justify,
 							  suffix);
+			else if (!xstrncasecmp(token, "containerid",
+					       strlen("containerid")))
+				step_format_add_container_id(params.format_list,
+							     field_size,
+							     right_justify,
+							     suffix);
 			else if (!xstrncasecmp(token, "numtasks",
 					       strlen("numtask")))
 				step_format_add_num_tasks( params.format_list,
@@ -1169,6 +1178,15 @@ extern int parse_long_format( char* format_long )
 							    field_size,
 							    right_justify,
 							    suffix);
+			else if (!xstrcasecmp(token, "container"))
+				job_format_add_container(params.format_list,
+							 field_size,
+							 right_justify, suffix);
+			else if (!xstrcasecmp(token, "containerid"))
+				job_format_add_container_id(params.format_list,
+							    field_size,
+							    right_justify,
+							    suffix);
 			else if (!xstrcasecmp(token, "delayboot"))
 				job_format_add_delay_boot(params.format_list,
 							  field_size,
@@ -1222,6 +1240,11 @@ extern int parse_long_format( char* format_long )
 							params.format_list,
 							field_size,
 							right_justify, suffix);
+			else if (!xstrcasecmp(token, "prefer"))
+				job_format_add_prefer(params.format_list,
+						      field_size,
+						      right_justify,
+						      suffix);
 			else if (!xstrcasecmp(token, "arrayjobid"))
 				job_format_add_array_job_id(
 					params.format_list,
@@ -1360,12 +1383,6 @@ extern int parse_long_format( char* format_long )
 							    field_size,
 							    right_justify,
 							    suffix );
-			else if (!xstrcasecmp(token, "selectjobinfo"))
-				job_format_add_select_jobinfo(
-					params.format_list,
-					field_size,
-					right_justify,
-					suffix );
 			else if (!xstrcasecmp(token, "starttime"))
 				job_format_add_time_start( params.format_list,
 							   field_size,
@@ -1820,7 +1837,7 @@ _parse_long_token( char *token, char *sep, int *field_size, bool *right_justify,
 static void
 _print_options(void)
 {
-	ListIterator iterator;
+	list_itr_t *iterator;
 	int i;
 	char *license, *name, *part;
 	uint32_t *user;
@@ -1858,7 +1875,9 @@ _print_options(void)
 	printf( "users       = %s\n", params.users );
 	printf( "verbose     = %d\n", params.verbose );
 
-	if ((params.verbose > 1) && params.job_list) {
+	if (params.verbose <= 1)
+		goto endit;
+	if (params.job_list) {
 		i = 0;
 		iterator = list_iterator_create( params.job_list );
 		while ( (job_step_id = list_next( iterator )) ) {
@@ -1875,7 +1894,7 @@ _print_options(void)
 	}
 
 
-	if ((params.verbose > 1) && params.name_list) {
+	if (params.name_list) {
 		i = 0;
 		iterator = list_iterator_create( params.name_list );
 		while ( (name = list_next( iterator )) ) {
@@ -1884,7 +1903,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.licenses_list) {
+	if (params.licenses_list) {
 		i = 0;
 		iterator = list_iterator_create( params.licenses_list );
 		while ( (license = list_next( iterator )) ) {
@@ -1893,7 +1912,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.part_list) {
+	if (params.part_list) {
 		i = 0;
 		iterator = list_iterator_create( params.part_list );
 		while ( (part = list_next( iterator )) ) {
@@ -1902,7 +1921,9 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.state_list) {
+	if (params.all_states) {
+		printf( "state_list = all\n");
+	} else if (params.state_list) {
 		i = 0;
 		iterator = list_iterator_create( params.state_list );
 		while ( (state_id = list_next( iterator )) ) {
@@ -1912,7 +1933,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.step_list) {
+	if (params.step_list) {
 		char tmp_char[34];
 		i = 0;
 		iterator = list_iterator_create( params.step_list );
@@ -1939,7 +1960,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.user_list) {
+	if (params.user_list) {
 		i = 0;
 		iterator = list_iterator_create( params.user_list );
 		while ( (user = list_next( iterator )) ) {
@@ -1947,7 +1968,7 @@ _print_options(void)
 		}
 		list_iterator_destroy( iterator );
 	}
-
+endit:
 	printf( "-----------------------------\n\n\n" );
 } ;
 
@@ -1957,10 +1978,9 @@ _print_options(void)
  * IN str - comma separated list of job_ids
  * RET List of job_ids (uint32_t)
  */
-static List
-_build_job_list( char* str )
+static list_t *_build_job_list(char *str)
 {
-	List my_list;
+	list_t *my_list;
 	char *end_ptr = NULL, *job = NULL, *tmp_char = NULL;
 	char *my_job_list = NULL;
 	int job_id, array_id;
@@ -1998,10 +2018,9 @@ _build_job_list( char* str )
  * IN str - comma separated list of strings
  * RET List of strings
  */
-static List
-_build_str_list(char* str)
+static list_t *_build_str_list(char *str)
 {
-	List my_list;
+	list_t *my_list;
 	char *elem, *tok = NULL, *tmp_char = NULL, *my_str = NULL;
 
 	if (str == NULL)
@@ -2023,17 +2042,19 @@ _build_str_list(char* str)
  * IN str - comma separated list of job states
  * RET List of enum job_states values
  */
-static List
-_build_state_list( char* str )
+static list_t *_build_state_list(char *str)
 {
-	List my_list;
+	list_t *my_list;
 	char *state = NULL, *tmp_char = NULL, *my_state_list = NULL;
 	uint32_t *state_id = NULL;
 
 	if (str == NULL)
 		return NULL;
-	if (xstrcasecmp( str, "all") == 0)
-		return _build_all_states_list ();
+	if (!xstrcasecmp(str, "all")) {
+		params.all_states = true;
+		return NULL;
+	}
+	params.all_states = false;
 
 	my_list = list_create(NULL);
 	my_state_list = xstrdup(str);
@@ -2050,47 +2071,14 @@ _build_state_list( char* str )
 
 }
 
-static void _append_state_list(List my_list, uint32_t state_id)
-{
-	uint32_t *state_rec;
-
-	state_rec = xmalloc(sizeof(uint32_t));
-	*state_rec = state_id;
-	list_append(my_list, state_rec);
-}
-
-/*
- * _build_all_states_list - build a list containing all possible job states
- * RET List of uint16_t values
- */
-static List
-_build_all_states_list( void )
-{
-	List my_list;
-	uint32_t i;
-
-	my_list = list_create( NULL );
-	for (i = 0; i < JOB_END; i++)
-		_append_state_list(my_list, i);
-
-	_append_state_list(my_list, JOB_COMPLETING);
-	_append_state_list(my_list, JOB_CONFIGURING);
-	_append_state_list(my_list, JOB_REVOKED);
-	_append_state_list(my_list, JOB_SPECIAL_EXIT);
-
-	return my_list;
-
-}
-
 /*
  * _build_step_list- build a list of job/step_ids
  * IN str - comma separated list of job_id[array_id].step_id values
  * RET List of job/step_ids (structure of uint32_t's)
  */
-static List
-_build_step_list( char* str )
+static list_t *_build_step_list(char *str)
 {
-	List my_list;
+	list_t *my_list;
 	char *end_ptr = NULL, *step = NULL, *tmp_char = NULL, *tmps_char = NULL;
 	char *job_name = NULL, *step_name = NULL, *my_step_list = NULL;
 	int job_id, array_id, step_id;
@@ -2138,10 +2126,9 @@ _build_step_list( char* str )
  * IN str - comma separated list of user names
  * RET List of UIDs (uint32_t)
  */
-static List
-_build_user_list( char* str )
+static list_t *_build_user_list(char *str)
 {
-	List my_list;
+	list_t *my_list;
 	char *user = NULL;
 	char *tmp_char = NULL, *my_user_list = NULL;
 
@@ -2167,71 +2154,20 @@ _build_user_list( char* str )
 	return my_list;
 }
 
-static void _usage(void)
-{
-	printf("\
-Usage: squeue [-A account] [--clusters names] [-i seconds] [--job jobid]\n\
-              [-n name] [-o format] [-p partitions] [--qos qos]\n\
-              [--reservation reservation] [--sort fields] [--start]\n\
-              [--step step_id] [-t states] [-u user_name] [--usage]\n\
-              [-L licenses] [-w nodes] [--federation] [--local] [--sibling]\n\
-	      [-ahjlrsv]\n");
-}
-
 static void _help(void)
 {
-	printf("\
-Usage: squeue [OPTIONS]\n\
-  -A, --account=account(s)        comma separated list of accounts\n\
-				  to view, default is all accounts\n\
-  -a, --all                       display jobs in hidden partitions\n\
-      --array-unique              display one unique pending job array\n\
-                                  element per line\n\
-      --federation                Report federated information if a member\n\
-                                  of one\n\
-  -h, --noheader                  no headers on output\n\
-      --hide                      do not display jobs in hidden partitions\n\
-  -i, --iterate=seconds           specify an interation period\n\
-  -j, --job=job(s)                comma separated list of jobs IDs\n\
-                                  to view, default is all\n\
-      --json                      Produce JSON output\n\
-      --local                     Report information only about jobs on the\n\
-                                  local cluster. Overrides --federation.\n\
-  -l, --long                      long report\n\
-  -L, --licenses=(license names)  comma separated list of license names to view\n\
-  -M, --clusters=cluster_name     cluster to issue commands to.  Default is\n\
-                                  current cluster.  cluster with no name will\n\
-                                  reset to default. Implies --local.\n\
-  -n, --name=job_name(s)          comma separated list of job names to view\n\
-      --noconvert                 don't convert units from their original type\n\
-                                  (e.g. 2048M won't be converted to 2G).\n\
-  -o, --format=format             format specification\n\
-  -O, --Format=format             format specification\n\
-  -p, --partition=partition(s)    comma separated list of partitions\n\
-				  to view, default is all partitions\n\
-  -q, --qos=qos(s)                comma separated list of qos's\n\
-				  to view, default is all qos's\n\
-  -R, --reservation=name          reservation to view, default is all\n\
-  -r, --array                     display one job array element per line\n\
-      --sibling                   Report information about all sibling jobs\n\
-                                  on a federated cluster. Implies --federation.\n\
-  -s, --step=step(s)              comma separated list of job steps\n\
-				  to view, default is all\n\
-  -S, --sort=fields               comma separated list of fields to sort on\n\
-      --start                     print expected start times of pending jobs\n\
-  -t, --states=states             comma separated list of states to view,\n\
-				  default is pending and running,\n\
-				  '--states=all' reports all states\n\
-  -u, --user=user_name(s)         comma separated list of users to view\n\
-      --name=job_name(s)          comma separated list of job names to view\n\
-  -v, --verbose                   verbosity level\n\
-  -V, --version                   output version information and exit\n\
-  -w, --nodelist=hostlist         list of nodes to view, default is \n\
-				  all nodes\n\
-      --yaml                      Produce YAML output\n\
-\nHelp options:\n\
-  --help                          show this help message\n\
-  --usage                         display a brief summary of squeue options\n");
+	char *txt;
+	static_ref_to_cstring(txt, help_txt);
+	printf("%s", txt);
+	xfree(txt);
+}
+
+static void _usage(void)
+{
+	char *txt;
+	static_ref_to_cstring(txt, usage_txt);
+	printf("%s", txt);
+	xfree(txt);
 }
 
 /*
@@ -2240,8 +2176,8 @@ Usage: squeue [OPTIONS]\n\
 static void _filter_nodes(void)
 {
 	char *name = NULL, *nodename = NULL;
-	hostset_t nodenames = hostset_create(NULL);
-	List clusters_nodes = NULL;
+	hostset_t *nodenames = hostset_create(NULL);
+	list_t *clusters_nodes = NULL;
 
 	/* Retrieve node_info from controllers */
 	if (!(clusters_nodes = _load_clusters_nodes()))
@@ -2282,10 +2218,10 @@ static void _node_info_list_del(void *data)
  *
  * NOTE: caller must free the returned list if not NULL.
  */
-static List _load_clusters_nodes(void)
+static list_t *_load_clusters_nodes(void)
 {
-	List node_info_list = NULL;
-	ListIterator iter = NULL;
+	list_t *node_info_list = NULL;
+	list_itr_t *iter = NULL;
 	node_info_msg_t *node_info = NULL;
 
 	node_info_list = list_create(_node_info_list_del);
@@ -2321,11 +2257,11 @@ static List _load_clusters_nodes(void)
  *
  * NOTE: caller must xfree() the returned name.
  */
-static char *_map_node_name(List clusters_node_info, char *name)
+static char *_map_node_name(list_t *clusters_node_info, char *name)
 {
 	char *nodename = NULL;
 	node_info_msg_t *node_info;
-	ListIterator node_info_itr;
+	list_itr_t *node_info_itr;
 
 	if (!name)
 		return NULL;
