@@ -339,7 +339,7 @@ START_TEST(test_data_job)
 	data_set_string(arg, "gpu:10");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_GRES, arg,
 						errors) == 0, "gres");
-	ck_assert_msg(!xstrcmp(opt.gres, "gres:gpu:10"), "gres value");
+	ck_assert_msg(!xstrcmp(opt.gres, "gres/gpu:10"), "gres value");
 
 	data_set_string(arg, "invalid");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_GRES_FLAGS, arg,
@@ -583,8 +583,8 @@ START_TEST(test_data_job)
 	opt.req_switch = 1;
 	data_set_null(arg);
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_SWITCHES, arg,
-						errors) == 0, "switches");
-	ck_assert_msg(opt.req_switch == 0, "switches value");
+						errors) != 0, "switches");
+	ck_assert_msg(opt.req_switch == 1, "switches value");
 	ck_assert_msg(opt.wait4switch == 12345, "wait 4 switches value");
 	opt.wait4switch = 12345;
 	opt.req_switch = 1;
@@ -705,7 +705,7 @@ START_TEST(test_data_job)
 
 	data_set_null(arg);
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_WAIT_ALL_NODES,
-						arg, errors) != 0,
+						arg, errors) == 0,
 		      "wait-all-nodes");
 	data_set_string(arg, "0");
 	ck_assert_msg(slurm_process_option_data(&opt, LONG_OPT_WAIT_ALL_NODES,
@@ -785,9 +785,8 @@ int main(void)
 	}
 
 	/* Do not load any plugins, we are only testing slurm_opt */
-	if (slurm_conf_init_load(slurm_unit_conf_filename, false) !=
-	    SLURM_SUCCESS) {
-		error("slurm_conf_init_load() failed");
+	if (slurm_conf_init(slurm_unit_conf_filename) != SLURM_SUCCESS) {
+		error("slurm_conf_init() failed");
 		return EXIT_FAILURE;
 	}
 
@@ -795,12 +794,6 @@ int main(void)
 	xfree(slurm_unit_conf_filename);
 	close(fd);
 
-	/* data_init() is necessary on this test */
-	if (data_init()) {
-		error("data_init_static() failed");
-		return EXIT_FAILURE;
-	}
-
 	/* Start the actual libcheck code */
 	int number_failed;
 	SRunner *sr = srunner_create(slurm_opt_suite());
@@ -809,8 +802,5 @@ int main(void)
 	number_failed = srunner_ntests_failed(sr);
 	srunner_free(sr);
 
-	/* Cleanup */
-	data_fini();
-
 	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
